@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_arch_c =
- *   "$Id: arch.c,v 1.28 1997/03/09 04:03:48 master Exp $";
+ *   "$Id: arch.c,v 1.33 1998/09/28 02:13:18 master Exp master $";
  */
 
 /*
@@ -28,6 +28,10 @@
 #include <global.h>
 #include <arch.h>
 #include <funcpoint.h>
+#include <loader.h>
+
+/* IF set, does a little timing on the archetype load. */
+#define TIME_ARCH_LOAD 0
 
 static archetype *arch_table[ARCHTABLE];
 int arch_cmp=0;		/* How many strcmp's */
@@ -73,7 +77,7 @@ void clear_archetable() {
 
 void init_archetable() {
   archetype *at;
-  LOG(llevDebug,"Setting up archetable...");
+  LOG(llevDebug," Setting up archetable...");
   for(at=first_archetype;at!=NULL;at=(at->more==NULL)?at->next:at->more)
     add_arch(at);
   LOG(llevDebug,"done\n");
@@ -84,11 +88,7 @@ void init_archetable() {
  */
 
 void dump_arch(archetype *at) {
-  unsigned char i;
   dump_object(&at->clone);
-  LOG(llevDebug,"Arch: %s [%d] = %s\n",at->name,at->animations,errmsg);
-  for(i=0; i < (int)at->animations; i++)
-    LOG(llevDebug,"Anim %d=%d\n",i,at->faces[i]);
 }
 
 /*
@@ -109,10 +109,15 @@ void free_all_archs()
     int i=0,f=0;
 
     for (at=first_archetype; at!=NULL; at=next) {
+#if 0	/* Right now, we could probably free the entries in the animation
+	 * indexes.  However, we should really call a seperate function
+	 * to do that.
+	 */
 	if (at->faces) {
 		free(at->faces);
 		f++;
 	}
+#endif
 	if (at->more) next=at->more;
 	else next=at->next;
 	free(at);
@@ -131,10 +136,8 @@ archetype *get_archetype_struct() {
   new=(archetype *)CALLOC(1,sizeof(archetype));
   if(new==NULL)
     fatal(OUT_OF_MEMORY);
-  new->faces=NULL;
   new->next=NULL;
   new->name=NULL;
-/*  new->randomitems=NULL;*/
   new->clone.other_arch=NULL;
   new->clone.name=NULL;
   new->clone.title=NULL;
@@ -146,7 +149,6 @@ archetype *get_archetype_struct() {
   SET_FLAG((&new->clone), FLAG_REMOVED); /* doesn't copy these flags... */
   new->head=NULL;
   new->more=NULL;
-  new->animations=0;
   return new;
 }
 
@@ -157,25 +159,28 @@ archetype *get_archetype_struct() {
 void first_arch_pass(FILE *fp) {
   object *op;
   archetype *at,*prev=NULL,*last_more=NULL;
-  int i;
+  int i,first=2;
 
   op=get_object();
   op->arch=first_archetype=at=get_archetype_struct();
-  while((i=load_object(fp,op))) {
+  while((i=load_object(fp,op,first))) {
+    first=0;
     copy_object(op,&at->clone);
     at->clone.speed_left= -0.1;
     switch(i) {
-    case 1: /* A new archetype, just link it with the previous */
+    case LL_NORMAL: /* A new archetype, just link it with the previous */
       if(last_more!=NULL)
         last_more->next=at;
       if(prev!=NULL)
         prev->next=at;
       prev=last_more=at;
       break;
+#if 0
     case 2:
       LOG(llevError,"Error: Archetype with inventory is illegal!\n");
       exit(-1);
-    case 3: /* Another part of the previous archetype, link it correctly */
+#endif
+    case LL_MORE: /* Another part of the previous archetype, link it correctly */
       at->head=prev;
       at->clone.head = &prev->clone;
       if(last_more!=NULL) {
@@ -250,33 +255,55 @@ void check_generators() {
  */
 
 void load_archetypes() {
-  FILE *fp;
-  char filename[MAX_BUF];
-  int comp;
+    FILE *fp;
+    char filename[MAX_BUF];
+    int comp;
+#if TIME_ARCH_LOAD
+    struct timeval tv1,tv2;
+#endif
 
-  sprintf(filename,"%s/%s",settings.libdir,settings.archetypes);
-  LOG(llevDebug,"Reading archetypes from %s...\n",filename);
-  if((fp=open_and_uncompress(filename,0,&comp))==NULL) {
-    LOG(llevError,"Can't open archetype file.\n");
-    return;
-  }
-  /*setlinebuf (fp);*/
-  clear_archetable();
-  LOG(llevDebug,"arch-pass 1...");
-  first_arch_pass(fp);
-  LOG(llevDebug,"done\n");
+    sprintf(filename,"%s/%s",settings.libdir,settings.archetypes);
+    LOG(llevDebug,"Reading archetypes from %s...\n",filename);
+    if((fp=open_and_uncompress(filename,0,&comp))==NULL) {
+	LOG(llevError," Can't open archetype file.\n");
+	return;
+    }
+    clear_archetable();
+    LOG(llevDebug," arch-pass 1...");
+#if TIME_ARCH_LOAD
+    gettimeofday(&tv1, NULL);
+#endif
+    first_arch_pass(fp);
+#if TIME_ARCH_LOAD
+    { int sec, usec;
+	gettimeofday(&tv2, NULL);
+	sec = tv2.tv_sec - tv1.tv_sec;
+	usec = tv2.tv_usec - tv1.tv_usec;
+	if (usec<0) { usec +=1000000; sec--;}
+	LOG(llevDebug,"Load took %d.%06d seconds\n", sec, usec);
+     }
+#endif
+
+  LOG(llevDebug," done\n");
   init_archetable();
   warn_archetypes=1;
-  rewind(fp);
-  LOG(llevDebug,"loading treasure...");
+
+  /* do a close and reopen instead of a rewind - necessary in case the
+   * file has been compressed.
+   */
+  close_and_delete(fp, comp);
+  fp=open_and_uncompress(filename,0,&comp);
+
+  LOG(llevDebug," loading treasure...");
   load_treasures();
-  LOG(llevDebug,"done\narch-pass 2...");
+  LOG(llevDebug,"done\n arch-pass 2...");
   second_arch_pass(fp);
   LOG(llevDebug,"done\n");
 #ifdef DEBUG
   check_generators();
 #endif
   close_and_delete(fp, comp);
+  LOG(llevDebug,"done\n");
 }
 
 /*
