@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_button_c =
- *   "$Id: button.c,v 1.29 1996/12/01 07:11:21 master Exp $";
+ *   "$Id: button.c,v 1.32 1998/05/16 01:59:28 master Exp $";
  */
 
 /*
@@ -67,24 +67,28 @@ void push_button(object *op) {
       update_ob_speed(tmp);
       break;
     case HANDLE:
-	tmp->face = &new_faces[tmp->arch->faces
-	    [tmp->value=tmp->stats.maxsp?!op->value:op->value]];
-      update_object(tmp);
-      break;
+	SET_ANIMATION(tmp, (tmp->value=tmp->stats.maxsp?!op->value:op->value));
+	update_object(tmp);
+	break;
     case SIGN:
-	(*info_map_func)(NDI_UNIQUE | NDI_NAVY,tmp->map,tmp->msg);
+	if (tmp->stats.food && ++tmp->last_eat>tmp->stats.food) {
+	    (*info_map_func)(NDI_UNIQUE | NDI_NAVY,tmp->map,	
+			    "Nothing is written on it.");
+	} else {
+	    (*info_map_func)(NDI_UNIQUE | NDI_NAVY,tmp->map,tmp->msg);
+	}
 	break;
     case ALTAR:
-      tmp->value = 1;
-      tmp->face = &new_faces[tmp->arch->faces[tmp->value]];
-      update_object(tmp);
-      break;
+	tmp->value = 1;
+	SET_ANIMATION(tmp, tmp->value);
+	update_object(tmp);
+	break;
     case BUTTON:
     case PEDESTAL:
-      tmp->value=op->value;
-      tmp->face = &new_faces[tmp->arch->faces[tmp->value]];
-      update_object(tmp);
-      break;
+	tmp->value=op->value;
+	SET_ANIMATION(tmp, tmp->value);
+	update_object(tmp);
+	break;
     case MOOD_FLOOR:
 	do_mood_floor(tmp, op);
 	break;
@@ -119,43 +123,45 @@ void push_button(object *op) {
  */
 
 void update_button(object *op) {
-  object *ab,*tmp;
-  int tot,any_down=0;
-  objectlink *ol;
+    object *ab,*tmp;
+    int tot,any_down=0;
+    objectlink *ol;
 
-  /* LOG(llevDebug, "update_button: %s (%d)\n", op->name, op->count); */
-  for (ol = get_button_links(op); ol; ol = ol->next) {
-    if (!ol->ob || ol->ob->count != ol->id) {
-      LOG(llevError, "Internal error in update_button (%s).\n", op->name);
-      continue;
+    /* LOG(llevDebug, "update_button: %s (%d)\n", op->name, op->count); */
+    for (ol = get_button_links(op); ol; ol = ol->next) {
+	if (!ol->ob || ol->ob->count != ol->id) {
+	    LOG(llevError, "Internal error in update_button (%s).\n", op->name);
+	    continue;
+	}
+	tmp = ol->ob;
+	if (tmp->type==BUTTON) {
+	    for(ab=tmp->above,tot=0;ab!=NULL;ab=ab->above)
+		if(!QUERY_FLAG(ab,FLAG_FLYING)) /* Why was nrof removed? */
+		    tot+=ab->weight*(ab->nrof?ab->nrof:1)+ab->carrying;
+	    tmp->value=(tot>=tmp->weight)?1:0;
+	    if(tmp->value)
+		any_down=1;
+	} else if (tmp->type == PEDESTAL) {
+	    tmp->value = 0;
+	    for(ab=tmp->above; ab!=NULL; ab=ab->above) {
+		if(ab->head!=NULL)
+		    ab=ab->head;
+		if ( (!QUERY_FLAG(ab,FLAG_FLYING) || 
+		      QUERY_FLAG(tmp,FLAG_FLY_ON)) &&
+		     (ab->race==tmp->slaying ||
+		      (!strcmp (tmp->slaying, "player") && 
+		       ab->type == PLAYER)))
+			    tmp->value = 1;
+	    }
+	    if(tmp->value)
+		any_down=1;
+	}
     }
-    tmp = ol->ob;
-    if (tmp->type==BUTTON) {
-      for(ab=tmp->above,tot=0;ab!=NULL;ab=ab->above)
-        if(!QUERY_FLAG(ab,FLAG_FLYING)) /* Why was nrof removed? */
-          tot+=ab->weight*(ab->nrof?ab->nrof:1)+ab->carrying;
-      tmp->value=(tot>=tmp->weight)?1:0;
-      if(tmp->value)
-        any_down=1;
-    } else if (tmp->type == PEDESTAL) {
-      tmp->value = 0;
-      for(ab=tmp->above; ab!=NULL; ab=ab->above) {
-        if(ab->head!=NULL)
-          ab=ab->head;
-        if ( (!QUERY_FLAG(ab,FLAG_FLYING) || QUERY_FLAG(tmp,FLAG_FLY_ON))
-	   && (ab->race==tmp->slaying ||
-             (!strcmp (tmp->slaying, "player") && ab->type == PLAYER)))
-          tmp->value = 1;
-      }
-      if(tmp->value)
-        any_down=1;
-    }
-  }
-  if(any_down) /* If any other buttons were down, force this to remain down */
-    op->value=1;
-  op->face= &new_faces[op->arch->faces[op->value]];
-  update_object(op);
-  push_button(op); /* Make all other buttons the same */
+    if(any_down) /* If any other buttons were down, force this to remain down */
+	op->value=1;
+    SET_ANIMATION(op, op->value);
+    update_object(op);
+    push_button(op); /* Make all other buttons the same */
 }
 
 /*
@@ -199,10 +205,10 @@ void use_trigger(object *op)
 
 void animate_turning(object *op) /* only one part objects */
 {
-    if (++op->state >= op->arch->animations/8)
-      op->state=0;
-    op->face = &new_faces[op->arch->faces[(op->stats.sp-1) *
-               op->arch->animations / 8 + op->state]];
+    if (++op->state >= NUM_ANIMATIONS(op)/8)
+      op->state=0; 
+    SET_ANIMATION(op, (op->stats.sp-1) * NUM_ANIMATIONS(op) / 8 + 
+		  op->state);
     update_object(op);
 }
 
@@ -215,6 +221,9 @@ void animate_turning(object *op) /* only one part objects */
  * unnecessarily - we can see if there is sufficient money, see if something
  * needs to be identified, and then remove money if needed. (via check_altar)
  * Check_altar uses this function also.
+ * 0.93.4: Linked objects (ie, objects that are connected) can not be
+ * sacrificed.  This fixes a bug of trying to put multiple altars/related
+ * objects on the same space that take the same sacrifice.
  */
  
 object *check_altar_sacrifice(object *altar)
@@ -222,7 +231,8 @@ object *check_altar_sacrifice(object *altar)
   object *op;
 
   for (op=altar->above; op; op=op->above)
-    if (!QUERY_FLAG(op,FLAG_ALIVE) && op->type != PLAYER) {
+    if (!QUERY_FLAG(op,FLAG_ALIVE) && !QUERY_FLAG(op,FLAG_IS_LINKED) &&
+	op->type != PLAYER) {
       if ((ARCH_SACRIFICE(altar) == op->arch->name  ||
           ARCH_SACRIFICE(altar) == op->name ||
 	  ARCH_SACRIFICE(altar) == op->slaying) 
@@ -279,21 +289,21 @@ int check_altar (object *altar)
 
 void trigger_move (object *op, int state) /* 1 down and 0 up */
 {
-  op->stats.wc = state;
-  op->face=&new_faces[op->arch->faces[op->stats.wc]];
-  update_object(op);
-  if (state) {
+    op->stats.wc = state;
+    SET_ANIMATION(op,op->stats.wc);
+    update_object(op);
+    if (state) {
 
-    /* Push button now does it all. */
-    use_trigger(op);
+	/* Push button now does it all. */
+	use_trigger(op);
 
-    op->speed =  op->arch->clone.speed;
-    update_ob_speed(op);
-    op->speed_left = -1;
-  } else {
-    op->speed = 0;
-    update_ob_speed(op);
-  }
+	op->speed =  op->arch->clone.speed;
+	update_ob_speed(op);
+	op->speed_left = -1;
+    } else {
+	op->speed = 0;
+	update_ob_speed(op);
+    }
 }
 
 void check_trigger (object *op) {
@@ -546,3 +556,22 @@ object *match;
     else if (!match && !trig->last_sp)
 	use_trigger(trig);
 }
+
+
+/* This does a minimal check of the button link consistency for object
+ * map.  All it really does it much sure the object id link that is set
+ * matches what the object has.
+ */
+void verify_button_links(mapstruct *map) {
+    oblinkpt *obp;
+    objectlink *ol;
+
+    if (!map) return;
+
+    for (obp = map->buttons; obp; obp = obp->next) {
+	for (ol=obp->link; ol; ol=ol->next) {
+	    if (ol->id!=ol->ob->count)
+	        LOG(llevError,"verify_button_links: object %s on list is corrupt (%d!=%d)\n",ol->ob->name, ol->id, ol->ob->count);
+	}
+    }
+}
