@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_glue_c =
- *   "$Id: glue.c,v 1.18 1996/10/29 09:57:42 master Exp $";
+ *   "$Id: glue.c,v 1.21 1998/09/28 02:13:56 master Exp master $";
  */
 
 /*
@@ -42,21 +42,18 @@ type_func_ob	remove_friendly_object_func;
 type_func_void	process_active_maps_func;
 type_func_map	update_buttons_func;
 type_func_int_int_ob_cchar	draw_info_func;
-type_func_ob	draw_stats_func;
-type_func_ob	draw_inventory_func;
 type_func_ob	draw_look_func;
-type_int_func_ob_ob	apply_func;
+type_int_func_ob_ob_int	apply_func;
 type_func_ob	draw_func;
 type_func_ob_ob	monster_check_apply_func;
-type_func_ob	draw_inventory_faces_func;
-type_func_ob	draw_look_faces_func;
 type_func_void	init_blocksview_players_func;
 type_func_int_map_char	info_map_func;
 type_func_ob	move_teleporter_func;
 type_func_ob_int  trap_adjust_func;
 type_func_ob    move_creator_func;
 type_func_ob_ob esrv_send_item_func;
-type_func_int_int esrv_del_item_func;
+type_func_player_int esrv_del_item_func;
+type_func_int_ob_ob esrv_update_item_func;
 
 static char *fatalmsgs[80]={
   "Failed to allocate memory",
@@ -82,12 +79,8 @@ void init_function_pointers() {
   process_active_maps_func = dummy_function;
   update_buttons_func = dummy_function_map;
   draw_info_func = dummy_draw_info;
-  draw_stats_func = dummy_function_ob;
-  draw_inventory_faces_func = dummy_function_ob;
-  draw_look_faces_func = dummy_function_ob;
-  draw_inventory_func = dummy_function_ob;
   draw_look_func = dummy_function_ob;
-  apply_func = dummy_function_ob2int;
+  apply_func = dummy_int_function_ob_ob_int;
   draw_func = dummy_function_ob;
   monster_check_apply_func = dummy_function_ob2;
   init_blocksview_players_func = dummy_function;
@@ -96,7 +89,8 @@ void init_function_pointers() {
   trap_adjust_func = dummy_function_ob_int;
   move_creator_func = dummy_function_ob;
   esrv_send_item_func = dummy_function_ob2;
-  esrv_del_item_func = dummy_function_int_int;
+  esrv_del_item_func = dummy_function_player_int;
+  esrv_update_item_func = dummy_function_int_ob_ob;
 
 }
 
@@ -162,42 +156,6 @@ void set_draw_info(type_func_int_int_ob_cchar addr) {
   draw_info_func = addr;
 }
 
-/*
- * Specify which function to call to update the text in the
- * stat-window of a player.
- */
-
-void set_draw_stats(type_func_ob addr) {
-  draw_stats_func = addr;
-}
-
-/*
- * Specify which function to call to update the graphics in the
- * inventory-window of a player.
- */
-
-void set_draw_inventory_faces(type_func_ob addr) {
-  draw_inventory_faces_func = addr;
-}
-
-/*
- * Specify which function to call to update the graphics
- * in the look-window of a player.
- */
-
-void set_draw_look_faces(type_func_ob addr) {
-  draw_look_faces_func = addr;
-}
-
-/*
- * Specify which function to call to update the content
- * in the inventory-window of a player.
- */
-
-void set_draw_inventory(type_func_ob addr) {
-  draw_inventory_func = addr;
-}
-
 /*
  * Specify which function to call to update the content
  * of the look-window of a player.
@@ -211,7 +169,7 @@ void set_draw_look(type_func_ob addr) {
  * Specify which function to call to apply an object.
  */
 
-void set_apply(type_int_func_ob_ob addr) {
+void set_apply(type_int_func_ob_ob_int addr) {
   apply_func = addr;
 }
 
@@ -261,7 +219,11 @@ void set_esrv_send_item (type_func_ob_ob addr) {
   esrv_send_item_func = addr;
 }
 
-void set_esrv_del_item(type_func_int_int addr) {
+void set_esrv_update_item (type_func_int_ob_ob addr) {
+  esrv_update_item_func = addr;
+}
+
+void set_esrv_del_item(type_func_player_int addr) {
   esrv_del_item_func = addr;
 }
 
@@ -293,6 +255,9 @@ void dummy_function_int(int i) {
 void dummy_function_int_int(int i, int j) {
 }
 
+void dummy_function_player_int(player *p, int j) {
+}
+
 
 void dummy_function() {
 }
@@ -328,5 +293,9 @@ void dummy_function_mapstr(int a, mapstruct *map, char *str) {
 void dummy_function_int_ob_ob (int n, object *ob, object *ob2) {
 }
 
+int dummy_int_function_ob_ob_int (object *ob, object *ob2, int n) {
+return 0;
+}
+
 #endif
 
