@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_image_c =
- *   "$Id: image.c,v 1.15 1997/03/09 04:05:21 master Exp $";
+ *   "$Id: image.c,v 1.18 1998/09/28 02:24:32 master Exp master $";
  */
 
 /*
@@ -55,6 +55,7 @@ MapLook blank_look;
  * it is easier to keep them like this.
  */
 New_Face *blank_face, *blocked_face, *stipple1_face, *stipple2_face;
+New_Face *dark_faces[3];
 New_Face *potion_face;
 
 
@@ -73,6 +74,28 @@ New_Face *inv_curse_face, *inv_damn_face, *inv_equip_face, *inv_lock_face,
  */
 int nroffiles = 0, nrofpixmaps=0;
 
+/* This now merges the color alias and color name table together.
+ * The first entry is the color as it is referred to in objects,
+ * the second is the actual screen/X11 color to use.  NUM_COLORS is now
+ * defined in global.h - if adding colors, increase that value, and everything
+ * else should work appropriately.
+ */
+char *colorname[NUM_COLORS][2] = {
+{"black",	"Black"},		/* 0  */
+{"white",	"White"},		/* 1  */
+{"blue",	"Navy"},		/* 2  */
+{"red",		"Red"},			/* 3  */
+{"orange",	"Orange"},		/* 4  */
+{"light_blue",	"DodgerBlue"},		/* 5  */
+{"dark_orange",	"DarkOrange2"},		/* 6  */
+{"green",	"SeaGreen"},		/* 7  */
+{"light_green",	"DarkSeaGreen"},	/* 8  - Used for window background color */
+{"grey",	"Grey50"},		/* 9  */
+{"brown",	"Sienna"},		/* 10 */
+{"yellow",	"Gold"},		/* 11 */
+{"khaki",	"Khaki"}		/* 12 */ 
+};
+
 static int compar (struct bmappair *a, struct bmappair *b) {
     return strcmp (a->name, b->name);
 }
@@ -233,7 +256,7 @@ int ReadBmapNames () {
 
     ReadFaceData();
 
-    for (i = 0; i <= nrofpixmaps; i++) {
+    for (i = 0; i < nrofpixmaps; i++) {
 	if (new_faces[i].magicmap==255) {
 #if 0	/* Useful for initial debugging, not needed now */
 	    LOG(llevDebug,"Face %s still had default magicmap, resetting to black\n",
@@ -252,6 +275,10 @@ int ReadBmapNames () {
     blank_face->bg = find_color ("khaki");
     blank_face->magicmap = find_color ("khaki") | FACE_FLOOR;
 
+	 dark_faces[0] = &new_faces[FindFace (DARK_FACE1_NAME,0)];
+	 dark_faces[1] = &new_faces[FindFace (DARK_FACE2_NAME,0)];
+	 dark_faces[2] = &new_faces[FindFace (DARK_FACE3_NAME,0)];
+
     blocked_face = &new_faces[FindFace (BLOCKED_FACE_NAME,0)];
     blocked_face->fg = find_color ("black");
     blocked_face->bg = find_color ("black");
