@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_init_c =
- *   "$Id: init.c,v 1.20 1997/03/09 04:06:13 master Exp $";
+ *   "$Id: init.c,v 1.26 1998/09/28 02:14:47 master Exp master $";
  */
 
 /*
@@ -33,24 +33,21 @@
  * correspond to.
  */
 struct Settings settings = {
-NULL, "logfile", PORT, CSPORT,	/*chrfont, logfile, port, new cs port */
-SERVER_OFF, 1, Dm_Font,		/* -server option, fixfontpath, displaymode */
-1, 0, 0,			/* synchronize, bw only, noadd */
+"logfile",			/* Logfile */
+CSPORT,				/* Client/server port */
+
+/* Debug level */
 #ifdef DEBUG
   llevDebug,
 #else
   llevError,
 #endif
-0, 0, 0,    /* split, dumpvalues, daemonmode */
+
+0, NULL, 0,    /* dumpvalues, dumparg, daemonmode */
 0, /* argc */
 NULL, /* argv */
-LIBDIR, FONTDIR, PLAYERDIR, MAPDIR, ARCHETYPES,TREASURES, 
-#ifdef ITEMS_DIR
-ITEMS_DIR ,
-#else
-"",
-#endif
-TMPDIR
+LIBDIR, PLAYERDIR, MAPDIR, ARCHETYPES,TREASURES, 
+UNIQUE_DIR, TMPDIR
 };
 
 
@@ -72,6 +69,7 @@ void init_library() {
     init_vars();
     init_block();
     ReadBmapNames ();
+    init_anim();    /* Must be after we read in the bitmaps */
     init_archetypes();	/* Reads all archetypes from file */
     init_dynamic ();
 }
@@ -81,13 +79,11 @@ void init_library() {
  * overwrite these if specified.
  */
 void init_environ() {
-#ifndef SECURE
     char *cp;
 
+#ifndef SECURE
     cp=getenv("CROSSFIRE_LIBDIR");
     if (cp) settings.libdir=cp;
-    cp=getenv("CROSSFIRE_FONTDIR");
-    if (cp) settings.fontdir=cp;
     cp=getenv("CROSSFIRE_PLAYERDIR");
     if (cp) settings.playerdir=cp;
     cp=getenv("CROSSFIRE_MAPDIR");
@@ -96,13 +92,11 @@ void init_environ() {
     if (cp) settings.archetypes=cp;
     cp=getenv("CROSSFIRE_TREASURES");
     if (cp) settings.treasures=cp;
-#ifdef UNIQUE_ITEMS
-    cp=getenv("CROSSFIRE_ITEMSDIR");
+    cp=getenv("CROSSFIRE_UNIQUEDIR");
     if (cp) settings.uniquedir=cp;
-#endif
-#endif
     cp=getenv("CROSSFIRE_TMPDIR");
     if (cp) settings.tmpdir=cp;
+#endif
 }
     
 
@@ -112,44 +106,28 @@ void init_environ() {
  */
 
 void init_globals() {
-  logfile = stderr;
-  exiting = 0;
-  delete_last_file=0;
-  first_player=NULL;
-  first_friendly_object=NULL;
-  first_map=NULL;
-  first_treasurelist=NULL;
-  first_artifactlist=NULL;
-  first_archetype=NULL;
-  warn_archetypes=0;
-  first_map=NULL;
-#if 0
-  write_state=0;
-  write_pos=0;
-  write_d=0;
-  button_flag=0;
-  last_bard=0;
-  fix_walls=0;
-  monsters = 1;
-  generators = 1;
-  hidden = 0;
-  others = 1;
-#endif
-  last_micro = 0;
-  nroftreasures = 0;
-  nrofartifacts = 0;
-  nrofallowedstr=0;
-  ring_arch = NULL;
-  amulet_arch = NULL;
-  staff_arch = NULL;
-  undead_name = add_string("undead");
-  name = strdup_local("crossfire");
-  active_socket = (sockets *) NULL;
-  first_socket = (sockets *) NULL;
-  font_graphic = FONTNAME;
-  trying_emergency_save = 0;
-
-  init_defaults();
+    logfile = stderr;
+    exiting = 0;
+    first_player=NULL;
+    first_friendly_object=NULL;
+    first_map=NULL;
+    first_treasurelist=NULL;
+    first_artifactlist=NULL;
+    first_archetype=NULL;
+    warn_archetypes=0;
+    first_map=NULL;
+    nroftreasures = 0;
+    nrofartifacts = 0;
+    nrofallowedstr=0;
+    ring_arch = NULL;
+    amulet_arch = NULL;
+    staff_arch = NULL;
+    undead_name = add_string("undead");
+    trying_emergency_save = 0;
+    num_animations=0;
+    animations=NULL;
+    animations_allocated=0;
+    init_defaults();
 }
 
 /*
@@ -187,7 +165,6 @@ void init_objects() {
 
 void init_defaults() {
   editor=0;
-  maplevel=0; /* What level the map we're editing has */
   nroferrors=0;
 }
 
