@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_item_c =
- *   "$Id: item.c,v 1.40 1997/01/05 09:38:44 master Exp $";
+ *   "$Id: item.c,v 1.43 1998/09/28 02:15:20 master Exp master $";
  */
 
 /*
@@ -28,6 +28,7 @@
 #include <global.h>
 #include <funcpoint.h>
 #include <living.h>
+#include <spells.h>
 
 #define	DESCRIBE_ABILITY(retbuf, variable, name) \
     if(variable) { \
@@ -45,7 +46,7 @@
     }
 
 static char numbers[21][20] = {
-  "no","a","two","three","four","five","six","seven","eight","nine","ten",
+  "no","","two","three","four","five","six","seven","eight","nine","ten",
   "eleven","twelve","thirteen","fourteen","fifteen","sixteen","seventeen",
   "eighteen","nineteen","twenty"
 };
@@ -67,8 +68,6 @@ static char levelnumbers_10[11][20] = {
   "seventieth","eightieth","ninetieth"
 };
 
-extern spell spells[NROFREALSPELLS];
-
 /*
  * query_weight(object) returns a character pointer to a static buffer
  * containing the text-representation of the weight of the given object.
@@ -211,11 +210,12 @@ char *query_short_name(object *op)
     if(op->nrof) {
 	strcpy(buf, get_number(op->nrof));
 
+#if 0
 	/* if nrof==1, then get_number returns 'a'. Add an 'n' if appropriate*/
 	if(op->nrof==1&&QUERY_FLAG(op,FLAG_AN))
 	    strcat(buf,"n");
-
-	strcat(buf, " ");
+#endif
+	if (op->nrof!=1) strcat(buf, " ");
 	strcat(buf,op->name);
 	if (op->nrof != 1) {
 	    char *buf3 = strstr(buf, " of ");
@@ -378,7 +378,8 @@ char *query_name(object *op) {
  * buffer which contains a verbose textual representation of the name
  * of the given object.  The buffer will be overwritten at the next
  * call to query_base_name().   This is a lot like query_name, but we
- * don't include the item count or item status.  Used for inventory sorting.
+ * don't include the item count or item status.  Used for inventory sorting
+ * and sending to client.
  */
 char *query_base_name(object *op) {
     static char buf[MAX_BUF];
@@ -577,6 +578,10 @@ char *describe_item(object *op) {
           sprintf(buf,"(ac%+d)",op->stats.ac);
           strcat(retbuf,buf);
         }
+	if (op->type==WEAPON && op->level>0) {
+	  sprintf(buf,"(improved %d/%d)",op->last_eat,op->level);
+          strcat(retbuf,buf);
+	}
         break;
       default:
         break;
@@ -878,6 +883,7 @@ int look_up_spell_name( char * spname ){
  */
 
 void identify(object *op) {
+  object *pl;
 
   SET_FLAG(op,FLAG_IDENTIFIED);
   CLEAR_FLAG(op, FLAG_KNOWN_MAGICAL);
@@ -909,4 +915,12 @@ void identify(object *op) {
 
   if (op->map) /* The shop identifies items before they hit the ground */
     update_object(op);
+  else {
+    pl = is_player_inv(op->env);
+    if (pl)
+	/* A lot of the values can change from an update - might as well send
+	 * it all.
+	 */
+	esrv_send_item_func(pl, op);
+  }
 }
