@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_living_c =
- *   "$Id: living.c,v 1.42 1997/02/04 09:48:32 master Exp $";
+ *   "$Id: living.c,v 1.48 1998/09/28 02:15:57 master Exp master $";
  */
 
 /*
@@ -26,7 +26,6 @@
 */
 
 #include <global.h>
-#include <living.h>
 #include <funcpoint.h>
 
 
@@ -48,48 +47,7 @@ static int grace_bonus[MAX_STAT +1] = {
   30,40,50,70,100
 };
 
-/* At around a 59 bonus, under the present system, it possible to make
- * money by buying an item then selling it.  I smoothed out the
- * increments some, making the gains more linear.  Before, in some cases,
- * the gain of a point might increase the bonus by 4, but the next point
- * gained only increased it by 2.  Now, for almost all positive bonuses,
- * it is increased by 3 per point.
- * Cost of buying an item is (6*(100-cha_bonus))/100
- * Cost of selling an item is 100/(100-cha_bonus)
- * This, it costs (60000-1200cha_bonus+6*cha_bonus^2)/10000 to buy something
- * than to sell something.
- * Various numbers of interest:
- * bonus	buy/sell value	cha to have that bonus
- * -100		24
- * -83		20.09		 (0)
- * -59		15.17		 (2)
- * -29		9.98		 (5)
- *   0		6.00		(12)
- *   8		5.08		(14)
- *  18		4.03		(18)
- *  29		3.02		(21)
- *  42		2.02		(26)
- *  59		1.009		(30+)
- *
- * What this shows is that a penalty is very harsh, and actual benefit
- * is of a diminishing nature.  Thus, I changed the cha_bonus so that
- * penalties are no longer so bad - a player should never have to pay more
- * than about 10 times to buy an item than what he gets for selling it.
- */
-#if 0	/* old bonus */
-int cha_bonus[MAX_STAT + 1]={
-  -80,-70,-60,-50,-40,-30,-25,-20,-15,-10,-7,-3,0,4,7,10,13,16,19,22,25,28,31,
-   34,37,40,43,46,49,52,55
-};
-
-/* old bonus, but not as old as the one above */
-int cha_bonus[MAX_STAT + 1]={
-  -35,-32,-29,-26,-23,-20,-17,-14,-11,-8,-5,-2,1,4,7,10,13,16,19,22,25,28,31,
-   34,37,40,43,46,49,52,55
-};
-#endif
-
-/* 0.92.7 Changed way charisma works (ignore above.)  Values now 
+/* 0.92.7 Changed way charisma works.  Values now 
  * represent how much more it costs to buy something than to sell it
  * (10, a value of 10 means it is that if it costs 50 gp to buy, you
  * would only get 5 gp when you sell.)  Let query_cost do the calculations
@@ -114,21 +72,47 @@ float cha_bonus[MAX_STAT + 1]={10.0, 10.0, 9.0, 8.0, 7.0, 6.0, /*<-5*/
 int dex_bonus[MAX_STAT + 1]={
   -4,-3,-2,-2,-1,-1,-1,0,0,0,0,0,0,0,1,1,1,2,2,2,3,3,3,4,4,4,5,5,6,6,7
 };
+
+/* speed_bonus uses dex as its stat */
 float speed_bonus[MAX_STAT + 1]={
   -0.4, -0.4, -0.3, -0.3, -0.2, -0.2, -0.2, -0.1, -0.1, -0.1, -0.05, 0, 0, 0,
   0.05, 0.1, 0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.8, 1.0, 1.2, 1.4,
   1.6, 1.8, 2.0, 2.5, 3.0
 };
+
+/* dam_bonus, thaco_bonus, max_carry, weight limit all are based on
+ * strength.
+ */
 int dam_bonus[MAX_STAT + 1]={
   -2,-2,-2,-1,-1,-1,0,0,0,0,0,0,1,1,1,2,2,2,3,3,3,4,4,5,5,6,6,7,8,10,15
 };
 int thaco_bonus[MAX_STAT + 1]={
   -2,-2,-1,-1,0,0,0,0,0,0,0,0,0,0,1,1,1,2,2,2,3,3,3,4,4,5,5,6,7,8,10
 };
+
+/* Max you can carry before you start getting extra speed penalties */
 int max_carry[MAX_STAT + 1]={
   2,4,7,11,16,22,29,37,46,56,67,79,92,106,121,137,154,172,191,211,232,254,277,
   301,326,352,400,450,500,600,1000
 };
+
+/* weight_limit - the absolute most a character can carry - a character can't
+ * pick stuff up if it would put him above this limit.
+ * value is in grams, so we don't need to do conversion later
+ * These limits are probably overly generous, but being there were no values
+ * before, you need to start someplace.
+ */
+
+int weight_limit[MAX_STAT+ 1] = {
+    200000,  /* 0 */
+    250000,300000,350000,400000,500000,	    /* 5*/
+    600000,700000,800000,900000,1000000,    /* 10 */
+    1100000,1200000,1300000,1400000,1500000,/* 15 */
+    1650000,1800000,1950000,2100000,2250000,/* 20 */
+    2400000,2550000,2700000,2850000,3000000, /* 25 */
+    3250000,3500000,3750000,4000000,4500000  /*30 */ 
+};
+
 int learn_spell[MAX_STAT + 1]={
   0,0,0,1,2,4,8,12,16,25,36,45,55,65,70,75,80,85,90,95,100,100,100,100,100,
   100,100,100,100,100,100
@@ -538,6 +522,7 @@ int change_abil(object *op, object *tmp) {
         (*draw_info_func)(NDI_UNIQUE, 0, op,"You float a little lower in the air.");
       else {
         (*draw_info_func)(NDI_UNIQUE, 0, op,"You float down to the ground.");
+	check_walk_on(op);
       }
     }
   }
@@ -774,7 +759,6 @@ void drain_specific_stat(object *op, int deplete_stats) {
   (*draw_info_func)(NDI_UNIQUE, 0, op, drain_msg[deplete_stats]);
   change_attr_value(&tmp->stats, deplete_stats, -1);
   fix_player(op);
-  if (op->type==PLAYER) (void)(*draw_stats_func)(op);
 }
 
 /*
@@ -805,6 +789,14 @@ void change_luck(object *op, int value) {
         LOG(llevDebug, "Internal error in change_luck().\n");
         return;
       }
+      /* Randomly change the players luck.  Basically, we move it
+       * back neutral (if greater>0, subtract, otherwise add)
+       * I believe this is supposed to be > and not >= - this means
+       * if your luck is -1/1, it won't get adjusted - only when your
+       * luck is worse can you hope for improvment.
+       * note that if we adjusted it with it is -1/1, that check above
+       * for 0 luck will happen, resulting in error.
+       */
       if (RANDOM()%(FABS(tmp->stats.luck)) > RANDOM()%30)
         tmp->stats.luck += tmp->stats.luck>0?-1:1;
     }
@@ -924,11 +916,11 @@ void fix_player(object *op) {
   for(tmp=op->inv;tmp!=NULL;tmp=tmp->below) 
     if(QUERY_FLAG(tmp,FLAG_APPLIED) && tmp->type!=CONTAINER && tmp->type!=CLOSE_CON) {
       if(op->type==PLAYER
-/*#ifdef ALLOW_SKILLS*/ /* The meaning of stats in skill or experience objects is different -
+#ifdef ALLOW_SKILLS /* The meaning of stats in skill or experience objects is different -
 		     * we use them solely to link skills to experience, thus it is 
 		     * inappropriate to allow these applied objects to change stats */ 
-/*           && tmp->type!=EXPERIENCE 
-#endif*/
+           && tmp->type!=EXPERIENCE 
+#endif
       ) {
         for(i=0;i<7;i++)
           change_attr_value(&(op->stats),i,get_attr_value(&(tmp->stats),i));
@@ -1157,11 +1149,12 @@ void fix_player(object *op) {
     if(!mana_obj || !mana_obj->level || op->type!=PLAYER) mana_obj = op;
     for(i=1,op->stats.maxsp=0;i<=mana_obj->level&&i<=10;i++) {
       j=op->contr->levsp[i]+sp_bonus[op->stats.Pow]/2;
-      if((i%2) && (sp_bonus[op->stats.Pow]%2))
-      if (sp_bonus[op->stats.Pow]>0)
+      if((i%2) && (sp_bonus[op->stats.Pow]%2)) {
+	if (sp_bonus[op->stats.Pow]>0)
             j++;
-      else
-          j--;
+	else
+	    j--;
+      }
       op->stats.maxsp+=j>1?j:1;
     }
     for(i=11;i<=mana_obj->level;i++)
@@ -1255,6 +1248,11 @@ void fix_player(object *op) {
     op->speed=max;
 
   if(op->type == PLAYER) {
+    /* f is a number the represents the number of kg above (positive num)
+     * or below (negative number) that the player is carrying.  If above
+     * weight limit, then player suffers a speed reduction based on how
+     * much above he is, and what is max carry is
+     */
     f=(op->carrying/1000)-max_carry[op->stats.Str];
     if(f>0) op->speed=op->speed/(1.0+f/max_carry[op->stats.Str]);
   }
@@ -1277,8 +1275,6 @@ void fix_player(object *op) {
     if(K<=0) K=0.01;
       S=op->speed/(K*s);
     op->contr->weapon_sp=S;
-    if(op->stats.hp!=-10000 && op->type==PLAYER)
-      (*draw_stats_func)(op);
   }
   /* I want to limit the power of small monsters with big weapons: */
   if(op->type!=PLAYER&&op->arch!=NULL&&
@@ -1537,7 +1533,7 @@ void player_lvl_adj(object *who, object *op) {
       if(who) fix_player(who);
       if(op->level>1 && op->type==EXPERIENCE) {
         sprintf(buf,"You are now level %d in %s based skills.",op->level,op->name);
-        if(who) (*draw_info_func)(NDI_UNIQUE, 0, who,buf);
+        if(who) (*draw_info_func)(NDI_UNIQUE|NDI_RED, 0, who,buf);
       }  
       player_lvl_adj(who,op); /* To increase more levels */
     } else if(op->level>1&&op->stats.exp<level_exp(op->level,op->expmul)) {
@@ -1545,7 +1541,7 @@ void player_lvl_adj(object *who, object *op) {
       if(who) fix_player(who);
       if(op->type==EXPERIENCE) {
         sprintf(buf,"You are now level %d in %s based skills.",op->level,op->name);
-        if(who) (*draw_info_func)(NDI_UNIQUE, 0, who,buf);
+        if(who) (*draw_info_func)(NDI_UNIQUE|NDI_RED, 0, who,buf);
       }
       player_lvl_adj(who,op); /* To decrease more levels */
     }
