@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_loger_c =
- *   "$Id: logger.c,v 1.3 1997/03/09 04:13:24 master Exp master $ ";
+ *   "$Id: logger.c,v 1.5 1998/09/28 02:16:22 master Exp master $ ";
  */
 
 /*
@@ -25,7 +25,6 @@
     The author can be reached via e-mail to frankj@ifi.uio.no.
 */
 
-#ifndef NO_LOG
 
 #include <stdarg.h>
 #include <global.h>
@@ -40,36 +39,20 @@
 
 void LOG (LogLevel logLevel, char *format, ...)
 {
-#ifdef SERVER
   char buf[20480];  /* This needs to be really really big - larger
 		     * than any other buffer, since that buffer may
 		     * need to be put in this one.
 		     */
-  sockets *s;
-#endif
 
   va_list ap;
   va_start(ap, format);
 
-#ifdef SERVER
   buf[0] = '\0';
   if (logLevel <= settings.debug)
   {
     vsprintf(buf, format, ap);
     fputs(buf, logfile);
   }
-  for (s = first_socket; s != (sockets *) NULL; s = s->next)
-    if(s->debug && logLevel <= s->debug)
-    {
-      if(buf[0] == '\0')
-        vsprintf(buf, format, ap);
-      if(write(s->fd,buf,strlen(buf)) == -1)
-        perror("log/write");
-    }
-#else
-  if (logLevel <= settings.debug)
-    vfprintf(logfile, format, ap);
-#endif
   if (!exiting && !trying_emergency_save &&
       logLevel == llevError && ++nroferrors > MAX_ERRORS) {
     exiting = 1;
@@ -80,4 +63,3 @@ void LOG (LogLevel logLevel, char *format, ...)
   }
   va_end(ap);
 }
-#endif
