@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_map_c =
- *   "$Id: map.c,v 1.48 1997/03/09 04:15:09 master Exp master $";
+ *   "$Id: map.c,v 1.55 1998/09/28 02:16:42 master Exp master $";
  */
 
 /*
@@ -28,13 +28,17 @@
 
 #include <global.h>
 #include <funcpoint.h>
-#if defined(UNIQUE_ITEMS) && defined (LOCK_ITEMS)
-#include <dirent.h>
-#endif
+
+#include <loader.h>
+
+#include <unistd.h>
 
 extern int nrofallocobjects,nroffreeobjects;
 
 #if defined (MACH) || defined (NeXT) || defined (__MACH__)
+#ifndef S_ISGID
+#define S_ISGID 0002000
+#endif
 #ifndef S_IWOTH
 #define S_IWOTH 0000200
 #endif
@@ -98,6 +102,7 @@ mapstruct *has_been_loaded (char *name) {
  * This makes a path absolute outside the world of Crossfire.
  * In other words, it prepends LIBDIR/MAPDIR/ to the given path
  * and returns the pointer to a static array containing the result.
+ * it really should be called create_mapname
  */
 
 char *create_pathname (char *name) {
@@ -113,7 +118,6 @@ char *create_pathname (char *name) {
     return (buf);
 }
 
-#ifdef UNIQUE_ITEMS
 /*
  * This makes absolute path to the itemfile where unique objects
  * will be saved. Converts '/' to '@'. I think it's essier maintain
@@ -138,63 +142,29 @@ static char *create_items_path (char *s) {
     return (buf);  
 }
 
-#ifdef LOCK_ITEMS
-/*
- * Return 1 if lock is created, 0 if file is locked, -1 other error
- */
-static int lock_unique_items(mapstruct *map, int v)
-{
-    char buf[MAX_BUF];
-    int fd;
-
-    sprintf (buf, "%s.l%02d", create_items_path(map->path), v);
-    fd = open (buf, O_CREAT|O_EXCL, 0644);
-    if (fd < 0)
-      if (errno == EEXIST)
-          return 0;
-        else
-          return -1;
-    close (fd);
-    map->lock_items = v;
-    return 1;
-}
-
-static void unlock_unique_items(mapstruct *m) {
-    char buf[MAX_BUF];
-
-    if (m->lock_items) {
-      sprintf (buf, "%s.l%02d", create_items_path(m->path), m->lock_items);
-      m->lock_items = 0;
-      unlink(buf);
-    }
-}
- 
-void clean_lockfiles() {
-    mapstruct *m;
- 
-    LOG(llevDebug, "Cleaning unique items filelocks.\n");
-    for(m=first_map; m!=NULL; m=m->next)
-      if (m->lock_items)
-          unlock_unique_items(m);
-}
-#endif /* LOCK_ITEMS */
-#endif /* UNIQUE_ITEMS */
-
 
 /*
  * This function checks if a file with the given path exists.
- * -1 is returned if it failes, otherwise the mode of the file
+ * -1 is returned if it fails, otherwise the mode of the file
  * is returned.
  * It tries out all the compression suffixes listed in the uncomp[] array.
+ *
+ * If prepend_dir is set, then we call create_pathname (which prepends
+ * libdir & mapdir).  Otherwise, we assume the name given is fully
+ * complete.
  */
 
-int check_path (char *name)
+int check_path (char *name, int prepend_dir)
 {
     char buf[MAX_BUF], *endbuf;
     struct stat statbuf;
     int mode = 0, i;
 
-    strcpy (buf, create_pathname(name));
+    if (prepend_dir)
+	strcpy (buf, create_pathname(name));
+    else
+	strcpy(buf, name);
+
     if ((endbuf = strchr(buf, '\0')) == NULL)
       return (-1);
    
@@ -345,6 +315,8 @@ int blocked_link(object *ob, int x, int y) {
  * Eneq(@csd.uu.se): This is a new version of blocked, this one handles objects
  * that can be passed through by monsters with the CAN_PASS_THRU defined.
  * Returns 1 if the object can not pass through that space.
+ * This code really needs to be rewritten so it would at least be somewhat
+ * reasonable to read.
  */
 
 int blocked_two(object *op, int x,int y) {
@@ -352,12 +324,28 @@ int blocked_two(object *op, int x,int y) {
   if(out_of_map(op->map,x,y))
     return 1;
   tmp=get_map_ob(op->map,x,y);
-  for(tmp=get_map_ob(op->map,x,y);tmp!=NULL;tmp=tmp->above)
+  for(tmp=get_map_ob(op->map,x,y);tmp!=NULL;tmp=tmp->above){
     if((QUERY_FLAG(tmp,FLAG_ALIVE)&&tmp->type!=DOOR)||
-	(QUERY_FLAG(tmp,FLAG_NO_PASS)&&!QUERY_FLAG(tmp,FLAG_PASS_THRU))||
+	(QUERY_FLAG(tmp,FLAG_NO_PASS)&&!QUERY_FLAG(tmp,FLAG_PASS_THRU)&&tmp->type!=CHECK_INV)||
         (QUERY_FLAG(tmp,FLAG_NO_PASS)&&QUERY_FLAG(tmp,FLAG_PASS_THRU)&&
-	!QUERY_FLAG(op,FLAG_CAN_PASS_THRU)))
-      		return 1;
+	!QUERY_FLAG(op,FLAG_CAN_PASS_THRU)&&tmp->type!=CHECK_INV))
+      return 1;
+    /* check inv doors by Florian Beck */
+    /* if check_inv has last_sp=1, the player can only move if he has the 
+     * object 
+     */
+    if (tmp->type==CHECK_INV&&QUERY_FLAG(tmp,FLAG_NO_PASS)) {
+      if (tmp->last_sp){ /* player needs object */
+	if (check_inv_recursive(op,tmp)==NULL) /* player doesn't have object 
+						  so no pass */
+	  return 1;
+      } else
+	/* player must *not* have object */
+	if (check_inv_recursive(op,tmp)!=NULL) /* player has object */
+	  return 1;
+    }
+  }
+  /* end fb */
   return 0;
 }
 
@@ -575,7 +563,7 @@ void load_objects (mapstruct *m, FILE *fp, int block) {
     op=get_object();
     op->map = m; /* To handle buttons correctly */
 
-    while((i=load_object(fp,op))) {
+    while((i=load_object(fp,op,LO_REPEAT))) {
 	/* if the archetype for the object is null, means that we
 	 * got an invalid object.  Don't do anythign with it - the game
 	 * or editor will not be able to do anything with it either.
@@ -594,17 +582,21 @@ void load_objects (mapstruct *m, FILE *fp, int block) {
 		LOG(llevDebug,"Baptized altar to %s\n",op->title);
 #endif
 	switch(i) {
-	  case 1:
+	  case LL_NORMAL:
 	    insert_ob_in_map(op,m);
+#if 1
+	    if (op->inv) sum_weight(op);
+#else
 	    if (op->inv) fix_container(op);
+#endif
 	    prev=op,last_more=op;
 	    break;
-
+#if 0	/* internal inventories should be handled by load routine */
 	  case 2:
 	    op = insert_ob_in_ob(op,prev);
 	    break;
-
-	  case 3:
+#endif
+	  case LL_MORE:
 	    insert_ob_in_map(op,m);
 	    op->head=prev,last_more->more=op,last_more=op;
 	    break;
@@ -733,25 +725,33 @@ void save_objects (mapstruct *m, FILE *fp, FILE *fp2) {
  * Opens the file "filename" and reads information about the map
  * from the given file, and stores it in a newly allocated
  * mapstruct.  A pointer to this structure is returned, or NULL on failure.
- * If block is true, it will block the game while reading the objects.
+ * flags correspond to those in map.h.  Main ones used are
+ * MAP_PLAYER_UNIQUE, in which case we don't do any name changes, and
+ * MAP_BLOCK, in which case we block on this load.
  */
 
-mapstruct *load_original_map(char *filename, int block) {
+mapstruct *load_original_map(char *filename, int flags) {
     FILE *fp;
     mapstruct *m;
     object *op;
     int comp;
+    char pathname[MAX_BUF];
     
     LOG(llevDebug, "load_original_map: %s\n", filename);
-    if((fp=open_and_uncompress(create_pathname(filename), 0, &comp))==NULL) {
-      LOG(llevError,"Can't open %s\n", create_pathname(filename));
+    if (flags & MAP_PLAYER_UNIQUE) 
+	strcpy(pathname, filename);
+    else
+	strcpy(pathname, create_pathname(filename));
+
+    if((fp=open_and_uncompress(pathname, 0, &comp))==NULL) {
+	LOG(llevError,"Can't open %s\n", pathname);
 	perror("Can't read map file");
 	return (NULL);
     }
         
     op = get_object();
 
-    if (!load_object(fp, op) || op->type != MAP) {
+    if (!load_object(fp, op,LO_NEWFILE) || op->type != MAP) {
 	LOG(llevError,"Error in map (%s) - map object not found\n", filename);
         close_and_delete(fp, comp);
 	return (NULL);
@@ -775,7 +775,7 @@ mapstruct *load_original_map(char *filename, int block) {
     op->map = m;
 
     m->in_memory=MAP_LOADING;
-    load_objects (m, fp, block);
+    load_objects (m, fp, (flags & MAP_BLOCK?1:0));
     close_and_delete(fp, comp);
     m->in_memory=MAP_IN_MEMORY;
     m->difficulty=calculate_difficulty(m);
@@ -788,9 +788,11 @@ mapstruct *load_original_map(char *filename, int block) {
 
 /*
  * Loads a map, which has been loaded earlier, from file.
+ * Return the map object we load into (this can change from the passed
+ * option if we can't find the original map)
  */
 
-void load_temporary_map(mapstruct *m) {
+static mapstruct *load_temporary_map(mapstruct *m) {
     FILE *fp;
     object *op;
     int comp;
@@ -802,7 +804,7 @@ void load_temporary_map(mapstruct *m) {
 	delete_map(m);
         m = load_original_map(buf,0);
 	(*fix_auto_apply_func)(m); /* Chests which open as default */
-	return;
+	return m;
     }
 
     if((fp=open_and_uncompress(m->tmpname,0, &comp))==NULL) {
@@ -812,16 +814,16 @@ void load_temporary_map(mapstruct *m) {
 	delete_map(m);
         m = load_original_map(buf,0);
 	(*fix_auto_apply_func)(m); /* Chests which open as default */
-	return;
+	return m;
     }
     
     op = get_object();
 
-    load_object(fp,op);
+    load_object(fp,op,LO_NEWFILE);
     if (op->arch == NULL || op->type != MAP) {
 	LOG(llevError,"Error in temporary map '%s'\n", m->path);
         m = load_original_map(m->path,0);
-	return;
+	return m;
     }
 
     m->compressed = comp;
@@ -838,49 +840,15 @@ void load_temporary_map(mapstruct *m) {
     load_objects (m, fp, 0);
     close_and_delete(fp, comp);
     m->in_memory=MAP_IN_MEMORY;
+    return m;
 }
 
-#ifdef UNIQUE_ITEMS
-void merge_files (char *s1, char *s2)
-{
-    char buf[MAX_BUF];
-    FILE *fp1, *fp2;
-    int i;
+/******************************************************************************
+ * This is the start of unique map handling code
+ *****************************************************************************/
 
-    if ((fp1 = fopen (s1, "a+")) == NULL) {
-      LOG(llevError, "Can't open file for append '%s'\n", s1);
-      return;
-    }
-    if ((fp2 = fopen (s2, "r")) == NULL) {
-      LOG(llevError, "Can't open file for reading '%s'\n", s2);
-      fclose (fp1);
-      return;
-    }
-    while ((i = fread(buf, 1, MAX_BUF, fp2)) > 0)
-      fwrite (buf, 1, i, fp1);
-    fclose (fp1);
-    fclose (fp2);
-}
-
-/*
- * returns version number if s1 is filename which is for map s2
- */
-int match_filename(char *s1, char *s2)
-{
-    int n;
-
-    if (*s2 == '/')
-      s2++;
-    
-    while (*s2) {
-      if (*s1 != *s2 && *s2 != '/' && *s1 != '@')
-          return 0;
-      s1++, s2++;
-    }
-    return sscanf(s1, ".v%d", &n) == 1 ? n : 0;
-}
-
-void delete_unique_items(mapstruct *m)
+/* This goes through map 'm' and removed any unique items on the map. */
+static void delete_unique_items(mapstruct *m)
 {
     int i,j,unique=0;
     object *op, *next;
@@ -906,103 +874,59 @@ void delete_unique_items(mapstruct *m)
 
 /*
  * Loads unique objects from file(s) into the map which is in memory
+ * m is the map to load unique items into.
  */
-void load_unique_objects(mapstruct *m) {
+static void load_unique_objects(mapstruct *m) {
     FILE *fp;
-    int comp;
+    int comp,count;
     char firstname[MAX_BUF];
 
-#ifdef LOCK_ITEMS
-    int v, firstlock = 0;
-    DIR *dirptr;
-    struct xdirect *dp;
-
-    if ((dirptr = opendir(create_items_path(""))) == NULL) {
-      LOG(llevDebug,"Can't open dir %s\n", create_items_path(""));
+    for (count=0; count<10; count++) {
+	sprintf(firstname, "%s.v%02d", create_items_path(m->path), count);
+	if (!access(firstname, R_OK)) break;
+    }
+    if ((fp=open_and_uncompress(firstname, 0, &comp))==NULL) {
+	/* There is no expectation that every map will have unique items, but this
+	 * is debug output, so leave it in.
+	 */
+	LOG(llevDebug, "Can't open unique items file for %s\n", create_items_path(m->path));
       return;
     }
-    while ((dp = xreaddir(dirptr,0)) != NULL ) {
-      if ((v = match_filename(dp->d_name,  m->path))) {
-          if (lock_unique_items(m, v) != 1) {
-              LOG(llevDebug,"Unique items are already locked.\n");
-              continue;
-          }
-          if (firstlock == 0) {
-              strcpy (firstname, create_items_path(dp->d_name));
-              firstlock = v;
-          } else {
-              merge_files (firstname, create_items_path(dp->d_name));
-              unlink (create_items_path(dp->d_name));
-              unlock_unique_items(m);     /* clears lock_items, so */
-              m->lock_items = firstlock; /* put original lock back */
-          }
-      }
-    }   
-    closedir(dirptr);
-    if (firstlock == 0)
-      return;
-#else 
-    strcpy (firstname, create_items_path(m->path));
-    strcat (firstname, ".v00");
 
-#endif /* LOCK_ITEMS */
-
-    if(! (fp=open_and_uncompress(firstname, 0, &comp))) {
-      LOG(llevDebug, "Can't open unique items file %s\n", firstname);
-      return;
-    }
     m->in_memory=MAP_LOADING;
     if (m->tmpname == NULL)    /* if we have loaded unique items from */
       delete_unique_items(m); /* original map before, don't duplicate them */
+    load_object(fp, NULL, LO_NOREAD);
     load_objects (m, fp, 0);
     close_and_delete(fp, comp);
     m->in_memory=MAP_IN_MEMORY;
 }
-#endif
 
-#define MAXQUEUE 10000
-static object* ob_queue[MAXQUEUE];
-static int ob_queue_start=0;
-static int ob_queue_end=0;
-    
-void queue_push(object* m_ob){
-    ob_queue[ob_queue_end++]=m_ob;
-    if(ob_queue_end==MAXQUEUE)
-      ob_queue_end=0;
-}
-    
-object *queue_pop(void){
-    object* m_ob;
-    if(ob_queue_start==ob_queue_end)
-      return(NULL);
-    m_ob=ob_queue[ob_queue_start++];
-    if(ob_queue_start==MAXQUEUE)
-      ob_queue_start=0;
-    return(m_ob);
-}
 
 /*
  * Saves a map to file.  If flag is set, it is saved into the same
  * file it was (originally) loaded from.  Otherwise a temporary
  * filename will be genarated, and the file will be stored there.
  * The temporary filename will be stored in the mapstructure.
+ * If the map is unique, we also save to the filename in the map
+ * (this should have been updated when first loaded)
  */
 
 int new_save_map(mapstruct *m, int flag) {
-    FILE *fp;
-    char filename[MAX_BUF];
-#ifdef UNIQUE_ITEMS
-    FILE *fp2;
-    char buf[MAX_BUF];
-#endif
+    FILE *fp, *fp2;
+    char filename[MAX_BUF],buf[MAX_BUF];
     
     if (flag && !*m->path) {
 	LOG(llevError,"Tried to save map without path.\n");
 	return -1;
     }
     
-    if(flag) {
-	strcpy (filename, create_pathname (m->path));
+    if (flag || QUERY_FLAG(m->map_object, FLAG_UNIQUE)) {
+	if (!QUERY_FLAG(m->map_object, FLAG_UNIQUE))
+	    strcpy (filename, create_pathname (m->path));
+	else
+	    strcpy (filename, m->path);
+
 	/* If the compression suffix already exists on the filename, don't
 	 * put it on again.  This nasty looking strcmp checks to see if the
 	 * compression suffix is at the end of the filename already.
@@ -1014,83 +938,67 @@ int new_save_map(mapstruct *m, int flag) {
 	make_path_to_file (filename);
     } else {
 	if (!m->tmpname)
-	    m->tmpname = tempnam(settings.tmpdir,NULL);
+	    m->tmpname = tempnam_local(settings.tmpdir,NULL);
 	strcpy(filename, m->tmpname);
     }
     LOG(llevDebug,"Saving map %s\n",m->path);
     m->in_memory = MAP_SAVING;
-    if (m->compressed && flag) { /* Compress if it isn't a temporary save */
-      char buf[MAX_BUF];
-      strcpy(buf, uncomp[m->compressed][2]);
-      strcat(buf, " > ");
-      strcat(buf, filename);
-      fp = popen(buf, "w");
-    } else
-      fp = fopen(filename, "w");
-    if(fp == NULL) {
-      perror("Can't open file for saving");
-      return -1;
-    }
-    
+
     if (!m->map_object) {
 	LOG(llevError, "no map object for map %s!\n", m->path);
 	return -1;
     }
 
+    if (m->compressed && flag) { /* Compress if it isn't a temporary save */
+	char buf[MAX_BUF];
+	strcpy(buf, uncomp[m->compressed][2]);
+	strcat(buf, " > ");
+	strcat(buf, filename);
+	fp = popen(buf, "w");
+    } else
+	fp = fopen(filename, "w");
+
+    if(fp == NULL) {
+	perror("Can't open file for saving");
+	return -1;
+    }
+    
     save_object (fp, m->map_object, 3);
 
-#ifdef UNIQUE_ITEMS
     /* In the game save unique items in the different file, but
      * in the editor save them to the normal map file.
+     * If unique map, save files in the proper destination (set by
+     * player)
      */
     fp2 = fp; /* save unique items into fp2 */
-    if (flag == 0) {
-      int lock = 0;
-#ifdef LOCK_ITEMS
-      lock = m->lock_items;
-      if (lock == 0) {
-          do {
-              lock++;
-          } while (lock < 10 && lock_unique_items(m, lock) != 1);
-          if (lock >= 10) {
-              LOG(llevError, "Over 10 item locks in map %s!\n", m->path);
-          }
-      }
-#endif
-      strcpy (buf, create_items_path (m->path));
-      sprintf (buf + strlen(buf), ".v%02d", lock);
-      if ((fp2 = fopen (buf, "w")) == NULL) {
-          LOG(llevError, "Can't open items file %s\n", buf);
-      }
-      save_objects (m, fp, fp2);
-      if (fp2 != NULL) {
-      if (ftell (fp2) == 0) {
-        /* remove the file if it's empty */
-        fclose (fp2);
-        unlink (buf);
-      } else {
-        fclose (fp2);
-        chmod (buf, SAVE_MODE);
-      }
-      }
-#ifdef LOCK_ITEMS
-      unlock_unique_items(m);
-#endif
+    if (flag == 0 && !QUERY_FLAG(m->map_object, FLAG_UNIQUE)) {
+	sprintf (buf,"%s.v00",create_items_path (m->path));
+	if ((fp2 = fopen (buf, "w")) == NULL) {
+	    LOG(llevError, "Can't open unique items file %s\n", buf);
+	}
+	save_objects (m, fp, fp2);
+	if (fp2 != NULL) {
+	    if (ftell (fp2) == 0) {
+		fclose (fp2);
+		unlink (buf);
+	    } else {
+		fclose (fp2);
+		chmod (buf, SAVE_MODE);
+	    }
+	}
     } else { /* save same file when not playing, like in editor */
-      save_objects (m, fp, fp);
+	save_objects (m, fp, fp);
     }
-#else
-    save_objects (m, fp, fp);
-#endif /* UNIQUE_ITEMS */
 
     if (m->compressed && !flag)
-      pclose(fp);
+	pclose(fp);
     else
-      fclose(fp);
+	fclose(fp);
 
-    chmod (filename, 0644);
+    chmod (filename, SAVE_MODE);
     return 0;
 }
+
 /*
  * If any directories in the given path doesn't exist, they are created.
  */
@@ -1106,18 +1014,19 @@ void make_path_to_file (char *filename)
     LOG(llevDebug, "make_path_tofile %s...", filename);
     while ((cp = strchr (cp + 1, (int) '/'))) {
 	*cp = '\0';
-	LOG(llevDebug, "Checking %s...", buf);
-	if (stat (buf, &statbuf) || !S_ISDIR (statbuf.st_mode)) {
+	LOG(llevDebug, "\n Checking %s...", buf);
+	if (stat(buf, &statbuf) || !S_ISDIR (statbuf.st_mode)) {
 	    LOG(llevDebug, "Was not dir...");
 	    if (mkdir (buf, 0777)) {
 		perror ("Couldn't make path to file");
 		return;
 	    }
-	    LOG(llevDebug, "Made dir.\n");
+	    LOG(llevDebug, "Made dir.");
 	} else
-	    LOG(llevDebug, "Was dir\n");
+	    LOG(llevDebug, "Was dir");
 	*cp = '/';
     }
+    LOG(llevDebug,"\n");
 }
 
 
@@ -1134,11 +1043,9 @@ void clear_map(mapstruct *m) {
     endptr=m->floor+m->mapx*m->mapy;
     for(aptr=m->floor; aptr < endptr; ++aptr)
 	*aptr = blank_look;
-#ifdef DOUBLE_FLOOR
     endptr=m->floor2+m->mapx*m->mapy;
     for(aptr=m->floor2; aptr < endptr; ++aptr)
 	*aptr = blank_look;
-#endif
     memset(m->map_ob, 0,sizeof(object *)*m->mapx*m->mapy);
 }
 
@@ -1209,28 +1116,6 @@ void clean_object(object *op)
     }
 }
 
-#if 0 /* Why make it so difficult?  I believe the below is buggy */
-
-void clean_object(object *op) {
-  object *tmp;
-  while((tmp=op->inv)!=NULL) {
-    clean_object(tmp);
-
-/* Eneq(@csd.uu.se): We'll have to move up the inventory or else op->inv
-   always will point to the same object. */
-    
-    if (tmp->below!=NULL)
-      tmp->below->above=NULL;
-    op->inv=tmp->below;
-    tmp->below=NULL;
-
-    remove_ob(tmp);
-    free_object(tmp);
-  }
-}
-
-#endif
-
 /*
  * Remove and free all objects in the given map.
  */
@@ -1297,6 +1182,13 @@ void delete_map(mapstruct *m) {
       return;
     if (m->in_memory == MAP_IN_MEMORY)
 	free_map (m, 1);
+    /* move this out of free_map, since tmpname can still be needed if
+     * the map is swapped out.
+     */
+    if (m->tmpname) {
+	free(m->tmpname);
+	m->tmpname=NULL;
+    }
     if (m == first_map)
 	first_map = m->next;
     else {
@@ -1325,17 +1217,13 @@ void free_map(mapstruct *m,int flag) {
     CFREE(m->map_ob);
   if (m->floor !=NULL)
     CFREE(m->floor);
-#ifdef DOUBLE_FLOOR
   if (m->floor2 !=NULL)
     CFREE(m->floor2);
-#endif
   m->map = NULL;/* It's being accessed again, incorrectly.  Maybe this will */
 		/* make it get loaded back in. */
   m->map_ob = NULL;
   m->floor = NULL;
-#ifdef DOUBLE_FLOOR
   m->floor2 = NULL;
-#endif
   if (m->map_object)
       free_object (m->map_object);
   m->map_object=NULL;
@@ -1374,9 +1262,6 @@ mapstruct *get_linked_map(char *path) {
   map->pending = (objectlink *) NULL;
   map->map_object = NULL;
   map->buttons = NULL;
-#if defined(UNIQUE_ITEMS) && defined(LOCK_ITEMS)
-  map->lock_items = 0;
-#endif
   map->compressed = 0;
 
   return map;
@@ -1393,14 +1278,9 @@ void allocate_map(mapstruct *m) {
   m->map=(MapLook *) CALLOC(m->mapx*m->mapy,sizeof(MapLook));
   m->floor=(MapLook *) CALLOC(m->mapx*m->mapy,sizeof(MapLook));
   m->map_ob=(object **) CALLOC(m->mapx*m->mapy,sizeof(object *));
-#ifdef DOUBLE_FLOOR
   m->floor2=(MapLook *) CALLOC(m->mapx*m->mapy,sizeof(MapLook));
   if(m->map==NULL||m->map_ob==NULL || m->floor==NULL || m->floor2==NULL)
     fatal(OUT_OF_MEMORY);
-#else
-  if(m->map==NULL||m->map_ob==NULL || m->floor==NULL)
-    fatal(OUT_OF_MEMORY);
-#endif
 }
 
 /*
@@ -1429,12 +1309,17 @@ mapstruct *get_empty_map (int sizex, int sizey) {
 
 /*
  * Makes sure the given map is loaded and swapped in.
- * name is path name of the map.  flush is true if called from
- * the editor, false otherwise.
+ * name is path name of the map.
+ * flags meaning:
+ * 0x1 (MAP_FLUSH): flush the map - always load from the map directory,
+ *   and don't do unique items or the like.
+ * 0x2 (MAP_PLAYER_UNIQUE) - this is a unique map for each player.
+ *   dont do any more name translation on it.
+ *
  * Returns a pointer to the given map.
  */
 
-mapstruct *ready_map_name(char *name, int flush) {
+mapstruct *ready_map_name(char *name, int flags) {
     mapstruct *m;
 
     if (!name)
@@ -1444,9 +1329,9 @@ mapstruct *ready_map_name(char *name, int flush) {
     m = has_been_loaded (name);
 
     if (m && (m->in_memory == MAP_LOADING || m->in_memory == MAP_IN_MEMORY))
-      return m;
+	return m;
 
-    if (flush || !m || (m->reset_time != -1 && seconds() > m->reset_time)) {
+    if ((flags & (MAP_FLUSH|MAP_PLAYER_UNIQUE)) || !m || (m->reset_time != -1 && seconds() > m->reset_time)) {
 
 	/* first visit or time to reset */
 	if (m) {
@@ -1455,22 +1340,27 @@ mapstruct *ready_map_name(char *name, int flush) {
 	}
 
 	/* create and load a map */
-        LOG(llevDebug, "Trying to load map %s.\n", create_pathname(name));
-      if (!(m = load_original_map(name, 0)))
+	if (flags & MAP_PLAYER_UNIQUE)
+	    LOG(llevDebug, "Trying to load map %s.\n", name);
+	else
+	    LOG(llevDebug, "Trying to load map %s.\n", create_pathname(name));
+
+	if (!(m = load_original_map(name, (flags & MAP_PLAYER_UNIQUE))))
 	    return (NULL);
 
 	(*fix_auto_apply_func)(m); /* Chests which open as default */
 
-#ifdef UNIQUE_ITEMS 
-      if (!flush) /* load these only in the game */
-          load_unique_objects(m);
-#endif
-    } else {
+	/* If a player unique map, no extra unique object file to load.
+	 * if from the editor, likewise.
+	 */
+	if (! (flags & (MAP_FLUSH|MAP_PLAYER_UNIQUE))) 
+	    load_unique_objects(m);
 
-	load_temporary_map (m);
-#ifdef UNIQUE_ITEMS 
-      load_unique_objects(m);
-#endif
+    } else {
+	/* If in this loop, we found a temporary map, so load it up. */
+
+	m=load_temporary_map (m);
+	load_unique_objects(m);
 
 	clean_tmp_map(m);
 	m->in_memory = MAP_IN_MEMORY;
@@ -1483,7 +1373,11 @@ mapstruct *ready_map_name(char *name, int flush) {
 	if (m->tmpname) free(m->tmpname);
 	m->tmpname = NULL;
 	/* It's going to be saved anew anyway */
-    }
+    } 
+
+    /* Below here is stuff common to both first time loaded maps and
+     * temp maps.
+     */
 
     /* In case other objects press some buttons down */
     (*update_buttons_func)(m);
