@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_object_c =
- *   "$Id: object.c,v 1.49 1997/03/09 04:19:06 master Exp master $";
+ *   "$Id: object.c,v 1.58 1998/09/28 02:17:56 master Exp master $";
  */
 
 /*
@@ -59,6 +59,91 @@ int freedir[SIZEOFFREE]= {
   0,1,2,3,4,5,6,7,8,1,2,2,2,3,4,4,4,5,6,6,6,7,8,8,8,
   1,2,2,2,2,2,3,4,4,4,4,4,5,6,6,6,6,6,7,8,8,8,8,8};
 
+
+/* Moved this out of define.h and in here, since this is the only file
+ * it is used in.  Also, make it an inline function for cleaner
+ * design.
+ *
+ * Function examines the 2 objects given to it, and returns true if
+ * they can be merged together.
+ *
+ * Note that this function appears a lot longer than the macro it
+ * replaces - this is mostly for clarity - a decent compiler should hopefully
+ * reduce this to the same efficiency.
+ *
+ * Check nrof variable *before* calling CAN_MERGE()
+ *
+ * Improvements made with merge:  Better checking on potion, and also
+ * check weight
+ */
+
+inline int CAN_MERGE(object *ob1, object *ob2) {
+
+    /* A couple quicksanity checks */
+    if ((ob1 == ob2) || (ob1->type != ob2->type)) return 0;
+
+    if (ob1->speed != ob2->speed) return 0;
+    /* Note sure why the following is the case - either the object has to
+     * be animated or have a very low speed.  Is this an attempted monster
+     * check?
+     */
+    if (!QUERY_FLAG(ob1,FLAG_ANIMATE) && FABS((ob1)->speed) > 0.001 ) return 0;
+
+    /* the 0x400000 on flags2 is FLAG_INV_LOCK.  I don't think something
+     * being locked in inventory should prevent merging.
+     */
+    if ((ob1->arch != ob2->arch) || 
+	(ob1->stats.sp != ob2->stats.sp) ||
+	(ob1->flags[0] != ob2->flags[0]) || 
+	(ob1->flags[1] != ob2->flags[1]) ||
+	((ob1->flags[2] & ~0x400000) != (ob2->flags[2] & ~ 0x400000)) ||
+	(ob1->flags[3] != ob2->flags[3]) || 
+	(ob1->name != ob2->name) || 
+	(ob1->title != ob2->title) ||
+	(ob1->msg != ob2->msg) || 
+	(ob1->weight != ob2->weight) ||
+	(ob1->stats.food != ob2->stats.food) ||
+	(ob1->protected != ob2->protected) || 
+	(ob1->immune != ob2->immune) ||
+	(ob1->vulnerable != ob2->vulnerable) || 
+	(ob1->magic != ob2->magic) ||
+	(ob1->slaying != ob2->slaying) ||
+	(ob1->value != ob2->value) ||
+	(ob1->animation_id != ob2->animation_id)
+	) 
+	    return 0;
+
+    switch (ob1->type) {
+	case SCROLL:
+	    if (ob1->level != ob2->level) return 0;
+	    break;
+
+	case RING:
+	    /* Don't merge applied rings - easier to keep them seperate, and
+	     * it makes more sense (can easily unapply one ring).  Rings are
+	     * the only objects that need this special code, as they are the
+	     * only objects of the same type in which more than 1 can be
+	     * applied at a time.
+	     *
+	     * Note - there is no break so we fall into the POTION/AMULET
+	     * check below.
+	     */
+	    if (QUERY_FLAG(ob1, FLAG_APPLIED) || QUERY_FLAG(ob2, FLAG_APPLIED))
+		return 0;
+
+	case POTION:
+	case AMULET:
+	    /* This should compare the value of the stats, and not the pointer
+	     * itself.  There can be cases were potions seem to loose their
+	     * plus
+	     */
+	    if (memcmp(&ob1->stats,&ob2->stats, sizeof(living))) return 0;
+	    break;
+    }
+    /* Everything passes, must be OK. */
+    return 1;
+}
+
 /*
  * sum_weight() is a recursive function which calculates the weight
  * an object is carrying.  It goes through in figures out how much
@@ -162,46 +247,6 @@ void dump_all_objects() {
   }
 }
 
-/*
- * animate_object(object) updates the face-variable of an object.
- * If the object is the head of a multi-object, all objects are animated.
- * If the object has the IS_TURNING() flag, that is taken into consideration.
- */
-
-void animate_object(object *op) {
-  if(op->arch==NULL) {
-    dump_object(op);
-    LOG(llevError,"Tried to animate object without archetype:\n%s\n", errmsg);
-    return;
-  }
-  if(!op->arch->animations) {
-    LOG(llevError,"Archtype lacks animation.\n");
-    dump_arch(op->arch);
-  }
-  if((unsigned char) ++op->state >=
-     (int) (QUERY_FLAG(op,FLAG_IS_TURNING) ? op->arch->animations/2 : op->arch->animations))
-    op->state=0;
-  op->face=&new_faces[
-	op->arch->faces[op->state+(QUERY_FLAG(op,FLAG_IS_TURNING)?
-           ((op->head!=NULL?op->head:op)->value?op->arch->animations/2:0):0)]];
-  if(op->face==blank_face)
-    op->invisible=1;
-#if 1
-  else if(QUERY_FLAG((&op->arch->clone),FLAG_ALIVE)) {
-    if(op->face->number==0) {
-      op->invisible=1;
-      CLEAR_FLAG(op, FLAG_ALIVE);
-    } else {
-      op->invisible=0;
-      SET_FLAG(op, FLAG_ALIVE);
-    }
-  }
-#endif
-  update_object(op);
-  if(op->more)
-    animate_object(op->more);
-}
-
 /*
  * get_nearest_part(multi-object, object 2) returns the part of the
  * multi-object 1 which is closest to the second object.
@@ -297,29 +342,51 @@ object *get_owner(object *op) {
  * Also checkpoints a backup id-scheme which detects freeing (and reusage)
  * of the owner object.
  * See also get_owner()
+ * The real purpose of setting an owner is so we know who kills something
+ * so we can properly credit EXP.
  */
 
 void set_owner(object *op, object *owner) {
-  if(owner==NULL||op==NULL)
-    return;
-  /* next line added to allow objects which own objects */ 
-  while(owner->owner&&owner!=owner->owner) owner=owner->owner;
+
+    if(owner==NULL||op==NULL)
+	return;
+    /* next line added to allow objects which own objects */ 
+    /* Add a check for ownercounts in here, as I got into an endless loop
+     * with the fireball owning a poison cloud which then owned the
+     * fireball.  I believe that was caused by one of the objects getting
+     * freed and then another object replacing it.  Since the ownercounts
+     * didn't match, this check is valid and I believe that cause is valid.
+     */
+    while (owner->owner && owner!=owner->owner && 
+	owner->ownercount==owner->owner->count) owner=owner->owner;
+
+    /* IF the owner still has an owner, we did not resolve to a final owner.
+     * so lets not add to that.
+     */
+    if (owner->owner) return;
+
     op->owner=owner;
 
-  op->ownercount=owner->count;
-  owner->refcount++;
+    op->ownercount=owner->count;
+    owner->refcount++;
 
 #ifdef ALLOW_SKILLS /* set the pointers in op to inherit owners skill, exp_obj */ 
-  if(owner->type==PLAYER&&owner->chosen_skill) {
-        op->chosen_skill = owner->chosen_skill;
-        op->exp_obj = owner->chosen_skill->exp_obj;
+    if(owner->type==PLAYER&&owner->chosen_skill) {
+	op->chosen_skill = owner->chosen_skill;
+	op->exp_obj = owner->chosen_skill->exp_obj;
 
-	/* unfortunately, we can't allow summoned monsters skill use
-         * because we will need the chosen_skill field to pick the
-         * right skill/stat modifiers for calc_skill_exp(). See
-         * hit_player() in server/attack.c -b.t. */
-  } else if(op->type!=PLAYER && QUERY_FLAG(op,FLAG_READY_SKILL))
-               CLEAR_FLAG(op,FLAG_READY_SKILL);
+	    /* unfortunately, we can't allow summoned monsters skill use
+	     * because we will need the chosen_skill field to pick the
+	     * right skill/stat modifiers for calc_skill_exp(). See
+	     * hit_player() in server/attack.c -b.t. 
+	     */
+	if(QUERY_FLAG(op,FLAG_CAN_USE_SKILL))
+	    CLEAR_FLAG(op,FLAG_CAN_USE_SKILL);
+	if(QUERY_FLAG(op,FLAG_READY_SKILL))
+	    CLEAR_FLAG(op,FLAG_READY_SKILL);
+
+    } else if(op->type!=PLAYER && QUERY_FLAG(op,FLAG_READY_SKILL))
+	CLEAR_FLAG(op,FLAG_READY_SKILL);
 #endif
 }
 
@@ -365,6 +432,8 @@ void clear_object(object *op) {
   }
   for (tmp=0; tmp<=(NUM_FLAGS/32); tmp++)
 	op->flags[tmp] = 0;
+  /* This is more or less true */
+  SET_FLAG(op, FLAG_REMOVED);
   op->map=NULL;
   op->below=NULL;
   op->above=NULL;
@@ -400,10 +469,6 @@ void clear_object(object *op) {
   op->expmul=1.0;
   op->direction=0;
   op->glow_radius=0;
-#ifdef NPC_PROG
-  op->npc_status=0;
-  op->npc_program=0;
-#endif
   op->stats.Str=op->stats.Dex=op->stats.Con=0;
   op->stats.Wis=op->stats.Cha=op->stats.Int=op->stats.Pow=0;
   op->material=op->magic=op->state=op->type=0;
@@ -428,6 +493,7 @@ void clear_object(object *op) {
 #endif
   op->randomitems=NULL;
   op->spellitem = NULL;
+  op->animation_id=0;
 
 }
 
@@ -549,10 +615,10 @@ object *get_object() {
  */
 
 void update_turn_face(object *op) {
-  if(!QUERY_FLAG(op,FLAG_IS_TURNABLE)||op->arch==NULL)
-    return;
-  op->face=&new_faces[op->arch->faces[op->direction]];
-  update_object(op);
+    if(!QUERY_FLAG(op,FLAG_IS_TURNABLE)||op->arch==NULL)
+	return;
+    SET_ANIMATION(op, op->direction);
+    update_object(op);
 }
 
 /*
@@ -578,7 +644,7 @@ void update_ob_speed(object *op) {
     if (arch_init) {
 	return;
     }
-    if (op->speed) {
+    if (FABS(op->speed)>0.001) {
 	/* If already on active list, don't do anything */
 	if (op->active_next || op->active_prev || op==active_objects)
 	    return;
@@ -620,7 +686,7 @@ void update_ob_speed(object *op) {
  */
 
 void update_object(object *op) {
-  object *tmp;
+  object *tmp, *player;
 
   if(op->env!=NULL) {
     tmp=is_player_inv(op->env);
@@ -628,14 +694,34 @@ void update_object(object *op) {
      * or something.  Ideally, client should handle animations on its
      * own.
      */
-    if (tmp && tmp->contr->eric_server<=0)
-      (*draw_inventory_faces_func)(tmp);
     return;
   }
 
  /* Can be null if the player has quit but window still exists. */
-  if (op->map->map != NULL)
-      update_position (op->map, op->x, op->y);
+  if (op->map->map != NULL) {
+    player=update_position (op->map, op->x, op->y);
+    /*
+     * insert_ob_in_map calls update_ob.  insert_ob_in_map is called with
+     * the player object anytime he moves - as such, we compare the object
+     * being updated against the player object so that we don't send
+     * unnecessary update commands.  Also, the client doesn't do anything
+     * with the players face anyways.
+     * As a note - we assume that we get here because the face has changed -
+     * this might not always be the case, but seems to be a good amount of
+     * the time, and its easier to do this than track down all the places in
+     * the code that changes the face.
+     *
+     * Add a check so that if the item that is changing is hidden from
+     * the player (ie, below the floor), we don't send an update to the
+     * player.
+     */
+    if (player && player!=op) {
+	for (tmp=op; tmp!=NULL; tmp=tmp->above)
+	    if (QUERY_FLAG(tmp,FLAG_IS_FLOOR) && !tmp->invisible) break;
+
+	if (!tmp) esrv_update_item_func(UPD_FACE, player, op);
+    }
+  }
 
   if(op->more!=NULL)
     update_object(op->more);
@@ -649,14 +735,18 @@ void update_object(object *op) {
  * so that we now have a visibility flag, and if that is set, then the
  * highest object with that visibility gets put into the addition object
  * slot.
+ *
+ * Return player object ifwe find one.
+ * update_object (which calls this) needs to know if it should send an 
+ * update to the client.  Update object could check for that, but
+ * since this looks through the space anyways, might as well this function
+ * return it for us.
  */
-void update_position (mapstruct *m, int x, int y) {
-    object *tmp, *last = NULL;
+object *update_position (mapstruct *m, int x, int y) {
+    object *tmp, *last = NULL, *player=NULL;
     unsigned int flags = 0;
     MapLook f=blank_look,floor=blank_look;
-#ifdef DOUBLE_FLOOR
     MapLook f1 = blank_look;
-#endif
 
     for (tmp = get_map_ob (m, x, y); tmp; last = tmp, tmp = tmp->above) {
 
@@ -669,23 +759,18 @@ void update_position (mapstruct *m, int x, int y) {
 	 * Always put the player down for drawing.
 	 */
 	if (tmp->type==PLAYER) {
-	    if (tmp->contr->eric_server<=0)
-		(*draw_look_faces_func)(tmp);
 	    f.face = tmp->face;
+	    player=tmp;
 	}
 	else if (QUERY_FLAG(tmp,FLAG_IS_FLOOR)) {
-#ifdef DOUBLE_FLOOR
 	    /* can't see anything that is below the floor, so reset it. */
 	    f1=blank_look;
-#endif
 	    if (!tmp->invisible)
 		floor.face = tmp->face;
 	}
 
-#ifdef DOUBLE_FLOOR
 	else if (f1.face->visibility < tmp->face->visibility && !tmp->invisible)
 		f1.face = tmp->face;
-#endif
 	if (tmp==tmp->above) {
 	    LOG(llevError, "Error in structure of map\n");
 	    exit (-1);
@@ -720,17 +805,15 @@ void update_position (mapstruct *m, int x, int y) {
 /* Basically, we want to try and set the floors first, since that some
  * object that comes along later can be put in the top spot.
  */
-#ifdef DOUBLE_FLOOR
     if (f.face == f1.face)
 	f1.face = blank_face;
     set_map_floor2(m, x, y, &f1);
-#endif
 
     f.flags = flags;
     set_map_floor(m,x,y,&floor);
     set_map (m, x, y, &f);
 
-    return;
+    return player;
 }
 
 /*
@@ -743,6 +826,14 @@ void update_position (mapstruct *m, int x, int y) {
 
 void free_object(object *ob) {
   object *tmp,*op;
+
+    if (!QUERY_FLAG(ob,FLAG_REMOVED)) {
+	LOG(llevDebug,"Free object called with non removed object\n");
+	dump_object(ob);
+#ifdef MANY_CORES
+	abort();
+#endif
+    }
   if(QUERY_FLAG(ob,FLAG_FRIENDLY)) {
     LOG(llevMonster,"Warning: tried to free friendly object.\n");
     (*remove_friendly_object_func)(ob);
@@ -943,10 +1034,6 @@ void remove_ob(object *op) {
     op->ox=op->x,op->oy=op->y;
     op->map=op->env->map;
     op->above=NULL,op->below=NULL;
-    if(otmp!=NULL) {
-      if (otmp->type==PLAYER && otmp->contr && !otmp->contr->eric_server)
-	(*draw_inventory_func)(otmp);
-    }
     op->env=NULL;
     return;
   }
@@ -987,11 +1074,20 @@ void remove_ob(object *op) {
     /* No point updating the players look faces if he is the object
      * being removed.
      */
-    if(tmp->type==PLAYER && tmp!=op)
+    if(tmp->type==PLAYER && tmp!=op) {
+	/* If a container that the player is currently using somehow gets
+	 * removed (most likely destroyed), update the player view
+	 * appropriately.
+	 */
+	if (tmp->container==op) {
+	    CLEAR_FLAG(op, FLAG_APPLIED);
+	    tmp->container=NULL;
+	}
       (*draw_look_func)(tmp);
+    }
     if(QUERY_FLAG(op,FLAG_FLYING)?QUERY_FLAG(tmp,FLAG_FLY_OFF):
       QUERY_FLAG(tmp,FLAG_WALK_OFF))
-        (*apply_func)(op,tmp);
+        (*apply_func)(op,tmp,0);
 
     /* Eneq(@csd.uu.se): Fixed this to skip tmp->above=tmp */
 
@@ -1033,7 +1129,7 @@ object *merge_ob(object *op, object *top) {
     if (CAN_MERGE(op,top))
     {
       top->nrof+=op->nrof;
-      CLEAR_FLAG(top,FLAG_STARTEQUIP);
+/*      CLEAR_FLAG(top,FLAG_STARTEQUIP);*/
       op->weight = 0; /* Don't want any adjustements now */
       remove_ob(op);
       free_object(op);
@@ -1140,6 +1236,15 @@ void insert_ob_in_map_simple(object *op, mapstruct *m)
   else
     set_map_ob(op->map,op->x,op->y,op);   /* Tell the map that we're here */
   update_object(op);
+
+/* comment out, 0.93.7 - insert_ob_in_map_simple should only be called in
+ * some special places with objects - in most all places, the object
+ * was already around, and is being re-inserted.  This will hopefully fix
+ * a problem with dropping objects on a slow move space (otherwise,
+ * the slow move penalty was applied for each object.
+ */
+
+#if 0
   /* Only check this if we are the head of the object */
   if (!op->head) {
       check_walk_on(op);
@@ -1156,6 +1261,7 @@ void insert_ob_in_map_simple(object *op, mapstruct *m)
       }
 
   }
+#endif
   if(op->type==PLAYER)
     op->contr->do_los=1;
   for(tmp=get_map_ob(op->map,op->x,op->y);tmp!=NULL;tmp=tmp->above)
@@ -1163,11 +1269,6 @@ void insert_ob_in_map_simple(object *op, mapstruct *m)
     case PLAYER:
       (*draw_look_func)(tmp);
       break;
-    case BUTTON:
-    case PEDESTAL:
-      if (op!=tmp)
-        update_button(tmp);
-      break;
     }
 }
 
@@ -1206,7 +1307,7 @@ void insert_ob_in_map(object *op,mapstruct *m) {
       if (CAN_MERGE(op,tmp))
       {
         op->nrof+=tmp->nrof;
-        CLEAR_FLAG(op,FLAG_STARTEQUIP);
+/*        CLEAR_FLAG(op,FLAG_STARTEQUIP);*/
         remove_ob(tmp);
         free_object(tmp);
       }
@@ -1318,11 +1419,6 @@ void insert_ob_in_map(object *op,mapstruct *m) {
     case PLAYER:
       (*draw_look_func)(tmp);
       break;
-    case BUTTON:
-    case PEDESTAL:
-      if (op!=tmp)
-        update_button(tmp);
-      break;
     }
   /* build up linked list of light sources in each map. We do
    * this even for non-dark maps, as down the line we might make
@@ -1348,39 +1444,31 @@ void insert_ob_in_map(object *op,mapstruct *m) {
  */
 
 object *get_split_ob(object *orig_ob,int nr) {
-  object *tmp, *otmp;
-  if(orig_ob->nrof<nr) {
-    sprintf(errmsg,"There are only %d %ss.",
+    object *newob;
+
+    if(orig_ob->nrof<nr) {
+	sprintf(errmsg,"There are only %d %ss.",
             orig_ob->nrof?orig_ob->nrof:1, orig_ob->name);
-    return NULL;
-  }
-  tmp=get_object();
-  copy_object(orig_ob,tmp);
-  if((orig_ob->nrof-=nr)<1) {
-    remove_ob(orig_ob);
-    free_object(orig_ob);
-  }
-  else if(!QUERY_FLAG(orig_ob,FLAG_REMOVED)){
-    if(orig_ob->env!=NULL)
-      sub_weight (orig_ob->env,orig_ob->weight*nr);
-    if((otmp=is_player_inv (orig_ob->env)) && otmp->contr->eric_server <= 0)
-      (*draw_inventory_func)(otmp);
-    else if (orig_ob->env == NULL) {
-      object *tmp;
-      if (orig_ob->map->in_memory!=MAP_IN_MEMORY) {
-        strcpy(errmsg, "Tried to split object whose map is not in memory.");
-	LOG(llevDebug,
-            "Error, Tried to split object whose map is not in memory.\n");
-        return NULL;
-      }
-      for(tmp=get_map_ob(orig_ob->map,orig_ob->x,orig_ob->y);
-          tmp!=NULL;tmp=tmp->above)
-        if(tmp->type==PLAYER && tmp->contr->eric_server <= 0)
-          (*draw_look_func)(tmp);
+	return NULL;
     }
-  }
-  tmp->nrof=nr;
-  return tmp;
+    newob=get_object();
+    copy_object(orig_ob,newob);
+    if((orig_ob->nrof-=nr)<1) {
+	remove_ob(orig_ob);
+	free_object(orig_ob);
+    }
+    else if(!QUERY_FLAG(orig_ob,FLAG_REMOVED)){
+	if(orig_ob->env!=NULL)
+	    sub_weight (orig_ob->env,orig_ob->weight*nr);
+	if (orig_ob->env == NULL && orig_ob->map->in_memory!=MAP_IN_MEMORY) {
+	    strcpy(errmsg, "Tried to split object whose map is not in memory.");
+	    LOG(llevDebug,
+		    "Error, Tried to split object whose map is not in memory.\n");
+	    return NULL;
+	}
+    }
+    newob->nrof=nr;
+    return newob;
 }
 
 /*
@@ -1391,8 +1479,20 @@ object *get_split_ob(object *orig_ob,int nr) {
 
 void decrease_ob_nr(object *op,int i) {
   object *tmp;
+  
+  /* nrof is split out into a long so it is handled ok on 64 bit machines.
+   * I think it might work to just do op->nrof-=i and cast that to an sint32
+   * to see if it is negative.  But if so, we might as well just nrof in
+   * the object structure an sint32, since that would still set a maximum
+   * value.
+   */
+  long nrof=op->nrof;
 
-  if((long)(op->nrof-=i)<1) {
+  nrof -= i;
+  if (nrof<0) nrof=0;
+  op->nrof = nrof;
+
+  if(!op->nrof) {
     op->nrof=0;	/* just to be sure that remove_ob handles weight properly */
     /* We don't have remove object send delete to the new client, because
      * in many areas, more intelligent strategies are employed.  However, if
@@ -1402,8 +1502,10 @@ void decrease_ob_nr(object *op,int i) {
      * environment.
      */
     tmp=is_player_inv (op->env);
-    if(tmp!=NULL && tmp->contr->eric_server)
-	(*esrv_del_item_func)(tmp->contr->eric_server, op->count);
+    if(tmp!=NULL) {
+	(*esrv_del_item_func)(tmp->contr, op->count);
+	(*esrv_update_item_func)(UPD_WEIGHT, tmp, tmp);
+    }
     remove_ob(op);
     free_object(op);
   }
@@ -1412,18 +1514,15 @@ void decrease_ob_nr(object *op,int i) {
       sub_weight(op->env,op->weight*i);
       tmp=is_player_inv (op->env);
       if(tmp!=NULL) {
-	if (tmp->contr->eric_server)
 	    (*esrv_send_item_func)(tmp, op);
-	else
-	    (*draw_inventory_func)(tmp);
+	    (*esrv_update_item_func)(UPD_WEIGHT, tmp, tmp);
       }
     } else {
       for(tmp=op->above;tmp!=NULL;tmp=tmp->above)
-        if(tmp->type==PLAYER) 
-	  if (tmp->contr->eric_server)
+        if(tmp->type==PLAYER) {
 	    (*esrv_send_item_func)(tmp, op);
-	  else
-	    (*draw_look_func)(tmp);
+	    (*esrv_update_item_func)(UPD_WEIGHT, tmp, tmp);
+	}
     }
 }
 
@@ -1458,7 +1557,6 @@ void add_weight (object *op, signed long weight) {
 
 object *insert_ob_in_ob(object *op,object *where) {
   object *tmp, *otmp;
-  int sort_inventory=0;
 
   if(!QUERY_FLAG(op,FLAG_REMOVED)) {
     dump_object(op);
@@ -1513,7 +1611,6 @@ object *insert_ob_in_ob(object *op,object *where) {
   if (otmp&&otmp->contr!=NULL) {
     if (!QUERY_FLAG(otmp,FLAG_NO_FIX_PLAYER))
       fix_player(otmp);
-    sort_inventory = otmp->contr->sort_inventory;
   }
 
   op->map=NULL;
@@ -1536,110 +1633,15 @@ object *insert_ob_in_ob(object *op,object *where) {
   }
 #endif /* USE_LIGHTING */ 
 
+  /* Client has no idea of ordering so lets not bother ordering it here.
+   * It sure simplifies this function...
+   */
   if (where->inv==NULL)
       where->inv=op;
   else {
-    tmp = where->inv;
-    if (! tmp->invisible || where->type==PLAYER) {
-      /*
-       * Rule 1: Move to the similar type
-       */
-      while (tmp->below != NULL &&
-             tmp->type != op->type)
-        tmp = tmp->below;
-
-/*	LOG(llevDebug,"insert_ob_in_ob: Found same type at %s\n",query_base_name(tmp));*/
-
-      /*
-       * Rule 2: Within similar type, sort by archetype
-       */
-      while (tmp->below != NULL &&
-             tmp->below->type == op->type &&
-             tmp->arch != op->arch)
-        tmp = tmp->below;
-
-/*	LOG(llevDebug,"insert_ob_in_ob: Found same arch at %s\n",query_base_name(tmp));*/
-	if (!sort_inventory) {
-	    /*
-	     * Rule 3: Within similar type & archetype, sort by magic
-	     * First check a special case, then scroll down until
-	     * we are below items with more magic.
-	     */
-	    if (tmp->type == op->type &&
-		tmp->arch == op->arch &&
-		abs(tmp->magic) < abs(op->magic))
-			tmp = tmp->above; /* Beware: tmp might be NULL now. */
-	    else {
-		while (tmp != NULL &&
-		   tmp->below != NULL &&
-		   tmp->below->type == op->type &&
-		   tmp->below->arch == op->arch &&
-		   abs(tmp->magic) > abs(op->magic))
-			tmp = tmp->below;
-
-		/*
-		 * Rule 4: Insert what seems like a similar item below.
-		 */
-		while (tmp != NULL &&
-		   tmp->below != NULL &&
-		   tmp->below->type == op->type &&
-		   tmp->below->arch == op->arch &&
-		   abs(tmp->below->magic) == abs(op->magic))
-			tmp = tmp->below;
-	    }
-	} else { /* If we do sort inventory */
-	    /*
-	     * Rule 5: Try to sort alphabetically.  This is probably a costly
-	     * operation.  It might be worthwhile to store the complete strings
-	     * in the object structure
-	     */
-	    char obuf[MAX_BUF],*nbuf;
-
-	    strcpy(obuf, query_base_name(op));
-
-	    /* We insert after tmp.  As such, we may want to insert before
-	     * the first object that we have matched so far, so lets back up
-	     */
-	    if (tmp->type == op->type &&
-		tmp->arch == op->arch &&
-		abs(tmp->magic) <= abs(op->magic) &&
-		(strcmp(obuf, query_base_name(tmp))<0))
-		    tmp = tmp->above; /* Beware: tmp might be NULL now. */
-	    else {
-		while (tmp != NULL &&
-		   tmp->below != NULL &&
-		   tmp->below->type == op->type &&
-		   tmp->below->arch == op->arch &&
-		   abs(tmp->below->magic) == abs(op->magic)) {
-			nbuf=query_base_name(tmp->below);
-			if (strcmp(obuf,nbuf)>0) {
-				tmp = tmp->below;
-			}
-			else break;
-		}
-	    }
-	} /*end of inventory sort */
-    } /* if this is a playr */
-    /*
-     * Now link the new object into the two-way list, below tmp.
-     */
-    if (tmp == NULL) {
       op->below = where->inv;
       op->below->above = op;
       where->inv = op;
-    } else if (tmp->below != NULL) {
-      tmp->below->above = op;
-      op->below = tmp->below;
-      tmp->below = op;
-      op->above = tmp;
-    } else {
-      tmp->below = op;
-      op->above = tmp;
-    }
-  }
-  tmp=is_player_inv(where);
-  if (tmp && tmp->contr && tmp->contr->eric_server <= 0) {
-      (*draw_inventory_func)(tmp);
   }
   return op;
 }
@@ -1656,37 +1658,35 @@ object *insert_ob_in_ob(object *op,object *where) {
   */
 
 void check_walk_on(object *op) {
-  object *tmp;
-  if(QUERY_FLAG(op,FLAG_NO_APPLY))
-    return;
-  for(tmp=op->below;tmp!=NULL;tmp=tmp->below) {
-    if (tmp==tmp->below)	/* why? mta */
-       break;
-    if(QUERY_FLAG(tmp,FLAG_SLOW_MOVE)&&!QUERY_FLAG(op,FLAG_FLYING)) { 
-	if(op->type==PLAYER && op->chosen_skill) { 
-             if((QUERY_FLAG(tmp,FLAG_IS_HILLY) && op->contr->shoottype == range_skill 
-		&& op->chosen_skill->stats.sp==SK_CLIMBING) 
-	        || (QUERY_FLAG(tmp,FLAG_IS_WOODED) && op->contr->shoottype == range_skill
-		&& op->chosen_skill->stats.sp==SK_WOODSMAN))
-      		    op->speed_left -= (SLOW_PENALTY(tmp)*FABS(op->speed))/4;
-             else 
-		op->speed_left -= SLOW_PENALTY(tmp)*FABS(op->speed);
-	} else 
-		op->speed_left -= SLOW_PENALTY(tmp)*FABS(op->speed);
+    object *tmp;
+
+    if(QUERY_FLAG(op,FLAG_NO_APPLY))
+	return;
+    for(tmp=op->below;tmp!=NULL;tmp=tmp->below) {
+	if (tmp==tmp->below)	/* if we have a map loop, exit */
+	    break;
+	if(QUERY_FLAG(tmp,FLAG_SLOW_MOVE)&&!QUERY_FLAG(op,FLAG_FLYING)) {
+	    float diff;
+
+	    diff=(SLOW_PENALTY(tmp)*FABS(op->speed));
+	    if (op->type==PLAYER) {
+		if ((QUERY_FLAG(tmp,FLAG_IS_HILLY) && find_skill(op,SK_CLIMBING)) ||
+		    (QUERY_FLAG(tmp,FLAG_IS_WOODED) && find_skill(op,SK_WOODSMAN)))  {
+			diff=diff/4.0;
+		}
+	    }
+	    op->speed_left -= diff;
+	}
+	if(QUERY_FLAG(op,FLAG_FLYING)?QUERY_FLAG(tmp,FLAG_FLY_ON):
+	   QUERY_FLAG(tmp,FLAG_WALK_ON)) {
+	    if((*apply_func)(op,tmp,0)==2)
+		break;
+	    /* IF the object has been destroyed, we want to stop anymore
+	     * processing on it right now.
+	     */
+	    if (QUERY_FLAG(op, FLAG_FREED)) break;
+	}
     }
-    if(QUERY_FLAG(op,FLAG_FLYING)?QUERY_FLAG(tmp,FLAG_FLY_ON):
-      QUERY_FLAG(tmp,FLAG_WALK_ON)) {
-#if 0
-/* the function that called this (a bit up the line) will draw the
- * window.  So don't do it here.
- */
-      if(op->type==PLAYER)
-        (*draw_func)(op);
-#endif
-      if((*apply_func)(op,tmp)==2)
-        break;
-      }
-  }
 }
 
 /*
@@ -1913,12 +1913,14 @@ int dirdiff(int dir1, int dir2) {
  *
  * Cf 0.91.3 - don't let WIZ's pick up anything - will likely cause
  * core dumps if they do.
+ *
+ * Add a check so we can't pick up invisible objects (0.93.8)
  */
 
 int can_pick(object *who,object *item) {
   return /*QUERY_FLAG(who,FLAG_WIZ)||*/
          (item->weight>0&&!QUERY_FLAG(item,FLAG_NO_PICK)&&
-	 !QUERY_FLAG(item,FLAG_ALIVE)&&
+	 !QUERY_FLAG(item,FLAG_ALIVE)&&!item->invisible && 
           (who->type==PLAYER||item->weight<who->weight/3));
 }
 
