@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_shstr_c =
- *   "$Id: shstr.c,v 1.18 1997/02/04 10:02:23 master Exp $";
+ *   "$Id: shstr.c,v 1.20 1998/09/28 02:23:41 master Exp master $";
  *
  * shstr.c
  *
@@ -30,7 +30,7 @@
 
 #include "shstr.h"
 
-#ifdef DMALLOC
+#ifdef HAVE_LIBDMALLOC
 #include <dmalloc.h>
 #endif
 
@@ -393,3 +393,22 @@ ss_dump_table(int what) {
     }
     return NULL;
 }
+
+/* buf_overflow() - we don't want to exceed the buffer size of 
+ * buf1 by adding on buf2! Returns true if overflow will occur.
+ */
+
+int 
+buf_overflow (char *buf1, char *buf2, int bufsize)
+{
+    int     len1 = 0, len2 = 0;
+
+    if (buf1)
+	len1 = strlen (buf1);
+    if (buf2)
+	len2 = strlen (buf2);
+    if ((len1 + len2) >= bufsize)
+	return 1;
+    return 0;
+}
+
