@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_time_c =
- *    "$Id: time.c,v 1.11 1996/03/04 09:25:10 master Exp $";
+ *    "$Id: time.c,v 1.14 1998/08/01 09:22:42 master Exp $";
  */
 
 /*
@@ -50,12 +50,18 @@ long pticks;
 long process_utime_long_count;
 #endif
 
-/* Solaris 2.5 wants 2 arguments for gettimeofday - fake it out here until
- * I can determine if earlier version of solaris do in fact only have 1
- * argument.
+/* 0.94.1 - change to GETTIMEOFDAY macro - SNI systems only one one option.
+ * rather than have complex #ifdefs throughout the file, lets just figure
+ * it out once, here at the top.
+ * Have no idea if that is the right symbol to check on for NetBSD,
+ * but NetBSD does use 2 params.
  */
-#if defined(__sun__) && defined(SVR4)
-#define gettimeofday(last_time) gettimeofday(last_time, (struct timezone *) NULL);
+
+#if defined(__sun__) || defined(linux) || defined(__NetBSD__) \
+   || defined(__FreeBSD__)
+#define GETTIMEOFDAY(last_time) gettimeofday(last_time, (struct timezone *) NULL);
+#else
+#define GETTIMEOFDAY(last_time) gettimeofday(last_time);
 #endif
 
 /*
@@ -76,11 +82,7 @@ reset_sleep()
   pticks = 0;
 #endif
 
-#if defined(__sun__) && defined(SVR4)
-  (void) gettimeofday(&last_time);
-#else
-  (void) gettimeofday(&last_time, (struct timezone *) NULL);
-#endif
+  (void) GETTIMEOFDAY(&last_time);
 }
 
 void
@@ -110,11 +112,8 @@ enough_elapsed_time()
   static struct timeval new_time;
   long elapsed_utime;
 
-#if defined(__sun__) && defined(SVR4)
-  (void) gettimeofday(&new_time);
-#else
-  (void) gettimeofday(&new_time, (struct timezone *) NULL);
-#endif
+  (void) GETTIMEOFDAY(&new_time);
+
   elapsed_utime = (new_time.tv_sec - last_time.tv_sec) * 1000000 +
                   new_time.tv_usec - last_time.tv_usec;
   if (elapsed_utime > max_time) {
@@ -139,11 +138,8 @@ sleep_delta()
   static struct timeval new_time;
   long sleep_sec, sleep_usec;
 
-#if defined(__sun__) && defined(SVR4)
-  (void) gettimeofday(&new_time);
-#else
-  (void) gettimeofday(&new_time, (struct timezone *) NULL);
-#endif
+  (void) GETTIMEOFDAY(&new_time);
+
   sleep_sec = last_time.tv_sec - new_time.tv_sec;
   sleep_usec = max_time - (new_time.tv_usec - last_time.tv_usec);
 
@@ -244,10 +240,6 @@ seconds()
 {
   struct timeval now;
 
-#if defined(__sun__) && defined(SVR4)
-  (void) gettimeofday(&now);
-#else
-  (void) gettimeofday(&now, (struct timezone *) 0);
-#endif
+  (void) GETTIMEOFDAY(&now);
   return now.tv_sec;
 }
