@@ -1,7 +1,7 @@
 
 /*
  * static char *rcs_treasure_c =
- *   "$Id: treasure.c,v 1.48 1997/03/09 04:26:52 master Exp master $";
+ *   "$Id: treasure.c,v 1.52 1998/08/01 09:23:02 master Exp $";
  */
 
 /*
@@ -42,6 +42,7 @@
 #include <treasure.h>
 #include <spellist.h>
 #include <funcpoint.h>
+#include <loader.h>
 
 /*
  * Initialize global archtype pointers:
@@ -504,14 +505,20 @@ void set_magic(int difficulty,object *op, int max_magic) {
  */
 
 void set_ring_bonus(object *op,int bonus) {
-  int r=RANDOM()%(bonus>0?24:10);
-  if(op->type==AMULET)
+
+  int r=RANDOM()%(bonus>0?25:11);
+
+  if(op->type==AMULET) {
     if(!(RANDOM()%21))
-      r=19+RANDOM()%2;
-    else if(RANDOM()&2)
-      r=9;
-    else
-      r=10+RANDOM()%9;
+	r=20+RANDOM()%2;
+    else {
+	if(RANDOM()&2)
+	    r=10;
+	else
+	    r=11+RANDOM()%9;
+    }
+  }
+
   switch(r) {
   case 0:
     if(op->stats.Str)
@@ -544,26 +551,31 @@ void set_ring_bonus(object *op,int bonus) {
     op->stats.Wis=bonus;
     break;
   case 6:
+    if(op->stats.Pow)
+      return;
+    op->stats.Pow=bonus;
+    break;
+  case 7:
     if(op->stats.dam)
       return;
     op->stats.dam=bonus;
     break;
-  case 7:
+  case 8:
     if(op->stats.wc)
       return;
     op->stats.wc=bonus;
     break;
-  case 8:
+  case 9:
     if(op->stats.food)
       return;
     op->stats.food=bonus; /* hunger/sustenance */
     break;
-  case 9:
+  case 10:
     if(op->stats.ac)
       return;
     op->stats.ac=bonus;
     break;
-  case 10:
+  case 11:
     if(bonus==2) { /* Maybe make an artifact of this? */
       op->immune|=AT_PARALYZE;
       op->immune|=AT_SLOW;
@@ -572,35 +584,35 @@ void set_ring_bonus(object *op,int bonus) {
     } else
       op->protected|=AT_PARALYZE;
     break;
-  case 11:
+  case 12:
     op->protected|=AT_MAGIC;
     op->value*=2;
     break;
-  case 12:
+  case 13:
     op->protected|=AT_ELECTRICITY;
     break;
-  case 13:
+  case 14:
     op->protected|=AT_FIRE;
     op->vulnerable|=AT_COLD;
     op->value=(op->value*3)/2;
     break;
-  case 14:
+  case 15:
     op->protected|=AT_DRAIN;
     break;
-  case 15:
+  case 16:
     op->protected|=AT_SLOW;
     break;
-  case 16:
+  case 17:
     op->protected|=AT_COLD;
     op->vulnerable|=AT_FIRE;
     break;
-  case 17:
+  case 18:
     op->protected|=AT_POISON;
     break;
-  case 18:
+  case 19:
     op->protected|=AT_FEAR;
     break;
-  case 19:
+  case 20:
     if(op->type==AMULET) {
       SET_FLAG(op,FLAG_REFL_SPELL);
       op->value*=11;
@@ -609,7 +621,7 @@ void set_ring_bonus(object *op,int bonus) {
       op->value*=4;
     }
     break;
-  case 20:
+  case 21:
     if(op->type==AMULET) {
       SET_FLAG(op,FLAG_REFL_MISSILE);
       op->value*=9;
@@ -618,7 +630,7 @@ void set_ring_bonus(object *op,int bonus) {
       op->value*=3;
     }
     break;
-  case 21:
+  case 22:
     if(op->stats.exp)
       return;
     op->stats.exp=bonus; /* Speed! */
@@ -746,9 +758,8 @@ void fix_generated_item(object *op,object *creator,int difficulty, int max_magic
           set_ring_bonus(op,d);
         }
       }
-      if(op->arch->animations)
-        op->face=&new_faces[op->arch->faces
-		[RANDOM()%((int) op->arch->animations)]];
+      if(GET_ANIM_ID(op))
+	SET_ANIMATION(op, RANDOM()%((int) NUM_ANIMATIONS(op)));
       break;
     case BOOK:
       /* Is it an empty book?, if yes lets make a special 
@@ -757,12 +768,12 @@ void fix_generated_item(object *op,object *creator,int difficulty, int max_magic
        */
       if(!op->msg&&RANDOM()%10) { 
 	/* set the book level properly */
-	if(creator->level==0 || QUERY_FLAG(creator,FLAG_ALIVE))
+	if(creator->level==0 || QUERY_FLAG(creator,FLAG_ALIVE)) {
             if(op->map&&op->map->difficulty) 
 	      op->level=RANDOM()%(op->map->difficulty)+RANDOM()%10+1;
             else
 	      op->level=RANDOM()%20+1;
-	else 
+	} else 
 	    op->level=RANDOM()%creator->level;
 
 	tailor_readable_ob(op,(creator&&creator->stats.sp)?creator->stats.sp:-1);
@@ -954,6 +965,84 @@ void dump_artifacts() {
   fprintf(logfile,"\n");
 }
 
+/*
+ * For debugging purposes.  Dumps all treasures recursively (see below).
+ */
+void dump_monster_treasure_rec (char *name, treasure *t, int depth)
+{
+  treasurelist *tl;
+  int           i;
+
+  if (depth > 100)
+    return;
+  while (t != NULL)
+    {
+      if (t->name != NULL)
+	{
+	  for (i = 0; i < depth; i++)
+	    fprintf (logfile, "  ");
+	  fprintf (logfile, "{   (list: %s)\n", t->name);
+	  tl = find_treasurelist (t->name);
+	  dump_monster_treasure_rec (name, tl->items, depth + 2);
+	  for (i = 0; i < depth; i++)
+	    fprintf (logfile, "  ");
+	  fprintf (logfile, "}   (end of list: %s)\n", t->name);
+	}
+      else
+	{
+	  for (i = 0; i < depth; i++)
+	    fprintf (logfile, "  ");
+	  if (t->item->clone.type == FLESH)
+	    fprintf (logfile, "%s's %s\n", name, t->item->clone.name);
+	  else
+	    fprintf (logfile, "%s\n", t->item->clone.name);
+	}
+      if (t->next_yes != NULL)
+	{
+	  for (i = 0; i < depth; i++)
+	    fprintf (logfile, "  ");
+	  fprintf (logfile, " (if yes)\n");
+	  dump_monster_treasure_rec (name, t->next_yes, depth + 1);
+	}
+      if (t->next_no != NULL)
+	{
+	  for (i = 0; i < depth; i++)
+	    fprintf (logfile, "  ");
+	  fprintf (logfile, " (if no)\n");
+	  dump_monster_treasure_rec (name, t->next_no, depth + 1);
+	}
+      t = t->next;
+    }
+}
+
+/*
+ * For debugging purposes.  Dumps all treasures for a given monster.
+ * Created originally by Raphael Quinet for debugging the alchemy code.
+ */
+
+void dump_monster_treasure (char *name)
+{
+  archetype *at;
+  int        found;
+
+  found = 0;
+  fprintf (logfile, "\n");
+  for (at = first_archetype; at != NULL; at = at->next) 
+    if (! strcasecmp (at->clone.name, name) && at->clone.title == NULL)
+      {
+	fprintf (logfile, "treasures for %s (arch: %s)\n", at->clone.name,
+		 at->name);
+	if (at->clone.randomitems != NULL)
+	  dump_monster_treasure_rec (at->clone.name,
+				     at->clone.randomitems->items, 1);
+	else
+	  fprintf (logfile, "(nothing)\n");
+	fprintf (logfile, "\n");
+	found++;
+      }
+  if (found == 0)
+    fprintf (logfile, "No objects have the name %s!\n\n", name);
+}
 
 /*
  * Builds up the lists of artifacts from the file in the libdir.
@@ -1009,7 +1098,7 @@ void init_artifacts() {
     else if (!strncmp(cp, "Object",6)) {
 	art->item = (object *) malloc(sizeof(object));
 	reset_object(art->item);
-        if (!load_object(fp, art->item))
+        if (!load_object(fp, art->item,LO_LINEMODE))
 	   LOG(llevError,"Init_Artifacts: Could not load object.\n");
 	art->item->name = add_string((strchr(cp, ' ')+1));
 	al=find_artifactlist(art->item->type);
@@ -1128,40 +1217,46 @@ void add_abilities(object *op, object *change) {
     op->level = -(change->level);
   else
     op->level += change->level;
-  if (change->armour)
+  if (change->armour) {
     if (change->armour < 0)
       op->armour = (-change->armour);
     else
       op->armour  = (signed char) (((int)op->armour * (int)change->armour)/10);
-  if (change->stats.dam)
+  }
+  if (change->stats.dam) {
     if (change->stats.dam < 0)
       op->stats.dam = (-change->stats.dam);
     else if (op->stats.dam) {
       tmp = (signed char) (((int)op->stats.dam * (int)change->stats.dam)/10);
-      if (tmp == op->stats.dam)
+      if (tmp == op->stats.dam) {
         if (change->stats.dam < 10)
           op->stats.dam--;
         else
           op->stats.dam++;
+      }
       else
         op->stats.dam = tmp;
     }
-  if (change->weight)
+  }
+  if (change->weight) {
     if (change->weight < 0)
       op->weight = (-change->weight);
     else
       op->weight = (op->weight    * (change->weight)) / 100;
-  if (change->last_sp)
+  }
+  if (change->last_sp) {
     if (change->last_sp < 0)
       op->last_sp = (-change->last_sp);
     else
       op->last_sp = (signed char) (((int)op->last_sp * (int)change->last_sp) / (int) 100);
-  if (change->last_heal)
+  }
+  if (change->last_heal) {
     if (change->last_heal < 0)
       op->last_heal = (-change->last_heal);
     else
       op->last_heal = (signed char) (((int)op->last_heal * ((int)change->last_heal))
                       / (int)100);
+  }
   op->value *= change->value;
 
   if(change->material)  op->material = change->material;
