@@ -38,6 +38,8 @@
 #include "debug.h"
 #include "util.h"
 
+#include <proto.h>
+
 Pixmap *pixmaps;	/* list of pixmaps */
 Pixmap *masks;		/* list of masks */
 #define E_EDITABLE   (E_MONSTER | E_EXIT | E_TREASURE | E_BACKGROUND | \
@@ -553,6 +555,23 @@ static void FillCb (Widget w, XtPointer client, XtPointer call)
     AppSelectUnset(self);
 }
 
+/* 
+ * callback: FillBelow
+ */
+static void FillBelowCb (Widget w, XtPointer client, XtPointer call)
+{
+    App self = (App)client;
+    
+    debug0 ("AppFillCb()\n");
+    if(!self->look.edit) {
+	CnvNotify("Select point to Fill","Continue",NULL);
+	return;
+    }
+    EditPerformFillBelow(self->look.edit,self->look.rect.x,self->look.rect.y);
+    EditModified(self->look.edit);
+    AppSelectUnset(self);
+}
+
 /* 
  * callback: Box
  */
@@ -588,6 +607,24 @@ static void WipeCb (Widget w, XtPointer client, XtPointer call)
     AppSelectUnset(self);
 }
 
+/* 
+ * callback: WipeBelow
+ */
+static void WipeBelowCb (Widget w, XtPointer client, XtPointer call)
+{
+    App self = (App)client;
+    
+    debug0 ("AppWipeBelowCb()\n");
+    if(!self->look.edit) {
+	CnvNotify("Select area to Wipe","Continue",NULL);
+	return;
+    }
+    
+    EditShaveRectangleBelow(self->look.edit, self->look.rect);
+    EditModified(self->look.edit);
+    AppSelectUnset(self);
+}
+
 /*
  * menu definition
  */
@@ -597,8 +634,10 @@ static CnvMenuRec editMenu[] = {
     {"paste",PasteCb},
     {"-----",NULL},
     {"fill" ,FillCb},
+    {"fillbelow" ,FillBelowCb},
     {"box" ,BoxCb},
     {"wipe" ,WipeCb},
+    {"wipebelow" ,WipeBelowCb},
     {"",     NULL}
 }; 
 
@@ -783,21 +822,25 @@ App AppCreate(XtAppContext appCon,
     /*** images & colors ***/
     InitializeColors(XtDisplay(self->shell));
 
-    if (self->res.usePixmaps) settings.displaymode=Dm_Bitmap;
-#ifdef Xpm_Pix
-    else if (self->res.useColorPixmaps) settings.displaymode=Dm_Pixmap;
+    if (self->res.usePixmaps) displaymode=Dm_Bitmap;
+#ifdef HAVE_LIBXPM
+    else if (self->res.useColorPixmaps) displaymode=Dm_Pixmap;
+#else
+    else if (self->res.useColorPixmaps) {
+	fprintf(stderr,"Crossedit not compiled with Xpm Support.\n");
+    }
 #endif
 
     CnvInitialize(self->shell);
 
-    if(settings.displaymode==Dm_Pixmap) {
+    if(displaymode==Dm_Pixmap) {
 	if (ReadPixmaps(self->display, &pixmaps, &masks, &colormap)) {
 	    /* We really should do something better than this */
 	    fprintf(stderr,"Not enough space in colormap - switch colormap.\n");
 /*	    exit(1);*/
 	}
     } else {
-	if (settings.displaymode==Dm_Bitmap)
+	if (displaymode==Dm_Bitmap)
 	    pixmaps = ReadBitmaps(self->display);
 	else if (!check_font_path(XtDisplay (self->shell))) {
 	    fprintf(stderr, "Trying to fix fontpath...\n");
