@@ -120,7 +120,8 @@ int GetType (object *tmp)
 	return T_Map;
     case DIRECTOR:
     case FIREWALL:
-	return (tmp->arch->animations > 0) ? T_Director : T_Default;
+	return (NUM_ANIMATIONS(tmp) > 0) 
+	    ? T_Director : T_Default;
     case CONTAINER:
 	return T_Container;
     default:
@@ -386,7 +387,7 @@ char *allowed_variables[] = {
     "can_use_horn", "expmul", "unique", "make_invisible", "is_wooded",
     "is_hilly", "has_ready_skill", "has_ready_weapon", "no_skill_ident",
     "glow_radius", "is_blind", "can_see_in_dark", "is_cauldron",
-    "randomitems", "is_dust", "no_steal",
+    "randomitems", "is_dust", "no_steal", "one_hit",
 #ifdef NPC_PROG
     "npc_status", "npc_program", 
 #endif
@@ -948,20 +949,26 @@ void AttrApply(Attr self)
     String str,var;
     object *ob;
     char buf[BUFSIZ];
-    int len, i, mask;
+    int len, i, mask,set_all=1;
 
     /* check out, that object exist */
     if(!self->op) {
 	return;
     }
-    
-    for(ob = self->op; ob; ob = ob->more) {
+
+    for(ob = self->op; ob && set_all; ob = ob->more) {
 	/*** variable ***/
 	XtVaGetValues(self->iw.var,
 		      XtNstring,&var,
 		      NULL);
 	if(var && *var) {
 	    debug1("AttrApply(), %s\n",var);
+
+	    /* This is a pretty gross hack, but somewhat necessary.  Otherwise,
+	     * all pieces of a multisquare monster will get the treasure, which
+	     * is really not what we want.
+	     */
+	    if (!strncmp(var, "randomitems", 11)) set_all=0;
 	    for (i = 0; i < ALLOWED_VARIABLES; i++)
 		if (!strncmp (allowed_variables[i], var, 
 			      strlen(allowed_variables[i]))) {
@@ -1010,7 +1017,7 @@ void AttrApply(Attr self)
 		}
 	    }
 	}
-    }
+    } /* for all parts of the object */
 
     /*** clear variables ***/
     XtVaSetValues(self->iw.var,XtNstring,NULL,NULL);
