@@ -7,7 +7,6 @@
 
 #include "global.h"
 #include "debug.h"
-#include "xio.h"
 
 #include "Cnv.h"
 #include "CrEditP.h"
@@ -297,7 +296,7 @@ static void AnimateCursor (Widget w, int x, int y) {
     int yd = y - old_y;
     int i; 
     
-    if (settings.displaymode==Dm_Bitmap && XtIsRealized (w)) {	
+    if (displaymode==Dm_Bitmap && XtIsRealized (w)) {	
 	if (!cursors_initialized) { 	    
 	    archetype *wizu = find_archetype ("editcursor");
 
@@ -307,22 +306,22 @@ static void AnimateCursor (Widget w, int x, int y) {
 	    for (i = 0; i < 4; i++)
 		wizc[i] =  XCreatePixmapCursor 
 		    (XtDisplay(w), 
-		     pixmaps[wizu->faces[i]], 
-		     pixmaps[wizu->faces[i + 4]],
+		     pixmaps[GET_ANIMATION((&wizu->clone), i)], 
+		     pixmaps[GET_ANIMATION((&wizu->clone), (i+4))],
 		     discolor, discolor + 12, 12, 12);
 	    cursors_initialized++;
 	}
-	if (abs (xd) > abs (yd))
+	if (abs (xd) > abs (yd)) {
 	    if (xd < 0)
 		XDefineCursor (XtDisplay(w), XtWindow(w), wizc[3]);
 	    else
 		XDefineCursor (XtDisplay(w), XtWindow(w), wizc[1]);
-	else
+	} else {
 	    if (yd < 0)
 		XDefineCursor (XtDisplay(w), XtWindow(w), wizc[0]);
 	    else
 		XDefineCursor (XtDisplay(w), XtWindow(w), wizc[2]);
-	
+	}
 	old_x = x;
 	old_y = y;
     }
@@ -393,7 +392,8 @@ static void UpdatePosition (Widget w, int x, int y,Boolean inv)
 	    while (op) {
 		New_Face f;
 		if (QUERY_FLAG(op, FLAG_TEAR_DOWN) && self->crEdit.show_weak_walls)
-		    f=new_faces[op->arch->faces[op->arch->animations/2]];
+		    f=new_faces[GET_ANIMATION(op,
+			NUM_ANIMATIONS(op)/2)];
 		else f = *op->face;
 		if (inv  == True) {
 		    int tmp = f.fg;
@@ -416,24 +416,22 @@ static void UpdatePosition (Widget w, int x, int y,Boolean inv)
 	}
     } else { /* Normal map drawing routine */
         New_Face f;
-        if (settings.displaymode==Dm_Pixmap) {
+        if (displaymode==Dm_Pixmap) {
           f = *(get_map_floor (self->crEdit.map, x, y)->face);
           FaceDraw (w, self->crEdit.gc, &f,
                     x * self->crEdit.fontSize,
                     y * self->crEdit.fontSize);
-#ifdef DOUBLE_FLOOR
           f = *(get_map_floor2 (self->crEdit.map, x, y)->face);
 	  if (f.number != blank_face->number)
 	    FaceDraw (w, self->crEdit.gc, &f,
                     x * self->crEdit.fontSize,
                     y * self->crEdit.fontSize);
-#endif
 	}
 	op=get_map_ob(self->crEdit.map, x, y);
 	while (op && op->above) op=op->above;
 	if (!op) f=*blank_face;
 	else if (QUERY_FLAG(op, FLAG_TEAR_DOWN) && self->crEdit.show_weak_walls)
-		f=new_faces[op->arch->faces[op->arch->animations/2]];
+		f=new_faces[GET_ANIMATION(op,NUM_ANIMATIONS(op)/2)/2];
 
 	else f = *op->face;
 
@@ -442,7 +440,7 @@ static void UpdatePosition (Widget w, int x, int y,Boolean inv)
 	    f.fg = f.bg;
 	    f.bg = tmp;
 	}
-	if (settings.displaymode!=Dm_Pixmap || f.number != blank_face->number)
+	if (displaymode!=Dm_Pixmap || f.number != blank_face->number)
 	    FaceDraw (w, self->crEdit.gc, &f, 
 		  x * self->crEdit.fontSize, 
 		  y * self->crEdit.fontSize);
