@@ -317,7 +317,7 @@ static Boolean Load(Edit self, char *name)
     char buf[BUFSIZ];
     
     strcpy(path, name);
-    if((mask = check_path (path)) < 4) {
+    if((mask = check_path (path,1)) < 4) {
 	sprintf(buf,"You can't access %s",path);
 	CnvNotify(buf,"Continue",NULL);
 	return False;
@@ -332,7 +332,7 @@ static Boolean Load(Edit self, char *name)
     if(self->emap)
 	strcpy(save,self->emap->path); /* klydge */
     EdFreeMap (self);
-    if(!(tmp = ready_map_name (path, 1))) {
+    if(!(tmp = ready_map_name (path, MAP_FLUSH))) {
 	/* Make the map null, since right now we don't have a valid map
 	 * in memory.  The CnfNotify call will force a redraw - we
 	 * can check for Null map data pretty easy, but can't really check
@@ -341,7 +341,7 @@ static Boolean Load(Edit self, char *name)
 	XtVaSetValues(self->w, XtNmap, NULL, NULL);
 	sprintf(buf,"Cannot load map %s",path);
 	CnvNotify(buf,"Continue",NULL);
-	if(!(tmp = ready_map_name(save,1))) return False;
+	if(!(tmp = ready_map_name(save,MAP_FLUSH))) return False;
     }
 
     /*** initialiaze map ***/
@@ -1418,6 +1418,23 @@ void EditPerformFill(Edit self, int x, int y)
     }
 }
 
+/*
+ * member: make fill
+ * x,y   : point to start fill
+ */
+void EditPerformFillBelow(Edit self, int x, int y)
+{
+    int dx, dy;
+    if (!self->emap) return;
+
+    for (dx=x; dx< x+self->app->look.rect.width; dx++) {
+	for (dy=y; dy < y+self->app->look.rect.height; dy++) {
+	    if (!out_of_map (self->emap, dx, dy))
+		EditInsert(self, dx, dy, -1);
+	}
+    }
+}
+
 /*
  * member: fill
  * x,y   : point to start fill
@@ -1475,6 +1492,21 @@ void EditShaveRectangle(Edit self, XRectangle rec)
     return;
 }
 
+void EditShaveRectangleBelow(Edit self, XRectangle rec)
+{
+    int x,y;
+
+    if (!self || !self->emap || self->read_only)
+	return;
+
+    for(x=0; x < rec.width; x++)
+	for(y=0; y < rec.height; y++)
+	    EditObjectDelete (self, x + rec.x, y + rec.y, -1);
+
+    CrEditRefresh (self->w, rec);
+    return;
+}
+
 /*
  * member: copy rectangle area (x,y,width,height) from src to
  *         self to point (x,y)
@@ -1611,11 +1643,21 @@ Boolean EditObjectDelete (Edit self, int x, int y, int z)
     object *obj;
 
     obj = MapGetObjectZ(self->emap, x, y, z);
-    if (self->read_only || z < 0 || !obj)
+    if (self->read_only || !obj)
       return False;
 
     if (obj->head)
-	obj = obj->head;
+		obj = obj->head;
+
+    if (z < 0) {
+	if (obj->below)
+	    obj = obj->below;
+	else
+	    return False;
+    }
+    if (z > 0)
+        obj = obj->above;
+
     if (self->app->attr && self->app->attr->op == obj)
 	AttrChange(self->app->attr, NULL, 0, 0);
     if (AppItemGetEdit(self->app) == self)
@@ -1625,7 +1667,8 @@ Boolean EditObjectDelete (Edit self, int x, int y, int z)
 
     remove_ob (obj);
     free_object (obj);
-    update_position (self->emap, x, y);
+    /* remove_ob should do this for us */
+    /*update_position (self->emap, x, y);*/
     return True;
 }
 
@@ -1689,6 +1732,12 @@ Boolean EditInsert (Edit self,int x, int y, int z)
 	if (op && op->arch == self->app->item.clone->arch)
 	    return False;
     }
+    /* Hopefully this doesn't break anything */
+    else {
+	op = get_map_ob (self->emap, x, y);
+	if (op && op->arch == self->app->item.clone->arch)
+	    return False;
+    }
 
     debug3("EditInsert() %dx%dx%d\n",x,y,z);
 
@@ -1722,7 +1771,6 @@ Boolean EditInsert (Edit self,int x, int y, int z)
 }
 
 
-
 /*
  * member: insert object to point of map of editor
  * object:
