@@ -60,6 +60,8 @@ String fallback_resources[] = {
     NULL
 };
 
+enum DisplayMode displaymode;
+
 void EditDirtyAc(Widget w, XEvent * event, String * argv, Cardinal * argc) {
     debug0 ("EditDirtyAc\n");
 }
@@ -135,6 +137,7 @@ void main (int argc, char **argv, char **env)
 
     XtRealizeWidget (app->shell);
     XtAppMainLoop (app_con);
+    exit(0);
 }
 
 /*** End of crossfire.c ***/
