@@ -2,7 +2,7 @@
  * reference: Portable C Software / Mark Horton, ISBN 0-13-868959-7
  */
 
-#ifndef _Posix_h
+#if !defined(_Posix_h) && !defined(__CEXTRACT__)
 #define _Posix_h
 
 /* sys/types needs to be included before POSIX_SOURCE is defined on
