@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_commands_h =
- *   "$Id: commands.h,v 1.11 1997/01/05 09:57:08 master Exp $";
+ *   "$Id: commands.h,v 1.12 1998/09/28 02:43:00 master Exp master $";
  */
 
 /*
@@ -43,9 +43,19 @@
  * be initialized in one of the source files.
  */
 
+typedef int (*CommFunc)(object *op, char *params);
+
+typedef struct {		/* global list's structure */
+  char *name;
+  CommFunc func;
+  float	time;			/* How long it takes to execute this command */
+} CommArray_s;
+
+
 extern CommArray_s Commands[],NewServerCommands [],SocketCommands[],
 	WizCommands [];
 
 extern const int CommandsSize,NewServerCommandSize,
 	SocketCommandsSize, WizCommandsSize;
 
+
