@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_define_h =
- *   "$Id: define.h,v 1.54 1997/03/09 04:52:49 master Exp master $";
+ *   "$Id: define.h,v 1.60 1998/09/28 02:44:07 master Exp master $";
  */
 
 /*
@@ -41,13 +41,10 @@
 #error - Your ANSI C compiler should be defining __STDC__
 #endif
 
+#include <autoconf.h>
 
-/* The offsetof macro is part of ANSI C, but many compilers lack it, for
- * example "gcc -ansi"
- */
-#if !defined (offsetof) && !defined(sgi)
-#define offsetof(type, member) (int)&(((type *)0)->member)
-#endif
+#define FONTDIR ""
+#define FONTNAME ""
 
 /* Decstations have trouble with fabs()... */
 #define FABS(x) (x<0?-x:x)
@@ -140,6 +137,10 @@
 #define BLINDNESS	49
 #define GOD		50
 
+/*  peterm:  detector is an object which notices the presense of
+	 another object and is triggered like buttons.  */
+#define DETECTOR 51
+
 #define SPEEDBALL	52
 #define DEAD_OBJECT	53
 #define DRINK		54
@@ -284,7 +285,7 @@
 #define AT_GODPOWER	0x00100000 /*1048576  peterm@soda.berkeley.edu */
 #define AT_HOLYWORD	0x00200000 /*2097152 race selective attack thomas@astro.psu.edu */ 
 #define AT_BLIND	0x00400000 /*4194304 thomas@astro.psu.edu */ 
-
+#define AT_INTERNAL	0x00800000 /* Only used for internal calculations */
 
 #define PATH_NULL	0x00000000
 #define PATH_PROT	0x00000001
@@ -401,7 +402,8 @@
 #define FLAG_APPLY_ONCE		20 /* Will dissapear when applied */
 #define FLAG_SEE_INVISIBLE 	21 /* Will see invisible player */
 #define FLAG_CAN_ROLL		22 /* Object can be rolled */
-#define FLAG_IS_TURNING		23 /* Object will turn after player */
+/* FLAG_IS_TURNING is no longer used */
+/*#define FLAG_IS_TURNING		23 *//* Object will turn after player */
 #define FLAG_IS_TURNABLE 	24 /* Object can change face with direction */
 #define FLAG_WALK_OFF		25 /* Object is applied when left */
 #define FLAG_FLY_ON		26 /* As WALK_ON, but only with FLAG_FLYING */
@@ -477,7 +479,10 @@
 #define FLAG_IS_CAULDRON	94 /* container can make alchemical stuff */
 #define FLAG_DUST		95 /* item is a 'powder', effects throwing */
 #define FLAG_NO_STEAL		96 /* Item can't be stolen */
-#define NUM_FLAGS		96 /* Should always be equal to the last
+#define FLAG_ONE_HIT		97 /* Monster can only hit once before going
+				    * away (replaces ghosthit)
+				    */
+#define NUM_FLAGS		97 /* Should always be equal to the last
 				    * defined flag */
 
 /* Values can go up to 127 before the size of the flags array in the
@@ -520,22 +525,12 @@
 #define EXIT_X(xyz)     (xyz)->stats.hp
 #define EXIT_Y(xyz)     (xyz)->stats.sp
 
-#define MAP_TIMEOUT(m)	((m)->map_object->value)
-#define MAP_RESETTIME(m)	((m)->map_object->weight)
-
-/* This is only used when loading the map.  For internal use (treausre
- * generation, etc). the map structure difficulty field should be used.
- */
-#define MAP_DIFFICULTY(m)	((m)->map_object->level)
-
 /* for use by the lighting code */
 #define MAX_LIGHT_RADII 	4	/* max radii for 'light' object, really
 					 * large values allow objects that can
 					 * slow down the game */
 #define MAX_DARKNESS		5	/* maximum map darkness, there is no
 					 * practical reason to exceed this */ 
-#define MAP_DARKNESS(m)	   	(m)->map_object->invisible>MAX_DARKNESS? \
-				  MAX_DARKNESS:(m)->map_object->invisible;
 #define BRIGHTNESS(xyz)		(xyz)->glow_radius>MAX_LIGHT_RADII? \
 				  MAX_LIGHT_RADII:(xyz)->glow_radius;
 
@@ -554,61 +549,6 @@
 #define WEAPON_SPEED(xyz)	(xyz)->last_sp
 
 
-/*#ifdef USE_BUTTONS*/
-#ifndef NUMDIRBUTTS
-#define NUMDIRBUTTS 9
-#define NW_button 0
-#define NO_button 1
-#define NE_button 2
-#define WE_button 3
-#define BR_button 4
-#define EA_button 5
-#define SW_button 6
-#define SO_button 7
-#define SE_button 8
-#define NUMOPBUTTS 4
-#define CHANGE_button 0
-#define APPLY_button 1
-#define PEACE_button 2
-#define TALK_button 3
-#endif
-/*#endif*/
-
-#define MAP_IN_MEMORY 1
-#define MAP_SWAPPED 2
-#define MAP_LOADING 3
-#define MAP_SAVING 4
-
-/*
- * Check nrof variable *before* calling CAN_MERGE()
- * stats.sp is used in wands, spellbooks and scrolls.
- * the 0x400000 on flags2 is FLAG_INV_LOCK.  I don't think something
- * being locked in inventory should prevent merging.
- */
-
-
-#define CAN_MERGE(ob1,ob2) \
-  ((ob1) != (ob2) && \
-   (ob1)->type == (ob2)->type && \
-   ((ob1)->speed == (ob2)->speed) && \
-   (QUERY_FLAG((ob1),FLAG_ANIMATE) || FABS((ob1)->speed) < 0.001 ) && \
-   (ob1)->arch == (ob2)->arch && \
-   (ob1)->stats.sp == (ob2)->stats.sp && \
-   (ob1)->flags[0] == (ob2)->flags[0] && \
-   (ob1)->flags[1] == (ob2)->flags[1] && \
-   ((ob1)->flags[2] & ~0x400000) == ((ob2)->flags[2] & ~ 0x400000) && \
-   (ob1)->flags[3] == (ob2)->flags[3] && \
-   (ob1)->magic == (ob2)->magic && \
-   (ob1)->name == (ob2)->name && \
-   (ob1)->title == (ob2)->title && \
-   (ob1)->msg == (ob2)->msg && \
-   ((ob1)->type!=SCROLL || (ob1)->level==(ob2)->level) && \
-   (ob1)->stats.food == (ob2)->stats.food && \
-   (ob1)->protected == (ob2)->protected && \
-   (ob1)->immune == (ob2)->immune && \
-   (ob1)->vulnerable == (ob2)->vulnerable \
-)
-
 /*
  * Flags to generate_treasures():
  */
@@ -678,15 +618,6 @@
 #define LO4     15 /* bitmasks for upper and lower 4 bits from 8 bit fields */
 #define HI4    240
 
-/*
- * Use of the global variable "server_mode":
- */
-
-#define SERVER_OFF	0
-#define SERVER_ENABLED	1
-#define SERVER_ABORTED	2
-#define SERVER_DISABLED	3
-
 /*
  * Use of the state-variable in player objects:
  */
@@ -700,7 +631,6 @@
 #define ST_GET_NAME     6
 #define ST_GET_PASSWORD 7
 #define ST_CONFIRM_PASSWORD     8
-#define ST_MENU_MORE	9
 #define ST_GET_PARTY_PASSWORD 10
 
 #define BLANK_FACE_NAME "blank.111"
@@ -716,6 +646,10 @@
 #define INV_MAGIC_FACE_NAME	"inv-magic.111"
 #define INV_UNPAID_FACE_NAME	"inv-unpaid.111"
 
+#define DARK_FACE1_NAME "dark1.111"
+#define DARK_FACE2_NAME "dark2.111"
+#define DARK_FACE3_NAME "dark3.111"
+
 /* separated this from the common/item.c file. b.t. Dec 1995 */
 
 #define DESCRIBE_ABILITY(retbuf, variable, name) \
@@ -748,3 +682,36 @@
         } \
       strcat(retbuf,")"); \
     }
+
+#define AP_APPLY 1
+#define AP_UNAPPLY 2
+
+
+/*
+ * random() is much better than rand().  If you have random(), use it instead.
+ * You shouldn't need to change any of this
+ *
+ * 0.93.3: It looks like linux has random (previously, it was set below
+ * to use rand).  Perhaps old version of linux lack rand?  IF you run into
+ * problems, add || defined(linux) the #if immediately below.
+ *
+ * 0.94.2 - you probably shouldn't need to change any of the rand stuff
+ * here.
+ */
+
+#ifdef HAVE_SRANDOM
+#define RANDOM() random()
+#define SRANDOM(xyz) srandom(xyz)
+#else
+#  ifdef HAVE_SRAND48
+#  define RANDOM() lrand48()
+#  define SRANDOM(xyz) srand48(xyz)
+#  else
+#    ifdef HAVE_SRAND
+#      define RANDOM() rand()
+#      define SRANDOM(xyz) srand(xyz)
+#    else
+#      "Could not find a usuable random routine"
+#    endif
+#  endif
+#endif
