@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_global_h =
- *   "$Id: global.h,v 1.34 1997/03/09 04:53:44 master Exp master $";
+ *   "$Id: global.h,v 1.40 1998/09/28 02:46:33 master Exp master $";
  */
 
 /*
@@ -34,32 +34,81 @@
 
 #include "includes.h"
 
-#define NUM_COLORS		13
-#define DEFAULT_PLAYER_FG	0
-#define DEFAULT_PLAYER_BG	8
-#define DEFAULT_PLAYER_FIELD	12
 
-#define XDRAWIMAGESTRING(disp,win,gc,x,y,str,len) \
-XDrawImageString16(disp,win,gc,x,y,str,len)
-#define FontindexToXChar(s) fontindex_to_XChar2b(s)
-#define XCharToFontindex(s) ((Fontindex) ((s).byte1*256 +  (s).byte2))
+/* Type defines for specific signed/unsigned variables of a certain number
+ * of bits.  Not really used anyplace, but if a certain number of bits
+ * is required, these type defines should then be used.  This will make
+ * porting to systems that have different sized data types easier.
+ *
+ * Note: The type defines should just mean that the data type has at
+ * least that many bits.  if a uint16 is actually 32 bits, no big deal,
+ * it is just a waste of space.
+ *
+ * Note2:  When using something that is normally stored in a character
+ * (ie strings), don't use the uint8/sint8 typdefs, use 'char' instead.
+ * The signedness for char is probably not universal, and using char
+ * will probably be more portable than sint8/unit8
+ */
+ 
+typedef unsigned int	uint32;
+typedef signed int	sint32;
+typedef unsigned short	uint16;
+typedef signed short	sint16;
+typedef unsigned char	uint8;
+typedef signed char	sint8;
 typedef unsigned short Fontindex;
-typedef XChar2b XChar; 
+
+/* This blob, in this order, is needed to actually define maps */
+#include "face.h"
+#include "living.h"
+#include "object.h"
+#include "map.h"
+
+/* Pull in the socket structure - used in the player structure */
+#include "newserver.h"
+
+/* Pull in the player structure */
+#include "player.h"
+
+/* pull in treasure structure */
+#include "treasure.h"
+
+/*
+ * So far only used when dealing with artifacts.
+ * (now used by alchemy and other code too. Nov 95 b.t).
+ */
+typedef struct linked_char {
+  char *name;
+  struct linked_char *next;
+} linked_char;
 
 
-#include "structs.h"
-#if defined(SOUND_EFFECTS) && !defined(__CEXTRACT__)
-#include <rplay.h> /* RPLAY structure */
-#endif
+/* Pull in artifacts */
+#include "artifact.h"
+
+/* Now for gods */
+#include "god.h"
+
+/* Now for races */
+#include "race.h"
+
+/* Now for recipe/alchemy */
+#include "recipe.h"
+
+/* Now for spells */
+#include "spells.h"
+
+#include "funcpoint.h"
 
 /*****************************************************************************
  * GLOBAL VARIABLES:							     *
  *****************************************************************************/
 
 
-extern Settings settings;
+#define NUM_COLORS		13
 
 extern char *colorname[NUM_COLORS][2];
+
 extern New_Face *new_faces;
 
 /*
@@ -71,7 +120,6 @@ EXTERN treasurelist *first_treasurelist;
 EXTERN artifactlist *first_artifactlist;
 EXTERN archetype *first_archetype;
 EXTERN objectlink *first_friendly_object;	/* Objects monsters will go after */
-EXTERN sockets *first_socket;			/* Start of linked list of sockets */
 EXTERN godlink *first_god;
 EXTERN racelink *first_race;
 
@@ -85,8 +133,6 @@ EXTERN long editor;     /* if true, edit maps instead of playing (almost obsolet
  * Variables set by different flags (see init.c):
  */
 
-EXTERN int maplevel;
-EXTERN long delete_last_file;
 EXTERN long warn_archetypes;	/* If true, write warnings when failing */
 				/* to find archetypes when loading from file */
 EXTERN long init_done;			/* Ignores signals until init_done is true */
@@ -100,8 +146,6 @@ extern long pticks;		/* used by various function to determine */
  */
 EXTERN FILE *logfile;			/* Used by server/daemon.c */
 EXTERN int exiting;			/* True if the game is about to exit */
-EXTERN char *name;			/* The name of the game, usually "crossfire" */
-EXTERN long last_micro;		/* When process_active_maps() was last exec */
 EXTERN long nroftreasures;		/* Only used in malloc_info() */
 EXTERN long nrofartifacts;		/* Only used in malloc_info() */
 EXTERN long nrofallowedstr;		/* Only used in malloc_info() */
@@ -120,11 +164,6 @@ EXTERN archetype *empty_archetype;	/* Nice to have fast access to it */
 EXTERN archetype *map_archeytpe;
 EXTERN char first_map_path[MAX_BUF];	/* The start-level */
 
-#ifdef LONGJUMP
-EXTERN jmp_buf jump_addr;
-#endif
-EXTERN Atom    Protocol_atom, Kill_atom;
-EXTERN XClientMessageEvent *cmev;
 EXTERN char errmsg[HUGE_BUF];
 EXTERN long ob_count;
 /*
@@ -132,13 +171,28 @@ EXTERN long ob_count;
  */
 EXTERN archetype *ring_arch,*amulet_arch,*staff_arch,*crown_arch;
 EXTERN char *undead_name;	/* Used in hit_player() in main.c */
-EXTERN sockets *active_socket;
+
+EXTERN Animations *animations;
+EXTERN int num_animations,animations_allocated;
+
+
+#define SET_ANIMATION(ob,newanim) ob->face=&new_faces[animations[ob->animation_id].faces[newanim]]
+#define GET_ANIMATION(ob,anim) (animations[ob->animation_id].faces[anim])
+#define GET_ANIM_ID(ob) (ob->animation_id)
+/* NUM_ANIMATIONS returns the number of animations allocated.  The last
+ * usuable animation will be NUM_ANIMATIONS-1 (for example, if an object
+ * has 8 animations, NUM_ANIMATIONS will return 8, but the values will
+ * range from 0 through 7.
+ */
+#define NUM_ANIMATIONS(ob) (animations[ob->animation_id].num_animations)
+#define NUM_FACINGS(ob) (animations[ob->animation_id].facings)
 
 extern int freearr_x[SIZEOFFREE], freearr_y[SIZEOFFREE];
 extern int maxfree[SIZEOFFREE], freedir[SIZEOFFREE];
 
 extern New_Face *blank_face;
 extern New_Face *blocked_face;
+extern New_Face *dark_faces[];
 extern New_Face *stipple1_face;
 extern New_Face *stipple2_face;
 extern New_Face *potion_face;
@@ -147,16 +201,11 @@ extern New_Face *inv_curse_face, *inv_damn_face, *inv_equip_face, *inv_lock_face
          *inv_magic_face, *inv_unpaid_face;
 
 
-#ifdef SOUND_EFFECTS
-EXTERN RPLAY	*sound_table[NROF_SOUNDS];
-#endif
-
 extern long max_time;	/* loop time */
 
 EXTERN char *font_graphic;
 
 #ifndef __CEXTRACT__
-#include <xdir.h>
 #include "libproto.h"
 #endif
 
@@ -172,10 +221,8 @@ EXTERN char *font_graphic;
 #define get_map_floor(M,X,Y)	(&(M)->floor[(X)+((M)->mapx*(Y))])
 #define set_map_floor(M,X,Y,C)	((M)->floor[(X)+((M)->mapx*(Y))]=*(C))
 
-#ifdef DOUBLE_FLOOR
 #define get_map_floor2(M,X,Y)	(&(M)->floor2[(X)+((M)->mapx*(Y))])
 #define set_map_floor2(M,X,Y,C)	((M)->floor2[(X)+((M)->mapx*(Y))]=*(C))
-#endif
 
 #define decrease_ob(xyz) decrease_ob_nr(xyz,1)
 
@@ -191,65 +238,43 @@ EXTERN char *font_graphic;
 # define CFREE(x)	free(x)
 #endif
 
-/*
- * Made this a macro since it's mostly used in time-critical areas:
- * (Hopefully a good optimizer will be able to move the if() out of
- * any loop where this macro is used.  That wouldn't have been possible
- * if this was a function!)
- */
-
-/*
- * Have two version of this macro.  One if Xpm_Pix is defined, another
- * if Xpm_Pix is not defined.  IT would be better to actually have the
- * #ifdef's inside the macro itself, instead of having two versions, but
- * I was not able to get that to work.
- * Mark Wedel (mark@pyramid.com)
- */
-
-#ifdef Xpm_Pix
-#define draw_face(pl,win,gc,x,y,face) \
-{ \
-  if (pl->color_pixmaps) \
-  { \
-    XSetClipMask(pl->gdisp, gc, pl->masks[(face)]); \
-    XSetClipOrigin(pl->gdisp, gc, x, (y) - 24); \
-    XCopyArea(pl->gdisp, pl->pixmaps[(face)], win, gc, 0, 0, 24, 24, \
-	(unsigned int) (x), (unsigned int) ((y) - 24)); \
-  } \
-  else if(pl->use_pixmaps) \
-  { \
-    XCopyPlane(pl->gdisp,pl->pixmaps[(face)],win,gc,0,0,24,24, \
-               (unsigned int) (x),(unsigned int) ((y) - 24),1); \
-  } \
-  else \
-  { \
-    XChar buf; \
-    buf = FontindexToXChar((Fontindex) (face)); \
-    XDRAWIMAGESTRING(pl->gdisp,win,gc,x,y,&buf,1); \
-  } \
-}
+#if HAVE_DIRENT_H
+# include <dirent.h>
+# define NAMLEN(dirent) strlen((dirent)->d_name)
 #else
-#define draw_face(pl,win,gc,x,y,face) \
-{ \
-  if(pl->use_pixmaps) \
-  { \
-    XCopyPlane(pl->gdisp,pl->pixmaps[(face)],win,gc,0,0,24,24, \
-               (unsigned int) (x),(unsigned int) ((y) - 24),1); \
-  } \
-  else \
-  { \
-    XChar buf; \
-    buf = FontindexToXChar((Fontindex) (face)); \
-    XDRAWIMAGESTRING(pl->gdisp,win,gc,x,y,&buf,1); \
-  } \
-}
+# define dirent direct
+# define NAMLEN(dirent) (dirnet)->d_namlen
+# if HAVE_SYS_NDIR_H
+#  include <sys/ndir.h>
+# endif
+# if HAVE_SYS_DIR_H
+#  include <sys/dir.h>
+# endif
+# if HAVE_NDIR_H
+#  include <ndir.h>
+# endif
 #endif
 
-/*
- * Have to define LOG() to nothing if NO_LOG is defined.
- */
-#ifdef NO_LOG
-#define LOG()
-#endif
+enum DisplayMode {Dm_Font, Dm_Bitmap, Dm_Pixmap};
+
+typedef struct Settings {
+    char    *logfilename;   /* logfile to use */
+    uint16  csport;	    /* port for new client/server */
+    LogLevel debug;	    /* Default debugging level */
+    uint8   dumpvalues;	    /* Set to dump various values/tables */
+    char    *dumparg;       /* additional argument for some dump functions */
+    uint8   daemonmode;     /* If true, detach and become daemon */
+    int	    argc;	    /* parameters that were passed to the program */
+    char    **argv;	    /* Only used by xio.c, so will go away at some time */
+    char    *libdir;	    /* Library location (ARchetypes, scores, etc. */
+    char    *playerdir;	    /* Where the player files are */
+    char    *mapdir;	    /* Where the map files are */
+    char    *archetypes;    /* name of the archetypes file - libdir is prepended */
+    char    *treasures;	    /* location of the treasures file. */
+    char    *uniquedir;	    /* directory for the unique items */
+    char    *tmpdir;	    /* Directory to use for temporary files */
+} Settings;
+
+extern Settings settings;
 
 #endif /* GLOBAL_H */
