@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_includes_h =
- *   "$Id: includes.h,v 1.19 1997/01/05 09:58:36 master Exp $";
+ *   "$Id: includes.h,v 1.23 1998/09/28 02:47:45 master Exp master $";
  */
 
 /*
@@ -30,6 +30,7 @@
 #define INCLUDES_H
 
 #define ASSERT(x)
+
 #if 0
 #define ASSERT(x) if (!(x)) { kill(getpid(),11);}
 #endif
@@ -52,6 +53,8 @@
 #  define __ultrix__
 #endif
 
+/* Include this first, because it lets us know what we are missing */
+#include <autoconf.h>
 
 #include <stdio.h>
 #include <string.h>
@@ -59,69 +62,36 @@
 #include <errno.h>
 #include <signal.h>
 #include <setjmp.h>
-#include <fcntl.h>
 #include <stdlib.h>
-#ifdef DMALLOC
+
+#ifdef HAVE_FCNTL_H
+#include <fcntl.h>
+#endif
+
+#ifdef HAVE_LIBDMALLOC
 #include <dmalloc.h>
 #endif
 
-/* tempnam() is not declared on NeXT's, linux with gcc -ansi */
-#if defined(NeXT) || (defined(linux) && defined(__STRICT_ANSI__))
-extern char *tempnam(char *dir, char *pfx);
-#endif
-
-
-#if defined(__sun__) && defined(SVR4)
+#ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
 
-#include <sys/types.h>
+#ifdef HAVE_SYS_TIME_H
 #include <sys/time.h>
-
-#if defined(_IBMR2) 
-#  include <time.h>
-#  include <math.h> /* Ack!  Can anyone with IBM tell me what requires this? */
 #endif
 
-#ifndef sgi
-#  include <sys/timeb.h>
+/* stddef is for offsetof */
+#ifdef HAVE_STDDEF_H
+#include <stddef.h>
 #endif
 
+#include <sys/types.h>
 
 #include <sys/stat.h>
-#include <X11/Xlib.h>
-#include <X11/Xutil.h>
+
 #include "config.h"
 #include "define.h"
 #include "logger.h"
 #include "newclient.h"
 
-#ifdef __osf__
-#  include <unistd.h>
-#  include "osf.h"
-#endif
-
-#ifdef __sgi__
-#  include <unistd.h>
-#  include <math.h>
-#  include "sgi.h"
-#endif
-
-#ifdef linux
-#  include <unistd.h>
-#endif
-
-#if defined (__sun__) && defined(StupidSunHeaders)
-#  include "sunos.h"   /* Prototypes for standard libraries, sunos lack those */
-#endif
-
-#ifdef pyrSVR4
-FILE *popen (const char *command, const char *type);
-char *tempnam (const char *dir, const char *pfx);
-#endif
-
-#ifndef _Xconst /* In case it's older than X11R4 */
-#define _Xconst const
-#endif
-
 #endif /* INCLUDES_H */
