@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_living_h =
- *   "$Id: living.h,v 1.20 1997/01/05 09:58:46 master Exp $";
+ *   "$Id: living.h,v 1.22 1998/09/28 02:48:25 master Exp master $";
  */
 
 /*
@@ -25,6 +25,9 @@
     The author can be reached via e-mail to mark@pyramid.com.
 */
 
+#ifndef LIVING_H
+#define LIVING_H
+
 #define STR 0
 #define DEX 1
 #define CON 2
@@ -54,3 +57,22 @@ extern char *statname[7];
 extern char *short_stat_name[7];
 extern char *lose_msg[7];
 extern float speed_bonus[MAX_STAT + 1];
+extern int weight_limit[MAX_STAT + 1];
+
+typedef struct liv { /* Mostly used by "alive" objects */
+  sint8		Str,Dex,Con,Wis,Cha,Int,Pow;
+  sint8		wc,ac;		/* Weapon Class and Armour Class */
+  sint16	hp;		/* Hit Points. */
+  sint16	maxhp;
+  sint16	sp;		/* Spell points.  Used to cast mage spells. */
+  sint16	maxsp;		/* Max spell points. */
+  sint16	grace;		/* Grace.  Used to invoke clerical prayers. */
+  sint16	maxgrace;	/* Grace.  Used to invoke clerical prayers. */
+  sint32	exp;		/* Experience.  Killers gain 1/10. */
+  sint16	food;		/* How much food in stomach.  0 = starved. */
+  sint16	dam;		/* How much damage this object does when hitting */
+  sint8		luck;		/* Affects thaco and ac from time to time */
+} living;
+
+
+#endif
