@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_object_h =
- *   "$Id: loader.h,v 1.28 1997/01/05 09:58:53 master Exp $";
+ *   "$Id: loader.h,v 1.31 1998/08/01 09:33:56 master Exp $";
  */
 
 /*
@@ -30,7 +30,7 @@
 enum varnrs {
   V_OBJECT,V_NAME,V_RACE,V_SLAYING,V_MSG,V_MSGEND,
   V_INVENTORY,V_ARCH,V_OTHER_ARCH,V_MORE,
-  V_ANIM,V_MINA,V_END,V_LAST_HEAL,V_LAST_SP,V_LAST_GRACE,V_LAST_EAT,
+  V_ANIMATION,V_END,V_LAST_HEAL,V_LAST_SP,V_LAST_GRACE,V_LAST_EAT,
   V_SPEED,V_SPEED_LEFT,V_SLOW_MOVE,
   V_FACE,V_STR,V_DEX,V_CON,V_WIS,V_CHA,V_INT,V_POW,V_HP,V_MAXHP,V_SP,V_MAXSP,
   V_GRACE,V_MAXGRACE,
@@ -58,13 +58,21 @@ enum varnrs {
   V_HAS_READY_HORN,V_CAN_USE_HORN,V_EXPMUL,V_UNIQUE,V_MAKE_INVIS,V_INV_LOCKED,
   V_IS_WOODED,V_IS_HILLY,V_HAS_READY_SKILL,V_HAS_READY_WEAPON,
   V_NO_SKILL_IDENT,V_GLOW_RADIUS,V_BLIND,V_SEE_IN_DARK,
-  V_IS_CAULDRON,V_RANDOMITEMS,V_DUST,V_NO_STEAL,
-#ifdef NPCPROG
-  V_NPC_STATUS,V_NPC_PROGRAM,
-#endif
+  V_IS_CAULDRON,V_RANDOMITEMS,V_DUST,V_NO_STEAL, V_ONE_HIT,
   NR_OF_VARIABLES
 };
 
+#define LL_IGNORED  -1
+#define LL_EOF	    0
+#define LL_MORE	    1
+#define LL_NORMAL   2
+
+/* see loader.l for more details on this */
+#define LO_REPEAT   0
+#define LO_LINEMODE 1
+#define LO_NEWFILE  2
+#define	LO_NOREAD   3
+
 extern char *variables[NR_OF_VARIABLES];
 
 extern char **xbm_names;
