@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_skills_h =
- *   "$Id: skills.h,v 1.6 1997/01/05 10:00:17 master Exp $";
+ *   "$Id: skills.h,v 1.7 1998/09/28 02:52:09 master Exp master $";
  */
 
 /*
@@ -128,5 +128,17 @@ enum skillnrs {
 #endif
 };
 
+typedef struct skill_struct {
+      char *name;       /* how to describe it to the player */
+      short category;   /* the experience category to which this skill belongs */
+      short time;       /* base number of ticks it takes to use the skill */
+      long bexp;        /* base exp gain for this skill */
+      float lexp;       /* level multiplier of exp gain for using this skill */
+      short stat1;      /* primary stat effecting use of this skill */
+      short stat2;      /* secondary stat for this skill */
+      short stat3;      /* tertiary stat for this skill */
+} skill;
+
+
 extern skill skills[];
 
