@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_spells_h =
- *   "$Id: spells.h,v 1.27 1997/02/04 10:39:38 master Exp $";
+ *   "$Id: spells.h,v 1.28 1998/09/28 02:52:34 master Exp master $";
  */
 
 /*
@@ -25,6 +25,9 @@
     The author can be reached via e-mail to mark@pyramid.com.
 */
 
+#ifndef SPELLS_H
+#define SPELLS_H
+
 extern Fontindex fbface[];
 extern Fontindex lface[];
 extern Fontindex  mmface[];
@@ -32,6 +35,32 @@ extern int turn_bonus[];
 extern int fear_bonus[];
 extern int cleric_chance[];
 
+typedef struct spell_struct {
+  char name[BIG_NAME];
+  short level;           /* Level required to cast this spell */
+  short sp;              /* Spellpoint-cost to cast it */
+  short charges;         /* If it can be used in wands, max # of charges */
+  float time;            /* How many ticks it takes to cast the spell */
+  short scrolls;         /* If it can be used from scrolls, max # of scrolls */
+  short scroll_chance;   /* 1-10 probability of finding this as scroll */
+  short books;           /* 1-10 probability of finding this as spellbook */
+  unsigned range:1;      /* True if this is a range attack spell */
+  unsigned defensive:1;  /* True if it is a defensive spell */
+  unsigned cleric:1;     /* True if it is a cleric-spell (wis-chance failure)*/
+  unsigned onself:1;     /* Should monsters cast this spell on themselves */
+  uint32 path;           /* Path this spell belongs to */
+  char *archname;	 /* Pointer to archetype used by spell */
+} spell;
+
+typedef struct
+{
+  sint16 bdam;  /*  base damage  */
+  sint16 bdur;  /*  base duration  */
+  sint16 ldam;  /*  damage adjustment for level  */
+  sint16 ldur;  /*  duration adjustment for level  */
+  sint16 spl;
+} spell_parameters;
+
 extern spell spells[NROFREALSPELLS];
 
 /* When adding new spells, don't insert into the middle of the list - 
@@ -167,3 +196,10 @@ extern char *range_name[range_size];
 extern spell_parameters SP_PARAMETERS[];
 extern char *spellpathnames[NRSPELLPATHS];
 extern archetype *spellarch[NROFREALSPELLS];
+
+typedef enum SpellTypeFrom {
+  spellNormal, spellWand, spellRod, spellHorn, spellScroll, spellPotion
+} SpellTypeFrom;
+
+
+#endif
