@@ -1,36 +1,59 @@
+#!/usr/local/bin/perl
+
 require "util.pl";
 
 $root = $ARGV[0];
 $archetypes = "archetypes";
 $bmaps = "bmaps";
 $faces = "faces";
+$animations = "animations";
 $paths = $bmaps."."."paths";
 $faceExt = "\\.[a-zA-Z0-9][A-Z0-9][A-Z0-9]";
 
 ### main
 &info("looking ...");
 &traverse($root);
-&info("writing ...");
+
+&info("writing ...$archetypes");
 open(ARCH,">".$archetypes) || &die("cannot open ".$archetypes);
 &archsOut;
 close(ARCH);
+
+&info("$bmaps");
 open(BMAPS,">".$bmaps) || &die("cannot open ".$bmaps);
 &bmapsOut;
 close(BMAPS);
 open(BMAPS,">".$paths) || &die("cannot open ".$paths);
 &pathsOut;
 close(BMAPS);
+
+&info("$faces");
 open(FACES,">".$faces) || &die("cannot open ".$faces);
 &facesOut;
 close(FACES);
+
+&info("$animations");
+open(ANIM,">".$animations) || &die("cannot open ".$animations);
+&animOut;
+close(ANIM);
+
 &stats;
 exit 0;
 
 sub traverse {
     local($dir) = shift;
     local($file,$name);
-    foreach $file (<$dir/*>) {
+    local( $tfile);
+
+    opendir(THISDIR, $dir) || die "couldn't open $dir";
+    local(@allfiles) = readdir(THISDIR);
+    closedir(THISDIR);
+
+    foreach $tfile (@allfiles) {
+	next if $tfile =~ /^\./;
+	$file = $dir."/".$tfile;
 	$name = &basename($file,""); # DIR
+
         if( -d $file && $name ne "dev" && $name ne "trashbin") {
 	    &traverse($file);
 	} elsif ( -d $file && ( $name eq "dev" || $name eq "trashbin" ) ) {
@@ -40,7 +63,7 @@ sub traverse {
 	    push(@archs,$file);
 	} elsif( $file =~ /.*$faceExt$/) { # FACE
 	    $facesNum++;
-	    &warn("dublicate face ".$name." in ".$dir)
+	    &warn("duplicate face ".$name." in ".$dir." and $faces{$name}")
 		if $faces{$name};
 	    $faces{$name} = $file;
 	    # &warn($file." has no archetype") 
@@ -50,14 +73,13 @@ sub traverse {
 	} elsif ( $file =~ /.*\.face$/) {	# Face information file
 	    $facesFileNum++;
 	    push(@face_files, $file);
-	}else {
+	} 
+	# ignore a couple of the more common 'junk' files that are not
+	# really junk.
+	elsif ($file ne "README") {
 	    $trashNum++;
 	    print "Warning: $file might be a junk file\n";
 	}
-# Supress warning messages.  Almost all of the .xpm files are over
-# 14 characters
-#	&warn($file." is over 14 charactes")
-#	    if length($name) > 14;
     }
 }
 
@@ -125,13 +147,22 @@ line:	while(<ARC>) {
 		$lvis = "";
 		$mm = "";
 		$floor = "";
+		$walkon = 0;
+		$nopick = 0;
+		$arch = join "_", @values;
 	    }
-	    if ($var eq "end" &&  $#lface !=0) {
-		$#lface--;
-		foreach $face (@lface) {
-		    &storeFaceInfo($face, $lfg, $lbg, $lvis,$mm,$floor);
+	    if ($var eq "end") {
+		if ($#lface !=0) {
+		    $#lface--;
+		    foreach $face (@lface) {
+		        &storeFaceInfo($face, $lfg, $lbg, $lvis,$mm,$floor);
+		    }
+	 	}
+		if ($walkon && !$nopick) {
+			&warn("File $arch has an object with walk_on set which can be picked up\n");
 		}
 	    }
+	    # Process the color/face info now
 	    if ($var eq "color_fg") {
 		$lfg = $values[0];
 		next line;
@@ -153,20 +184,38 @@ line:	while(<ARC>) {
 		# is_floor is also needed for archs, so let it pass
 		# through
 	    }
-	    $lface[$#lface++] = $values[0]
-		if ($var eq "face");
-	    $anim = 0 if $var eq "mina";
-	    if ($anim) {
-		if (! $faces{$var}) {
-		    &warn($arch." is missing face ".$var);
+	    elsif ($var eq "no_pick") {
+		$nopick = $values[0];
+	    } elsif ($var eq "walk_on") {
+		$walkon = $values[0];
+	    }
+	    elsif ($var eq "face") {
+		$lface[$#lface++] = $values[0]
+	    }
+	    elsif ($var eq "anim") {
+		if ($anim{$arch}) {
+		    &warn("$arch is a duplicate animation name");
+		    $anim{$arch}="";
 		}
-		else {
-		    $lface[$#lface++] = $var;
+		while (<ARC>) {
+		    chomp;
+		    $var = $_;
+		    last if ($var =~ "mina\s*");
+		    if ($var =~ /facings \S+$/) { }
+		    elsif (! $faces{$var}) {
+			&warn($arch." is missing face ".$var);
+		    }
+		    else {
+			$lface[$#lface++] = $var;
+		    }
+		    $anim{$arch} .= "$var\n";
 		}
+		print ARCH "animation $arch\n";
+		next line;	# don't want the mina
+	    }
+	    if ($var eq "face" && ! $faces{$values[0]}) {
+		&warn($arch." is missing face ".$values[0])
 	    }
-	    &warn($arch." is missing face ".$values[0])
-		if $var eq "face" && ! $faces{$values[0]};
-	    $anim = 1 if $var eq "anim";
 	    print ARCH $_,"\n";
 	}
 	close(ARC);
@@ -228,15 +277,37 @@ sub facesOut {
 	    elsif ($var eq "visibility") {
 		$lvis = $values[0];
 	    }
-	    if ($var eq "magicmap") {
+	    elsif ($var eq "magicmap") {
 		$mm = $values[0];
 	    }
-	    if ($var eq "is_floor") {
+	    elsif ($var eq "is_floor") {
 		$floor = $values[0];
 	    }
 	    elsif ($var eq "end") {
 		&storeFaceInfo($lface, $lfg, $lbg, $lvis, $mm, $floor);
 	    }
+	    elsif ($var eq "animation") {
+		$animation=$values[0];
+		if ($anim{$1}) {
+		    &warn("$animation is a duplicate animation name");
+		    $anim{$animation}="";
+		}
+		while (<FACE>) {
+		    chomp;
+		    $var = $_;
+		    last if ($var =~ /^mina\s*$/);
+		    if ($var !~ /^facings/ ) {
+		    	if (! $faces{$var}) {
+				&warn($arch." is missing face ".$var);
+			}
+			else {
+				$lface[$#lface++] = $var;
+			}
+		    }
+		    $anim{$animation} .= "$var\n";
+		}
+		next;	# don't want the mina
+	    }
 	}
 	close(FACE);
     }
@@ -261,10 +332,18 @@ sub facesOut {
     }
 }
 
+sub animOut {
+    foreach $anim (sort keys %anim) {
+	print ANIM "anim $anim\n$anim{$anim}mina\n";
+	$animationsNum++;
+    }
+}
+
 ### print out statical information
 sub stats {
     &info(Archs.":\t".$archsNum);
     &info(Images.":\t".$facesNum);
     &info(Faces.":\t".$facesFileNum);
+    &info(Animations.":\t".$animationsNum);
     &info(Trash.":\t".$trashNum);
 }
