@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_apply_c =
- *   "$Id: apply.c,v 1.55 1997/01/05 10:05:02 master Exp $";
+ *   "$Id: apply.c,v 1.63 1998/09/28 02:59:03 master Exp $";
  */
 /*
     CrossFire, A Multiplayer game for X-windows
@@ -36,9 +36,10 @@
 #ifndef __CEXTRACT__
 #include <sproto.h>
 #endif
-#ifdef SOUND_EFFECTS
+
+/* Want this regardless of rplay. */
 #include <sounds.h>
-#endif
+
 
 #if defined(vax) || defined(ibm032)
 size_t strftime(char *, size_t, const char *, const struct tm *);
@@ -229,9 +230,9 @@ int apply_potion(object *op, object *tmp)
     if (tmp->stats.sp) {
       if(QUERY_FLAG(tmp, FLAG_CURSED) || QUERY_FLAG(tmp, FLAG_DAMNED)) {
         new_draw_info(NDI_UNIQUE,0,op, "Yech!  Your lungs are on fire!");
-        cast_spell(op,tmp, 0, 3, 0, spellPotion,NULL);
+        cast_spell(op,tmp, 0, 3, 1, spellPotion,NULL);
       } else
-        cast_spell(op,tmp, 0, tmp->stats.sp, 0, spellPotion,NULL);
+        cast_spell(op,tmp, op->facing, tmp->stats.sp, 1, spellPotion,NULL);
       decrease_ob(tmp);
       /* if youre dead, no point in doing this... */
       if(!QUERY_FLAG(op,FLAG_REMOVED)) fix_player(op);
@@ -304,57 +305,73 @@ void eat_item(object *op,char *item)
   }
 }
 
-int check_sacrifice(object *op,object *improver)
+/* This checks to see of the player (who) is sufficient level to use a weapon
+ * with improvs improvements (typically last_eat).  We take an int here
+ * instead of the object so that the improvement code can pass along the
+ * increased value to see if the object is usuable.
+ * we return 1 (true) if the player can use the weapon.
+ */
+int check_weapon_power(object *who, int improvs)
 {
-  int count;
-#ifndef NEW_IMPROVE_WEAPON
-  if (check_item(op,"food")<5) {
-    new_draw_info(NDI_UNIQUE,0,op,"The gods want more food.");
-    return 0;
-  }
-  if (check_item(op,"booze")<10) {
-    new_draw_info(NDI_UNIQUE,0,op,"The gods want more booze.");
-    return 0;
-  }
-#endif
-  count = 0;
-  if (improver->slaying!=NULL) {
-    count = check_item(op,improver->slaying);
-    if (count<1) {
-      char buf[200];
-      sprintf(buf,"The gods want more %ss",improver->slaying);
-      new_draw_info(NDI_UNIQUE,0,op,buf);
-      return 0;
-    }
-#ifndef NEW_IMPROVE_WEAPON
-    eat_item(op,improver->slaying);
-#endif
-  }
-  else
-    count=1;
+    int level=who->level;
 
-#ifndef NEW_IMPROVE_WEAPON
-  eat_item(op,"food");
-  eat_item(op,"booze");
+    /* The skill system hands out wc and dam bonuses to fighters
+     * more generously than the old system (see fix_player). Thus
+     * we need to curtail the power of player enchanted weapons. 
+     * I changed this to 1 improvement per "fighter" level/5 -b.t. 
+     * Note:  Nothing should break by allowing this ratio to be different or
+     * using normal level - it is just a matter of play balance.
+     */
+#ifdef ALLOW_SKILLS
+    if(who->type==PLAYER) { 
+      object *wc_obj=NULL;
+
+      for(wc_obj=who->inv;wc_obj;wc_obj=wc_obj->below)
+	if(wc_obj->type==EXPERIENCE&&wc_obj->stats.Str) break;
+      if(!wc_obj) 
+	LOG(llevError,"Error: Player: %s lacks wc experience object\n",who->name);
+      else
+	level=wc_obj->level; 
+    }
+    return (improvs <= ((level/5)+5));
+#else
+    return (improvs <= (level+5));
 #endif
-  return count;
+}
+
+/* Returns the object count that of the number of objects found that
+ * improver wants.
+ */
+static int check_sacrifice(object *op,object *improver)
+{
+    int count=0;
+
+    if (improver->slaying!=NULL) {
+	count = check_item(op,improver->slaying);
+	if (count<1) {
+	    char buf[200];
+	    sprintf(buf,"The gods want more %ss",improver->slaying);
+	    new_draw_info(NDI_UNIQUE,0,op,buf);
+	    return 0;
+	}
+    }
+    else
+	count=1;
+
+    return count;
 }
 
 int improve_weapon_stat(object *op,object *improver,object *weapon,
 			signed char *stat,int sacrifice_count,char *statname)
 {
 
-#ifndef NEW_IMPROVE_WEAPON
-  if (sacrifice_count<2)
-    return 0;
-  sacrifice_count = isqrt(sacrifice_count/2);
-#endif
   new_draw_info(NDI_UNIQUE,0,op,"Your sacrifice was accepted.");
   *stat += sacrifice_count;
   weapon->last_eat++;
   new_draw_info_format(NDI_UNIQUE,0,op,
 	"Weapon's bonus to %s improved by %d",statname,sacrifice_count);
   decrease_ob(improver);
+
   /* So it updates the players stats and the window */
   fix_player(op);
   return 1;
@@ -374,161 +391,6 @@ int improve_weapon_stat(object *op,object *improver,object *weapon,
 #define IMPROVE_POW 11
 
 
-#ifndef NEW_IMPROVE_WEAPON
-/* build_weapon returns 0 if it was not able to work. */
-/* #### We are hiding extra information about the weapon in the level and
-   last_eat numbers for an object.  Hopefully this won't break anything ?? 
-   level == max improve last_eat == current improve*/
-int improve_weapon(object *op,object *improver,object *weapon)
-{
-  int sacrifice_count;
-  char *tmp;
-
-  if(improver->stats.sp==IMPROVE_PREPARE) {
-    if (weapon->level!=0) {
-      new_draw_info(NDI_UNIQUE,0,op,"Weapon already prepared.");
-      return 0;
-    }
-    /* Could check for other things.  I'm assuming all things which provide
-       other benefits are also magic. */
-    if (weapon->magic!=0) {
-      new_draw_info(NDI_UNIQUE,0,op,"Cannot prepare magic weapons.");
-      return 0;
-    }
-    sacrifice_count=check_sacrifice(op,improver);
-    if (sacrifice_count<=0)
-      return 0;
-    sacrifice_count = isqrt(sacrifice_count);
-    weapon->level=sacrifice_count;
-    new_draw_info(NDI_UNIQUE,0,op,"Your sacrifice was accepted.");
-
-    new_draw_info_format(NDI_UNIQUE, 0, op,"Your *%s may be improved %d times.",
-	    weapon->name,sacrifice_count);
-
-    tmp = (char *) malloc(strlen(weapon->name)+strlen(op->name) + 4);
-    sprintf(tmp,"%s's %s",op->name,weapon->name);
-    weapon->name=tmp;
-    weapon->nrof=0;  /*  prevents preparing n weapons in the same
-			 slot at once! */
-    decrease_ob(improver);
-    weapon->last_eat=0;
-    return 1;
-  }
-  if (!(weapon->level>0)) {
-    new_draw_info(NDI_UNIQUE, 0,op,"This weapon has not been prepared.");
-    return 0;
-  }
-  if (weapon->level==weapon->last_eat) {
-    new_draw_info(NDI_UNIQUE, 0,op,"This weapon cannot be improved any more.");
-    return 0;
-  }
-    /* The skill system hands out wc and dam bonuses to fighters
-     * more generously than the old system (see fix_player). Thus
-     * we need to curtail the power of player enchanted weapons. 
-     * I changed this to 1 improvement per "fighter" level/5 -b.t. 
-     * Note:  Nothing should break by allowing this ratio to be different or
-     * using normal level - it is just a matter of play balance.
-     */
-  if (QUERY_FLAG(weapon, FLAG_APPLIED)) {
-#ifdef ALLOW_SKILLS
-      object *wc_obj=NULL;
-      int level=0;
-
-      for(wc_obj=op->inv;wc_obj;wc_obj=wc_obj->below)
-	if(wc_obj->type==EXPERIENCE&&wc_obj->stats.Str) break;
-      if(!wc_obj) 
-	LOG(llevError,"Error: Player: %s lacks wc experience object\n",op->name);
-      else
-	level=wc_obj->level; 
-    if(weapon->last_eat>((level/5)+5)) {  
-#else
-    if(weapon->last_eat>(op->level+5)) {
-#endif
-	new_draw_info(NDI_UNIQUE, 0,op,"Improving the weapon will make it");
-	new_draw_info(NDI_UNIQUE, 0,op,"powerful to use.  Unready it if you");
-	new_draw_info(NDI_UNIQUE, 0,op,"really want to improve it.");
-	return 0;
-    }
-  }
-  sacrifice_count = check_sacrifice(op,improver);
-  if (sacrifice_count<=0)
-    return 0;
-  switch (improver->stats.sp) {
-   case IMPROVE_DAMAGE:
-    sacrifice_count = isqrt(sacrifice_count);
-    if ((sacrifice_count+weapon->stats.dam)>70)
-	sacrifice_count = 70 - weapon->stats.dam;
-    weapon->stats.dam += sacrifice_count;
-    weapon->weight += sacrifice_count*2000;
-    new_draw_info(NDI_UNIQUE, 0,op,"Your sacrifice was accepted.");
-    new_draw_info_format(NDI_UNIQUE, 0, op,
-	"Damage is increased by %d, weight by %6.1f kg",
-	    sacrifice_count,(float)sacrifice_count*2.0);
-
-    weapon->last_eat++;
-    decrease_ob(improver);
-    return 1;
-   case IMPROVE_WEIGHT:
-    sacrifice_count = isqrt(sacrifice_count)*750;
-    if (weapon->weight<sacrifice_count)
-      weapon->weight=1;
-    else
-      weapon->weight-=sacrifice_count;
-    new_draw_info(NDI_UNIQUE, 0,op,"Your sacrifice was accepted.");
-    new_draw_info_format(NDI_UNIQUE, 0, op,
-	"Weapon weight reduced by %6.1f kg to %6.1f kg",
-	(float)sacrifice_count/1000.0,(float)weapon->weight/1000.0);
-    weapon->last_eat++;
-    decrease_ob(improver);
-    return 1;
-   case IMPROVE_ENCHANT:
-    new_draw_info(NDI_UNIQUE, 0,op,"Your sacrifice was accepted.");
-    weapon->magic++;
-    weapon->last_eat++;
-    if (weapon->weight>1) weapon->weight=(weapon->weight*90)/100;
-
-    new_draw_info_format(NDI_UNIQUE, 0, op
-	,"Weapon magic increased to %d, weight lowered to %6.1f kg",
-	weapon->magic,(float)weapon->weight/1000.0);
-    decrease_ob(improver);
-    return 1;
-
-   case IMPROVE_STR:
-    return improve_weapon_stat(op,improver,weapon,
-                               (signed char *) &(weapon->stats.Str),
-			       sacrifice_count,(char *) "strength");
-   case IMPROVE_DEX:
-    return improve_weapon_stat(op,improver,weapon,
-                               (signed char *) &(weapon->stats.Dex),
-			       sacrifice_count,(char *) "dexterity");
-   case IMPROVE_CON:
-    return improve_weapon_stat(op,improver,weapon,
-                               (signed char *) &(weapon->stats.Con),
-			       sacrifice_count,(char *) "constitution");
-   case IMPROVE_WIS:
-    return improve_weapon_stat(op,improver,weapon,
-                               (signed char *) &(weapon->stats.Wis),
-			       sacrifice_count,(char *) "wisdom");
-   case IMPROVE_CHA:
-    return improve_weapon_stat(op,improver,weapon,
-                               (signed char *) &(weapon->stats.Cha),
-			       sacrifice_count,(char *) "charisma");
-   case IMPROVE_INT:
-    return improve_weapon_stat(op,improver,weapon,
-                               (signed char *) &(weapon->stats.Int),
-			       sacrifice_count,(char *) "intelligence");
-   case IMPROVE_POW:
-    return improve_weapon_stat(op,improver,weapon,
-                               (signed char *) &(weapon->stats.Pow),
-			       sacrifice_count,(char *) "power");
-
-   default:
-    new_draw_info(NDI_UNIQUE, 0,op,"Unknown improvement type.");
-    return 0;
-  }
-}
-#else
-
 /* This does the prepare weapon scroll */
 
 int prepare_weapon(object *op, object *improver, object *weapon)
@@ -574,46 +436,6 @@ int prepare_weapon(object *op, object *improver, object *weapon)
 }
 
 
-/* this code is by b.t. (thomas@nomad.astro.psu.edu) -
- * only 'enchantment' of armour is possible - improving
- * the stats of a player w/ armour as well as a weapon
- * will probably horribly unbalance the game. Magic enchanting
- * depends on the level of the character - ie the plus
- * value (magic) of the armour can never be increased beyond
- * the level of the character / 10 -- rounding upish, nor may
- * the armour value of the piece of equipment exceed either 
- * the users level or 99)
- */
- 
-int improve_armour(object *op, object *improver, object *armour)
-{
-    int addarm;
- 
-    addarm = armour->armour/25 + op->level/20 + 1;
-
-    if (armour->magic>=(op->level/10+1) || ((armour->armour + 
-		addarm) >= op->level )) {
-        new_draw_info(NDI_UNIQUE, 0,op,"You are not yet powerfull enough");
-        new_draw_info(NDI_UNIQUE, 0,op,"to improve this armour.");
-        return 0;
-    }
-
-    if( (armour->armour + addarm) <= 99)  {
-        armour->magic++;
-	armour->armour+=addarm;
-	armour->weight+=armour->weight*0.05;
-        decrease_ob(improver);
-        return 1;
-    } else {
-        armour->magic++;
-        new_draw_info(NDI_UNIQUE, 0,op,"The armour value of this equipment");
-        new_draw_info(NDI_UNIQUE, 0,op,"cannot be further improved.");
-        decrease_ob(improver);
-	return 1;
-    } 
-
- }
-
 /* This is the new improve weapon code */
 /* build_weapon returns 0 if it was not able to work. */
 /* #### We are hiding extra information about the weapon in the level and
@@ -634,26 +456,12 @@ int improve_weapon(object *op,object *improver,object *weapon)
     new_draw_info(NDI_UNIQUE, 0,op,"This weapon cannot be improved any more.");
     return 0;
   }
-  if (QUERY_FLAG(weapon, FLAG_APPLIED)) {
-#ifdef ALLOW_SKILLS
-      object *wc_obj=NULL;
-      int level=0;
-
-      for(wc_obj=op->inv;wc_obj;wc_obj=wc_obj->below)
-	if(wc_obj->type==EXPERIENCE&&wc_obj->stats.Str) break;
-      if(!wc_obj) 
-	LOG(llevError,"Error: Player: %s lacks wc experience object\n",op->name);
-      else
-	level=wc_obj->level; 
-      if(weapon->last_eat>((level/5)+5)) {  
-#else
-      if(weapon->last_eat>(op->level+5)) {
-#endif
+  if (QUERY_FLAG(weapon, FLAG_APPLIED) && 
+      !check_weapon_power(op, weapon->last_eat+1)) {
 	new_draw_info(NDI_UNIQUE, 0,op,"Improving the weapon will make it too");
 	new_draw_info(NDI_UNIQUE, 0,op,"powerful for you to use.  Unready it if you");
 	new_draw_info(NDI_UNIQUE, 0,op,"really want to improve it.");
 	return 0;
-    }
   }
   /* This just increases damage by 5 points, no matter what.  No sacrifice
    * is needed.  Since stats.dam is now a 16 bit value and not 8 bit,
@@ -740,7 +548,6 @@ int improve_weapon(object *op,object *improver,object *weapon)
   LOG(llevError,"improve_weapon: Got to end of function\n");
   return 0;
 }
-#endif
 
 int check_improve_weapon (object *op, object *tmp)
 {
@@ -757,18 +564,6 @@ int check_improve_weapon (object *op, object *tmp)
 	return 0;
     }
     otmp=find_marked_object(op);
-#if 0
-    if (op->inv==NULL) {
-      new_draw_info(NDI_UNIQUE, 0,op,"First item in inventory is not a weapon\n");
-      return 0;
-    }
-    for (otmp=op->inv; (otmp!=NULL && otmp->invisible); otmp=otmp->below) {
-	if (!otmp) {
-	    new_draw_info(NDI_UNIQUE,0,op,"You have no weapon to enchant.\n");
-	    return 0;
-	}
-    }
-#endif
     if(!otmp) {
       new_draw_info(NDI_UNIQUE, 0, op, "You need to mark a weapon object.");
       return 0;
@@ -779,13 +574,51 @@ int check_improve_weapon (object *op, object *tmp)
     }
     new_draw_info(NDI_UNIQUE, 0,op,"Applied weapon builder.");
     improve_weapon(op,tmp,otmp);
-    if (op->contr->eric_server > 0)
-      esrv_send_item(op, tmp);
-    else
-      draw_inventory(op);
+    esrv_send_item(op, otmp);
     return 1;
 }
 
+/* this code is by b.t. (thomas@nomad.astro.psu.edu) -
+ * only 'enchantment' of armour is possible - improving
+ * the stats of a player w/ armour as well as a weapon
+ * will probably horribly unbalance the game. Magic enchanting
+ * depends on the level of the character - ie the plus
+ * value (magic) of the armour can never be increased beyond
+ * the level of the character / 10 -- rounding upish, nor may
+ * the armour value of the piece of equipment exceed either 
+ * the users level or 99)
+ */
+ 
+int improve_armour(object *op, object *improver, object *armour)
+{
+    int addarm;
+ 
+    addarm = armour->armour/25 + op->level/20 + 1;
+
+    if (armour->magic>=(op->level/10+1) || ((armour->armour + 
+		addarm) >= op->level )) {
+        new_draw_info(NDI_UNIQUE, 0,op,"You are not yet powerfull enough");
+        new_draw_info(NDI_UNIQUE, 0,op,"to improve this armour.");
+        return 0;
+    }
+
+    if( (armour->armour + addarm) <= 99)  {
+        armour->magic++;
+	armour->armour+=addarm;
+	armour->weight+=armour->weight*0.05;
+        decrease_ob(improver);
+        return 1;
+    } else {
+        armour->magic++;
+        new_draw_info(NDI_UNIQUE, 0,op,"The armour value of this equipment");
+        new_draw_info(NDI_UNIQUE, 0,op,"cannot be further improved.");
+        decrease_ob(improver);
+	return 1;
+    } 
+
+ }
+
+
 /* archetypes don't contain any MONEY_CHANGER object,
  * so it this function really used/useful?
  * Converters seems to make almost same.  -Tero
@@ -986,8 +819,7 @@ int apply_container (object *op, object *sack)
 	}
     }
     new_draw_info (NDI_UNIQUE, 0, op, buf);
-    draw_inventory(op);
-    draw_all_look(op);
+    draw_look(op);
     return 1;
 }
 
@@ -1180,12 +1012,17 @@ int make_gravestone (object *op, object *grave)
     return 1;
 }
 
-/* apply returns 0 if it wasn't possible to apply that object */
+/* apply returns 0 if it wasn't possible to apply that object, 1
+ * if the object was applied, 2 if the object is now a different
+ * object.
+ */
 /* op is the object that is causing object to be applied, tmp is the object
  * being applied.
+ * aflag is special (always apply/unapply) flags.  Nothing is done with
+ * them in this function - they are passed to apply_special
  */
 
-int apply(object *op, object *tmp) {
+int apply(object *op, object *tmp, int aflag) {
   char buf[MAX_BUF];
   int inven;
 
@@ -1207,9 +1044,7 @@ int apply(object *op, object *tmp) {
   if(tmp==NULL)
     return 0;
   if(QUERY_FLAG(tmp,FLAG_WAS_WIZ)&&!QUERY_FLAG(op,FLAG_WAS_WIZ)&&op->type==PLAYER) {
-#ifdef SOUND_EFFECTS
     play_sound_map(op->map, op->x, op->y, SOUND_OB_EVAPORATE);
-#endif
     new_draw_info(NDI_UNIQUE, 0,op,"The object disappears in a puff of smoke!");
     new_draw_info(NDI_UNIQUE, 0,op,"It must have been an illusion.");
     remove_ob(tmp);
@@ -1224,7 +1059,15 @@ int apply(object *op, object *tmp) {
   case PLAYERMOVER:
     if (tmp->attacktype && (tmp->level || op->type!=PLAYER)) {
 	if (!tmp->stats.maxsp) tmp->stats.maxsp=2.0;
+	/* Is this correct?  From the docs, it doesn't look like it
+	 * should be divided by tmp->speed
+	 */
 	op->speed_left = -FABS(tmp->stats.maxsp*op->speed/tmp->speed);
+	/* Just put in some sanity check.  I think there is a bug in the
+	 * above with some objects have zero speed, and thus the player
+	 * getting permanently paralyzed.
+	 */
+	if (op->speed_left<-50.0) op->speed_left=-50.0;
 /*	fprintf(stderr,"apply, playermove, player speed_left=%f\n", op->speed_left);*/
     }
     return 1;
@@ -1300,9 +1143,7 @@ int apply(object *op, object *tmp) {
     check_fired_arch(tmp);
     return 1;
   case TRAPDOOR:
-#ifdef SOUND_EFFECTS
     play_sound_map(op->map, op->x, op->y, SOUND_FALL_HOLE);
-#endif
     {
       int max;
       object *ab;
@@ -1313,7 +1154,7 @@ int apply(object *op, object *tmp) {
             tot+=ab->weight+ab->carrying;
         if(!(tmp->value=(tot>tmp->weight)?1:0))
           return 1;
-        tmp->face=&new_faces[tmp->arch->faces[tmp->value]];
+	SET_ANIMATION(tmp, tmp->value);
         update_object(tmp);
       }
       for(ab=tmp->above,max=100;--max&&ab&&!QUERY_FLAG(ab, FLAG_FLYING);ab=ab->above) {
@@ -1328,11 +1169,9 @@ int apply(object *op, object *tmp) {
     return 0;
   case HANDLE:
     new_draw_info(NDI_UNIQUE, 0,op,"You turn the handle.");
-#ifdef SOUND_EFFECTS
     play_sound_map(op->map, op->x, op->y, SOUND_TURN_HANDLE);
-#endif
     tmp->value=tmp->value?0:1;
-    tmp->face=&new_faces[tmp->arch->faces[tmp->value]];
+    SET_ANIMATION(tmp, tmp->value);
     update_object(tmp);
     push_button(tmp);
     return 1;
@@ -1351,15 +1190,22 @@ int apply(object *op, object *tmp) {
   case HOLE:
     if(tmp->stats.wc>0) /* Is the hole open? */
       return 1; /* Nope */
+    /* Is this a multipart monster and not the head?  If so, return.
+     * Processing will happen if the head runs into the pit
+     */
+    if (op->head) return 1;
     {
-    int i=find_free_spot(op->arch,op->map,EXIT_X(tmp),EXIT_Y(tmp),0,SIZEOFFREE);
-      remove_ob(op);
-      op->x=EXIT_X(tmp)+freearr_x[i],op->y=EXIT_Y(tmp)+freearr_y[i];
-      insert_ob_in_map(op,op->map);
+	int i=find_free_spot(op->arch,op->map,EXIT_X(tmp),EXIT_Y(tmp),0,SIZEOFFREE);
+	object *part;
+
+	remove_ob(op);
+	for (part=op; part!=NULL; part=part->more) {
+	    part->x=EXIT_X(tmp)+freearr_x[i] + (part->arch?part->arch->clone.x:0);
+	    part->y=EXIT_Y(tmp)+freearr_y[i] + (part->arch?part->arch->clone.y:0);
+	}
+	insert_ob_in_map(op,op->map);
     }
-#ifdef SOUND_EFFECTS
     play_sound_map(op->map, op->x, op->y, SOUND_FALL_HOLE);
-#endif
     new_draw_info(NDI_UNIQUE, 0,op,"You fall through the hole!\n");
     return 1;
   case EXIT:
@@ -1416,6 +1262,10 @@ int apply(object *op, object *tmp) {
       return 1;
     }
   case SIGN:
+    if (tmp->stats.food && ++tmp->last_eat>tmp->stats.food) {
+	new_draw_info(NDI_UNIQUE, 0, op, "Nothing is written on it.");
+	return 1;
+    }
     if(QUERY_FLAG(op, FLAG_BLIND)&&!QUERY_FLAG(op,FLAG_WIZ))
       new_draw_info(NDI_UNIQUE, 0,op,"You are unable to read while blind.");
     else if(tmp->msg==NULL)
@@ -1454,7 +1304,7 @@ int apply(object *op, object *tmp) {
 	  if(!QUERY_FLAG(tmp,FLAG_IDENTIFIED)) {
 	    /*exp_gain *= 2; because they just identified it too */
 	    SET_FLAG(tmp,FLAG_IDENTIFIED);
-	    if(tmp->env) draw_inventory(op);
+	    if(tmp->env) esrv_update_item(UPD_FLAGS|UPD_NAME, op,tmp);
 	    else draw_look(op);
 	  }
           add_exp(op,exp_gain);
@@ -1467,15 +1317,15 @@ int apply(object *op, object *tmp) {
       else if (lev_diff < 2)
 	new_draw_info(NDI_UNIQUE, 0,op,"This book is just barely beyond your comprehension.");
       else if (lev_diff < 3)
-	new_draw_info(NDI_UNIQUE, 0,op,"This book is a little beyond your comprehension.");
+	new_draw_info(NDI_UNIQUE, 0,op,"This book is slightly beyond your comprehension.");
       else if (lev_diff < 5)
-	new_draw_info(NDI_UNIQUE, 0,op,"This book is a beyond your comprehension.");
+	new_draw_info(NDI_UNIQUE, 0,op,"This book is beyond your comprehension.");
       else if (lev_diff < 8)
-	new_draw_info(NDI_UNIQUE, 0,op,"This book is a quite a bit beyond your comprehension.");
+	new_draw_info(NDI_UNIQUE, 0,op,"This book is quite a bit beyond your comprehension.");
       else if (lev_diff < 15)
-	new_draw_info(NDI_UNIQUE, 0,op,"This book is a way beyond your comprehension.");
+	new_draw_info(NDI_UNIQUE, 0,op,"This book is way beyond your comprehension.");
       else 
-	new_draw_info(NDI_UNIQUE, 0,op,"This book is a totally beyond your comprehension.");
+	new_draw_info(NDI_UNIQUE, 0,op,"This book is totally beyond your comprehension.");
 #endif
     }
     return 1;
@@ -1510,7 +1360,6 @@ int apply(object *op, object *tmp) {
 	  break;
     }
     decrease_ob(tmp);
-    draw_inventory(op);
 #endif
     return 1; 
   case SPELLBOOK:
@@ -1559,20 +1408,20 @@ int apply(object *op, object *tmp) {
 	"The spellbook contains the %s level spell %s.",
             get_levelnumber(spells[tmp->stats.sp].level),
             spells[tmp->stats.sp].name);
-    if(check_spell_known(op,tmp->stats.sp)) {
-      new_draw_info(NDI_UNIQUE, 0,op,"You already know that spell.\n");
 
-      if (!QUERY_FLAG(tmp, FLAG_IDENTIFIED)) {
+    if (!QUERY_FLAG(tmp, FLAG_IDENTIFIED)) {
 	identify(tmp);
-	if (op->contr->eric_server > 0)
-	  esrv_send_item(op, tmp);
-	else if (tmp->env)
-	  draw_inventory(op);
+	if (tmp->env)
+	    esrv_update_item(UPD_FLAGS|UPD_NAME,op,tmp);
 	else
-	  draw_look(op);
-      }
-      return 1;
+	    draw_look(op);
     }
+
+    if(check_spell_known(op,tmp->stats.sp)) {
+	new_draw_info(NDI_UNIQUE, 0,op,"You already know that spell.\n");
+	return 1;
+    }
+
     /* I changed spell learning in 3 ways:
      *
      *  1- MU spells use Int to learn, Cleric spells use Wisdom
@@ -1590,9 +1439,7 @@ int apply(object *op, object *tmp) {
       scroll_failure(op,RANDOM()%(spells[tmp->stats.sp].level+1),spells[tmp->stats.sp].sp);
     } else if(QUERY_FLAG(tmp,FLAG_STARTEQUIP) || RANDOM()%150-(2*SK_level(op)) <
 	learn_spell[spells[tmp->stats.sp].cleric ? op->stats.Wis : op->stats.Int]) {
-#ifdef SOUND_EFFECTS
-      play_sound_player_only(op->contr, SOUND_LEARN_SPELL);
-#endif
+      play_sound_player_only(op->contr, SOUND_LEARN_SPELL,0,0);
       new_draw_info(NDI_UNIQUE, 0,op,"You succeed in learning the spell!");
       op->contr->known_spells[op->contr->nrofknownspells++]=tmp->stats.sp;
       if(op->contr->nrofknownspells == 1)
@@ -1604,13 +1451,10 @@ int apply(object *op, object *tmp) {
       if(op->type==PLAYER) add_exp(op,calc_skill_exp(op,tmp));
 #endif
     } else {
-#ifdef SOUND_EFFECTS
-      play_sound_player_only(op->contr, SOUND_FUMBLE_SPELL);
-#endif
+      play_sound_player_only(op->contr, SOUND_FUMBLE_SPELL,0,0);
       new_draw_info(NDI_UNIQUE, 0,op,"You fail to learn the spell.\n");
     }
-    remove_ob(tmp);
-    free_object(tmp);
+    decrease_ob(tmp);
     return 1;
   case SCROLL: {
       int scroll_spell=tmp->stats.sp, old_spell=0;
@@ -1652,7 +1496,6 @@ int apply(object *op, object *tmp) {
 	    else /* something bad */
 		scroll_failure(op,-5*(RANDOM()%(tmp->level+1)-1),spells[scroll_spell].sp);
           decrease_ob(tmp);
-	    draw_inventory(op);
 	    return 0;
 	} 
 	else if((exp_gain = calc_skill_exp(op,tmp)))
@@ -1675,8 +1518,8 @@ int apply(object *op, object *tmp) {
 
     new_draw_info_format(NDI_BLACK, 0, op,
         "The scroll of %s turns to dust.", spells[tmp->stats.sp].name);
-    new_draw_info_format(NDI_ALL, 6, op,
-        "%s reads a scroll of %s.",op->name,spells[tmp->stats.sp].name);
+    sprintf(buf, "%s reads a scroll of %s.",op->name,spells[tmp->stats.sp].name);
+    new_info_map(NDI_ORANGE, op->map, buf);
 
     cast_spell(op,tmp,0,scroll_spell,0,spellScroll,NULL);
     decrease_ob(tmp);
@@ -1685,7 +1528,6 @@ int apply(object *op, object *tmp) {
         op->contr->shoottype=old_shoot;
 	op->contr->chosen_spell = old_spell;
       }
-      draw_stats(op);
     }
     break;
   }
@@ -1698,12 +1540,12 @@ int apply(object *op, object *tmp) {
 
 /* Eneq(@csd.uu.se): Handle apply on containers. */
   case CLOSE_CON:
-    if (op->contr->eric_server > 0)
+    if (op->type==PLAYER)
       return esrv_apply_container (op, tmp->env);
     else
       return apply_container (op, tmp->env);
   case CONTAINER:
-    if (op->contr->eric_server > 0)
+    if (op->type==PLAYER)
       return esrv_apply_container (op, tmp);
     else
       return apply_container (op, tmp);
@@ -1745,7 +1587,7 @@ int apply(object *op, object *tmp) {
     break;
   }
   case WAND:
-    if(apply_special(op,tmp))
+    if(apply_special(op,tmp,aflag))
       return 1;
     if(op->type==PLAYER) {
       if(QUERY_FLAG(tmp, FLAG_APPLIED)) {
@@ -1765,7 +1607,7 @@ int apply(object *op, object *tmp) {
     }
     return 1;
   case ROD:
-    if(apply_special(op,tmp))
+    if(apply_special(op,tmp,aflag))
       return 1;
     if(op->type==PLAYER) {
       if(QUERY_FLAG(tmp, FLAG_APPLIED)) {
@@ -1785,7 +1627,7 @@ int apply(object *op, object *tmp) {
     }
     return 1;
   case HORN:
-    if(apply_special(op,tmp))
+    if(apply_special(op,tmp,aflag))
       return 1;
     if(op->type==PLAYER) {
       if(QUERY_FLAG(tmp, FLAG_APPLIED)) {
@@ -1805,7 +1647,7 @@ int apply(object *op, object *tmp) {
     }
     return 1;
   case BOW:
-    if(apply_special(op,tmp))
+    if(apply_special(op,tmp,aflag))
       return 0;
     if(QUERY_FLAG(tmp, FLAG_APPLIED)) {
       new_draw_info_format(NDI_UNIQUE, 0, op,
@@ -1819,7 +1661,7 @@ int apply(object *op, object *tmp) {
     return 1;
 #ifdef ALLOW_SKILLS
   case SKILL:   /* allows skill tools  to be used -b.t */
-    if(apply_special(op,tmp))
+    if(apply_special(op,tmp,aflag))
       return 0;
     if((!tmp->invisible||op->type!=PLAYER)&&QUERY_FLAG(tmp, FLAG_APPLIED)) {
         SET_FLAG(op, FLAG_READY_SKILL);
@@ -1830,7 +1672,6 @@ int apply(object *op, object *tmp) {
           op->contr->shoottype=range_skill;
 	  link_player_skill(op, tmp);
           fix_player(op);
-          draw_stats(op);
         }
     } else { 
         CLEAR_FLAG(op, FLAG_READY_SKILL);
@@ -1851,7 +1692,7 @@ int apply(object *op, object *tmp) {
   case CLOAK:
 /* Mol(mol@meryl.csd.uu.se) compressed return & apply_s into one statement */
 /* Frank: If done this way, a ! must be prepended */
-    return !apply_special(op,tmp);
+    return !apply_special(op,tmp,aflag);
   case DRINK:
   case FOOD:
   case FLESH:
@@ -1885,7 +1726,6 @@ int apply(object *op, object *tmp) {
       }
       /* special food hack -b.t. */
       if(tmp->title || QUERY_FLAG(tmp,FLAG_CURSED)) eat_special_food(op,tmp);
-      draw_stats(op);
     }
     decrease_ob(tmp);
     return 1;
@@ -1899,9 +1739,7 @@ int apply(object *op, object *tmp) {
       new_draw_info(NDI_UNIQUE, 0,op,"You should pay for it first.");
       return 1;
     }
-#ifdef SOUND_EFFECTS
-    play_sound_player_only(op->contr, SOUND_DRINK_POISON);
-#endif
+    play_sound_player_only(op->contr, SOUND_DRINK_POISON,0,0);
 /*    op->stats.hp+=tmp->stats.hp;*/
     /* I think this is better */
     LOG(llevDebug,"Trying to poison player for %d hp\n", tmp->stats.hp);
@@ -1910,7 +1748,6 @@ int apply(object *op, object *tmp) {
     if(op->type==PLAYER) {
       new_draw_info(NDI_UNIQUE, 0,op,"Yech!  That tasted poisonous!");
       strcpy(op->contr->killer,"poisonous booze");
-      draw_stats(op);
     }
     decrease_ob(tmp);
     return 1;
@@ -1976,12 +1813,8 @@ int apply(object *op, object *tmp) {
     {
     	    new_draw_info(NDI_UNIQUE, 0,op,"Applying armour enchantment.");
     	    improve_armour(op,tmp,armor);
-   	     if (op->contr->eric_server > 0)
-      			esrv_send_item(op, tmp);
-    	     else
-      			draw_inventory(op);
-
-    	     return 1;
+	    esrv_send_item(op, tmp);
+	    return 1;
     } 
 
     new_draw_info(NDI_UNIQUE, 0,op,"Your marked item is not armour!\n");
@@ -2005,12 +1838,10 @@ int apply(object *op, object *tmp) {
     money_change(op,tmp->slaying);
     return 1;
    case CLOCK: {
-       time_t t = time(NULL);
+	time_t t = time(NULL);
 	strftime(buf, sizeof(buf), "Time is %I:%M %p", localtime(&t));
-#ifdef SOUND_EFFECTS
-       play_sound_player_only(op->contr, SOUND_CLOCK);
-#endif
-       new_draw_info(NDI_UNIQUE, 0,op, buf);
+	play_sound_player_only(op->contr, SOUND_CLOCK,0,0);
+	new_draw_info(NDI_UNIQUE, 0,op, buf);
 	return 1;
    }
   case MENU: 
@@ -2042,19 +1873,38 @@ int apply(object *op, object *tmp) {
   return 1;
 }
 
-void apply_below(object *op) {
-  object *tmp;
+/* Apply_Below attempts to apply the object 'below' the player (op).
+ * If the player has an open container, we use that for below, otherwise
+ * we use the ground.
+ */
 
-  /* If we are using a container, then lets apply what is in the container
-   * instead of whats on the ground.
-   */
-  if (op->container!=NULL) {
-    for(tmp=op->container->inv;tmp!=NULL&&!apply(op,tmp);tmp=tmp->below);
-  } else
-    for(tmp=op->below;tmp!=NULL&&!apply(op,tmp);tmp=tmp->below);
+void apply_below(object *op) {
+    object *tmp, *next;
+
+    /* If using a container, set the starting item to be the top
+     * item in the container.  Otherwise, use the map.
+     */
+    if (op->container!=NULL) tmp=op->container->inv;
+    else tmp=op->below;
+
+    /* This is perhaps more complicated.  However, I want to make sure that
+     * we don't use a corrupt pointer for the next object, so we get the
+     * next object in the stack before applying.  This is probably only a 
+     * problem if apply actually uses the object but does not return a
+     * proper value.
+     */
+    for ( ; tmp!=NULL; tmp=next) {
+	next = tmp->below;
+	if (apply(op, tmp, 0)) return;
+    }
 }
 
-int apply_special(object *who,object *op) { /* wear/wield */
+/* who is the object using the object.
+ * op is the object they are using.
+ * aflags is special flags (0 - normal/toggel, 1=always apply, 2=always unapply
+ * (see define.h, AP_* values)
+ */
+int apply_special(object *who,object *op, int aflags) { /* wear/wield */
   object *tmp;
   char buf[MAX_BUF];
   int i;
@@ -2069,6 +1919,8 @@ int apply_special(object *who,object *op) { /* wear/wield */
     return 1;
   }
   if(QUERY_FLAG(op,FLAG_APPLIED)) {
+    /* always apply, so no reason to unapply */
+    if (aflags == AP_APPLY) return 0;
     if (QUERY_FLAG(op, FLAG_CURSED) || QUERY_FLAG(op, FLAG_DAMNED))
     {
       new_draw_info_format(NDI_UNIQUE, 0, who,
@@ -2155,36 +2007,38 @@ int apply_special(object *who,object *op) { /* wear/wield */
     new_draw_info(NDI_UNIQUE, 0,who,buf);
     tmp=merge_ob(op,NULL);
     fix_player(who);
-    if(who->type==PLAYER)
-      if (who->contr->eric_server > 0) {
+    if(who->type==PLAYER) {
 	  if (tmp) {  /* it was merged */
-	      esrv_del_item (who->contr->eric_server, op->count);
+	      esrv_del_item (who->contr, op->count);
 	      op = tmp;
 	  }
 	  esrv_send_item(who, op);
-      } else
-	  draw_inventory(who);
+    }
 
     return 0;
   }
+  if (aflags == AP_UNAPPLY) return 0;
   if (QUERY_FLAG(op, FLAG_UNPAID)) {
     new_draw_info(NDI_UNIQUE, 0,who, "You should pay for it first.\n");
     return 0;
   }
   i=0;
+  /* This goes through and checks to see if the player already has something
+   * of that type applied - if so, unapply it.
+   */
   for(tmp=who->inv;tmp!=NULL;tmp=tmp->below)
-    if(tmp->type==op->type&&QUERY_FLAG(tmp, FLAG_APPLIED)&&tmp!=op)
+    if(tmp->type==op->type&&QUERY_FLAG(tmp, FLAG_APPLIED)&&tmp!=op) {
       if(tmp->type==RING&&!i)
         i=1;
-      else if(apply_special(who,tmp))
+      else if(apply_special(who,tmp,0))
         return 0;
+    }
   if(op->nrof > 1)
     tmp = get_split_ob(op,op->nrof - 1);
   else
     tmp = NULL;
   switch(op->type) {
   case WEAPON: {
-    int level=who->level; 
 
     if(!QUERY_FLAG(who,FLAG_USE_WEAPON)) {
       sprintf(buf,"You can't use %s.",query_name(op));
@@ -2193,36 +2047,7 @@ int apply_special(object *who,object *op) { /* wear/wield */
       new_draw_info(NDI_UNIQUE, 0,who,buf);
       return 0;
     }
-    /* The skill system hands out wc and dam bonuses to fighters
-     * more generously than the old system (see fix_player). Thus
-     * we need to curtail the power of player enchanted weapons. 
-     * I changed this to 1 improvement per "fighter" level/5 -b.t. 
-     * Note:  Nothing should break by allowing this ratio to be different or
-     * using normal level - it is just a matter of play balance.
-     */			
-#ifdef ALLOW_SKILLS
-    if(who->type==PLAYER) { 
-      object *wc_obj=NULL;
-
-      for(wc_obj=who->inv;wc_obj;wc_obj=wc_obj->below)
-	if(wc_obj->type==EXPERIENCE&&wc_obj->stats.Str) break;
-      if(!wc_obj) 
-	LOG(llevError,"Error: Player: %s lacks wc experience object\n",who->name);
-      else
-	level=wc_obj->level; 
-    } 
-    if(op->last_eat>((level/5)+5)) {  
-#else
-    if(op->last_eat>(level+5)) {	/* peterm:  last eat is used by
-					the prepare weapon scroll
-					to state how many times a weapon
-					has been improved.  It's sort
-					of like a weapon level.  Setting
-					a limit here prevents people
-					from using an enchanted weapon
-					too unreasonably powerful for them */
-#endif
-					
+    if (!check_weapon_power(who, op->last_eat)) {
       new_draw_info(NDI_UNIQUE, 0,who,	
 		    "That weapon is too powerful for you to use.");
       new_draw_info(NDI_UNIQUE, 0, who,	"It would consume your soul!.");
@@ -2353,14 +2178,10 @@ int apply_special(object *who,object *op) { /* wear/wield */
     }
   }
   if(who->type==PLAYER) {
-    if (who->contr->eric_server > 0) {
       /* if multiple objects were applied, update both slots */
       if (tmp)            
 	esrv_send_item(who, tmp);
       esrv_send_item(who, op);
-    } else {
-      draw_inventory(who);
-    }
   }
   return 0;
 }
@@ -2479,7 +2300,7 @@ void eat_special_food(object *who, object *food) {
   	if(food->protected&at_type[i]) cast_change_attr(who,who,99,sp_type[i]);
 
   /* check for hp, sp change */
-  if(food->stats.hp!=0) 
+  if(food->stats.hp!=0) {
      if(QUERY_FLAG(food, FLAG_CURSED)) { 
          new_draw_info(NDI_UNIQUE, 0,who,"Eck!...that was poisonous!");
  	 who->stats.hp -= food->stats.hp;
@@ -2490,7 +2311,8 @@ void eat_special_food(object *who, object *food) {
             new_draw_info(NDI_UNIQUE, 0,who,"Eck!...that was poisonous!");
   	 who->stats.hp += food->stats.hp;
      }
-  if(food->stats.sp!=0) 
+  }
+  if(food->stats.sp!=0) {
      if(QUERY_FLAG(food, FLAG_CURSED)) { 
          new_draw_info(NDI_UNIQUE, 0,who,"You are drained of mana!");
 	 who->stats.sp -= food->stats.sp; 
@@ -2500,6 +2322,7 @@ void eat_special_food(object *who, object *food) {
 	 who->stats.sp += food->stats.sp; 
 	/* place limit on max sp from food? */
      }
+  }
 }
 
 
@@ -2509,27 +2332,30 @@ void eat_special_food(object *who, object *food) {
  */
 
 void apply_lighter(object *who, object *lighter) {
-   object *item = who->inv;
+    object *item = who->inv;
+    int nrof;
 
-#if 0
-   for(item=who->inv;item;item=item->next)
-	if(!item->invisible&&item!=lighter) break;
-#endif
-   item=find_marked_object(who);
-   if(item) {  
+    item=find_marked_object(who);
+    if(item) {  
 	/* Perhaps we should split what we are trying to light on fire?
 	 * I can't see many times when you would want to light multiple
 	 * objects at once.
 	 */
-        new_draw_info_format(NDI_UNIQUE, 0,who,
-	   "You attempt to light the %s with the %s.",item->name,lighter->name);
         if(lighter->last_eat && lighter->stats.food) /* lighter is used up */ 
 	    lighter->stats.food--; 
         else if(lighter->last_eat) {           /* no charges left in lighter */ 
-	     new_draw_info_format(NDI_UNIQUE, 0,who,"Nothing happens.");
+	     new_draw_info_format(NDI_UNIQUE, 0,who,"You ateempt to light the %s with the %s and nothing happens.", item->name, lighter->name);
 	     return;
         }
+	nrof=item->nrof;
 	save_throw_object(item,AT_FIRE);
+	if ((nrof != item->nrof ) || QUERY_FLAG(item, FLAG_FREED)) {
+	    new_draw_info_format(NDI_UNIQUE, 0,who,
+		 "You light the %s with the %s.",item->name,lighter->name);
+	} else {
+	    new_draw_info_format(NDI_UNIQUE, 0,who,
+		 "You attempt to light the %s with the %s and fail.",item->name,lighter->name);
+	}
 
    } else /* nothing to light */ 
 	new_draw_info(NDI_UNIQUE, 0,who,"You need to mark a lightable object.");
@@ -2553,7 +2379,6 @@ void scroll_failure(object *op, int failure, int power)
      new_draw_info(NDI_UNIQUE, 0,op,"Your mana is drained!.");
      op->stats.sp -= RANDOM()%power;
      if(op->stats.sp<0) op->stats.sp = 0;
-     if(op->type==PLAYER) draw_stats(op);
     }
 
   /* even nastier effects continue...*/ 
