@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_c_misc_c =
- *   "$Id: c_misc.c,v 1.17 1997/03/09 05:06:46 master Exp master $";
+ *   "$Id: c_misc.c,v 1.20 1998/09/28 03:02:47 master Exp master $";
  */
 
 /*
@@ -91,13 +91,8 @@ int command_bug(object *op, char *params)
       new_draw_info(NDI_UNIQUE, 0,op,"what bugs?");
       return 1;
     }
-    if (op == NULL) {
-      strcpy(buf,active_socket->name);
-      strcat(buf," bug reports: ");
-    } else {
-      strcpy(buf,op->name);
-      strcat(buf," bug-reports: ");
-    }
+    strcpy(buf,op->name);
+    strcat(buf," bug-reports: ");
     strncat(buf,++params,MAX_BUF - strlen(buf) );
     buf[MAX_BUF - 1] = '\0';
     bug_report(buf);
@@ -110,7 +105,7 @@ int command_bug(object *op, char *params)
 
 void malloc_info(object *op) {
   int ob_used=count_used(),ob_free=count_free(),players,nrofmaps;
-  int nrm=0,mapmem=0,anr,anims,sum_alloc=0,sum_used=0,i,j,tlnr, alnr;
+  int nrm=0,mapmem=0,anr,anims,sum_alloc=0,sum_used=0,i,tlnr, alnr;
   treasurelist *tl;
   player *pl;
   mapstruct *m;
@@ -121,9 +116,11 @@ void malloc_info(object *op) {
   for(al=first_artifactlist, alnr=0; al!=NULL; al=al->next, alnr++);
 
   for(at=first_archetype,anr=0,anims=0;at!=NULL;
-      at=at->more==NULL?at->next:at->more,anr++)
-    if(at->faces!=NULL)
-      anims+=at->animations+1;
+      at=at->more==NULL?at->next:at->more,anr++);
+
+  for (i=1; i<num_animations; i++)
+    anims += animations[i].num_animations;
+
   for(pl=first_player,players=0;pl!=NULL;pl=pl->next,players++);
   for(m=first_map,nrofmaps=0;m!=NULL;m=m->next,nrofmaps++)
     if(m->in_memory == MAP_IN_MEMORY)
@@ -143,19 +140,8 @@ void malloc_info(object *op) {
   sprintf(errmsg,"%4d players:         %8d",players,i=(players*sizeof(player)));
   new_draw_info(NDI_UNIQUE, 0,op,errmsg);
   sum_alloc+=i; sum_used+=i;
-  for(pl=first_player,i=0;pl!=NULL;pl=pl->next)
-    i+=pl->infolines*pl->infochars*sizeof(char)+
-       pl->inv_chars*pl->inv_size*sizeof(char)+
-       pl->look_chars*pl->look_size*sizeof(char);
-  sprintf(errmsg,"%4d player windows:  %8d",players*3,i);
-  new_draw_info(NDI_UNIQUE, 0,op,errmsg);
-  sum_alloc+=i; sum_used+=i;
-  for(pl=first_player,i=0,j=0;pl!=NULL;pl=pl->next)
-    if(pl->pixmaps!=NULL)
-      i+=nrofpixmaps*sizeof(Pixmap), j++;
-  sprintf(errmsg,"%4d pixmap sets:     %8d",j,i);
-  new_draw_info(NDI_UNIQUE, 0,op,errmsg),
-  sum_alloc+=i; sum_used+=i;
+
+
   sprintf(errmsg,"%4d maps allocated:  %8d",nrofmaps,
           i=(nrofmaps*sizeof(mapstruct)));
   new_draw_info(NDI_UNIQUE, 0,op,errmsg);
@@ -245,9 +231,6 @@ int command_malloc_verify(object *op, char *parms)
 int command_who (object *op, char *params)
 {
     player *pl;
-#ifdef SERVER
-    sockets *s;
-#endif /* SERVER */
   char buf[MAX_BUF];
 
     if (first_player != (player *) NULL)
@@ -259,30 +242,21 @@ int command_who (object *op, char *params)
  * the same informaitn, except one display pl->ob->count.
  */
       if(op == NULL || QUERY_FLAG(op, FLAG_WIZ))
-        (void) sprintf(buf,"%s the %s (%s@%s) [%s]%s%s%s (%d)",pl->ob->name,
+        (void) sprintf(buf,"%s the %s (@%s) [%s]%s%s%s (%d)",pl->ob->name,
 		(pl->own_title[0]=='\0'?pl->title:pl->own_title),
-		pl->username,pl->name,
+		pl->socket.host,
 		pl->ob->map->path,
                 QUERY_FLAG(pl->ob,FLAG_WIZ)?" [WIZ]":"",pl->idle?" I":"",
                 pl->peaceful?"P":"W",pl->ob->count);
       else
-        (void) sprintf(buf,"%s the %s (%s@%s) [%s]%s%s%s",pl->ob->name,
+        (void) sprintf(buf,"%s the %s (@%s) [%s]%s%s%s",pl->ob->name,
                  (pl->own_title[0]=='\0'?pl->title:pl->own_title),
-	         pl->username,pl->name,
+		 pl->socket.host,
 		 pl->ob->map->path,
                  QUERY_FLAG(pl->ob,FLAG_WIZ)?" [WIZ]":"",pl->idle?" I":"",
                  pl->peaceful?"P":"W");
       new_draw_info(NDI_UNIQUE, 0,op,buf);
     }
-#ifdef SERVER
-    if(first_socket == (sockets *) NULL)
-      return 1;
-    new_draw_info(NDI_UNIQUE, 0,op,"Sockets:");
-    for(s = first_socket; s != (sockets *) NULL; s = s->next) {
-      (void) sprintf(buf, "%s (%s)%s", s->name,s->host,s->wiz?" [Wiz]":"");
-      new_draw_info(NDI_UNIQUE, 0,op,buf);
-    }
-#endif
     return 1;
   }
 
@@ -345,20 +319,13 @@ int command_debug (object *op, char *params)
   if(params==NULL || !sscanf(params, "%d", &i)) {
       sprintf(buf,"Global debug level is %d.",settings.debug);
       new_draw_info(NDI_UNIQUE, 0,op,buf);
-      if(op == NULL) {
-        sprintf(buf,"Socket debug level is %d.",active_socket->debug);
-        new_draw_info(NDI_UNIQUE, 0,op,buf);
-      }
       return 1;
     }
-  if((op != NULL && !QUERY_FLAG(op, FLAG_WIZ)) || (*params == '-' && !active_socket->wiz)) {
+  if(op != NULL && !QUERY_FLAG(op, FLAG_WIZ)) {
       new_draw_info(NDI_UNIQUE, 0,op,"Privileged command.");
       return 1;
     }
-  if (*params == '-' || op != NULL)
-      settings.debug = (enum LogLevel) FABS(i);
-    else
-      active_socket->debug = (enum LogLevel) i;
+    settings.debug = (enum LogLevel) FABS(i);
     sprintf(buf,"Set debug level to %d.", i);
     new_draw_info(NDI_UNIQUE, 0,op,buf);
     return 1;
@@ -450,30 +417,11 @@ int command_printlos (object *op, char *params)
   return 0;
 }
 
-int command_clearinfo (object *op, char *params)
-{
- clear_win_info(op);
- return 0;
-}
-
-int command_refresh (object *op, char *params)
-{
-      if(!op->contr->split_window)
-        XClearWindow(op->contr->gdisp,op->contr->win_root);
-      refresh(op);
-      op->contr->last_value= -1;
-      draw_stats(op);
-      draw_all_info(op);
-      draw_all_inventory(op);
-      draw_all_look(op);
-      draw_all_message(op);
-  return 0;
-}
 
 int command_version (object *op, char *params)
 {
-      version(op);
-  return 0;
+    version(op);
+    return 0;
 }
 
 
@@ -538,74 +486,15 @@ int command_listen (object *op, char *params)
 
   if(params==NULL || !sscanf(params, "%d", &i)) {
     new_draw_info_format(NDI_UNIQUE, 0, op,
-	"Set listen to what (presently %d)?", 
-	op==NULL? active_socket->listen_lev : (int) op->contr->listening);
+	"Set listen to what (presently %d)?", op->contr->listening);
       return 1;
     }
-    if(op == NULL) {
-      active_socket->listen_lev = i;
-      new_draw_info(NDI_UNIQUE, 0,op,"OK.");
-    }
-    else {
-      op->contr->listening=(char) i;
-      new_draw_info_format(NDI_UNIQUE, 0, op,
+    op->contr->listening=(char) i;
+    new_draw_info_format(NDI_UNIQUE, 0, op,
 	"Your verbose level is now %d.",i);
-    }
     return 1;
 }
 
-int command_sort_inventory(object *op, char *params)
-{
-  if (!params)
-    op->contr->sort_inventory =!op->contr->sort_inventory;
-  else
-    op->contr->sort_inventory =onoff_value(params);
-
-  if(op->contr->sort_inventory)
-    new_draw_info(NDI_UNIQUE, 0,op, "Picked up items will be inserted alphabetically.");
-  else
-    new_draw_info(NDI_UNIQUE, 0,op, "Picked up items will be inserted by type");
-
-  return 0;
-}
-
-int command_keyboard_flush (object *op, char *params)
-{
-  if (!params)
-    op->contr->keyboard_flush =!op->contr->keyboard_flush;
-  else
-    op->contr->keyboard_flush =onoff_value(params);
-
-  if(op->contr->keyboard_flush)
-    new_draw_info(NDI_UNIQUE, 0,op, "Keystrokes will not be buffered.");
-  else
-    new_draw_info(NDI_UNIQUE, 0,op, "Keystrokes will be buffered.");
-
-  return 0;
-}
-
-int command_show_inv_icon(object *pl, char *params)
-{
-  if (!params)
-    pl->contr->show_inv_icon =!pl->contr->show_inv_icon;
-  else
-    pl->contr->show_inv_icon =onoff_value(params);
-
-  if(pl->contr->show_inv_icon) {
-    new_draw_info(NDI_UNIQUE, 0,pl, "Inventory icons will now be shown");
-    pl->contr->inv_chars=(pl->contr->invhint.width-88)/FONTWIDTH;
-
-  }
-  else {
-    new_draw_info(NDI_UNIQUE, 0,pl, "Inventory icons will not be shown");
-    pl->contr->inv_chars=(pl->contr->invhint.width-60)/FONTWIDTH;
-  }
-  sprintf(pl->contr->format_inv,"%%-%d.%ds%%-6s",pl->contr->inv_chars-6,
-	  pl->contr->inv_chars-6);
-  draw_all_inventory(pl);
-  return 0;
-}
-
 /* Prints out some useful information for the character.  Everything we print
  * out can be determined by the docs, so we aren't revealing anything extra -
  * rather, we are making it convenient to find the values.  params have
@@ -637,24 +526,6 @@ int command_statistics(object *pl, char *params)
    return 0;
 }
 
-int command_use_buttons(object *pl, char *params)
-{
-  if (!params)
-    pl->contr->show_buttons =!pl->contr->show_buttons;
-  else
-    pl->contr->show_buttons =onoff_value(params);
-
-  if(!pl->contr->show_buttons) {
-    new_draw_info(NDI_UNIQUE, 0,pl, "Buttons will not be displayed.");
-  }
-  else {
-    new_draw_info(NDI_UNIQUE, 0,pl, "Buttons now displayed");
-    draw_all_message(pl);
-  }
-  draw_all_message(pl);
-  return 0;
-}
-
 int command_fix_me(object *op, char *params)
 {
     fix_player(op);
