@@ -6,6 +6,7 @@
 #include <global.h>
 #include <loader.h>
 #include <sproto.h>
+#include <living.h>
 
 /*
  * Object id parsing functions
@@ -63,7 +64,7 @@ int item_matched_string(object *pl, object *op, char *name)
 	/* Allow for things like '100 arrows' */
 	if ((count=atoi(cp))!=0) {
 	    cp=strchr(cp, ' ');
-	    while (cp[0]==' ') ++cp;	/* get rid of spaces */
+	    while (cp && cp[0]==' ') ++cp;	/* get rid of spaces */
 	}
 	else count=pl->contr->count;
 	if (!cp || cp[0]=='\0' || count<0) return 0;
@@ -122,209 +123,7 @@ object *find_best_object_match(object *pl, char *params)
     return best;
 }
 
-/* This looks for the first item in the players inventory.  In theory,
- * you can also use non player objects.  In all cases, it skips over
- * invisible objects.  In the case of players, it will also match up
- * with what the inventory window shows.
- */
 
-object *find_first_inventory_item(object *pl)
-{
-
-    object *inv;
-
-    inv= pl->inv;
-    if (pl->type!=PLAYER) {
-	while(inv&&inv->invisible)
-	    inv=inv->below;
-    }
-    else { /*show_what_object also looks for invisibility */
-	while (inv && !show_what_object(inv, pl->contr->show_what))
-	    inv=inv->below;
-    }
-    return inv;
-}
-
-#if 0
-/* returns nonzero if the appropriate action can be
- * done.  Really a stupid function - this should be handled in
- * the calling function.
- */
-static int may_do(object *op,object *ob,int state)
-{
-  if(!ob)
-    return 0;
-  switch(state)
-    {
-    case 0:			/* drop */
-      return (!QUERY_FLAG(ob,FLAG_NO_DROP) && !ob->invisible);
-
-    case 1:			/* take */
-      return (can_pick(op,ob) && !ob->invisible);
-
-    default:
-      return 0;
-    }
-}
-
-static objectlink *parse_id(object *op,object *from, char *name, int state)
-{
-  objectlink *first=NULL,*last=NULL;
-  int count=0,plural=0;
-  char buf[MAX_BUF];
-
-  while(name[0]==' ')		/* get rid of leading spaces ', ' */
-    name++;
-  if(strcmp(name,"all")==0)
-    {
-      while(from)
-	{
-	  if(may_do(op,from,state))
-	    {
-	      ADD_ITEM(from,0);
-	    }
-	  from=from->below;
-	}
-      return first; 
-    }
-  if(strcmp(name,"unpaid")==0)
-    {
-      while(from)
-	{
-	  if(QUERY_FLAG(from,FLAG_UNPAID) && may_do(op,from,state))
-	    {
-	      ADD_ITEM(from,0);
-	    }
-	  from=from->below;
-	}
-      return first; 
-    }
-  if((count=atoi(name))!=0)
-    name=strchr(name,' ');
-  else
-    if (op->contr->count>0 && op->contr->count<10000)
-      count=op->contr->count;
-  if(!name || strlen(name)==0 || count<0)
-    return NULL;
-  while(name[0]==' ')		/* get rid of leading spaces ', ' */
-    name++;
-  if(strlen(name)==0)
-    return NULL;
-  while(from)
-    {
-      if(may_do(op,from,state))
-	{
-	  strcpy(buf,from->name);
-	  if(QUERY_FLAG(from, FLAG_NEED_IE)) {
-	    char *cp=strrchr(buf,'y');
-	    if(cp!=NULL)
-	      *cp='\0'; /* Strip the 'y' */
-	    strcat(buf,"ies");
-	  } else
-	    strcat(buf,"s");
-	  if(strcasecmp(name,from->name)==0 && count<2)	/* singular */
-	    {
-	      ADD_ITEM(from,1);
-	      break;
-	    }
-	  else if(strcasecmp(name,buf)==0 && (count!=1 || plural)) /* plural */
-	    {
-	      plural=1;
-	      ADD_ITEM(from,count);
-	      count-=(from->nrof<1?1:from->nrof);
-	      if(count<=0)
-		{
-		  count=0;
-		  break;
-		}
-	    }
-	  else if(strcasecmp(name,query_name(from))==0 && count==0)
-	    {
-	      ADD_ITEM(from,from->nrof);
-	    }
-	}
-      from=from->below;
-    }
-  return first;
-}
-
-/* Tries to find a matching object.
- * op is the player/monster.
- * from is the potential source object.
- * ids is a string to match for
- * state is what we are tryign to do (0=drop, 1=pickup)
- */
-
-objectlink *id(object *op,object *from,char *ids,int state)
-{
-  objectlink *first=NULL,*last=NULL,*tmp;
-  char *name;
-  if(from==NULL)
-    return NULL;
-  if(ids==NULL || !ids[0])
-    {
-      int show_what=show_all;
-      if (op->type==PLAYER && state==0)  show_what=op->contr->show_what;
-
-      while(from && !show_what_object(from,show_what) && may_do(op, from, state)) {
-	from=from->below;
-      }
-      if(!from)
-	return NULL;
-      OBLINKMALLOC(first);
-      first->ob=from;
-      first->id=op->contr->count;
-      first->next=NULL;
-      return first;
-    }
-  name=strtok(ids,",");		/* pearls, gems, coins (split with commas) */
-  while(name)
-    {
-      if((tmp=parse_id(op,from,name,state)))
-	{
-	  if(!first)
-	    first=tmp;
-	  else
-	    last->next=tmp;
-	  last=tmp;
-	  while(last->next)	/* move to end of list */
-	    last=last->next;
-	}
-      name=strtok(NULL,",");
-    }
-  if(first && first->next)	/* get rid of duplicates */
-    {
-      objectlink *pos,*prev;
-      pos=first;
-      while(pos)
-	{
-	  tmp=pos->next;
-	  prev=pos;
-	  while(tmp)
-	    {
-	      if(tmp->ob == pos->ob)
-		{
-		  if(tmp->id==0 || pos->id==0)
-		    pos->id=0;
-		  else
-		    pos->id=pos->id + tmp->id;
-		  prev->next=tmp->next;
-		  free(tmp);
-		  tmp=prev->next;
-		}
-	      else
-		{
-		  prev=tmp;
-		  tmp=tmp->next;
-		}
-	    }
-	  pos=pos->next;
-	}
-    }
-  return first;
-}
-
-#endif
 int command_uskill ( object *pl, char *params) {
    if (!params) {
         new_draw_info(NDI_UNIQUE, 0,pl, "Usage: use_skill <skill name>");
@@ -360,10 +159,10 @@ void apply_inventory(object *op) {
       return;
     }
   } else {
-    inv= find_first_inventory_item(op);
+    for (inv=op->inv; inv && inv->invisible; inv=inv->below) ;
   }
   if(inv) {
-    if(!apply(op,inv)) {
+    if(!apply(op,inv,0)) {
       new_draw_info_format(NDI_UNIQUE, 0, op,
 	"I don't know how to apply the %s.",query_name(inv));
     }
@@ -379,9 +178,23 @@ int command_apply (object *op, char *params)
     return 0;
   }
   else {
-    object *inv=find_best_object_match(op, params);
+    int aflag=0;
+    object *inv;
+
+    while (*params==' ') params++;
+    if (!strncmp(params,"-a ",3)) {
+	aflag=AP_APPLY;
+	params+=3;
+    }
+    if (!strncmp(params,"-u ",3)) {
+	aflag=AP_UNAPPLY;
+	params+=3;
+    }
+    while (*params==' ') params++;
+
+    inv=find_best_object_match(op, params);
     if (inv) {
-	if(!apply(op,inv)) {
+	if(!apply(op,inv,aflag)) {
 	  new_draw_info_format(NDI_UNIQUE, 0, op,
 	    "I don't know how to apply the %s.",query_name(inv));
 	}
@@ -389,14 +202,43 @@ int command_apply (object *op, char *params)
 	  new_draw_info_format(NDI_UNIQUE, 0, op,
 	    "Could not find any match to %s.",params);
   }
-  /* By applying something what, the applied/unapplied state will change,
-   * so the window needs to be updated.
-   */
-  if (op->contr->show_what == show_applied || op->contr->show_what==show_unapplied)
-    draw_all_inventory(op);
   return 0;
 }
 
+/*
+ * Check if an item op can be put into a sack. If pl exists then tell
+ * a player the reason of failure.
+ * returns 1 if it will fit, 0 if it will not.  nrof is the number of
+ * objects (op) we want to put in.  We specify it separately instead of
+ * using op->nrof because often times, a player may have specified a
+ * certain number of objects to drop, so we can pass that number, and
+ * not need to use split_ob and stuff.
+ */
+int sack_can_hold (object *pl, object *sack, object *op, int nrof) {
+    char buf[MAX_BUF];
+    buf[0] = 0;
+
+    if (! QUERY_FLAG (sack, FLAG_APPLIED))
+	sprintf (buf, "%s is not active.", query_name(sack));
+    if (sack == op)
+	sprintf (buf, "You can't put %s into itself.", query_name(sack));
+    if (sack->race && (sack->race != op->race || op->type == CONTAINER
+		       || (sack->stats.food && sack->stats.food != op->type)))
+	sprintf (buf, "You can put only %s into %s.", sack->race,
+		 query_name(sack));
+    if (op->type == SPECIAL_KEY && sack->slaying && op->slaying)
+	sprintf (buf, "You don't want put the key into %s.", query_name(sack));
+    if (sack->weight_limit && sack->carrying + (nrof ? nrof : 1) * 
+	op->weight * (100 - sack->stats.Str) / 100  > sack->weight_limit)
+	sprintf (buf, "That won't fit in %s!", query_name(sack));
+    if (buf[0]) {
+	if (pl)
+	    new_draw_info(NDI_UNIQUE, 0,pl, buf);
+	return 0;
+    }
+    return 1;
+}
+
 /* Pick up commands follow */
 /* pl = player, op is the object to put tmp into, 
  * tmp is the object to pick up, nrof is the number to
@@ -405,6 +247,8 @@ int command_apply (object *op, char *params)
 void pick_up_object (object *pl, object *op, object *tmp, int nrof)
 {
     char buf[MAX_BUF];
+    object *env=tmp->env;
+    uint32 weight;
 
     /* IF the player is flying & trying to take the item out of a container 
      * that is in his inventory, let him.  tmp->env points to the container 
@@ -418,26 +262,38 @@ void pick_up_object (object *pl, object *op, object *tmp, int nrof)
     }
     if(!can_pick(pl,tmp)) {
 	if (tmp->name!=NULL) {
-	sprintf(buf,"You can't pick up a %s", tmp->name);
-	new_draw_info(NDI_UNIQUE, 0,pl, buf);
+	    sprintf(buf,"You can't pick up a %s", tmp->name);
+	    new_draw_info(NDI_UNIQUE, 0,pl, buf);
 	}
-      else
-	new_draw_info(NDI_UNIQUE, 0,pl,"You can't take that!");
-      return;
+	else
+	    new_draw_info(NDI_UNIQUE, 0,pl,"You can't take that!");
+	return;
     }
     if (QUERY_FLAG (tmp, FLAG_NO_DROP))
 	return;
     if(QUERY_FLAG(tmp,FLAG_WAS_WIZ) && !QUERY_FLAG(pl, FLAG_WAS_WIZ)) {
 	new_draw_info(NDI_UNIQUE, 0,pl, "The object disappears in a puff of smoke!");
 	new_draw_info(NDI_UNIQUE, 0,pl, "It must have been an illusion.");
-	if (pl->contr->eric_server > 0)
-	    esrv_del_item (pl->contr->eric_server, tmp->count);
+	esrv_del_item (pl->contr, tmp->count);
 	remove_ob(tmp);
 	free_object(tmp);
 	return;
     }
+    /* Figure out how much weight this object will add to the player */
+    weight = tmp->weight;
+    if (nrof) weight *=nrof;
+    else if (tmp->nrof) weight *= tmp->nrof;
+    if (op->type == CONTAINER) weight = weight * (100 - op->stats.Str) / 100;
+    if ((pl->weight + pl->carrying +weight) > weight_limit[pl->stats.Str]) {
+	new_draw_info(NDI_UNIQUE, 0,pl,"That item is too heavy for you to pick up.");
+	return;
+    }
+	
+
+#ifndef REAL_WIZ
     if(QUERY_FLAG(pl, FLAG_WAS_WIZ))
 	SET_FLAG(tmp, FLAG_WAS_WIZ);
+#endif
 
     if(nrof && nrof != tmp->nrof) {
 	object *tmp2 = tmp;
@@ -447,10 +303,9 @@ void pick_up_object (object *pl, object *op, object *tmp, int nrof)
 	    return;
 	}
 	/* Tell a client what happened rest of objects */
-	if (pl->contr->eric_server > 0)
-	    if (QUERY_FLAG(tmp2, FLAG_REMOVED))
-		esrv_del_item (pl->contr->eric_server, tmp2->count);
-	    else
+	if (QUERY_FLAG(tmp2, FLAG_REMOVED))
+		esrv_del_item (pl->contr, tmp2->count);
+	else
 		esrv_send_item (pl, tmp2);
     } else {
 	remove_ob(tmp); /* Unlink it */
@@ -464,17 +319,16 @@ void pick_up_object (object *pl, object *op, object *tmp, int nrof)
 
     tmp = insert_ob_in_ob(tmp, op);
     if(pl->type!=PLAYER) return;
-    if (pl->contr->eric_server > 0) {
-	esrv_send_item (pl, tmp);
-    } else {
-	pl->contr->last_used=tmp;
-	pl->contr->last_used_id=tmp->count;
 
-	draw_inventory(pl);
+    esrv_send_item (pl, tmp);
+    /* These are needed to update the weight for the container we
+     * are putting the object in, and the players weight, if different.
+     */
+    esrv_update_item (UPD_WEIGHT, pl, op);
+    if (op!=pl) esrv_send_item (pl, pl);
 
-	if (pl->container)
-	    draw_look (pl);
-    }
+    /* Update the container the object was in */
+    if (env && env!=pl && env!=op) esrv_update_item (UPD_WEIGHT, pl, env);
 }
 
 
@@ -536,8 +390,9 @@ void pick_up(object *op,object *alt)
 
 int command_take (object *op, char *params)
 {
-    object *tmp=op->below, *next;
+    object *tmp, *next;
 
+    tmp=(op->container) ? op->container->inv : op->below;
     if (tmp==NULL) {
 	new_draw_info(NDI_UNIQUE, 0,op,"Nothing to take!");
 	return 0;
@@ -546,8 +401,6 @@ int command_take (object *op, char *params)
     /* Makes processing easier */
     if (params && *params=='\0') params=NULL;
 
-    op->contr->freeze_inv=1;
-    op->contr->freeze_look=1;
     while (tmp) {
 	next=tmp->below;
 
@@ -580,9 +433,6 @@ int command_take (object *op, char *params)
 	if (!tmp) new_draw_info(NDI_UNIQUE, 0,op, "There is nothing to pick up.");
     }
 
-    op->contr->freeze_inv=0;
-    op->contr->freeze_look=0;
-    draw_inventory(op);
     draw_look(op);
     return 0;
 }
@@ -591,7 +441,7 @@ int command_take (object *op, char *params)
 /*
  *  This function was part of drop, now is own function. 
  *  Player 'op' tries to put object 'tmp' into sack 'sack', 
- *  if tmp is non zero, then nrof objects is tried to put into sack. 
+ *  if nrof is non zero, then nrof objects is tried to put into sack. 
  */
 void put_object_in_sack (object *op, object *sack, object *tmp, long nrof) 
 {
@@ -640,47 +490,54 @@ void put_object_in_sack (object *op, object *sack, object *tmp, long nrof)
     if(QUERY_FLAG(tmp, FLAG_APPLIED)) {
       int nrof = tmp->nrof, a;
       tmp->nrof = 0;
-      a = apply_special(op,tmp);
+      a = apply_special(op,tmp,0);
       tmp->nrof = nrof;
       if (a)
 	  return;
     }
 
+    /* we want to put some portion of the item into the container */
     if (nrof && tmp->nrof != nrof) {
-      object *tmp2 = tmp;
-      tmp = get_split_ob (tmp, nrof);
-      if(!tmp) {
-	  new_draw_info(NDI_UNIQUE, 0,op, errmsg);
-	  return;
-      }
-      /* Tell a client what happened other objects */ 
-      if (op->contr->eric_server > 0)
-	  if (QUERY_FLAG(tmp2, FLAG_FREED))
-	      esrv_del_item (op->contr->eric_server, tmp2->count);
-	  else
+	object *tmp2 = tmp;
+	tmp = get_split_ob (tmp, nrof);
+
+	if(!tmp) {
+	    new_draw_info(NDI_UNIQUE, 0,op, errmsg);
+	    return;
+	}
+	/* Tell a client what happened other objects */ 
+	if (QUERY_FLAG(tmp2, FLAG_FREED))
+	      esrv_del_item (op->contr, tmp2->count);
+	else	/* this can proably be replaced with an update */
 	      esrv_send_item (op, tmp2);
     } else
-      remove_ob(tmp);
+	remove_ob(tmp);
 
     sprintf(buf, "You put %s in ", query_name(tmp));
     strcat (buf, query_name(sack));
     strcat (buf, ".");
-    tmp = insert_ob_in_ob(tmp, sack);
+    tmp2 = insert_ob_in_ob(tmp, sack);
     new_draw_info(NDI_UNIQUE, 0,op,buf);
-    if (op->contr->eric_server > 0) {
-      esrv_send_item (op, tmp);
-    } else {
-      draw_look(op);
-      draw_inventory(op);
-    }
     fix_player(op); /* This is overkill, fix_player() is called somewhere */ 
 		  /* in object.c */
+
+    /* If an object merged (and thus, different object), we need to
+     * delete the original.
+     */
+    if (tmp2 != tmp) 
+	esrv_del_item (op->contr, tmp->count);
+	    
+    esrv_send_item (op, tmp2);
+    /* update the sacks and players weight */
+    esrv_update_item (UPD_WEIGHT, op, sack);
+    esrv_update_item (UPD_WEIGHT, op, op);
 }
 
 /*
  *  This function was part of drop, now is own function. 
  *  Player 'op' tries to drop object 'tmp', if tmp is non zero, then
  *  nrof objects is tried to dropped. 
+ * This is used when dropping objects onto the floor.
  */
 void drop_object (object *op, object *tmp, long nrof) 
 {
@@ -699,25 +556,29 @@ void drop_object (object *op, object *tmp, long nrof)
     if(QUERY_FLAG(tmp, FLAG_APPLIED)) {
       int nrof = tmp->nrof,a;
       tmp->nrof = 0;
-      a = apply_special (op, tmp);
+      a = apply_special (op, tmp,0);
       tmp->nrof = nrof;
       if (a)		/* can't unapply it */
 	  return;
     }
 
+    /* We are only dropping some of the items.  We split the current objec
+     * off
+     */
     if(nrof && tmp->nrof != nrof) {
-      object *tmp2 = tmp;
-      tmp = get_split_ob (tmp, nrof);
-      if(!tmp) {
-	  new_draw_info(NDI_UNIQUE, 0,op, errmsg);
-	  return;
-      }
-      /* Tell a client what happened rest of objects */ 
-      if (op->contr->eric_server > 0)
-	  if (QUERY_FLAG(tmp2, FLAG_FREED))
-	      esrv_del_item (op->contr->eric_server, tmp->count);
-	  else
-	      esrv_send_item (op, tmp);
+	object *tmp2 = tmp;
+	tmp = get_split_ob (tmp, nrof);
+	if(!tmp) {
+	    new_draw_info(NDI_UNIQUE, 0,op, errmsg);
+	    return;
+	}
+	/* Tell a client what happened rest of objects.  tmp2 is now the
+	 * original object
+	 */ 
+	if (QUERY_FLAG(tmp2, FLAG_FREED))
+	      esrv_del_item (op->contr, tmp2->count);
+	else
+	      esrv_send_item (op, tmp2);
     } else
       remove_ob (tmp);
 
@@ -725,8 +586,7 @@ void drop_object (object *op, object *tmp, long nrof)
       sprintf(buf,"You drop %s.", query_name(tmp));
       new_draw_info(NDI_UNIQUE, 0,op,buf);
       new_draw_info(NDI_UNIQUE, 0,op,"The gods who lent it to you retrieves it.");
-      if (op->contr->eric_server > 0)
-	  esrv_del_item (op->contr->eric_server, tmp->count);
+	esrv_del_item (op->contr, tmp->count);
       free_object(tmp);
       fix_player(op);
       return;
@@ -747,8 +607,7 @@ void drop_object (object *op, object *tmp, long nrof)
 	save_player(op,1); /* To avoid cheating */
 #endif
 #endif /* SAVE_INTERVAL */
-    
-    D_LOCK(op);
+
 
     floor = get_map_ob (op->map, op->x, op->y);
     if( floor && floor->type == SHOP_FLOOR && 
@@ -758,18 +617,22 @@ void drop_object (object *op, object *tmp, long nrof)
     tmp->x = op->x;
     tmp->y = op->y;
     insert_ob_in_map(tmp, op->map);
-    remove_ob(op);
-    insert_ob_in_map(op, op->map);
-    D_UNLOCK(op);
 
-    if (op->contr->eric_server > 0) {
-      esrv_send_item (op, tmp);
-    } else {
-      draw_look(op);
-      draw_inventory(op);
-    }
+    remove_ob(op);
+    insert_ob_in_map_simple(op, op->map);
+
+    /* Call this before we update the various windows/players.  At least
+     * that we, we know the weight is correct.
+     */
     fix_player(op); /* This is overkill, fix_player() is called somewhere */ 
 		    /* in object.c */
+
+
+    esrv_send_item (op, tmp);
+    /* Need to update the weight for the player */
+    esrv_send_item (op, op);
+
+
 #ifdef USE_LIGHTING
     if(tmp->glow_radius>0) remove_light_from_list(tmp,op);
 #endif
@@ -777,8 +640,25 @@ void drop_object (object *op, object *tmp, long nrof)
 
 void drop(object *op, object *tmp) 
 {
-    while(tmp!=NULL && tmp->invisible)
-      tmp=tmp->below;
+    /* Hopeful fix for disappearing objects when dropping from a container -
+     * somehow, players get an invisible object in the container, and the
+     * old logic would skip over invisible objects - works fine for the
+     * playes inventory, but drop inventory wants to use the next value.
+     */
+    if (tmp->invisible) {
+	/* if the following is the case, it must be in an container. */
+	if (tmp->env && tmp->env->type != PLAYER) {
+	    /* Just toss the object - probably shouldn't be around
+	     * around anyways
+	     */
+	    remove_ob(tmp);
+	    free_object(tmp);
+	    return;
+	} else {
+	    while(tmp!=NULL && tmp->invisible)
+		tmp=tmp->below;
+	}
+    }
 
     if (tmp==NULL) {
       new_draw_info(NDI_UNIQUE, 0,op,"You don't have anything to drop.");
@@ -830,8 +710,6 @@ int command_dropall (object *op, char *params) {
   }
   
   curinv = op->inv;
-  op->contr->freeze_inv=1;
-  op->contr->freeze_look=1;
 
   /* 
     This is the default.  Drops everything not locked or considered
@@ -923,9 +801,6 @@ int command_dropall (object *op, char *params) {
     }
   }
 
-  op->contr->freeze_inv=0;
-  op->contr->freeze_look=0;
-  draw_all_inventory(op);
   draw_look(op);
   return 0;
 }
@@ -937,17 +812,16 @@ int command_dropall (object *op, char *params) {
 int command_drop (object *op, char *params)
 {
     object  *tmp, *next;
-    int did_one=0, show_what=show_all;
+    int did_one=0;
 
-    op->contr->freeze_inv=1;
-    op->contr->freeze_look=1;
-    if (!params) drop(op,find_first_inventory_item(op));
-    else {
-	if (op->type==PLAYER) show_what=op->contr->show_what;
+    if (!params) {
+	new_draw_info(NDI_UNIQUE,0, op, "Drop what?");
+	return 0;
+    } else {
 	for (tmp=op->inv; tmp; tmp=next) {
 	    next=tmp->below;
 	    if (QUERY_FLAG(tmp,FLAG_NO_DROP) || 
-		!show_what_object(tmp,show_what)) continue;
+		!tmp->invisible) continue;
 	    if (item_matched_string(op,tmp,params)) {
 		drop(op, tmp);
 		did_one=1;
@@ -955,10 +829,7 @@ int command_drop (object *op, char *params)
 	}
 	if (!did_one) new_draw_info(NDI_UNIQUE, 0,op,"Nothing to drop.");
     }
-    op->contr->freeze_inv=0;
-    op->contr->freeze_look=0;
     op->contr->count=0;
-    draw_all_inventory(op);
     draw_look(op);
     return 0;
 }
@@ -1041,53 +912,3 @@ int command_mark(object *op, char *params)
     return 0;	/*shouldnt get here */
 }
 
-int command_lock (object *op, char *params)
-{
-    object  *tmp;
-    int show_what=show_all,did_one=0;
-
-    op->contr->freeze_inv=1;
-    op->contr->freeze_look=1;
-    if (!params) lock_inv(op,find_first_inventory_item(op));
-    else {
-	if (op->type==PLAYER) show_what=op->contr->show_what;
-	for (tmp=op->inv; tmp; tmp=tmp->below) {
-	    if (!show_what_object(tmp,show_what)) continue;
-	    if (item_matched_string(op,tmp,params)) {
-		lock_inv(op, tmp);
-		did_one=1;
-	    }
-	}
-	if (!did_one) new_draw_info(NDI_UNIQUE, 0,op,"Nothing to lock.");
-    }
-    op->contr->freeze_inv=0;
-    op->contr->count=0;
-    draw_inventory(op);
-    return 0;
-}
-
-int command_unlock (object *op, char *params)
-{
-    object  *tmp;
-    int show_what=show_all,did_one=0;
-
-    op->contr->freeze_inv=1;
-    op->contr->freeze_look=1;
-    if (!params) unlock_inv(op,find_first_inventory_item(op));
-    else {
-	if (op->type==PLAYER) show_what=op->contr->show_what;
-	for (tmp=op->inv; tmp; tmp=tmp->below) {
-	    if (!show_what_object(tmp,show_what)) continue;
-	    if (item_matched_string(op,tmp,params)) {
-		unlock_inv(op, tmp);
-		did_one=1;
-	    }
-	}
-	if (!did_one) new_draw_info(NDI_UNIQUE, 0,op,"Nothing to unlock.");
-    }
-    op->contr->freeze_inv=0;
-    op->contr->count=0;
-    draw_inventory(op);
-    return 0;
-}
-
