@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_input_c =
- *   "$Id: c_party.c,v 1.8 1997/02/04 10:44:29 master Exp $";
+ *   "$Id: c_party.c,v 1.11 1998/09/28 03:05:14 master Exp master $";
  */
 /*
     CrossFire, A Multiplayer game for X-windows
@@ -30,10 +30,8 @@
 #include <sproto.h>
 #endif
 #include <version.h>
-#include <main.h>
 #include <spells.h>
 
-#ifdef SIMPLE_PARTY_SYSTEM
 static partylist * firstparty=NULL; /* Keeps track of first party in list */
 static partylist * lastparty=NULL; /*Keeps track of last party in list */
 
@@ -52,6 +50,8 @@ partylist * form_party(object *op, char *params, partylist * firstparty, partyli
     newparty = (partylist *)malloc(sizeof(partylist));
     newparty->partyid = nextpartyid;
     newparty->partyname = strdup_local(params);
+    newparty->total_exp=0;
+    newparty->kills=0;
 
     newparty->passwd[0] = '\0';
     newparty->next = NULL;
@@ -73,14 +73,45 @@ char * find_party(int partynumber, partylist * party) {
     }
     return NULL;
 }
-#if 0
-void get_party_passwd(object *op) {
-    op->contr->write_buf[0]='\0';
-    op->contr->writing=0;
-    op->contr->state=ST_GET_PASSWORD;
-    draw_info(op,"What is your password?");
-    write_ch(op,':');
-    op->contr->no_echo=1;
+
+partylist *find_party_struct(int partynumber)
+{
+  partylist *party;
+
+  party=firstparty;
+  while(party!=NULL)
+    {
+      if(party->partyid==partynumber) return party;
+      else
+	party=party->next;
+    }
+  return NULL;
+}
+
+#ifdef PARTY_KILL_LOG
+void add_kill_to_party(int numb,char *killer,char *dead,long exp)
+{
+  partylist *party;
+  int i,pos;
+
+  party=find_party_struct(numb);
+  if(party==NULL) return;
+  if(party->kills>=PARTY_KILL_LOG)
+    {
+      pos=PARTY_KILL_LOG-1;
+      for(i=0;i<PARTY_KILL_LOG-1;i++)
+	memcpy(&(party->party_kills[i]),&(party->party_kills[i+1]),
+	       sizeof(party->party_kills[0]));
+    }
+  else
+    pos=party->kills;
+  party->kills++;
+  party->total_exp+=exp;
+  party->party_kills[pos].exp=exp;
+  strncpy(party->party_kills[pos].killer,killer,MAX_NAME);
+  strncpy(party->party_kills[pos].dead,dead,MAX_NAME);
+  party->party_kills[pos].killer[MAX_NAME]=0;
+  party->party_kills[pos].dead[MAX_NAME]=0;
 }
 #endif
 
@@ -102,16 +133,6 @@ int confirm_party_password(object *op) {
 
 void receive_party_password(object *op, char k) {
  
-  if(k!=13) {
-    if((k==8 || k==127)&&(op->contr->writing==1))
-      return;
-    if(k!=8 && k !=127 && (!((k>='a' && k<='z') || (k>='A' && k<='Z')) ||
-                           op->contr->writing > 8))
-      return;
-    write_ch(op,k);
-    return;
-  }
-  op->contr->writing = 0;
   if(confirm_party_password(op) == 0) {
     op->contr->party_number = op->contr->party_number_to_join;
     op->contr->party_number_to_join = (-1);
@@ -179,8 +200,59 @@ int command_party (object *op, char *params)
     new_draw_info(NDI_UNIQUE, 0,op,"There is an 8 character max");
     new_draw_info(NDI_UNIQUE, 0,op,"To talk to party members type: party say <msg>");
     new_draw_info(NDI_UNIQUE, 0,op,"To see who is in your party: party who");
+    new_draw_info(NDI_UNIQUE, 0,op,"To see what you've killed, type: party kills");
     return 1;
   }
+#ifdef PARTY_KILL_LOG
+  if(!strncmp(params, "kills",5))
+    {
+      int i,max;
+      char chr;
+      char buffer[80];
+      float exp;
+
+      if(op->contr->party_number<=0)
+	{
+	  new_draw_info(NDI_UNIQUE, 0,op,"You are not a member of any party.");
+	  return 1;
+	}
+      tmpparty = find_party_struct(op->contr->party_number);
+      if(!tmpparty->kills)
+	{
+	  new_draw_info(NDI_UNIQUE,0,op,"You haven't killed anything yet.");
+	  return 1;
+	}
+      max=tmpparty->kills-1;
+      if(max>PARTY_KILL_LOG-1) max=PARTY_KILL_LOG-1;
+      new_draw_info(NDI_UNIQUE,0,op,
+		    "Killed          |          Killer|     Exp"); 
+      new_draw_info(NDI_UNIQUE,0,op,
+		    "----------------+----------------+--------");
+      for(i=0;i<=max;i++)
+	{
+	  exp=tmpparty->party_kills[i].exp;
+	  chr=' ';
+	  if(exp>1000000) { exp/=1000000; chr='M'; }
+	  else
+	    if(exp>1000) { exp/=1000; chr='k'; }
+	  sprintf(buffer,"%16s|%16s|%6.1f%c",
+		  tmpparty->party_kills[i].dead,
+		  tmpparty->party_kills[i].killer,exp,chr);
+	  new_draw_info(NDI_UNIQUE,0,op,buffer);
+	}
+      exp=tmpparty->total_exp;
+      chr=' ';
+      if(exp>1000000) { exp/=1000000; chr='M'; }
+      else
+	if(exp>1000) { exp/=1000; chr='k'; }
+      new_draw_info(NDI_UNIQUE,0,op,
+		    "----------------+----------------+--------");
+      sprintf(buffer,"Totals: %d kills, %.1f%c exp",tmpparty->kills,
+	      exp,chr);
+      new_draw_info(NDI_UNIQUE,0,op,buffer);
+      return 1;
+    }
+#endif /* PARTY_KILL_LOG */
   if(strncmp(params, "say ", 4)==0)
     {
           if(op->contr->party_number<=0)
@@ -398,5 +470,4 @@ int command_party (object *op, char *params)
   return 1;
 }
 
-#endif /* SIMPLE_PARTY_SYSTEM */
 
