@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_c_wiz_c =
- *   "$Id: c_wiz.c,v 1.13 1996/09/30 03:48:38 master Exp $";
+ *   "$Id: c_wiz.c,v 1.18 1998/09/28 03:05:40 master Exp master $";
  */
 /*
     CrossFire, A Multiplayer game for X-windows
@@ -37,46 +37,130 @@
 #include <sproto.h>
 #endif
 #include <version.h>
-#include <main.h>
 #include <spells.h>
 #include <treasure.h>
+#include <skills.h>
+
+/* This finds and returns the object which matches the name or
+ * object nubmer (specified via num #whatever).
+ */
+
+static object *find_object_both(char *params)
+{
+    if (!params) return NULL;
+    if(params[0]=='#') return find_object(atol(params+1));
+    else return find_object_name(params);
+}
+
+/* Sets the god for some objects.  params should contain two values -
+ * first the object to change, followed by the god to change it to.
+ */
+int command_setgod(object *op, char *params)
+{
+    object *ob;
+    char    *str;
+
+    if (!params || !(str=strchr(params,' '))) {
+	new_draw_info(NDI_UNIQUE,0,op,"Usage: setgod object god");
+	return 0;
+    }
+    /* kill the space, and set string to the next param */
+    *str++='\0';
+    if(!(ob=find_object_both(params))) {
+    	new_draw_info_format(NDI_UNIQUE,0,op,"Set whose god - can not find object %s?",params);
+    	return 1;
+    }
+    /* Perhaps this is overly restrictive?  Should we perhaps be able
+     * to rebless altars and the like?
+     */
+    if (ob->type!=PLAYER) {
+    	new_draw_info_format(NDI_UNIQUE,0,op,"%s is not a player - can not change its god",ob->name);
+    	return 1;
+    }
+    change_skill(ob,SK_PRAYING);
+    if(!ob->chosen_skill||ob->chosen_skill->stats.sp!=SK_PRAYING) {
+    	new_draw_info_format(NDI_UNIQUE,0,op,"%s doesn't have praying skill.",ob->name);
+    	return 1;
+    }
+    if(find_god(str)==NULL) {
+    	new_draw_info_format(NDI_UNIQUE,0,op,"No such god %s.",str);
+    	return 1;
+    }
+    become_follower(ob,params);
+    return 1;
+}
+
+int command_kick (object *op, char *params)
+{
+  struct pl *pl;
+
+  for(pl=first_player;pl!=NULL;pl=pl->next) 
+    if((params==NULL || !strcmp(pl->ob->name,params)) && pl->ob!=op)
+      {
+	object *op;
+
+	op=pl->ob;
+	remove_ob(op);
+	op->direction=0;
+	op->contr->count_left=0;
+	new_draw_info_format(NDI_UNIQUE | NDI_ALL, 5, op,
+			     "%s is kicked out of the game.",op->name);
+	strcpy(op->contr->killer,"left");
+	check_score(op); /* Always check score */
+	(void)save_player(op,0);
+	play_again(op);
+	op->map->players--;
+#if MAP_MAXTIMEOUT
+	op->map->timeout = MAP_TIMEOUT(op->map);
+#endif
+      }
+  return 1;
+}
+
+int command_shutdown(object *op, char *params)
+{
+
+    if(op!=NULL && !QUERY_FLAG(op,FLAG_WIZ)) 
+    {
+	new_draw_info(NDI_UNIQUE,0,op,"Sorry, you can't shutdown the server.");
+	return 1;
+    }
+    command_kick(NULL,NULL);
+    while (first_player!=NULL)
+    {
+	leave(first_player);
+    }
+    cleanup();
+    /* not reached */
+    return 1;
+}
 
 int command_goto (object *op, char *params)
 {
-      char *name;
-      object *dummy;
+    char *name;
+    object *dummy;
 
-  if (!op)
-    return 0;
+    if (!op)
+	return 0;
 
-  if(params == NULL) {
+    if(params == NULL) {
         new_draw_info(NDI_UNIQUE, 0,op,"Go to what level?");
         return 1;
       }
-  name = params;
-      dummy=get_object();
-      dummy->map = op->map;
-      EXIT_PATH(dummy) = add_string (name);
-      enter_exit(op,dummy);
-      free_object(dummy);
-      if(op->contr->loading == NULL) {
+    name = params;
+    dummy=get_object();
+    dummy->map = op->map;
+    EXIT_PATH(dummy) = add_string (name);
+    dummy->name = add_string(name);
+
+    enter_exit(op,dummy);
+    free_object(dummy);
+    if(op->contr->loading == NULL) {
 	new_draw_info_format(NDI_UNIQUE, 0, op,
 	    "Difficulty: %d.",op->map->difficulty);
-        /*
-         * Now, if we entered a new, unloaded map, set the cheat-flag in
-         * all objects here.
-         */
-#ifdef SET_WIZCHEAT
-        if(op->map != prev && !m) {
-          object *tmp;
-          for(tmp = objects; tmp != NULL; tmp = tmp->next)
-            if(tmp->map == op->map)
-              SET_WAS_WIZ(tmp);
-        }
-#endif
-      }
-      return 1;
     }
+    return 1;
+}
 
 /* is this function called from somewhere ? -Tero */
 int command_generate (object *op, char *params)
@@ -94,7 +178,7 @@ int command_generate (object *op, char *params)
         while((tmp=generate_treasure(0,op->map->difficulty))==NULL && --retry);
         if (tmp != NULL) {
           tmp = insert_ob_in_ob(tmp, op);
-	  if (op->type == PLAYER && op->contr->eric_server > 0)
+	  if (op->type == PLAYER)
 	      esrv_send_item (op, tmp);
         }
       }
@@ -208,7 +292,9 @@ int command_create (object *op, char *params)
         if (set_nrof)
           tmp->nrof = nrof;
         tmp->map=op->map;
+#ifndef REAL_WIZ
 	SET_FLAG(tmp, FLAG_WAS_WIZ);
+#endif
         if (set_magic)
           set_abs_magic(tmp, magic);
         if (art)
@@ -218,17 +304,17 @@ int command_create (object *op, char *params)
 	  CLEAR_FLAG(tmp, FLAG_KNOWN_MAGICAL);
 	}
         tmp = insert_ob_in_ob(tmp,op);
-	if (op->contr->eric_server > 0)
-	    esrv_send_item(op, tmp);
+	esrv_send_item(op, tmp);
         return 1;
       } 
-      D_LOCK(op);
       for (i=0 ; i < (set_nrof ? nrof : 1); i++) {
         archetype *atmp;
         object *prev=NULL,*head=NULL;
         for (atmp=at;atmp!=NULL;atmp=atmp->more) {
           tmp=arch_to_object(atmp);
+#ifndef REAL_WIZ
 	  SET_FLAG(tmp, FLAG_WAS_WIZ);
+#endif
           if(head==NULL)
             head=tmp;
           tmp->x=op->x+tmp->arch->clone.x;
@@ -253,14 +339,7 @@ int command_create (object *op, char *params)
         if (at->clone.randomitems!=NULL)
           create_treasure(at->clone.randomitems,head,GT_INVENTORY,
                           op->map->difficulty,0);
-	if (op->contr->eric_server > 0) {
-	    esrv_send_item(op, head);
-	}
-      }
-      D_UNLOCK(op);
-      if (op->contr->eric_server <= 0) {
-	draw_all_inventory(op);
-	draw_all_look(op);
+	esrv_send_item(op, head);
       }
       return 1;
 }
@@ -343,9 +422,11 @@ int command_patch (object *op, char *params)
     }
     if((arg2=strchr(++arg,' ')))
       arg2++;
+#ifndef REAL_WIZ
     SET_FLAG(tmp, FLAG_WAS_WIZ); /* To avoid cheating */
+#endif
     if(set_variable(tmp,arg) == -1)
-      new_draw_info(NDI_UNIQUE, 0,op,errmsg);
+      new_draw_info_format(NDI_UNIQUE, 0,op,"Unknown variable %s", arg);
     else {
       new_draw_info_format(NDI_UNIQUE, 0, op,
 	"(%s#%d)->%s=%s",tmp->name,tmp->count,arg,arg2);
@@ -419,8 +500,9 @@ int command_addexp (object *op, char *params)
 #endif
     pl->ob->stats.exp += i;
     add_exp(pl->ob,0);
+#ifndef REAL_WIZ
     SET_FLAG(pl->ob, FLAG_WAS_WIZ);
-    draw_stats(pl->ob);
+#endif
     return 1;
   }
 
@@ -508,8 +590,10 @@ int command_abil (object *op, char *params)
       return 1;
     }
     for(pl=first_player;pl!=NULL;pl=pl->next) 
-       if(!strcmp(pl->ob->name,thing)){   
+       if(!strcmp(pl->ob->name,thing)){ 
+#ifndef REAL_WIZ  
 	  SET_FLAG(pl->ob, FLAG_WAS_WIZ);
+#endif
           if(!strcmp("str",thing2))
             pl->ob->stats.Str = iii,pl->orig_stats.Str = iii;
           if(!strcmp("dex",thing2))   
@@ -558,7 +642,7 @@ int command_reset (object *op, char *params)
 	    return 0;
 	}
 	for(pl=first_player;pl!=NULL;pl=pl->next)
-	    if(pl->ob == NULL || (!(QUERY_FLAG(pl->ob,FLAG_REMOVED)) && pl->ob->map == m))
+	    if(pl->ob == NULL || (!(QUERY_FLAG(pl->ob,FLAG_REMOVED)) && pl->ob->map == m)) {
 		if (QUERY_FLAG(pl->ob,FLAG_WIZ)) {
 		    /* does not work if there are several dms in same map */
 		    tmp = pl->ob;
@@ -570,6 +654,7 @@ int command_reset (object *op, char *params)
 		    remove_ob (tmp);
 		} else
 		    break;
+	    }
 	swap_map(m);
       }
 
@@ -599,23 +684,33 @@ int command_reset (object *op, char *params)
 
 int command_nowiz (object *op, char *params) /* 'noadm' is alias */
 {
-     if(op == NULL) {
-       active_socket->wiz = 0;
-       return 1;
-     }
      CLEAR_FLAG(op, FLAG_WIZ);
      CLEAR_FLAG(op, FLAG_WIZPASS);
      CLEAR_FLAG(op, FLAG_FLYING);
+#ifdef REAL_WIZ
+     CLEAR_FLAG(op, FLAG_WAS_WIZ);
+#endif
+     new_draw_info(NDI_UNIQUE | NDI_ALL, 1, NULL,
+	"The Dungeon Master is gone..");
      return 1;
   }
 
-
+/*
+ * object *op is trying to become dm.
+ * pl_name is name supplied by player.  Restrictive DM will make it harder
+ * for socket users to become DM - in that case, it will check for the players
+ * character name.
+ */
 static int checkdm(object *op, char *pl_name, char *pl_passwd, char *pl_host)
 {
   FILE  *dmfile;
   char  buf[MAX_BUF];
   char  line_buf[160], name[160], passwd[160], host[160];
 
+#ifdef RESTRICTIVE_DM
+  *pl_name=op->name?op->name:"*";
+#endif
+
   sprintf (buf, "%s/%s", LIBDIR, DMFILE);
   if ((dmfile = fopen(buf, "r")) == NULL) {
     LOG (llevDebug, "Could not find DM file.\n");
@@ -643,38 +738,10 @@ static int checkdm(object *op, char *pl_name, char *pl_passwd, char *pl_host)
 
 int command_dm (object *op, char *params)
 {
-  if (op==NULL) {
-    if (checkdm(op, active_socket->name, params?params:"*", active_socket->host)) {
-      active_socket->wiz=1;
-      new_draw_info(NDI_UNIQUE, 0,op, "Ok, you are the Dungeon Master!");
-      new_draw_info(NDI_UNIQUE | NDI_ALL, 1, NULL,
-	"The Dungeon Master has arrived!");
-    }
-    else {
-      new_draw_info(NDI_UNIQUE, 0,op, "Sorry Pal, I don't think so.");
-      LOG(llevDebug,"socket[%s]: DM mode attempted (%s %s)\n", 
-	  active_socket->name,params?params:"*", active_socket->host);
-    }
-    return 1;
-  }
-  /* If this isn't a player, quit out now */
-  else if (!op->contr) return 0;
+  if (!op->contr) return 0;
   else {
-    char *s1, player_host[160];
-    /* op->contr->name is actually the displayname */
-    if (op->contr->name) {
-	strcpy(player_host, op->contr->name);
-	/* Next line removes the :0 or :0.0 from the player hostname, if it
-	 * is there
-	 */
-	if ((s1=strchr(player_host, ':'))!=NULL) *s1='\0';
-    }
-    else strcpy(player_host,"*");
-    /* op->contr->username is the name supplied with the name command via
-     * the socket
-     */
-    if (checkdm(op, (op->contr->username?op->contr->username:"*"), 
-		(params?params:"*"), player_host)) {
+    if (checkdm(op, op->name,
+		(params?params:"*"), op->contr->socket.host)) {
       SET_FLAG(op, FLAG_WIZ);
       SET_FLAG(op, FLAG_WAS_WIZ);
       SET_FLAG(op, FLAG_WIZPASS);
@@ -683,12 +750,10 @@ int command_dm (object *op, char *params)
   	"The Dungeon Master has arrived!");
       SET_FLAG(op, FLAG_FLYING);
       clear_los(op);
-      op->contr->writing =0;
       op->contr->write_buf[0] ='\0';
       return 1;
     } else {
       new_draw_info(NDI_UNIQUE, 0,op, "Sorry Pal, I don't think so.");
-      op->contr->writing =0;
       op->contr->write_buf[0] ='\0';
       return 1;
     }
