@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_deamon_c =
- *   "$Id: daemon.c,v 1.10 1997/03/09 05:07:54 master Exp master $";
+ *   "$Id: daemon.c,v 1.12 1998/09/28 03:07:36 master Exp master $";
  */
 
 /*
@@ -41,7 +41,6 @@
 
 FILE *BecomeDaemon (char *filename)
 {
-#ifdef SERVER
   FILE *logfile;
   register int i;
   int forkresult;
@@ -87,34 +86,29 @@ FILE *BecomeDaemon (char *filename)
 	int zero = 0;
 	(void) ioctl (i, TIOCTTY, &zero);
 #else
-#ifndef TIOCNOTTY /********* !!!!!!!!! **********/
-#if defined(SVR4)
-#include <sys/termios.h>
-#else
-#include <sys/ttycom.h>
-#endif
-#endif
+
+#  ifdef HAVE_SYS_TERMIOS_H
+#    include <sys/termios.h>
+#  else
+#    ifdef HAVE_SYS_TTYCOM_H
+#      include <sys/ttycom.h>
+#    endif
+#  endif
 	(void) ioctl (i, TIOCNOTTY, (char *) 0);    /* detach, BSD style */
 #endif
 	(void) close (i);
     }
 
 
-#if defined(SYSV) || defined(SVR4)
-#if defined(__sun__) || defined(hpux) || defined(sgi) || defined(__osf__)
+#ifdef HAVE_SETSID
     setsid();
 #else
-    setpgrp (0, 0);
-#endif
-#else /* Non SYSV machines */
-#if defined(__osf__) || defined(linux)
-    setsid();
-#else
-    setpgrp (0, getpid());
-#endif
+/* Are these really different?  */
+#  if defined(SYSV) || defined(SVR4)
+      setpgrp (0, 0);
+#  else /* Non SYSV machines */
+      setpgrp (0, getpid());
+#  endif
 #endif
   return(logfile);
-#else
-  return (NULL);
-#endif
 }
