@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_apply_c =
- *   "$Id: gods.c,v 1.8 1997/01/05 10:05:57 master Exp $";
+ *   "$Id: gods.c,v 1.12 1998/09/28 03:09:56 master Exp master $";
  */
 /*
     CrossFire, A Multiplayer game for X-windows
@@ -116,13 +116,7 @@ void pray_at_altar(object *pl, object *altar) {
       * we lose experience from the clerical experience obj */
 
      loss = 0.1 * (float) pl->chosen_skill->exp_obj->stats.exp;
-     if (loss) loss = RANDOM()%(loss*angry);
-     if((loss = check_dm_add_exp_to_obj(pl->chosen_skill->exp_obj,loss))) { 
-       pl->chosen_skill->exp_obj->stats.exp -= loss;
-       pl->stats.exp -= loss;
-       add_exp(pl,0);
-       draw_stats(pl);
-     }
+     if(loss) lose_priest_exp(pl,RANDOM()%(loss*angry));
  
      /* May switch Gods, but its random chance based on our current level
       * note it gets harder to swap gods the higher we get */
@@ -227,7 +221,7 @@ void stop_using_item ( object *op, int type, int number ) {
 
   for(tmp=op->inv;tmp&&number;tmp=tmp->below)
     if(tmp->type==type&&QUERY_FLAG(tmp,FLAG_APPLIED)) { 
-	apply(op,tmp); /* this should unapply things properly */ 
+	apply(op,tmp,0); /* this should unapply things properly */ 
 	number--;
     }
 }
@@ -252,30 +246,41 @@ void update_priest_flag (object *god, object *exp_ob, uint32 flag) {
  */
 
 char *determine_god(object *op) {
-  int godnr = -1;
+    int godnr = -1;
 
-  /* spells */
-  if((op->type==FBULLET||op->type==CONE||op->type==FBALL)&&op->title)
-    if(lookup_god_by_name(op->title)>=0) return op->title;
-
-  if(op->type!= PLAYER && QUERY_FLAG(op,FLAG_ALIVE)) {
-    if(!op->title) {
-      godlink *gl=first_god;
-
-      godnr = (RANDOM()%gl->id) + 1;
-      while(gl) {
-	if(gl->id==godnr) break;
-	gl=gl->next;
-      }
-      op->title = add_string(gl->name);
+    /* spells */
+    if((op->type==FBULLET||op->type==CONE||op->type==FBALL)&&op->title) {
+	if(lookup_god_by_name(op->title)>=0) return op->title;
     }
-    return op->title;
-  } 
 
-  if((op->type==PLAYER && op->chosen_skill 
-      && op->chosen_skill->exp_obj && op->chosen_skill->exp_obj->title))
-  	     return  op->chosen_skill->exp_obj->title;
+    if(op->type!= PLAYER && QUERY_FLAG(op,FLAG_ALIVE)) {
+	if(!op->title) {
+	    godlink *gl=first_god;
 
+	    godnr = (RANDOM()%gl->id) + 1;
+	    while(gl) {
+		if(gl->id==godnr) break;
+		gl=gl->next;
+	    }
+	    op->title = add_string(gl->name);
+	}
+	return op->title;
+    }
+
+
+    /* If we are player, lets search a bit harder for the god.  This
+     * is a fix for perceive self (before, we just looked at the active
+     * skill.)
+     */
+    if(op->type==PLAYER) {
+	object *tmp;
+
+	for (tmp=op->inv; tmp!=NULL; tmp=tmp->below) {
+	    if (tmp->exp_obj && tmp->exp_obj->title) 
+		return tmp->exp_obj->title;
+	}
+    }
+	
   return ("none");
 } 
 
@@ -292,6 +297,10 @@ void god_intervention(object *op, object *god) {
   /*safety, shouldnt happen */
   if(!god) return;
 
+  /* lets do some checks of whether we are kosher 
+   * with our god */
+  if(god_examines_priest(op,god)<0) return;
+
   new_draw_info(NDI_UNIQUE|NDI_UNIQUE,0,op,"You feel a holy presence!");
 
   /* So, what can we do to help out? We do some minimal checking
@@ -336,16 +345,14 @@ void god_intervention(object *op, object *god) {
 
     for (tmp = op->inv; tmp; tmp = tmp->below)
       if (QUERY_FLAG(tmp, FLAG_CURSED)||QUERY_FLAG(tmp, FLAG_DAMNED)) {
-          success++;
-          if(QUERY_FLAG(tmp, FLAG_DAMNED)) CLEAR_FLAG(tmp, FLAG_DAMNED);
-          CLEAR_FLAG(tmp, FLAG_CURSED);
-          CLEAR_FLAG(tmp, FLAG_KNOWN_CURSED);
-          if (op->type == PLAYER && op->contr->eric_server > 0)
-                  esrv_send_item(op, tmp);
+	    success++;
+	    if(QUERY_FLAG(tmp, FLAG_DAMNED)) CLEAR_FLAG(tmp, FLAG_DAMNED);
+	    CLEAR_FLAG(tmp, FLAG_CURSED);
+	    CLEAR_FLAG(tmp, FLAG_KNOWN_CURSED);
+	    esrv_send_item(op, tmp);
        }  
     if (op->type==PLAYER&&success) {
 	new_draw_info(NDI_UNIQUE, 0,op, "You feel like someone is helping you.");
-	draw_inventory(op);
 	return;
     }
   }
@@ -409,8 +416,9 @@ void god_intervention(object *op, object *god) {
     for(weapon=op->inv;weapon;weapon=weapon->below) 
        if(weapon->type==WEAPON&&QUERY_FLAG(weapon,FLAG_APPLIED)) break;
 
-    /* allow the weapon to slay enemies */
-    if(weapon && !weapon->slaying&& god->slaying) {
+    if(weapon&&god_examines_item(god,weapon)>0) {
+      /* allow the weapon to slay enemies */
+      if(!weapon->slaying&& god->slaying) {
         char buf[MAX_BUF];
 
 	weapon->slaying = add_string(god->slaying);
@@ -421,25 +429,28 @@ void god_intervention(object *op, object *god) {
 	     weapon->name);
 	  sprintf(buf,"of %s",god->name);
 	  weapon->title=add_string(buf);
-          if(op->type==PLAYER) draw_inventory(op);
+          if(op->type==PLAYER) 
+	    esrv_update_item(UPD_NAME, op, weapon);
 	}
 	return;
-    }
+      }
  
     /* add the gods attacktype*/
-    if(weapon && !(RANDOM()%2)&&!(weapon->attacktype&god->attacktype)) {
+      if(!(RANDOM()%2)&&!(weapon->attacktype&god->attacktype)) {
         new_draw_info(NDI_UNIQUE,0,op,"Your weapon suddenly glows!");
 	weapon->attacktype=weapon->attacktype|god->attacktype;
 	return;
-    }
+      }
 
     /* higher magic value */
-    if(weapon && !(RANDOM()%2)&&weapon->magic<(level/5)) {
-       new_draw_info(NDI_UNIQUE,0,op,
+      if(!(RANDOM()%2)&&weapon->magic<(level/5)) {
+        new_draw_info(NDI_UNIQUE,0,op,
           "A phosphorescent glow envelops your weapon!");
-       weapon->magic++;
-       if(op->type==PLAYER) draw_inventory(op);
-       return;
+        weapon->magic++;
+        if(op->type==PLAYER) 
+	    esrv_update_item(UPD_NAME, op, weapon);
+        return;
+      }
     }
   }
 
@@ -487,9 +498,7 @@ void god_intervention(object *op, object *god) {
 	&& spells[spell].level <= level
 	&& !check_spell_known(op,spell)) { 
 
-#ifdef SOUND_EFFECTS 
-    	play_sound_player_only(op->contr, SOUND_LEARN_SPELL); 
-#endif   
+    	play_sound_player_only(op->contr, SOUND_LEARN_SPELL,0,0); 
     	new_draw_info_format(NDI_UNIQUE, 0,op,
 		"%s grants you use of a special prayer!",god->name); 
     	op->contr->known_spells[op->contr->nrofknownspells++]=spell; 
@@ -508,6 +517,63 @@ void god_intervention(object *op, object *god) {
 
 }
 
+int god_examines_priest (object *op, object *god) {
+  int reaction=1;
+  object *weapon=NULL;
+
+  for(weapon=op->inv;weapon;weapon=weapon->below)
+     if(weapon->type==WEAPON&&QUERY_FLAG(weapon,FLAG_APPLIED)) break;
+
+  if(weapon) 
+    reaction=god_examines_item(god,weapon)*(weapon->magic?abs(weapon->magic):1);
+
+  /* well, well. Looks like we screwed up. Time for god's revenge */
+  if(reaction<0) { 
+    char buf[MAX_BUF];
+    int loss = 10000000;
+    int angry = abs(reaction);
+    if(op->chosen_skill->exp_obj)
+      loss = 0.05 * (float) op->chosen_skill->exp_obj->stats.exp;
+    lose_priest_exp(op,RANDOM()%(loss*angry));
+    if(RANDOM()%(angry+1)) 
+      cast_mana_storm(op,SK_level(op)+(angry*3));
+    sprintf(buf,"%s becomes angry and punishes you!",god->name);
+    new_draw_info(NDI_UNIQUE|NDI_NAVY,0,op,buf); 
+  }
+
+  return reaction;
+}
+
+/* god_likes_item() - your god looks at the item you
+ * are using and we return either -1 (bad), 0 (neutral) or
+ * 1 (item is ok). If you are using the item of an enemy
+ * god, it can be bad...-b.t. */
+
+int god_examines_item(object *god, object *item) {
+  char buf[MAX_BUF];
+
+  if(!god||!item) return 0;
+
+  if(!item->title) return 1; /* unclaimed item are ok */
+
+  sprintf(buf,"of %s",god->name);
+  if(!strcmp(item->title,buf)) return 1; /* belongs to that God */ 
+
+  if(god->title) { /* check if we have any enemy blessed item*/ 
+    sprintf(buf,"of %s",god->title);
+    if(!strcmp(item->title,buf)) {
+      if(item->env) {
+        char buf[MAX_BUF];
+        sprintf(buf,"Heretic! You are using %s!",query_name(item));
+        new_draw_info(NDI_UNIQUE|NDI_NAVY,0,item->env,buf);
+      }
+      return -1;
+    }
+  }
+
+  return 0; /* item is sacred to a non-enemy god/or is otherwise magical */ 
+}
+
 /* get_god() - returns the gods index in linked list 
  * if exists, if not, it returns -1. -b.t.  */
 
@@ -568,3 +634,17 @@ int tailor_god_spell(object *spellop, object *caster) {
     return 1;
 }
 
+void lose_priest_exp(object *pl, int loss) {
+
+  if(!pl||pl->type!=PLAYER||!pl->chosen_skill
+     ||!pl->chosen_skill->exp_obj) 
+  {
+    LOG(llevError,"Bad call to lose_priest_exp() \n");
+    return;
+  }
+  if((loss = check_dm_add_exp_to_obj(pl->chosen_skill->exp_obj,loss))) {
+    pl->chosen_skill->exp_obj->stats.exp -= loss;
+    pl->stats.exp -= loss;
+    add_exp(pl,0);
+  }
+}
