@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_init_c =
- *   "$Id: init.c,v 1.48 1997/03/09 05:10:03 master Exp master $";
+ *   "$Id: init.c,v 1.53 1998/09/28 03:19:27 master Exp master $";
  */
 
 /*
@@ -28,27 +28,17 @@
 
 #include <global.h>
 #include <loader.h>
-#include <main.h>
 #ifndef __CEXTRACT__
 #include <sproto.h>
 #endif
 #include <version.h>
-#include <X11/keysym.h>
 
-void set_chrfont(char *val) {    settings.chrfont=val; }
 void set_logfile(char *val) { settings.logfilename=val; }
-void set_servermode() { settings.servermode=SERVER_ENABLED; }
-void set_fix_fontpath() { settings.fixfontpath=0; }
-void set_pixmaps() { settings.displaymode=Dm_Bitmap; }
-void set_synchronize() { settings.synchronize=0; }
 void call_version() { version(NULL); exit(0); }
 void showscores() { display_high_score(NULL,9999,NULL); exit(0); }
-void set_blackwhite() { settings.blackwhite=1; }
-void set_initflag() { settings.noadd=1; }
 void set_debug() { settings.debug=llevDebug; }
 void unset_debug() { settings.debug=llevError; }
 void set_mondebug() { settings.debug=llevMonster; }
-void set_splitwindow() { settings.splitwindows=1; }
 void set_dumpmon1() {settings.dumpvalues=1; }
 void set_dumpmon2() {settings.dumpvalues=2; }
 void set_dumpmon3() {settings.dumpvalues=3; }
@@ -57,10 +47,10 @@ void set_dumpmon5() {settings.dumpvalues=5; }
 void set_dumpmon6() {settings.dumpvalues=6; }
 void set_dumpmon7() {settings.dumpvalues=7; }
 void set_dumpmon8() {settings.dumpvalues=8; }
+void set_dumpmon9() {settings.dumpvalues=9; }
+void set_dumpmont(char *name) {settings.dumpvalues=10; settings.dumparg=name; }
 void set_daemon() {settings.daemonmode=1; }
-void set_xpm() { settings.displaymode=Dm_Pixmap; }
 void set_libdir(char *path) { settings.libdir=path; }
-void set_fontdir(char *path) { settings.fontdir=path; }
 void set_mapdir(char *path) { settings.mapdir=path; }
 void set_archetypes(char *path) { settings.archetypes=path; }
 void set_treasures(char *path) { settings.treasures=path; }
@@ -73,16 +63,6 @@ void showscoresparm(char *data) {
     exit(0); 
 }
 
-void set_port(char *val)
-{ 
-    settings.listen_port=atoi(val);
-    if (settings.listen_port<=0 || settings.listen_port>32765 ||
-	(settings.listen_port<1024 && getuid()!=0)) {
-	fprintf(stderr,"%d is an invalid port number.\n",settings.listen_port);
-	exit(1);
-    }
-}
-
 void set_csport(char *val)
 { 
     settings.csport=atoi(val);
@@ -93,20 +73,6 @@ void set_csport(char *val)
     }
 }
 
-void newplayer(char *val)
-{
-    char *di = (char *) getenv("DISPLAY");
-
-    /* If the default display matches one passed via the -display, then
-     * we don't want to add to the default display.
-     */
-    if (di && !strcmp(di, val))
-	settings.noadd=1;
-    if (add_player(val, NULL, first_map, 0))
-        LOG(llevDebug,"%s\n", errmsg);
-}
-
-
 /* Most of this is shamelessly stolen from XSysStats.  But since that is
  * also my program, no problem.
  */
@@ -138,7 +104,6 @@ struct Command_Line_Options options[] = {
 {"-mon", 0, 1, set_mondebug},
 #ifndef SECURE
 {"-lib",1,1, set_libdir},
-{"-fontdir", 1, 1, set_fontdir},
 {"-maps", 1, 1, set_mapdir},
 {"-arch", 1, 1, set_archetypes},
 {"-playerdir", 1, 1, set_playerdir},
@@ -150,28 +115,9 @@ struct Command_Line_Options options[] = {
 /* Pass 2 functions.  Most of these could probably be in pass 1, 
  * as they don't require much of anything to bet set up.
  */
-#ifdef CHRFONT
-{"-chrfont", 1, 2, set_chrfont},
-#endif
 {"-log", 1, 2, set_logfile},
-#ifdef SERVER
-{"-listen_port", 1, 2, set_port},
-#ifdef ERIC_SERVER
 {"-csport", 1, 2, set_csport},
-#endif
-{"-server", 0, 2, set_servermode},
-#endif
-{"-p", 0, 2, set_fix_fontpath},
-{"-pix", 0, 2, set_pixmaps},
-{"-f", 0, 2, set_synchronize},
-{"-l", 0, 2, set_initflag},
-{"-b", 0, 2, set_blackwhite},
-{"-w", 0, 2, set_splitwindow},
-{"-split", 0, 2, set_splitwindow},
 {"-detach", 0, 2, set_daemon},
-#if Xpm_Pix
-{"-xpm", 0, 2, set_xpm},
-#endif
 /* Start of pass 3 information. In theory, by pass 3, all data paths
  * and defaults should have been set up. 
  */
@@ -185,8 +131,9 @@ struct Command_Line_Options options[] = {
 {"-m6", 0, 3, set_dumpmon6},
 {"-m7", 0, 3, set_dumpmon7},
 {"-m8", 0, 3, set_dumpmon8},
+{"-m9", 0, 3, set_dumpmon9},
+{"-mt", 1, 3, set_dumpmont},
 #endif
-{"-display", 1, 3, newplayer},
 {"-s", 0, 3, showscores},
 {"-score", 1, 3, showscoresparm},
 };
@@ -242,7 +189,8 @@ static void parse_args(int argc, char *argv[], int pass)
  */
 
 void init(int argc, char **argv) {
-    char *di=NULL;
+
+    (void) umask(0);	/* We don't want to be affected by players' umask */
 
     init_done=0;		/* Must be done before init_signal() */
     logfile=stderr;
@@ -260,35 +208,21 @@ void init(int argc, char **argv) {
     fprintf(logfile,"Maintained locally by: %s\n",DM_MAIL);
     fprintf(logfile,"Questions and bugs should be mailed to above address.\n");
 #endif
-    (void) umask(0);	/* We don't want to be affected by players' umask */
     SRANDOM(time(NULL));
 
     init_startup();	/* Write (C), check shutdown/forbid files */
     init_signals();	/* Sets up signal interceptions */
     setup_library();	/* Set up callback function pointers */
-    setuperrors();	/* Sets up X11 error-handlers */
     init_commands();	/* Sort command tables */
     read_map_log();	/* Load up the old temp map files */
 
-#ifdef SOUND_EFFECTS
-    setup_sounds();	/* Sets up sound effects */
-#endif
-
     parse_args(argc, argv, 3);
 
-    di = (char *) getenv("DISPLAY");
-    if((!settings.noadd)&&di!=NULL&& settings.servermode != SERVER_ENABLED) {
-	if(add_player(di,NULL,first_map,0))
-	    LOG(llevError,"%s\n",errmsg);
-    }
-    if(first_player==NULL && settings.servermode != SERVER_ENABLED) {
-	usage();
-	exit(1);
-    }
-    if (settings.servermode == SERVER_ENABLED && settings.daemonmode)
+    if (settings.daemonmode)
 	logfile = BecomeDaemon(settings.logfilename);
+
+    init_beforeplay();
     init_ericserver();
-    init_socket();
     reset_sleep();
     init_done=1;
 }
@@ -302,11 +236,6 @@ void help() {
     usage();
     printf("Flags:\n");
     printf(" -b          Black/white instead of colors.\n");
-#ifdef CHRFONT
-/* Eneq(@csd.uu.se): Added -chrfont <fontname> */
-    printf(" -chrfont    Followed by fontname; will replace character\n");
-    printf("             rep. by that font.\n");
-#endif
     printf(" -d          Turns on some debugging.\n");
     printf(" -display <display> Open crossfire on the specified display.  Multiple\n");
     printf("             displays can be given.\n");
@@ -322,7 +251,9 @@ void help() {
     printf(" -m5         Dumps out skill information.\n");
     printf(" -m6         Dumps out race information.\n");
     printf(" -m7         Dumps out alchemy information.\n");
-    printf(" -m8         Dumps out more alchemy information.\n");
+    printf(" -m8         Dumps out gods information.\n");
+    printf(" -m9         Dumps out more alchemy information (formula checking).\n");
+    printf(" -mt <name>  Dumps out list of treasures for a monster.\n");
 #endif
     printf(" -mon        Turns on monster debugging.\n");
     printf(" -o          Prints out info on what was defined at compile time.\n");
@@ -332,20 +263,12 @@ void help() {
     printf(" -score <name or class> Displays all high scores with matching name/class.\n");
     printf(" -split      Makes split-windows default.\n");
     printf(" -v          Print version and contributors.\n");
-#ifdef Xpm_Pix
-    printf(" -xpm        Use color pixmaps (XPM) instead of bitmaps or fonts.\n");
-#endif
-#ifdef SERVER
-#ifdef ERIC_SERVER
     printf(" -csport <port> Specifies the port to use for the new client/server code.\n");
-#endif
     printf(" -detach     The server will go in the background, closing all\n");
     printf("             connections to the tty.\n");
-    printf(" -listen_port <port>  Specifies server listen port number\n");
     printf(" -log <file> Specifies which file to send output to.\n");
-    printf("             Only has meaning if -detatch is specified.\n");
+    printf("             Only has meaning if -detach is specified.\n");
     printf(" -server     Puts crossfire into server mode.\n");
-#endif
     exit(0);
 }
 
@@ -387,10 +310,13 @@ void init_beforeplay() {
     dump_alchemy();
     exit(0);
   case 8:
-    dump_prod_val_vs_cost();
+    dump_gods();
     exit(0);
   case 9:
-    dump_gods();
+    dump_alchemy_costs();
+    exit(0);
+  case 10:
+    dump_monster_treasure(settings.dumparg);
     exit(0);
   }
 #endif
@@ -452,11 +378,6 @@ void compile_info() {
   printf("Secure:\t\t<true>\n");
 #else
   printf("Secure:\t\t<false>\n");
-#endif
-#ifdef NO_LOG
-  printf("Logging:\t<false>\n");
-#else
-  printf("Logging:\t<true>\n");
 #endif
   printf("Libdir:\t\t%s\n",settings.libdir);
 #ifdef PERM_FILE
@@ -467,36 +388,14 @@ void compile_info() {
 #endif
   printf("Save player:\t<true>\n");
   printf("Save mode:\t%4.4o\n",SAVE_MODE);
-#ifdef SAVE_HOMEDIR
-  printf("Playerdir:\t%s/%s\n",(char *) getenv("HOME"),PlayerDir);
-  printf("Save homedir:\t<true>\n");
-#else
   printf("Playerdir:\t<LIB>/%s\n",settings.playerdir);
-  printf("Save homedir:\t<false>\n");
-#endif
-#ifdef LOCK_PLAYER
-  printf("Lock player:\t<true>\n");
-#else
-  printf("Lock player:\t<false>\n");
-#endif
-#ifdef UNIQUE_ITEMS
-  printf("Unique items:\t<true>\n");
   printf("Itemsdir:\t<LIB>/%s\n", settings.uniquedir);
-#ifdef LOCK_ITEMS
-  printf("Lock items:\t<true>\n");
-#else
-  printf("Lock items:\t<false>\n");
-#endif
-#else
-  printf("Unique items:\t<false>\n");
-#endif
 #ifdef USE_CHECKSUM
   printf("Use checksum:\t<true>\n");
 #else
   printf("Use checksum:\t<false>\n");
 #endif
   printf("Tmpdir:\t\t%s\n",settings.tmpdir);
-  printf("Fontdir:\t%s\n",settings.fontdir);
   printf("Compress:\t%s\n",COMPRESS);
   printf("Uncompress:\t%s\n",UNCOMPRESS);
   printf("Map max timeout:\t%d\n",MAP_MAXTIMEOUT);
@@ -511,34 +410,20 @@ void compile_info() {
 #else
   printf("Use_calloc:\t<false>\n");
 #endif
-#ifdef CHRFONT
-  printf("CHRFONT:\t<true>\n");
-#else
-  printf("CHRFONT:\t<false>\n");
-#endif
 
 #ifdef USE_SWAP_STATS
   printf("Use_swap_stats:\t<true>\n");
 #else
   printf("Use_swap_stats:\t<false>\n");
 #endif
-#ifdef SOUND_EFFECTS
-  printf("Sound_effects:\t<true>\n");
-#else
-  printf("Sound_effects:\t<false>\n");
-#endif
+
 #ifdef DM_MAIL
   printf("DM mail:\t%s\n",DM_MAIL);
 #endif
+
 #ifdef X_EDITOR
   printf("Editor:\t\t%s\n",X_EDITOR);
 #endif
-#ifdef SERVER
-  printf("Server:\t\t<true>\n");
-  printf("Port:\t\t%d\n",PORT);
-#else
-  printf("Server:\t\t<false>\n");
-#endif
 #ifdef EXPLORE_MODE
   printf("Explore mode:\t<true>\n");
 #else
@@ -553,11 +438,6 @@ void compile_info() {
   printf("Random encounter:\t<true>\n");
 #else
   printf("Random encounter:\t<false>\n");
-#endif
-#ifdef NEW_IMPROVE_WEAPON
-  printf("New improve weapon:\t<true>\n");
-#else
-  printf("New improve weapon:\t<false>\n");
 #endif
   printf("Max_time:\t%d\n",MAX_TIME);
   execl("/bin/uname", "uname", "-a", NULL);
@@ -622,8 +502,6 @@ void fatal_signal(int make_core, int close_sockets) {
   if(init_done) {
     emergency_save(0);
     clean_tmp_files();
-    if(close_sockets)
-      close_all_sockets();
   }
   if(make_core)
     abort();
@@ -654,21 +532,19 @@ void setup_library() {
   set_process_active_maps(process_active_maps);
   set_update_buttons(update_buttons);
   set_draw_info(new_draw_info);
-  set_draw_stats(draw_stats);
-  set_draw_inventory(draw_inventory);
   set_draw_look(draw_look);
   set_apply(apply);
   set_draw(draw);
   set_monster_check_apply(monster_check_apply);
-  set_draw_inventory_faces(draw_inventory_faces);
-  set_draw_look_faces(draw_look_faces);
   set_move_teleporter(move_teleporter);
   set_move_creator(move_creator);
   set_trap_adjust(trap_adjust);
   set_esrv_send_item(esrv_send_item);
   set_esrv_del_item(esrv_del_item);
+  set_esrv_update_item(esrv_update_item);
 /*  set_init_blocksview_players(init_blocksview_players); */
   set_info_map(new_info_map);
+
 }
 
 /* init_races() - reads the races file in the lib/ directory, then
