@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_input_c =
- *   "$Id: input.c,v 1.60 1997/03/09 05:11:10 master Exp master $";
+ *   "$Id: input.c,v 1.66 1998/09/28 03:22:27 master Exp master $";
  */
 /*
     CrossFire, A Multiplayer game for X-windows
@@ -30,14 +30,10 @@
 #include <sproto.h>
 #endif
 #include <version.h>
-#include <main.h>
 #include <spells.h>
 #include <skills.h>
 #include <newclient.h>
-
-#ifdef SET_TITLE
-extern void redraw_title(object *pl);
-#endif /* SET_TITLE */
+#include <commands.h>
 
 int onoff_value(char *line)
 {
@@ -77,142 +73,6 @@ object **find_objects(object *op, char *line, int flags)
   return NULL;
 }
 
-/*
- * Check if an item op can be put into a sack. If pl exists then tell
- * a player the reason of failure.
- * returns 1 if it will fit, 0 if it will not.  nrof is the number of
- * objects (op) we want to put in.  We specify it separately instead of
- * using op->nrof because often times, a player may have specified a
- * certain number of objects to drop, so we can pass that number, and
- * not need to use split_ob and stuff.
- */
-int sack_can_hold (object *pl, object *sack, object *op, int nrof) {
-    char buf[MAX_BUF];
-    buf[0] = 0;
-
-    if (! QUERY_FLAG (sack, FLAG_APPLIED))
-	sprintf (buf, "%s is not active.", query_name(sack));
-    if (sack == op)
-	sprintf (buf, "You can't put %s into itself.", query_name(sack));
-    if (sack->race && (sack->race != op->race || op->type == CONTAINER
-		       || (sack->stats.food && sack->stats.food != op->type)))
-	sprintf (buf, "You can put only %s into %s.", sack->race,
-		 query_name(sack));
-    if (op->type == SPECIAL_KEY && sack->slaying && op->slaying)
-	sprintf (buf, "You don't want put the key into %s.", query_name(sack));
-    if (sack->weight_limit && sack->carrying + (nrof ? nrof : 1) * 
-	op->weight * (100 - sack->stats.Str) / 100  > sack->weight_limit)
-	sprintf (buf, "That won't fit in %s!", query_name(sack));
-    if (buf[0]) {
-	if (pl)
-	    new_draw_info(NDI_UNIQUE, 0,pl, buf);
-	return 0;
-    }
-    return 1;
-}
-
-
-void lock_inv(object *op, object *item) {
-  SET_FLAG(item,FLAG_INV_LOCKED);
-  new_draw_info(NDI_UNIQUE, 0,op,"Item has been locked");
-  draw_inventory(op);
-  return;
-}
-
-void unlock_inv(object *op, object *item) {
-  CLEAR_FLAG(item,FLAG_INV_LOCKED);
-  new_draw_info(NDI_UNIQUE, 0,op,"Item has been unlocked");
-  draw_inventory(op);
-  return;
-}
-
-
-/* Eneq(@csd.uu.se): Had to remove rotate_right and replace it with a version
-   which doesn't use insert_ob_in_ob since I redid that to add things last in
-   the inventory, or next to objects of it's own type. This one now works just
-   like rotate_left but reversed :-) 
-   [ And now handles invisible objects. Tero.Haatanen@lut.fi ] */
-
-void rotate_right(object *op) {
-  object *tmp;
-  char buf[MAX_BUF];
-  int items=0;
-
-  for (tmp=op->inv;tmp;tmp=tmp->below)
-    if (!tmp->invisible)
-      items++;
-
-  if(items==0) {
-    new_draw_info(NDI_UNIQUE, 0,op,"You don't carry anything.");
-    return;
-  }
-  if(items==1) {
-    new_draw_info(NDI_UNIQUE, 0,op,"You only carry one thing.");
-    return;
-  }
-  op->contr->last_used=NULL;
-  op->contr->last_used_id=0;
-  for (tmp=op->inv; tmp && tmp->below; tmp=tmp->below);
-  while(tmp!=NULL&&tmp->invisible)
-    tmp=tmp->above;
-  /* tmp points now the last visible item in the inventory */
-  tmp->above->below=tmp->below;
-  if (tmp->below)
-    tmp->below->above=tmp->above;
-  tmp->above=NULL;
-  tmp->below=op->inv;
-  op->inv->above=tmp;
-  op->inv=tmp;
-
-  if(QUERY_FLAG(op, FLAG_WIZ))
-    sprintf(buf,"The %s (%d) is now on top.",
-            query_name(op->inv),op->inv->count);
-  else
-    sprintf(buf,"The %s is now on top.",query_name(op->inv));
-  new_draw_info(NDI_UNIQUE, 0,op,buf);
-  draw_inventory(op);
-}
-
-/*
- * [ Now rotate_left handles invisible objects too. Tero.Haatanen@lut.fi ]
- */
-
-void rotate_left(object *op) {
-  object *tmp,*tmp2;
-  char buf[MAX_BUF];
-  int items=0;
-
-  for (tmp=op->inv;tmp;tmp=tmp->below)
-    if (!tmp->invisible)
-      items++;
-
-  if(items==0) {
-    new_draw_info(NDI_UNIQUE, 0,op,"You don't carry anything.");
-    return;
-  }
-  if(items==1) {
-    new_draw_info(NDI_UNIQUE, 0,op,"You only carry one thing.");
-    return;
-  }
-  op->contr->last_used=NULL;
-  op->contr->last_used_id=0;
-  for (tmp=op->inv; tmp && tmp->below; tmp=tmp->below);
-  for (tmp2=op->inv->below; tmp2 && tmp2->invisible; tmp2=tmp2->below);
-
-  tmp2->above->below=NULL;
-  tmp2->above=NULL;
-  tmp->below=op->inv;
-  op->inv->above=tmp;
-  op->inv=tmp2;
-
-  if(QUERY_FLAG(op, FLAG_WIZ))
-    sprintf(buf,"The %s (%d) is now on top.",
-            query_name(op->inv),op->inv->count);
-  else
-    sprintf(buf,"The %s is now on top.",query_name(op->inv));
-  new_draw_info(NDI_UNIQUE, 0,op,buf);
-  draw_inventory(op);
-}
 
 void drop_inventory(object *op) {
   object *inv,*n=NULL;
@@ -327,26 +187,27 @@ void examine(object *op, object *tmp) {
   buf[0]='\0';
   switch(tmp->type) {
   case SPELLBOOK:
-    if(QUERY_FLAG(tmp, FLAG_IDENTIFIED) && tmp->stats.sp > 0 && tmp->stats.sp <= NROFREALSPELLS )
+    if(QUERY_FLAG(tmp, FLAG_IDENTIFIED) && tmp->stats.sp > 0 && tmp->stats.sp <= NROFREALSPELLS ) {
       if(!strcmp(tmp->arch->name,"cleric_book"))
         sprintf(buf,"%s is a %d level prayer.",
               spells[tmp->stats.sp].name,spells[tmp->stats.sp].level);
       else
         sprintf(buf,"%s is a %d level spell.",
               spells[tmp->stats.sp].name,spells[tmp->stats.sp].level);
+    }
     break;
   case BOOK:
     if(tmp->msg!=NULL)
       strcpy(buf,"Something is written in it.");
     break;
   case CONTAINER:
-    if(tmp->race!=NULL)
+    if(tmp->race!=NULL) {
       if(tmp->weight_limit && tmp->stats.Str<100)
         sprintf (buf,"It can hold only %s and its weight limit is %.1f kg.", 
 		 tmp->race, tmp->weight_limit/(10.0 * (100 - tmp->stats.Str)));
       else
         sprintf (buf,"It can hold only %s.", tmp->race);
-    else
+    } else
       if(tmp->weight_limit && tmp->stats.Str<100)
         sprintf (buf,"Its weight limit is %.1f kg.", 
 		tmp->weight_limit/(10.0 * (100 - tmp->stats.Str)));
@@ -480,6 +341,7 @@ void look_at(object *op,int dx,int dy) {
   char buf[MAX_BUF];
   int flag=0;
 
+  if (out_of_map(op->map, op->x+dx, op->y+dy)) return;
   if(op->above!=NULL) {
     remove_ob(op);
     insert_ob_in_map(op,op->map);
@@ -513,26 +375,20 @@ void look_at(object *op,int dx,int dy) {
       break;
     tmp=tmp->below;
   }
-  if(!flag)
+  if(!flag) {
     if(dx||dy)
       new_draw_info(NDI_UNIQUE, 0,op,"You see nothing there.");
     else
       new_draw_info(NDI_UNIQUE, 0,op,"You see nothing.");
+  }
 }
 
 
 
 void receive_player_name(object *op,char k) {
-  if(k!=13) {
-    if((k==8||k==127)&&(op->contr->writing==1))
-      return;
-    if(k!=8&&k!=127&&!((k>='a'&&k<='z')||(k>='A'&&k<='Z'))&&k!='-'&&k!='_')
-      return;
-    write_ch(op,k);
-    return;
-  }
-  if(op->contr->writing<=1) {
-    if (op->contr->eric_server>0) get_name(op);
+
+  if(strlen(op->contr->write_buf)<=1) {
+    get_name(op);
     return;
   }
   if(!check_name(op->contr,op->contr->write_buf+1)) {
@@ -544,31 +400,22 @@ void receive_player_name(object *op,char k) {
   op->name=add_string(op->contr->write_buf+1);
   new_draw_info(NDI_UNIQUE, 0,op,op->contr->write_buf);
   op->contr->last_value= -1; /* Flag: redraw all stats */
-  draw_stats(op);
   op->contr->name_changed=1;
   get_password(op);
 }
 
 void receive_player_password(object *op,char k) {
-  if(k!=13) {
-    if(k!=8&&k!=127&&(!((k>='a'&&k<='z')||(k>='A'&&k<='Z'))||
-       op->contr->writing>8)) /* Max 8 characters in password */
-      return;
-    write_ch(op,k);
-    return;
-  }
-  op->contr->no_echo=0;
-  if(op->contr->writing<=1) {
-    remove_lock(op->contr);
+
+  if(strlen(op->contr->write_buf)<=1) {
+    unlock_player(op->name);
     get_name(op);
     return;
   }
-  op->contr->writing=0;
   new_draw_info(NDI_UNIQUE, 0,op,"          "); /* To hide the password better */
   if(op->contr->state==ST_CONFIRM_PASSWORD) {
     if(!check_password(op->contr->write_buf+1,op->contr->password)) {
       new_draw_info(NDI_UNIQUE, 0,op,"The passwords did not match.");
-      remove_lock(op->contr);
+      unlock_player(op->name);
       get_name(op);
       return;
     }
@@ -635,49 +482,46 @@ int explore_mode() {
 
 static void help_topics(object *op, int what)
 {
-  XDIR *dirp;
-  struct xdirect *de;
-  char filename[MAX_BUF], line[80];
-  int namelen, linelen=0;
+    DIR *dirp;
+    struct dirent *de;
+    char filename[MAX_BUF], line[80];
+    int namelen, linelen=0;
   
-  switch (what) {
-  case 1:
-    sprintf(filename, "%s/wizhelp", settings.libdir);
-    new_draw_info(NDI_UNIQUE, 0,op, "      Wiz commands:");
-    break;
-  case 2:
-    sprintf(filename, "%s/sockethelp", settings.libdir);
-    new_draw_info(NDI_UNIQUE, 0,op, "      Socket commands:");
-    break;
-  case 3:
-    sprintf(filename, "%s/mischelp", settings.libdir);
-    new_draw_info(NDI_UNIQUE, 0,op, "      Misc help:");
-    break;
-  default:
-    sprintf(filename, "%s/help", settings.libdir);
-    new_draw_info(NDI_UNIQUE, 0,op, "      Commands:");
-    break;
-}
-  if (!(dirp=xopendir(filename)))
-    return;
-  line[0] ='\0';
-  for (de = xreaddir(dirp, 1); de; de = xreaddir(dirp, 1)) {
-    namelen = de->d_namlen;
-    if (namelen <= 2 && *de->d_name == '.' &&
-	(namelen == 1 || de->d_name[1] == '.' ) )
-      continue;
-    linelen +=namelen+1;
-    if (linelen > 42) {
-      new_draw_info(NDI_UNIQUE, 0,op, line);
-      sprintf(line, " %s", de->d_name);
-      linelen =namelen+1;
-      continue;
+    switch (what) {
+	case 1:
+	    sprintf(filename, "%s/wizhelp", settings.libdir);
+	    new_draw_info(NDI_UNIQUE, 0,op, "      Wiz commands:");
+	    break;
+	case 3:
+	    sprintf(filename, "%s/mischelp", settings.libdir);
+	    new_draw_info(NDI_UNIQUE, 0,op, "      Misc help:");
+	    break;
+	default:
+	    sprintf(filename, "%s/help", settings.libdir);
+	    new_draw_info(NDI_UNIQUE, 0,op, "      Commands:");
+	    break;
     }
-    strcat(line, " ");
-    strcat(line, de->d_name);
-  }	       
-  new_draw_info(NDI_UNIQUE, 0,op, line);
-  xclosedir(dirp);
+    if (!(dirp=opendir(filename)))
+	return;
+
+    line[0] ='\0';
+    for (de = readdir(dirp); de; de = readdir(dirp)) {
+	namelen = NAMLEN(de);
+	if (namelen <= 2 && *de->d_name == '.' &&
+		(namelen == 1 || de->d_name[1] == '.' ) )
+	    continue;
+	linelen +=namelen+1;
+	if (linelen > 42) {
+	    new_draw_info(NDI_UNIQUE, 0,op, line);
+	    sprintf(line, " %s", de->d_name);
+	    linelen =namelen+1;
+	    continue;
+	}
+	strcat(line, " ");
+	strcat(line, de->d_name);
+    }
+    new_draw_info(NDI_UNIQUE, 0,op, line);
+    closedir(dirp);
 }
 
 static void show_commands(object *op, int what)
@@ -694,11 +538,6 @@ static void show_commands(object *op, int what)
     size =WizCommandsSize;
     new_draw_info(NDI_UNIQUE, 0,op, "      Wiz commands:");
     break;
-  case 2:
-    ap =SocketCommands;
-    size = SocketCommandsSize;
-    new_draw_info(NDI_UNIQUE, 0,op, "      Socket commands:");
-    break;
   default:
     ap =Commands;
     size =CommandsSize;
@@ -758,12 +597,6 @@ int command_help (object *op, char *params)
    */
   if (!strcmp(params, "topics")) {
     help_topics(op, 3);
-    if (!op) {
-      help_topics(op, 2);
-      if (active_socket->wiz)
-	help_topics(op, 1);
-      return 0;
-    }
     help_topics(op, 0);
     if (QUERY_FLAG(op, FLAG_WIZ))
       help_topics(op, 1);
@@ -774,12 +607,6 @@ int command_help (object *op, char *params)
    * Commands list
    */
   if (!strcmp(params, "commands")) {
-    if (!op) {
-      show_commands(op, 2);
-      if (active_socket->wiz)
-	show_commands(op, 1);
-      return 0;
-  }
     show_commands(op, 0);
     if (QUERY_FLAG(op, FLAG_WIZ))
       show_commands(op, 1);
@@ -807,16 +634,6 @@ int command_help (object *op, char *params)
 	} else
 	  goto nohelp;
       }
-          } else {
-      sprintf(filename, "%s/sockethelp/%s", settings.libdir, params);
-      if (stat(filename, &st) || !S_ISREG(st.st_mode)) {
-	if (active_socket->wiz) {
-	  sprintf(filename, "%s/wizhelp/%s", settings.libdir, params);
-	  if (stat(filename, &st) || !S_ISREG(st.st_mode))
-	    goto nohelp;
-	} else
-	  goto nohelp;
-          }
   }
   }
 
@@ -850,23 +667,6 @@ int command_help (object *op, char *params)
 }
 
 
-int command_chrfont (object *op, char *params)
-{
-  XFontStruct *tmp_font;
-  
-  if (params != NULL) {
-    if ((tmp_font=XLoadQueryFont(op->contr->gdisp, params)) != NULL) {
-      strcpy (op->contr->font_str, params);
-      XFreeFont(op->contr->gdisp, tmp_font);
-    } else {
-      LOG(llevError, "Crossfire: Couldn't load font %s.\n", params);
-      new_draw_info(NDI_UNIQUE, 0,op, "Couldn't load font.");
-    }
-  } else new_draw_info(NDI_UNIQUE, 0,op, "What font?");
-      return 1;
-}
-
-
 int command_invoke(object *op, char *params)
 {
 	return command_cast_spell(op, params, 'i');
@@ -886,12 +686,19 @@ int command_prepare(object *op, char *params)
  * value of the spell in the spells array for a match, -1 if there is no
  * match, -2 if there are multiple matches.  Note that 0 is a valid entry, so
  * we can't use that as failure.
+ *
+ * Modified 03/24/98 - extra parameter 'options' specifies if the search is
+ * done with the length of the input spell name, or the length of the stored
+ * spell name.  This allows you to find out if the spell name entered had
+ * extra optional parameters at the end (ie: marking rune <text>)
+ *
  */
-static int find_spell_byname(object *op, char *params)
+static int find_spell_byname(object *op, char *params, int options)
 {
     int numknown; /* number of spells known by op */
     int spnum;  /* number of spell that is being cast */
     int match=-1,i;
+    int paramlen;
 
     if(QUERY_FLAG(op, FLAG_WIZ))
 	numknown = NROFREALSPELLS;
@@ -902,7 +709,10 @@ static int find_spell_byname(object *op, char *params)
 	if (QUERY_FLAG(op, FLAG_WIZ)) spnum = i;
 	else spnum = op->contr->known_spells[i];
 
-	if (!strncmp(params, spells[spnum].name, strlen(params))) {
+        if (!options)
+          paramlen=strlen(params);
+          
+	if (!strncmp(params, spells[spnum].name, options?strlen(spells[spnum].name):paramlen)) {
 	    /* We already found a match previously - thus params is not
 	     * not unique, so return -2 stating this.
 	     */
@@ -913,6 +723,7 @@ static int find_spell_byname(object *op, char *params)
     return match;
 }
 
+
 /* Shows all spells that op knows.  If params is supplied, the must match
  * that.  If cleric is 1, show cleric spells, if not set, show mage
  * spells.
@@ -978,6 +789,7 @@ int command_cast_spell (object *op, char *params, char command)
     if (command=='i') castnow = 1;
     if(params!=NULL) {
 
+#if 0
 	/* rune of fire, rune of ... are special cases, break it into 'rune' and
 	 * then put 'fire, marking, whatever' in cp. */
 	if (strncmp(params,"rune",4)) {
@@ -987,7 +799,24 @@ int command_cast_spell (object *op, char *params, char command)
 		cp +=4;
 	    }
 	}
-	spnum = find_spell_byname(op, params);
+#endif
+        /* This replaces the above.  It assumes simply that if the name of
+         * the spell being cast as input by the player is shorter than or
+         * equal to the length of the spell name, then there is no options
+         * but if it is longer, then everything after the spell name is
+         * an option.  It determines if the spell name is shorter or
+         * longer by first iterating through the actual spell names, checking
+         * to the length of the typed in name.  If that fails, then it checks
+         * to the length of each spell name.  If that passes, it assumes that
+         * anything after the length of the actual spell name is extra options
+         * typed in by the player (ie: marking rune Hello there) */
+	if ( ((spnum = find_spell_byname(op, params, 0)) < 0) && ((spnum = find_spell_byname(op, params, 1)) >= 0) ) {
+          params[strlen(spells[spnum].name)] = '\0';
+          cp = &params[strlen(spells[spnum].name)+1];
+          if (strncmp(cp,"of ",3) == 0)
+            cp += 3;
+        }	         
+
 	if (spnum>=0) {
 	    rangetype orig_rangetype=op->contr->shoottype;
 	    op->contr->shoottype=range_magic;
@@ -1028,7 +857,6 @@ int command_cast_spell (object *op, char *params, char command)
 	    /* We are not casting now */
 	    else op->contr->chosen_spell=spnum; 
 	    
-	    draw_stats(op);
 	    return 1;
 	} /* found a matching spell */
     } /* params supplied */
@@ -1049,34 +877,32 @@ int command_cast_spell (object *op, char *params, char command)
 #ifdef SET_TITLE
 int command_title (object *op, char *params)
 {
-      char buf[MAX_BUF];
-  if(params == NULL) {
+    char buf[MAX_BUF];
+
+    if(params == NULL) {
 	if(op->contr->own_title[0]=='\0')
-	  sprintf(buf,"Your title is '%s'.",
-		  op->contr->title);
+	    sprintf(buf,"Your title is '%s'.", op->contr->title);
 	else
-	  sprintf(buf,"Your title is '%s'.",
-		  op->contr->own_title);
+	    sprintf(buf,"Your title is '%s'.", op->contr->own_title);
 	new_draw_info(NDI_UNIQUE, 0,op,buf);
 	return 1;
-      }
-  if(strcmp(params, "clear")==0 || strcmp(params, "default")==0) {
+    }
+    if(strcmp(params, "clear")==0 || strcmp(params, "default")==0) {
 	if(op->contr->own_title[0]=='\0')
-	  new_draw_info(NDI_UNIQUE, 0,op,"Your title is the default title.");
+	    new_draw_info(NDI_UNIQUE, 0,op,"Your title is the default title.");
 	else
-	  new_draw_info(NDI_UNIQUE, 0,op,"Title set to default.");
+	    new_draw_info(NDI_UNIQUE, 0,op,"Title set to default.");
 	op->contr->own_title[0]='\0';
-	redraw_title(op);
 	return 1;
-      }
-  if((int)strlen(params) >= MAX_NAME) {
+    }
+
+    if((int)strlen(params) >= MAX_NAME) {
 	new_draw_info(NDI_UNIQUE, 0,op,"Title too long.");
 	return 1;
-      }
-  strcpy(op->contr->own_title, params);
-      redraw_title(op);
-      return 1;
     }
+    strcpy(op->contr->own_title, params);
+    return 1;
+}
 #endif /* SET_TITLE */
 
 int command_save (object *op, char *params)
@@ -1102,7 +928,6 @@ int command_search_items (object *op, char *params)
 	op->contr->search_str[0]='\0';
 	new_draw_info(NDI_UNIQUE, 0,op,"Search mode turned off.");
 	fix_player(op);
-	draw_stats(op);
 	return 1;
   }
   if((int)strlen(params) >= MAX_BUF) {
@@ -1113,7 +938,6 @@ int command_search_items (object *op, char *params)
       sprintf(buf,"Searching for '%s'.",op->contr->search_str);
       new_draw_info(NDI_UNIQUE, 0,op,buf);
   fix_player(op);
-  draw_stats(op);
       return 1;
     }
 #endif /* SEARCH_ITEMS */
@@ -1127,45 +951,6 @@ int command_peaceful (object *op, char *params)
       return 1;
     }
 
-int command_scroll (object *op, char *params)
-{
-      if((op->contr->scroll=!op->contr->scroll)) {
-	new_draw_info(NDI_UNIQUE, 0,op,"Scroll is enabled.");
-	if (op->contr->infofull) {
-		op->contr->infoline = op->contr->infolines - 1;
-		draw_all_info(op);
-	}
-      }
-      else {
-	/* simply put, in scroll mode, infopos does not have to be equal
-	 * to infoline.  However, infopos must equal infoline in wrap
-	 * mode.  A simple infopos=infoline solves this, but if a redraw
-	 * was needed, the redraw wouldn't look right, so we copy the
-	 * info array to make things right.
-	 * Mark Wedel (master@cats.ucsc.edu)
-	 */
-	int i;
-	char	**info;
-	if ((op->contr->infofull) &&
-	   (op->contr->infoline != op->contr->infopos)){
-		info = (char **) malloc(sizeof(char *) * op->contr->infolines);
-      for (i=0; i<(int)op->contr->infolines; i++) {
-			info[i] = malloc(sizeof(char) * (op->contr->infochars+1));
-	strncpy(info[i], op->contr->info[(i+op->contr->infopos) % (int)op->contr->infolines],op->contr->infochars);
-			info[i][op->contr->infochars] = '\0';
-		}
-      for (i=0; i<(int)op->contr->infolines; i++)
-			free(op->contr->info[i]);
-		free(op->contr->info);
-		op->contr->info = info;
-		op->contr->infopos = op->contr->infoline;
-		draw_all_info(op);
-	}
-        new_draw_info(NDI_UNIQUE, 0,op,"Scroll is disabled.");
-      }
-      return 1;
-    }
-
 int command_berzerk (object *op, char *params)
 {
       if((op->contr->berzerk=!op->contr->berzerk))
@@ -1205,133 +990,6 @@ int command_pickup (object *op, char *params)
       return 1;
 }
 
-int command_protocol (object *op, char *params)
-{
-  int i;
-  if (params == NULL || !sscanf(params, "%d", &i)) {
-      new_draw_info(NDI_UNIQUE, 0,op,"Set what protocol?");
-      return 1;
-  }
-  set_protocol(active_socket, i);
-  return 1;
-}
-
-int command_set (object *op, char *params)
-{
-  if (params == NULL) {
-	/* Lets tell what options are available */
-	new_draw_info(NDI_UNIQUE, 0,op, "Options you can set are:");
-	new_draw_info(NDI_UNIQUE, 0,op, "font:    Use normal font mode");
-	new_draw_info(NDI_UNIQUE, 0,op, "pixmaps: Use pixmaps instead of font");
-	new_draw_info(NDI_UNIQUE, 0,op, "split:   Start in split window mode");
-#ifdef Xpm_Pix
-	new_draw_info(NDI_UNIQUE, 0,op, "xpm: Use X Pixmaps for display");
-#endif
-        return 1;
-      }
-
-      /* Lets not be picky about case */
-
-  if(!strcasecmp(params, "font")) {
-        active_socket->use_pix = 0;
-	active_socket->color_pix = 0;
-        new_draw_info(NDI_UNIQUE, 0,op, "OK.");
-        return 1;
-      }
-  if(!strcasecmp(params, "pixmaps")) {
-        active_socket->use_pix = 1;
-	active_socket->color_pix = 0;
-        new_draw_info(NDI_UNIQUE, 0,op, "OK.");
-        return 1;
-      }
-  if(!strcasecmp(params, "split")) {
-        active_socket->split = 1;
-        new_draw_info(NDI_UNIQUE, 0,op, "OK.");
-        return 1;
-      }
-#ifdef Xpm_Pix
-  if (!strcasecmp(params, "xpm")) {
-	active_socket->color_pix = 1;
-        active_socket->use_pix = 0;
-	new_draw_info(NDI_UNIQUE, 0,op, "OK.");
-	return 1;
-      }
-#endif
-      new_draw_info(NDI_UNIQUE, 0,op,"Unknown option.\n");
-      return 1;
-    }
-
-int command_unset (object *op, char *params)
-{
-  if (params == NULL) {
-        new_draw_info(NDI_UNIQUE, 0,op, "Unset what option?");
-        return 1;
-      }
-  if(!strcmp(params,"pixmaps")) {
-        active_socket->use_pix = 0;
-        new_draw_info(NDI_UNIQUE, 0,op, "OK.");
-        return 1;
-      }
-  if(!strcmp(params,"split")) {
-        active_socket->split = 0;
-        new_draw_info(NDI_UNIQUE, 0,op, "OK.");
-        return 1;
-      }
-#ifdef Xpm_Pix
-  if (!strcasecmp(params,"xpm")) {
-	active_socket->color_pix = 0;
-	new_draw_info(NDI_UNIQUE, 0,op, "OK.");
-	return 1;
-      }
-#endif
-      new_draw_info(NDI_UNIQUE, 0,op,"Unknown option.\n");
-      return 1;
-    }
-
-/*
- * Now follows non-dm-commands which are also acceptable from sockets
- */
-
-int command_sync (object *op, char *params)
-{
-    int i;
-    char buf[MAX_BUF];
-  if (params==NULL || !sscanf(params, "%d", &i)) {
-      sprintf(buf, "Your current sync is %d.",
-#ifdef SERVER
-	    op ? op->contr->sync : active_socket->sync
-#else
-	    op->contr->sync
-#endif
-	    );
-      new_draw_info(NDI_UNIQUE, 0,op, buf);
-    }
-#ifdef SERVER
-    if (!op)
-      active_socket->sync = i;
-    else
-#endif
-      op->contr->sync = i;
-    new_draw_info(NDI_UNIQUE, 0,op,"OK.");
-    return 1;
-  }
-
-int command_name (object *op, char *params)
-{
-  if(params == NULL) {
-      new_draw_info(NDI_UNIQUE, 0,op,"Change name to what?");
-      return 1;
-    }
-    if(op == NULL) {
-    strncpy(active_socket->name, params, 8);
-      active_socket->name[8]='\0';
-      new_draw_info(NDI_UNIQUE, 0,NULL,"OK.");
-      return 1;
-    }
-    new_draw_info(NDI_UNIQUE, 0,op,"You can't change your name.");
-    return 1;
-  }
-
 
 int command_wimpy (object *op, char *params)
 {
@@ -1351,19 +1009,13 @@ int command_wimpy (object *op, char *params)
 
 int command_quit (object *op, char *params)
 {
-    if(op == NULL) {
-      new_draw_info(NDI_UNIQUE, 0,NULL,"Goodbye.");
-      active_socket->quit = 1;
-      return 1;
-    }
-    if(op->stats.exp&&!QUERY_FLAG(op, FLAG_WAS_WIZ)) {
-      new_draw_info(NDI_UNIQUE, 0,op,"This will delete your character!");
-      new_draw_info(NDI_UNIQUE, 0,op,"Are you still sure you want to quit(y/n)?");
-    } else
-      new_draw_info(NDI_UNIQUE, 0,op,"Are you sure you want to quit(y/n)?");
+    char *wiz_quit="This will delete your character!\nAre you still sure you want to quit(y/n)?",
+	*quit="Are you sure you want to quit(y/n)?",*cp;
 
-    if (op->contr->eric_server>0)
-	send_query(op->contr->eric_server,CS_QUERY_SINGLECHAR,"");
+    if(op->stats.exp&&!QUERY_FLAG(op, FLAG_WAS_WIZ)) cp=wiz_quit;
+    else cp=quit;
+
+    send_query(&op->contr->socket,CS_QUERY_SINGLECHAR,cp);
 
     op->contr->state = ST_CONFIRM_QUIT;
     return 1;
@@ -1393,88 +1045,38 @@ int command_explore (object *op, char *params)
     }
 #endif
 
-#ifdef SOUND_EFFECTS
 int command_sound (object *op, char *params)
 {
-    if ((op->contr->rplay_fd = - op->contr->rplay_fd) > 0)
-      new_draw_info(NDI_UNIQUE, 0,op, "The sounds are enabled.");
-    else
-      new_draw_info(NDI_UNIQUE, 0,op, "Silence is golden...");
+    if (op->contr->socket.sound) {
+        op->contr->socket.sound=0;
+        new_draw_info(NDI_UNIQUE, 0,op, "Silence is golden...");
+    }
+    else {
+        op->contr->socket.sound=1;
+        new_draw_info(NDI_UNIQUE, 0,op, "The sounds are enabled.");
+    }
     return 1;
-  }
-#endif
-
-int command_add (object *op, char *params)
-{
-    char username[9];
-    Settings oldsettings=settings;
-    int i;
-
-#ifdef EXPLORE_MODE
-    if (explore_mode()) {
-	new_draw_info(NDI_UNIQUE, 0,op,"You are not able to add new players in explore mode.");
-	return 1;
-    }
-#endif
-#ifdef NO_ADD
-    if(op != NULL) {
-      new_draw_info(NDI_UNIQUE, 0,op, "Add is disabled.  Use the client.");
-      return 1;
-    }
-#endif
-
-  if(params == NULL) {
-      if(op != NULL)
-        op->contr->writing=0;
-      new_draw_info(NDI_UNIQUE, 0,op,"Add which display?");
-      return 1;
-    }
-    if (active_socket != (sockets *) NULL)
-    {
-	if (active_socket->color_pix) settings.displaymode=Dm_Pixmap;
-	else if (active_socket->use_pix) settings.displaymode=Dm_Bitmap;
-	settings.splitwindows=active_socket->split;
-	settings.synchronize=active_socket->sync;
-	strcpy(username, active_socket->name);
-    }
-    else
-	*username = '\0';
-    i=add_player(params, *username ? username : NULL, NULL,0);
-    settings = oldsettings;
-
-    /* This player can be freed during a add_player() */
-    if(op == NULL || (op->type == PLAYER && !QUERY_FLAG(op,FLAG_FREED)))
-      new_draw_info(NDI_UNIQUE, 0,op, (i ? errmsg : "OK."));
-    return 1;
-  }
+}
 
 int command_shout (object *op, char *params)
 {
     char buf[MAX_BUF];
-  if (params == NULL) {
-      new_draw_info(NDI_UNIQUE, 0,op,"Shout what?");
-      return 1;
+    if (params == NULL) {
+	new_draw_info(NDI_UNIQUE, 0,op,"Shout what?");
+	return 1;
     }
-    if (op == NULL) {
-      strcpy(buf,active_socket->name);
-      strcat(buf," shouts: ");
-    } else {
-      strcpy(buf,op->name);
-      strcat(buf," shouts: ");
-    }
-  strncat(buf, params, MAX_BUF-30);
+    strcpy(buf,op->name);
+    strcat(buf," shouts: ");
+    strncat(buf, params, MAX_BUF-30);
     buf[MAX_BUF - 1] = '\0';
     new_draw_info(NDI_UNIQUE | NDI_ALL | NDI_RED, 1, NULL, buf);
     return 1;
-  }
+}
 
 int command_tell (object *op, char *params)
 {
     char buf[MAX_BUF],*name = NULL ,*msg = NULL;
     player *pl;
-#ifdef SERVER
-    sockets *s;
-#endif /* SERVER */
     if ( params != NULL){
         name = params;
         msg = strchr(name, ' ');
@@ -1495,27 +1097,14 @@ int command_tell (object *op, char *params)
     }
 
 
-    if (op == NULL)
-      sprintf(buf,"%s tells you: %s",active_socket->name,msg);
-    else
-      sprintf(buf,"%s tells you: %s",op->name,msg);
+    sprintf(buf,"%s tells you: %s",op->name,msg);
     for(pl=first_player;pl!=NULL;pl=pl->next)
       if(strncasecmp(pl->ob->name,name,MAX_NAME)==0)
       {
 	new_draw_info(NDI_UNIQUE | NDI_WHITE, 0, pl->ob, buf);
         return 1;
       }
-#ifdef SERVER
-    for(s = first_socket; s != (sockets *) NULL; s = s->next)
-      if(strncasecmp(s->name,name,MAX_NAME)==0)
-      {
-        draw_socket(s->fd,buf);
-        return 1;
-      }
-    new_draw_info(NDI_UNIQUE, 0,op,"No such player or socket.");
-#else
-    new_draw_info(NDI_UNIQUE, 0,op,"No suck player.");
-#endif /* SERVER */
+    new_draw_info(NDI_UNIQUE, 0,op,"No such player.");
     return 1;
   }
 
@@ -1523,9 +1112,6 @@ int command_bell (object *op, char *params)
 {
     char buf[MAX_BUF];
     player *pl;
-#ifdef SERVER
-    sockets *s;
-#endif /* SERVER */
   if (params == NULL) {
       new_draw_info(NDI_UNIQUE, 0,op,"Bell whom?");
       return 1;
@@ -1533,28 +1119,13 @@ int command_bell (object *op, char *params)
     for(pl=first_player;pl!=NULL;pl=pl->next)
     if(strncasecmp(pl->ob->name, params, MAX_NAME)==0)
       {
-        if (op == NULL)
-          sprintf(buf,"%s bells you.",active_socket->name);
-        else
-          sprintf(buf,"%s bells you.",op->name);
+        sprintf(buf,"%s bells you.",op->name);
         new_draw_info(NDI_UNIQUE, 0,pl->ob,buf);
-        XBell(pl->ob->contr->gdisp,100);
+	/* Need to play a bell sound here */
         return 1;
       }
-#ifdef SERVER
-    for(s = first_socket; s != (sockets *) NULL; s = s->next)
-    if(strncasecmp(s->name, params, MAX_NAME)==0)
-      {
-        if (op == NULL)
-          sprintf(buf,"%s bells you.%c",active_socket->name,7);
-        else
-          sprintf(buf,"%s bells you.%c",op->name,7);
-        draw_socket(s->fd,buf);
-        return 1;
-      }
-#endif /* SERVER */
   return 0;
-  }
+}
 
 /**************************************************************************/
 
@@ -1686,94 +1257,8 @@ void change_spell(object *op,char k) {
     break;
   }
   new_draw_info(NDI_UNIQUE, 0,op,buf);
-  draw_stats(op);
 }
 
-#ifdef SAVE_WINDOW_POSITIONS
-void get_window_coord(object *op, Window win,
-		 int *x,int *y,
-		 int *wx,int *wy,
-		 unsigned int *w,unsigned int *h)
-{
-  Window root,child;
-  unsigned int tmp;
-  XGetGeometry(op->contr->gdisp,win,
-	       &root,
-	       x,y,w,h,
-	       &tmp,&tmp);
-  XTranslateCoordinates(op->contr->gdisp,win,root,
-			0,0,wx,wy,
-			&child);
-}
-#endif /* SAVE_WINDOW_POSITIONS */
-
-int command_rotateinventory (object *op, char *params)
-{
-  int size, i, j=1;
-  object *head, *tail, *tmp;
-  char buf[MAX_BUF];
-
-  if (params && params[0])
-    sscanf(params, "%d", &j);
-
-  if (!j)
-    return 0;
-  if(!op->inv) {
-    new_draw_info(NDI_UNIQUE, 0,op,"You don't carry anything.");
-    return 0;
-  }
-
-  for (size=0,tmp=op->inv; tmp; tmp=tmp->below)
-    if (!tmp->invisible)
-      size++;
-  if (size < 2) {
-    new_draw_info(NDI_UNIQUE, 0,op,"You only carry one thing.");
-    return 0;
-  }
-  if (FABS(j) >= size) {
-    new_draw_info(NDI_UNIQUE, 0,op, "Value too big");
-    return 0;
-  }
-
-  op->contr->last_used =NULL;
-  op->contr->last_used_id =0;
-
-  if (j<0) {
-    for (i=0,head=NULL,tmp=op->inv; tmp->below; tmp=tmp->below)
-      if (!tmp->invisible)
-	if (i-- == j)
-	  head =tmp;
-    tail =tmp;
-  } else {
-    for (tmp=op->inv; tmp->below; tmp=tmp->below)
-      ;
-    tail =tmp;
-    for (i=0; tmp; tmp=tmp->above)
-      if (!tmp->invisible)
-	if (++i == j)
-      break;
-    head =tmp;
-  }
-  if (!head)
-    return 0;
-
-  tmp =op->inv;
-  op->inv =head;
-  head->above->below =NULL;
-  head->above =NULL;
-  tail->below =tmp;
-  tmp->above =tail;
-  
-  if(QUERY_FLAG(op, FLAG_WIZ))
-    sprintf(buf,"Rotate %d. The %s (%d) is now on top.",
-            j, query_name(op->inv),op->inv->count);
-  else
-    sprintf(buf,"Rotate %d. The %s is now on top.", j, query_name(op->inv));
-  new_draw_info(NDI_UNIQUE, 0,op,buf);
-  draw_inventory(op);
-      op->contr->count_left=0;
-  return 0;
-}
 
 int command_invisible (object *op, char *params)
 {
@@ -1794,35 +1279,6 @@ int command_rotateshoottype (object *op, char *params)
   return 0;
 }
 
-int command_show (object *op, char *params)
-{
-
-  if(!params) {
-      if (++op->contr->show_what > show_nonmagical)
-        op->contr->show_what = show_all;
-      draw_all_inventory(op);
-    return 1;
-  }
-
-  if (!strncmp(params, "all", strlen(params)))
-	op->contr->show_what = show_all;
-  else if (!strncmp(params, "applied", strlen(params)))
-	op->contr->show_what = show_applied;
-  else if (!strncmp(params, "unapplied", strlen(params)))
-	op->contr->show_what = show_unapplied;
-  else if (!strncmp(params, "unpaid", strlen(params)))
-	op->contr->show_what = show_unpaid;
-  else if (!strncmp(params, "cursed", strlen(params)))
-	op->contr->show_what = show_cursed;
-  else if (!strncmp(params, "magical", strlen(params)))
-	op->contr->show_what = show_magical;
-  else if (!strncmp(params, "nonmagical", strlen(params)))
-	op->contr->show_what = show_nonmagical;
-
-  draw_all_inventory(op);
-  return 1;
-}
-
 int command_throw (object *op, char *params)
 {
 
@@ -1830,7 +1286,7 @@ int command_throw (object *op, char *params)
    if(!change_skill(op,SK_THROWING))
         return 0;
    else {
-        int success = do_skill(op,op->facing,NULL);
+        int success = do_skill(op,op->facing,params);
         return success;
    }
 #else
@@ -1877,77 +1333,9 @@ int command_rotatespells (object *op, char *params)
 	i +=j + (int)pl->nrofknownspells;
 	i = i % (int)pl->nrofknownspells;
 	pl->chosen_spell=pl->known_spells[i];
-	draw_stats(op);
 	return 1;
 	    }
   pl->chosen_spell=pl->known_spells[0];
-  draw_stats(op);
   return 1;
 }
 
-#ifdef SAVE_WINDOW_POSITIONS
-int command_savewinpos (object *op, char *params)
-{
-  player *pl=op->contr;
-	int i,eq=1;
-
-  if(!(pl->split_window)) {
-	  new_draw_info(NDI_UNIQUE, 0,op,"You can't save window positions in this mode.");
-    return 0;
-	}
-  if(pl->valid_save_positions) {
-    pl->valid_save_positions=0;
-	  new_draw_info(NDI_UNIQUE, 0,op,"Window positions will no longer be saved.");
-    return 0;
-	}
-
-  get_window_coord(op,pl->win_game,
-		   &pl->win_pos[0].x,&pl->win_pos[0].y,
-		   &pl->win_pos[0].wx,&pl->win_pos[0].wy,
-		   &pl->win_pos[0].w,&pl->win_pos[0].h);
-  get_window_coord(op,pl->win_stats,
-		   &pl->win_pos[1].x,&pl->win_pos[1].y,
-		   &pl->win_pos[1].wx,&pl->win_pos[1].wy,
-		   &pl->win_pos[1].w,&pl->win_pos[1].h);
-  get_window_coord(op,pl->win_info,
-		   &pl->win_pos[2].x,&pl->win_pos[2].y,
-		   &pl->win_pos[2].wx,&pl->win_pos[2].wy,
-		   &pl->win_pos[2].w,&pl->win_pos[2].h);
-  get_window_coord(op,pl->win_inv,
-		   &pl->win_pos[3].x,&pl->win_pos[3].y,
-		   &pl->win_pos[3].wx,&pl->win_pos[3].wy,
-		   &pl->win_pos[3].w,&pl->win_pos[3].h);
-  get_window_coord(op,pl->win_look,
-		   &pl->win_pos[4].x,&pl->win_pos[4].y,
-		   &pl->win_pos[4].wx,&pl->win_pos[4].wy,
-		   &pl->win_pos[4].w,&pl->win_pos[4].h);
-  get_window_coord(op,pl->win_message,
-		   &pl->win_pos[5].x,&pl->win_pos[5].y,
-		   &pl->win_pos[5].wx,&pl->win_pos[5].wy,
-		   &pl->win_pos[5].w,&pl->win_pos[5].h);
-    for(i=0;i<5;i++)	/* try to figure out the window coords */
-    if(pl->win_pos[i].x!=pl->win_pos[i+1].x ||
-       pl->win_pos[i].y!=pl->win_pos[i+1].y)
-	    {
-	      eq=0;			/* window positions were different */
-	return 0;
-	    }
-	if(eq)			/* need to shift window positions (wm frame) */
-	  for(i=0;i<6;i++)
-	    {
-#ifdef FUNNY_WINDOW_MANAGER /* try defining this if the other doesn't work */
-	pl->win_pos[i].x=pl->win_pos[i].wx-
-	  pl->win_pos[i].x;
-	pl->win_pos[i].y=pl->win_pos[i].wy-
-	  pl->win_pos[i].y;
-#else /* FUNNY_WINDOW_MANAGER */
-	pl->win_pos[i].x=pl->win_pos[i].wx;
-	pl->win_pos[i].y=pl->win_pos[i].wy;
-#endif /* FUNNY_WINDOW_MANAGER */
-	    }
-    pl->valid_save_positions=1;
-	new_draw_info(NDI_UNIQUE, 0,op,"Current window positions will be saved.");
-        return 0;
-}
-#endif /* SAVE_WINDOW_POSITIONS */
-
