@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_login_c =
- *   "$Id: login.c,v 1.46 1997/03/09 05:11:32 master Exp master $";
+ *   "$Id: login.c,v 1.53 1998/09/28 03:23:47 master Exp master $";
  */
 
 /*
@@ -30,6 +30,7 @@
 #include <sproto.h>
 #endif
 #include <spells.h>
+#include <loader.h>
 
 extern spell spells[NROFREALSPELLS];
 extern void sub_weight (object *, signed long);
@@ -77,118 +78,108 @@ void emergency_save(int flag) {
 #else
   LOG(llevError,"Emergency saves disabled, no save attempted\n");
 #endif
-  /* After we complete the emergency saves, then try and free
-   * the pixmaps.  Without this effort, some X-Terminals (and maybe
-   * even workstations) XServers will not free the space used by
-   * the pixmaps.  This is more of a problem for the X-Terminals,
-   * because they don't have a lot of memory.  The pointer is set
-   * to NULL after we are finished to prevent another effort later.
-   */
+  /* If the game is exiting, remove the player locks */
   if (!flag) {
     for(pl=first_player;pl!=NULL;pl=pl->next) {
-      if(!pl->ob) {
-	LOG(llevError, "No name, ignoring this.\n");
-	continue;
-      }
-      if (pl->pixmaps) {
-	free_pixmaps(pl->gdisp, pl->pixmaps);
-	pl->pixmaps=NULL;
-      }
-      if (pl->masks) {
-	free_pixmaps(pl->gdisp, pl->masks);
-	pl->masks =NULL;
+      if(pl->ob) {
+	unlock_player(pl->ob->name);
       }
     }
   }
 }
 
-void delete_character(char *name) {
-  char buf[MAX_BUF];
+/* Delete character with name.  if new is set, also delete the new
+ * style directory, otherwise, just delete the old style playfile
+ * (needed for transition)
+ */
+void delete_character(char *name, int new) {
+    char buf[MAX_BUF];
 
-#ifdef SAVE_HOMEDIR
-  sprintf(buf,"%s/%s/%s.pl",(char *) getenv("HOME"),settings.playerdir,name);
-#else
-  sprintf(buf,"%s/%s/%s.pl",settings.libdir,settings.playerdir,name);
-#endif
-  if(unlink(buf)== -1 && settings.debug)
-    perror("crossfire (delete character)");
+    sprintf(buf,"%s/%s/%s.pl",settings.libdir,settings.playerdir,name);
+    if(unlink(buf)== -1 && settings.debug)
+	perror("crossfire (delete character)");
+    if (new) {
+	sprintf(buf,"%s/%s/%s",settings.libdir,settings.playerdir,name);
+	/* this effectively does an rm -rf on the directory */
+	remove_directory(buf);
+    }
 }
 
-void remove_lock(player *pl) {
-#ifdef LOCK_PLAYER
-  char buf[MAX_BUF];
+/* Lock/unlock player functions.  In reality, the only time they are
+ * really needed is if 2 players are logging in at the same time and
+ * trying to use the same name to create characters (not very likely,
+ * but...)  Otherwise, it checks against the existance of other active
+ * players for duplicate names.  In reality, these functions could
+ * probably be removed and it wouldn't create any problems.  However,
+ * they are minor enough and the extra safety the grant is probably
+ * worth keeping them around.
+ */
 
-  if(pl->ob == NULL)
-    return;
-#ifdef SAVE_HOMEDIR
-  sprintf(buf, "%s/%s", (char *) getenv("HOME"), settings.playerdir);
-  (void) create_savedir_if_needed(buf);
-  
-  sprintf(buf,"%s/%s/%s.lock",(char *) getenv("HOME"),settings.playerdir,pl->ob->name);
-#else
-  sprintf(buf,"%s/%s/%s.lock",settings.libdir,settings.playerdir,pl->ob->name);
-#endif
-  if(!rmdir(buf)) {
+/* Renamed from 'remove_lock' to unlock_player to better match with
+ * lock_player below.  Basically, given player 'pl', it removes the
+ * corresponding lock file.
+ */
+
+void unlock_player(char *name) {
+    char buf[MAX_BUF];
+
+    sprintf(buf,"%s/%s/%s.lock",settings.libdir,settings.playerdir,name);
+    if(!rmdir(buf)) {
 #ifdef DEBUG
-    perror("Couldn't remove lockfile(dir)");
-#endif
-  }
+	perror("Couldn't remove lockfile(dir)");
 #endif
+    }
 }
 
-int lock_player(char *name) {
-#ifdef LOCK_PLAYER
-  char buf[MAX_BUF];
+/* creates a lock for player 'name'.  returns 0 if the lock is successful,
+ * 1 otherwise.
+ */
 
-#ifdef SAVE_HOMEDIR
-  sprintf(buf, "%s/%s", (char *) getenv("HOME"), settings.playerdir);
-  (void) create_savedir_if_needed(buf);
-  
-  sprintf(buf,"%s/%s/%s.lock",(char *) getenv("HOME"),settings.playerdir,name);
-#else
-  sprintf(buf,"%s/%s/%s.lock",settings.libdir,settings.playerdir,name);
-#endif
-  if(!mkdir(buf,0770))
-    return 0;
-  if(errno != EEXIST) {
-    perror("Couldn't create lockfile(dir)");
+static int lock_player(char *name) {
+    char buf[MAX_BUF];
+
+    sprintf(buf,"%s/%s/%s.lock",settings.libdir,settings.playerdir,name);
+    if(!mkdir(buf,0770))
+	return 0;
+    if(errno != EEXIST) {
+	perror("Couldn't create lockfile(dir)");
+	return 1;
+    }
     return 1;
-  }
-  return 1;
-#else
-  return 0;
-#endif
 }
 
+/* Checks to see if anyone else by 'name' is currently playing.  We
+ * do this by a lockfile (playername.lock) in the save directory.  If
+ * we find that file or another character of some name is already in the
+ * game, we don't let this person join.  We return 0 if the name is
+ * in use, 1 otherwise.
+ */
+
 int check_name(player *me,char *name) {
-  player *pl;
+    player *pl;
+    char buf[MAX_BUF];
 
-#ifdef ONE_PLAYER_PR_UID
-  if (me->username == NULL)
-    LOG(llevError, "Player had no username.\n");
-  else
-    for (pl = first_player; pl != NULL; pl = pl->next)
-      if (pl != me && pl->username != NULL &&
-          !strcmp(pl->username, me->username)) {
-        new_draw_info(NDI_UNIQUE, 0,me->ob, "You are already playing the game.");
-        return 0;
-      }
-#endif
-  for(pl=first_player;pl!=NULL;pl=pl->next)
-    if(pl!=me&&pl->ob->name!=NULL&&!strcmp(pl->ob->name,name)) {
-      new_draw_info(NDI_UNIQUE, 0,me->ob,"That name is already in use.");
-      return 0;
+    for(pl=first_player;pl!=NULL;pl=pl->next)
+	if(pl!=me&&pl->ob->name!=NULL&&!strcmp(pl->ob->name,name)) {
+	    new_draw_info(NDI_UNIQUE, 0,me->ob,"That name is already in use.");
+	    return 0;
+	}
+
+    if(lock_player(name)) {
+	/* Include the 'locked' so that the error message is more descriptive.
+	 * This way, players can include a slightly more detailed message
+	 * on the problem they are having.
+	 */
+	new_draw_info(NDI_UNIQUE, 0,me->ob,"That name is already in use (locked).");
+	return 0;
     }
-  if(lock_player(name)) {
-    new_draw_info(NDI_UNIQUE, 0,me->ob,"That name is already in use.");
-    return 0;
-  }
-  if(!playername_ok(name)) {
-    new_draw_info(NDI_UNIQUE, 0,me->ob,"That name contains illegal characters.");
-    remove_lock(me);
-    return 0;
-  }
-  return 1;
+
+    if(!playername_ok(name)) {
+	unlock_player(name);
+	new_draw_info(NDI_UNIQUE, 0,me->ob,"That name contains illegal characters.");
+	return 0;
+    }
+    return 1;
 }
 
 int create_savedir_if_needed(char *savedir)
@@ -239,7 +230,7 @@ int save_player(object *op, int flag) {
   char filename[MAX_BUF], *tmpfilename,backupfile[MAX_BUF];
   object *tmp, *container=NULL;
   player *pl = op->contr;
-  int i;
+  int i,wiz=QUERY_FLAG(op,FLAG_WIZ);
   long checksum;
 #ifdef BACKUP_SAVE_AT_HOME
   sint16 backup_x, backup_y;
@@ -247,7 +238,7 @@ int save_player(object *op, int flag) {
 
   flag&=1;
 
-  if(QUERY_FLAG(op,FLAG_WAS_WIZ)||!pl->name_changed||(!flag&&!op->stats.exp)) {
+  if(!pl->name_changed||(!flag&&!op->stats.exp)) {
     if(!flag) {
       new_draw_info(NDI_UNIQUE, 0,op,"Your game is not valid,");
       new_draw_info(NDI_UNIQUE, 0,op,"Game not saved.");
@@ -258,15 +249,13 @@ int save_player(object *op, int flag) {
   if (flag == 0)
     terminate_all_pets(op);
 
-#ifdef SAVE_HOMEDIR
-  sprintf(filename,"%s/%s", (char *) getenv("HOME"), settings.playerdir);
-  (void) create_savedir_if_needed(filename);
-  
-  sprintf(filename,"%s/%s/%s.pl",(char *) getenv("HOME"),settings.playerdir,op->name);
-#else
+  /* Delete old style file */
   sprintf(filename,"%s/%s/%s.pl",settings.libdir,settings.playerdir,op->name);
-#endif
-  tmpfilename = tempnam(settings.tmpdir,NULL);
+  unlink(filename);
+
+  sprintf(filename,"%s/%s/%s/%s.pl",settings.libdir,settings.playerdir,op->name,op->name);
+  make_path_to_file(filename);
+  tmpfilename = tempnam_local(settings.tmpdir,NULL);
   fp=fopen(tmpfilename, "w");
   if(!fp) {
     new_draw_info(NDI_UNIQUE, 0,op, "Can't open file for save.");
@@ -286,16 +275,7 @@ int save_player(object *op, int flag) {
   if(pl->own_title[0]!='\0')
     fprintf(fp,"title %s\n",pl->own_title);
 #endif /* SET_TITLE */
-#ifdef SAVE_WINDOW_POSITIONS
-  if(pl->valid_save_positions) {
-    fprintf(fp,"win_pos");
-    for(i=0;i<6;i++)
-      fprintf(fp," %d %d %u %u",
-	      pl->win_pos[i].x,pl->win_pos[i].y,
-	      pl->win_pos[i].w,pl->win_pos[i].h);
-    fprintf(fp,"\n");
-  }
-#endif /* SAVE_WINDOW_POSITIONS */
+
 #ifdef EXPLORE_MODE
   fprintf(fp,"explore %d\n",pl->explore);
 #endif
@@ -307,15 +287,10 @@ int save_player(object *op, int flag) {
   fprintf(fp,"shoottype %d\n",pl->shoottype);
   fprintf(fp,"berzerk %d\n",pl->berzerk);
   fprintf(fp,"peaceful %d\n",pl->peaceful);
-  fprintf(fp,"scroll %d\n",pl->scroll);
   fprintf(fp,"digestion %d\n",pl->digestion);
   fprintf(fp,"pickup %d\n", pl->mode);
-  fprintf(fp,"keyboard_flush %d\n", pl->keyboard_flush);
-  fprintf(fp,"show_inv_icon %d\n", pl->show_inv_icon);
   fprintf(fp,"outputs_sync %d\n", pl->outputs_sync);
   fprintf(fp,"outputs_count %d\n", pl->outputs_count);
-  fprintf(fp,"sort_inventory %d\n", pl->sort_inventory);
-  fprintf(fp,"show_buttons %d\n", pl->show_buttons);
 
 #ifdef BACKUP_SAVE_AT_HOME
   if (op->map!=NULL && flag==0)
@@ -334,8 +309,6 @@ int save_player(object *op, int flag) {
   fprintf(fp,"Wis %d\n",pl->orig_stats.Wis);
   fprintf(fp,"Cha %d\n",pl->orig_stats.Cha);
 
-  dump_keys(pl, fp);
-
   fprintf(fp,"lev_array %d\n",op->level>10?10:op->level);
   for(i=1;i<=pl->last_level&&i<=10;i++) {
     fprintf(fp,"%d\n",pl->levhp[i]);
@@ -346,8 +319,8 @@ int save_player(object *op, int flag) {
     fprintf(fp,"known_spell %s\n",spells[pl->known_spells[i]].name);
   fprintf(fp,"endplst\n");
 
-  pl->freeze_inv = 1;
   SET_FLAG(op, FLAG_NO_FIX_PLAYER);
+  CLEAR_FLAG(op, FLAG_WIZ);
 #ifdef BACKUP_SAVE_AT_HOME
   if (flag) {
     backup_x = op->x;
@@ -401,17 +374,13 @@ int save_player(object *op, int flag) {
   else
 	unlink(backupfile);
 
-  pl->freeze_inv=0;
-
   /* Eneq(@csd.uu.se): Reveal the container if we have one. */
   if (flag&&container!=NULL) 
     op->container = container;
 
+  if (wiz) SET_FLAG(op,FLAG_WIZ);
   if(!flag)
-    if (op->contr->eric_server > 0)
 	esrv_send_inventory(op, op);
-    else
-	draw_all_inventory(op);
 
   chmod(filename,SAVE_MODE);
   return 1;
@@ -474,16 +443,25 @@ void check_login(object *op) {
   FILE *fp;
   char filename[MAX_BUF];
   char buf[MAX_BUF],bufall[MAX_BUF];
-  int i,value,x,y,comp;
+  int i,value,x,y,comp,has_keys=0;
   long checksum = 0;
   player *pl = op->contr;
 
   strcpy (pl->maplevel, first_map_path);
-#ifdef SAVE_HOMEDIR
-  sprintf(filename,"%s/%s/%s.pl",(char *) getenv("HOME"),settings.playerdir,op->name);
-#else
-  sprintf(filename,"%s/%s/%s.pl",settings.libdir,settings.playerdir,op->name);
-#endif
+
+  /* First, lets check for newest form of save */
+  sprintf(filename,"%s/%s/%s/%s.pl",settings.libdir,settings.playerdir,op->name,op->name);
+  if (access(filename, F_OK)==-1) {
+    /* not there,  Try the old style */
+
+    sprintf(filename,"%s/%s/%s.pl",settings.libdir,settings.playerdir,op->name);
+    /* Ok - old style exists.  Lets make the new style directory */
+    if (access(filename, F_OK)==0) {
+	sprintf(buf,"%s/%s/%s",settings.libdir,settings.playerdir,op->name);
+	make_path_to_file(buf);
+    }
+  }
+
   if ((fp=open_and_uncompress(filename,1,&comp)) == NULL) {
     confirm_password(op);
   } else {
@@ -505,23 +483,19 @@ void check_login(object *op) {
       new_draw_info(NDI_UNIQUE, 0,op," ");
       new_draw_info(NDI_UNIQUE, 0,op,"Wrong Password!");
       new_draw_info(NDI_UNIQUE, 0,op," ");
-      remove_lock(pl);
+      unlock_player(pl->ob->name);
       if(op->name!=NULL)
         free_string(op->name);
       op->name=add_string("noname");
       pl->last_value= -1;
-      draw_stats(op);
       get_name(op);
     } else {
+	unlock_player(pl->ob->name);
 #ifdef SAVE_INTERVAL
       pl->last_save_time=time(NULL);
 #endif /* SAVE_INTERVAL */
-#ifdef SIMPLE_PARTY_SYSTEM
       pl->party_number = (-1);
-#endif /* SIMPLE_PARTY_SYSTEM */
-#ifdef SAVE_WINDOW_POSITIONS
-      pl->valid_save_positions=0;
-#endif /* SAVE_WINDOW_POSITIONS */
+
 #ifdef SEARCH_ITEMS
       pl->search_str[0]='\0';
 #endif /* SEARCH_ITEMS */
@@ -533,34 +507,16 @@ void check_login(object *op) {
       pl->orig_stats.Pow=0;
       pl->orig_stats.Wis=0;
       pl->orig_stats.Cha=0;
-      load_default_keys(op->contr);
+
       while (fgets(bufall,MAX_BUF,fp)!=NULL) {
         sscanf(bufall,"%s %d\n",buf,&value);
         if (!strcmp(buf,"endplst"))
           break;
 #ifdef SET_TITLE
-	else if (!strcmp(bufall,"title"))
-          sscanf(bufall,"title %s[^\n]",pl->own_title);
+	else if (!strcmp(buf,"title"))
+          sscanf(bufall,"title %[^\n]",pl->own_title);
 #endif /* SET_TITLE */
-#ifdef SAVE_WINDOW_POSITIONS
-	else if (!strcmp(buf,"win_pos")) {
-	  pl->valid_save_positions=
-	    (sscanf(bufall,
-		    "win_pos %d %d %u %u %d %d %u %u %d %d %u %u %d %d %u %u %d %d %u %u %d %d %u %u",
-		    &pl->win_pos[0].x,&pl->win_pos[0].y,
-		    &pl->win_pos[0].w,&pl->win_pos[0].h,
-		    &pl->win_pos[1].x,&pl->win_pos[1].y,
-		    &pl->win_pos[1].w,&pl->win_pos[1].h,
-		    &pl->win_pos[2].x,&pl->win_pos[2].y,
-		    &pl->win_pos[2].w,&pl->win_pos[2].h,
-		    &pl->win_pos[3].x,&pl->win_pos[3].y,
-		    &pl->win_pos[3].w,&pl->win_pos[3].h,
-		    &pl->win_pos[4].x,&pl->win_pos[4].y,
-		    &pl->win_pos[4].w,&pl->win_pos[4].h,
-		    &pl->win_pos[5].x,&pl->win_pos[5].y,
-		    &pl->win_pos[5].w,&pl->win_pos[5].h)==24);
-	}
-#endif /* SAVE_WINDOW_POSITIONS */
+
 #ifdef EXPLORE_MODE
       else if (!strcmp(buf,"explore"))
         pl->explore = value;
@@ -581,25 +537,10 @@ void check_login(object *op) {
           pl->berzerk=value;
         else if (!strcmp(buf,"peaceful"))
           pl->peaceful=value;
-        else if (!strcmp(buf,"scroll"))
-          pl->scroll=value;
         else if (!strcmp(buf,"digestion"))
           pl->digestion=value;
 	else if (!strcmp(buf,"pickup"))
 	  pl->mode=value;
-        else if (!strcmp(buf,"keyboard_flush"))
-	  pl->keyboard_flush = value;
-	else if (!strcmp(buf,"sort_inventory"))
-	  pl->sort_inventory = value;
-	else if (!strcmp(buf,"show_buttons"))
-	  pl->show_buttons = value;
-        else if (!strcmp(buf,"show_inv_icon")) {
-	  pl->show_inv_icon = value;
-	  if (pl->show_inv_icon)
-	    strcpy(pl->format_inv,"%-20.20s%-6s"); /* This can be changed by resize */
-	  else
-	    strcpy(pl->format_inv,"%-24.24s%-6s"); /* This can be changed by resize */
-	}
 	else if (!strcmp(buf,"outputs_sync"))
 	  pl->outputs_sync = value;
 	else if (!strcmp(buf,"outputs_count"))
@@ -657,20 +598,19 @@ void check_login(object *op) {
 	    fgets(bufall, sizeof(bufall), fp);
         } else if (!strcmp(buf,"pushkey")) {
 	  LOG(llevDebug, "Ignoring old pushkey (%s)\n", bufall);
-	} else if (!strcmp(buf,"key")) {
-	  insert_key(op->contr, KEYF_USER, &bufall[4]);
-        } else {
-            LOG(llevError,"Warning, oldfashioned variable in save-file: %s\n",
-                buf);
         }
       }
 
+#if 0
       /* sigh */
       free_object (op);
       op = pl->ob = LoadObject (fp, NULL);
-      /* Somehow, the game worked before without this line - relied
-       * on getting the object back that we just freed? */
+#endif
+      reset_object(op);
       op->contr = pl;
+      pl->ob = op;
+      load_object(fp, op, LO_NEWFILE);
+
       CLEAR_FLAG(op, FLAG_NO_FIX_PLAYER);
 
       x=op->x; y=op->y;
@@ -688,11 +628,15 @@ void check_login(object *op) {
       /* If the map where the person was last saved does not exist,
        * restart them in the beginning town.  This is good for when
        * maps change between versions
+       * First, we check for partial path, then check to see if the full
+       * path (for unique player maps)
        */
 
-      if (check_path(pl->maplevel)==-1) {
-	strcpy(pl->maplevel, first_map_path);
-	x = -1;
+      if (check_path(pl->maplevel,1)==-1) {
+	if (check_path(pl->maplevel,0)==-1) {
+	    strcpy(pl->maplevel, first_map_path);
+	    x = -1;
+	}
       }
 
       enter_exit(op,NULL); /* This won't insert the player any longer! */
@@ -705,7 +649,7 @@ void check_login(object *op) {
         new_draw_info(NDI_UNIQUE, 0,op,"Your character was dead, when you quit.");
         new_draw_info(NDI_UNIQUE, 0,op," ");
 	play_again(op);
-        delete_character(op->name);
+        delete_character(op->name,0);
       } else { /* character is alive */
         pl->name_changed=1;
         pl->state = ST_PLAYING;
@@ -713,36 +657,26 @@ void check_login(object *op) {
 	pl->last_save_tick = pticks;
 #endif
 	op->carrying=0;
-	pl->freeze_inv=1;
-
-	pl->freeze_inv=0;
-	if (pl->eric_server > 0) {
-	    esrv_new_player(pl->eric_server, op->count, op->name, op->weight,
-			op->face->number);
-	    esrv_send_inventory(op, op);
-	} else {
-	    draw_all_inventory(op);
-	}
-
-	draw_all_inventory(op);
-
 	/* Need to call fix_player now - program modified so that it is not
 	 * called during the load process (FLAG_NO_FIX_PLAYER set when
-	 * saved
+	 * saved)
+	 * Moved ahead of the esrv functions, so proper weights will be
+	 * sent to the client.
 	 */
 	legal_range(op, op->contr->shoottype);
         fix_player(op);
 	fix_weight ();
+	esrv_new_player(op->contr,op->weight+op->carrying);
+	esrv_send_inventory(op, op);
+
+
 
 #ifdef ALLOW_SKILLS
 	(void) init_player_exp(op);
         (void) link_player_skills(op);
 #endif
 
-        if(!pl->split_window && pl->eric_server == 0)
-          XClearWindow(pl->gdisp,pl->win_root);
-        draw_all_info(op);
-        draw_all_message(op);
+
         new_draw_info(NDI_UNIQUE, 0,op,"Welcome Back!");
         if(pl->loading == NULL) {
           if(!out_of_map(op->map,x,y))
@@ -753,8 +687,7 @@ void check_login(object *op) {
                   op->map->path);
           pl->removed = 0; /* Pl. will be inserted when map is loaded */
         }
-        refresh(op);
-        draw_all_look(op);
+        draw_look(op);
         close_and_delete(fp, comp);
         LOG(llevDebug,"Checksums: %x %x\n",
                 checksum,calculate_checksum(filename,1));
@@ -777,43 +710,10 @@ void check_login(object *op) {
    */
   qsort((void *)pl->known_spells,pl->nrofknownspells,
 	sizeof(pl->known_spells[0]),(int (*)())spell_sort);
-  draw_stats(op);
-#ifdef SAVE_WINDOW_POSITIONS
-  if(pl->valid_save_positions && pl->split_window) {
-    unsigned long xwc_mask=0;
-    XWindowChanges xwc;
-    xwc_mask=CWX|CWY|CWWidth|CWHeight;
-    
-    xwc.x=pl->win_pos[0].x;xwc.y=pl->win_pos[0].y;
-    xwc.width=pl->win_pos[0].w;xwc.height=pl->win_pos[0].h;
-    XConfigureWindow(pl->gdisp,pl->win_game,xwc_mask,&xwc);
-
-    xwc.x=pl->win_pos[1].x;xwc.y=pl->win_pos[1].y;
-    xwc.width=pl->win_pos[1].w;xwc.height=pl->win_pos[1].h;
-    XConfigureWindow(pl->gdisp,pl->win_stats,xwc_mask,&xwc);
-
-    xwc.x=pl->win_pos[2].x;xwc.y=pl->win_pos[2].y;
-    xwc.width=pl->win_pos[2].w;xwc.height=pl->win_pos[2].h;
-    XConfigureWindow(pl->gdisp,pl->win_info,xwc_mask,&xwc);
-
-    xwc.x=pl->win_pos[3].x;xwc.y=pl->win_pos[3].y;
-    xwc.width=pl->win_pos[3].w;xwc.height=pl->win_pos[3].h;
-    XConfigureWindow(pl->gdisp,pl->win_inv,xwc_mask,&xwc);
-
-    xwc.x=pl->win_pos[4].x;xwc.y=pl->win_pos[4].y;
-    xwc.width=pl->win_pos[4].w;xwc.height=pl->win_pos[4].h;
-    XConfigureWindow(pl->gdisp,pl->win_look,xwc_mask,&xwc);
-
-    xwc.x=pl->win_pos[5].x;xwc.y=pl->win_pos[5].y;
-    xwc.width=pl->win_pos[5].w;xwc.height=pl->win_pos[5].h;
-    XConfigureWindow(pl->gdisp,pl->win_message,xwc_mask,&xwc);
-    sprintf(pl->format_inv,"%%-%d.%ds%%-6s",pl->inv_chars-6,
-          pl->inv_chars-6);
-    draw_all_inventory(pl->ob);
-
+  if (has_keys) {
+    new_draw_info(NDI_UNIQUE, 0,op,"Your old player keybindings will not be");
+    new_draw_info(NDI_UNIQUE, 0,op,"available - using default keybindings.");
   }
-    
-#endif /* SAVE_WINDOW_POSITIONS */
   return;
 }
   
