@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_main_c =
- *    "$Id: main.c,v 1.41 1997/03/09 05:11:52 master Exp master $";
+ *    "$Id: main.c,v 1.46 1998/09/28 03:26:39 master Exp master $";
  */
 
 /*
@@ -34,12 +34,15 @@
 extern char *crypt(const char *, const char *);
 #endif
 
+#if defined(linux)
+#include <crypt.h>
+#endif
+
 #include <version.h>
 #include <global.h>
 #ifndef __CEXTRACT__
 #include <sproto.h>
 #endif
-#include <main.h>
 #include <object.h>
 #if defined (_IBMR2) || defined (___IBMR2) /* Which is correct? */
 #include <time.h>
@@ -113,9 +116,6 @@ void info_keys(object *op) {
   new_draw_info(NDI_UNIQUE, 0,op,"C  = configure    s = brace       v = version");
   new_draw_info(NDI_UNIQUE, 0,op,"+- = change range <tab> = browse spells");
   new_draw_info(NDI_UNIQUE, 0,op,"x  = change inventory type");
-#ifdef SAVE_WINDOW_POSITIONS
-  new_draw_info(NDI_UNIQUE, 0,op,"X  = Toggle window position saving.");
-#endif /* SAVE_WINDOW_POSITIONS */
   new_draw_info(NDI_UNIQUE, 0,op,"Mouse: L = examine,  M = apply,  R = drop/get");
   new_draw_info(NDI_UNIQUE, 0,op,"'help  = info about extended commands.");
   new_draw_info(NDI_UNIQUE, 0,op,"Ctrl-R = refresh   Ctrl-C = clear");
@@ -209,181 +209,203 @@ char *normalize_path (char *src, char *dst) {
  */
 
 void enter_exit(object *op, object *exit_ob) {
-  char buf[MAX_BUF];
+    char buf[MAX_BUF], apartment[MAX_BUF]="";
+    mapstruct *oldmap, dummy_map;
+    int x=0, y=0, removed=QUERY_FLAG(op,FLAG_REMOVED);
+    char *newpath=NULL, *lastlevel=NULL;
+    int last_x = op->x, last_y = op->y, notunique=1;
 
-  /* having dummy_map be non static is better for two reasons - with it
-   * being static, only 1 instance of it was created, and thus, if this
-   * function was called recursively, dummy_map would not have an
-   * expected value.  Second, dummy_map is no longer needed once this
-   * function finishes, and as I understand it, its data will be accessible
-   * until that time --MSW
-   */
-  mapstruct *m, dummy_map;
-  int x=0, y=0, removed=QUERY_FLAG(op,FLAG_REMOVED);
-  char *newpath=NULL, *lastlevel=NULL;
-  int oldx = (-1), oldy = (-1), last_x = op->x, last_y = op->y;
+    dummy_map.pending = (objectlink *) NULL;
+    dummy_map.in_memory = MAP_LOADING;
 
-  dummy_map.pending = (objectlink *) NULL;
-  dummy_map.in_memory = MAP_LOADING;
+    /* First, lets figure out what map the player is going to go to */
+    if (exit_ob) {
+	x=EXIT_X(exit_ob);
+	y=EXIT_Y(exit_ob);
 
-  if (exit_ob) {
-    x=EXIT_X(exit_ob);
-    y=EXIT_Y(exit_ob);
-
-    if (EXIT_PATH(exit_ob)) {
-      newpath = normalize_path (exit_ob->map->path, EXIT_PATH(exit_ob));
-    } else {
-      if (EXIT_LEVEL (exit_ob) > 0)
+	if (EXIT_PATH(exit_ob)) {
+	    newpath = normalize_path (exit_ob->map->path, EXIT_PATH(exit_ob));
+	} else {
+	    if (EXIT_LEVEL (exit_ob) > 0)
 		LOG(llevError,"Number Map levels are no more supported\n");
+	    return;
+	}
+    } else
+	newpath = op->contr->maplevel;
+
+
+    /* If no map path has been found yet, just keep the player on the
+     * map he is on right now
+     */
+    if(!newpath)
+	newpath = op->map->path;
+
+    /* If the exit is marked unique, this is a special 'apartment' map -
+     * a unique one for each player.
+     */
+    if (op->type==PLAYER && exit_ob && QUERY_FLAG(exit_ob, FLAG_UNIQUE)) {
+	sprintf(apartment, "%s/%s/%s/%s", settings.libdir,
+		settings.playerdir, op->name, clean_path(newpath));
+
+	/* If the map is in memory or the destination (already loaded then
+         * saved) map exsits, then set the path appropriately and mark
+	 * this a unique map.  Otherwise, we fall through below where
+	 * depending on values where it checks for the original map again
+	 */
+	if (has_been_loaded(apartment) && check_path(apartment,0)!= -1) {
+	    newpath = apartment;
+	    notunique=0;
+	}
     }
 
-  } else
-    newpath = op->contr->maplevel;
+    /* If notunique has been cleared, then the stuff above has already
+     * found a map, so no need to check again.  Otherwise, check
+     * to see if the actual file exists.
+     * We check to see if a unique version also exists - needed if the
+     * player saves on the unique map.
+     */
+    if (notunique && !has_been_loaded(newpath) && 
+      check_path(newpath,notunique) == -1) {
+	if (check_path(newpath, 0) == -1) {
+	    new_draw_info_format(NDI_UNIQUE, 0,op, "The %s is closed.", newpath);
+	    return;
+	}
+	else {
+	    notunique=0;
+	}
+    }
 
+    /* Clear the player's count, and reset direction */
+    op->direction=0;
+    op->contr->count=0;
+    op->contr->count_left=0;
 
-  /* If no map path has been found yet, just keep the player on the
-   * map he is on right now
-   */
-  if(!newpath)
-    newpath = op->map->path;
-
-  /* See if the map actually exists */
-  if (!has_been_loaded(newpath) && check_path(newpath) == -1) {
- 	sprintf(buf,"The %s is closed.", exit_ob->name);
-	new_draw_info(NDI_UNIQUE, 0,op, buf);
-	return;
-  }
-  /* Clear the player's count, and reset direction */
-  op->direction=0;
-  op->contr->count=0;
-  op->contr->count_left=0;
-
-  /* For exits that cause damages (like pits) */
-  if(exit_ob && exit_ob->stats.dam && op->type==PLAYER)
+    /* For exits that cause damages (like pits) */
+    if(exit_ob && exit_ob->stats.dam && op->type==PLAYER)
        hit_player(op,exit_ob->stats.dam,exit_ob,exit_ob->attacktype);
 
-  /* Keep track of the map the player is on right now */
-  m=op->map;
+    /* Keep track of the map the player is on right now */
+    oldmap=op->map;
 
 
-  /* When logging in, the object is already removed.  If that is
-   * not the case, then the following is used */
-  if(!removed) {
-    remove_ob(op);
+    /* When logging in, the object is already removed.  If that is
+     * not the case, then the following is used */
+    if(!removed) {
+	remove_ob(op);
 
-    lastlevel = m->path;
+	lastlevel = oldmap->path;
+	op->map = NULL;
 
-    op->map = NULL;
+	/* If we are changing maps, do some extra logic.  In theory,
+	 * if the player is going to the same map, we could do this much
+	 * simply and just update the player x,y
+	 */
+	if(strcmp (newpath, oldmap->path)) {
+	    /* Remove any golems */
+	    if(op->contr->golem) {
+		remove_friendly_object(op->contr->golem);
+		remove_ob(op->contr->golem);
+		free_object(op->contr->golem);
+		op->contr->golem=NULL;
+	    }
 
-    if(strcmp (newpath, m->path)) {
-
-      /* Remove any golems */
-      if(op->contr->golem) {
-        remove_friendly_object(op->contr->golem);
-        remove_ob(op->contr->golem);
-        free_object(op->contr->golem);
-        op->contr->golem=NULL;
-      }
-
-      /* This stuff should probably be moved to ready_map: */
-      op->map = &dummy_map;
+	    /* This stuff should probably be moved to ready_map: */
+	    op->map = &dummy_map;
 
 #if MAP_MAXTIMEOUT
-      m->timeout = MAP_TIMEOUT(m);
-      if (m->timeout > MAP_MAXTIMEOUT) {
-	  m->timeout = MAP_MAXTIMEOUT;
-      }
-      swap_below_max (newpath);
-      /* Swap out the oldest map if low on mem */
+	    oldmap->timeout = MAP_TIMEOUT(oldmap);
+	    /* Do MINTIMEOUT first, so that MAXTIMEOUT is used if that is
+	     * lower than the min value.
+	     */
+#if MAP_MINTIMEOUT
+	    if (oldmap->timeout < MAP_MINTIMEOUT) {
+		oldmap->timeout = MAP_MINTIMEOUT;
+	    }
+#endif
+	    if (oldmap->timeout > MAP_MAXTIMEOUT) {
+		oldmap->timeout = MAP_MAXTIMEOUT;
+	    }
+	    /* Swap out the oldest map if low on mem */
+	    swap_below_max (newpath);
 #else
-      swap_map(m);
-#endif
-
+	    /* save out the map */
+	    swap_map(oldmap);
+#endif /* MAP_MAXTIMEOUT */
+	}
+	oldmap->players--;
     }
-    m->players--;
-  }
-  /* Do any processing to get the map loaded and ready */
-  op->map = ready_map_name(newpath,0);
 
-/*LOG(llevDebug, "enter_exit: %s\n", newpath); */
-
-  if (op->map==NULL) { /* Something went wrong, try to go back */
-    int x2,y2;
-    object *enc;
-    LOG(llevError,"Error, couldn't open map %s.\n", newpath);
-    op->map = ready_map_name (lastlevel, 0);
-    x = last_x;
-    y = last_y;
-    oldx = op->x, oldy = op->y;
-    LOG(llevDebug, "Trying to remove all entries to the map.\n");
-    for (x2 = (-2); x2 < 3; x2++)
-      for (y2 = (-2); y2 < 3; y2++) {
-        if (out_of_map(op->map, x2 + op->x, y2 + op->y))
-          continue;
-        enc = get_map_ob(op->map, x2 + op->x, y2 + op->y);
-        if (!enc)
-          continue;
-        if (enc->type != ENCOUNTER || enc->slaying == NULL ||
-            strcmp(enc->slaying, newpath))
-          continue;
-        free_string(enc->slaying);
-        enc->slaying = (char *) NULL;
-      }
-  } 
-#if 0	/* Are these lines really needed?  What map a player is on can
-	 * be found out with the maps command anyway.
-	 */
-  else if(exit_ob!=NULL && exit_ob->name) {
-    new_draw_info_format(NDI_UNIQUE | NDI_ALL, 0, op,
-	"%s enters %s.",op->name,exit_ob->name);
-  } else {
-    new_draw_info_format(NDI_UNIQUE | NDI_ALL, 0, op,   
-	"%s teleports to level %s.",op->name,
-              op->map->map_object->name);
-  }
-#endif
-/*  new_draw_info(NDI_UNIQUE, 0,op, buf);*/
-  op->map->players++;
+    /* Do any processing to get the map loaded and ready */
+    op->map = ready_map_name(newpath,(notunique?0:MAP_PLAYER_UNIQUE));
 
 
-  /* If objects are being updated while the map is loading, things like
-   * pets will be put into the players map, pending element.  That is
-   * really just the dummy_map.  Since the map has actually loaded now,
-   * we want to append that list to the 'real' loaded maps pending
-   * list -- MSW
-   */
-  if(dummy_map.pending != (objectlink *) NULL) {
-    objectlink *obl;
+    /* Did the load fail for some reason?  If so, put the player back on the
+     * map they came from and remove the exit that pointed the player to
+     * the bad map.
+     */
+    if (op->map==NULL) { /* Something went wrong, try to go back */
+	int x2,y2;
+	object *enc;
+
+	LOG(llevError,"Error, couldn't open map %s.\n", newpath);
+	op->map = ready_map_name (lastlevel, 0);
+	x = last_x;
+	y = last_y;
+	LOG(llevDebug, "Trying to remove all entries to the map.\n");
+
+	for (x2 = (-2); x2 < 3; x2++) {
+	    for (y2 = (-2); y2 < 3; y2++) {
+		if (out_of_map(op->map, x2 + op->x, y2 + op->y))
+		    continue;
+		enc = get_map_ob(op->map, x2 + op->x, y2 + op->y);
+		if (!enc)
+		    continue;
+		if (enc->type != ENCOUNTER || enc->slaying == NULL ||
+		    strcmp(enc->slaying, newpath))
+			continue;
+		free_string(enc->slaying);
+		enc->slaying = (char *) NULL;
+	    }
+	}
+    }
+    /* If we got the map we wanted and it is UNIQUE, we need to update
+     * it so it gets saved in the right place.  Set unique so that
+     * when we save it, it knows to save it in the right place
+     */
+    else if (exit_ob && QUERY_FLAG(exit_ob, FLAG_UNIQUE)) {
+	strcpy(op->map->path, apartment);
+	SET_FLAG(op->map->map_object, FLAG_UNIQUE);
+    }
+	    
+    op->map->players++;
+
+    /* If objects are being updated while the map is loading, things like
+     * pets will be put into the players map, pending element.  That is
+     * really just the dummy_map.  Since the map has actually loaded now,
+     * we want to append that list to the 'real' loaded maps pending
+     * list -- MSW
+     */
+    if(dummy_map.pending != (objectlink *) NULL) {
+	objectlink *obl;
+
 	for(obl = op->map->pending; obl!= NULL && obl->next != NULL; 
 	    obl=obl->next);
-    if(obl == NULL)
-      op->map->pending = dummy_map.pending;
-    else
-      obl->next = dummy_map.pending;
-  }
+	if(obl == NULL)
+	    op->map->pending = dummy_map.pending;
+	else
+	    obl->next = dummy_map.pending;
+    }
 
-  op->contr->loading = op->map;
-  if (oldx != (-1)) {
-    op->contr->new_x = oldx;
-    op->contr->new_y = oldy;
-  } else {
+    op->contr->loading = op->map;
     op->contr->new_x = x;
     op->contr->new_y = y;
-  }
-  op->contr->removed = removed;
-  op->map->timeout = 0;
+    op->contr->removed = removed;
+    op->map->timeout = 0;
 
-/* I don't think, at least the way things are now, that this can ever
- * happen.
- */
-  if(op->map->in_memory == MAP_LOADING) {
-    LOG(llevDebug, "Players map is still loading!?\n");
-    return; /* This will block the player until map has been loaded */
-  }
 #ifdef USE_LIGHTING
-  if(op->lights||op->glow_radius>0) remove_carried_lights(op,m);
+    if(op->lights||op->glow_radius>0) remove_carried_lights(op,oldmap);
 #endif
-  enter_map(op);
+    enter_map(op);
 }
 
 /*
@@ -514,7 +536,7 @@ void process_map(mapstruct *map) {
  * I moved it out of process events and process_map.  This was to some
  * extent for debugging as well as to get a better idea of the time used
  * by the various functions.  process_players1() does the processing before
- * objects have been updated, procesS_players2() does the processing that
+ * objects have been updated, process_players2() does the processing that
  * is needed after the players have been updated.
  */
 
@@ -542,15 +564,8 @@ void process_players1(mapstruct *map)
 	    }
 #endif
 	    if(pl->ob->speed_left>0) {
-		if (pl->eric_server>0) {
-		    if (handle_newcs_player(pl->ob))
-			flag=1;
-		}
-		else {
-		    if(--(pl->ob->speed_left)>0) /* Another loop needed */
-			flag=1;
-		    handle_player(pl->ob);
-		}
+		if (handle_newcs_player(pl->ob))
+		    flag=1;
 	    } /* end if player has speed left */
 	} /* end of for loop for all the players */
     } /* for flag */
@@ -586,10 +601,6 @@ void process_players2(mapstruct *map)
 	    if (pl->ob->map!=map) continue;
 	}
 
-#ifdef SOUND_EFFECTS
-	if (pl->play_count > 0)
-	    pl->play_count--;
-#endif
 	if(pl->has_hit) {
 	    float hit_speed = pl->ob->speed/pl->weapon_sp - pl->ob->speed;
 
@@ -602,19 +613,6 @@ void process_players2(mapstruct *map)
 	else if (pl->ob->speed_left>pl->ob->speed)
 	    pl->ob->speed_left = pl->ob->speed;
     }
-
-/* Then synchronize the different screens to avoid jerky movement */
-    for(pl=first_player;pl!=NULL;pl=pl->next) {
-	if (pl->eric_server > 0) {
-	    continue;
-	}
-	if (pl->sync && ++pl->cur_sync >= pl->sync) {
-	    XSync(pl->gdisp,False);
-	    pl->cur_sync = 0;
-	}
-	else
-	    XFlush(pl->gdisp);
-    }
 }
 
 #define SPEED_DEBUG
@@ -687,11 +685,8 @@ void process_events(mapstruct *map) {
 
 void clean_tmp_files() {
   mapstruct *m;
-  player *pl;
 
   LOG(llevError,"Cleaning up...\n");
-  for(pl=first_player;pl!=NULL;pl=pl->next)
-    remove_lock(pl);
 
   /* We save the maps - it may not be intuitive why, but if there are unique
    * items, we need to save the map so they get saved off.  Perhaps we should
@@ -712,66 +707,11 @@ void clean_tmp_files() {
     clean_tmp_map(m);
 #endif
   }
-/* I don't think we should care about lockfiles being cleaned up
- * if recycling the temp maps.  Not sure, however.
- */
-#if defined(UNIQUE_ITEMS) && defined(LOCK_ITEMS)
-  clean_lockfiles();
-#endif
 }
 
-
-
-void leave(player *pl) {
-  char buf[MAX_BUF];
-
-  /* Lets free this before you close the connection */
-  if (pl->pixmaps) free_pixmaps(pl->gdisp, pl->pixmaps);
-  if (pl->masks) free_pixmaps(pl->gdisp, pl->masks);
-
-
-  if (pl->eric_server > 0)
-    esrv_remove_player(pl->eric_server);
-  else {
-    XSync(pl->gdisp, False);
-    /* Free all the GCs we made */
-    if(pl->gc_root)
-  	XFreeGC(pl->gdisp, pl->gc_root);
-    if(pl->gc_game)
-  	XFreeGC(pl->gdisp, pl->gc_game);
-    if(pl->gc_stats)
-  	XFreeGC(pl->gdisp, pl->gc_stats);
-    if(pl->gc_info)
-  	XFreeGC(pl->gdisp, pl->gc_info);
-    if(pl->gc_inv_text)
-  	XFreeGC(pl->gdisp, pl->gc_inv_text);
-    if(pl->gc_inv_icon)
-  	XFreeGC(pl->gdisp, pl->gc_inv_icon);
-    if(pl->gc_look_text)
-  	XFreeGC(pl->gdisp, pl->gc_look_text);
-    if(pl->gc_look_icon)
-  	XFreeGC(pl->gdisp, pl->gc_look_icon);
-    if(pl->gc_message)
-  	XFreeGC(pl->gdisp, pl->gc_message);
-    if(pl->gc_xpm_floor)
-  	XFreeGC(pl->gdisp, pl->gc_xpm_floor);
-    if(pl->gc_xpm_object)
-  	XFreeGC(pl->gdisp, pl->gc_xpm_object);
-    XDestroyWindow(pl->gdisp,pl->win_game);
-    XCloseDisplay(pl->gdisp);
-  }
-
-  (void) sprintf(buf,"%s left the game.",pl->name);
-  if (pl->ob->map->in_memory==MAP_IN_MEMORY)
-	pl->ob->map->timeout = MAP_TIMEOUT(pl->ob->map);
-  pl->ob->map->players--;
-  pl->ob->map=NULL;
-  pl->ob->type = DEAD_OBJECT; /* To avoid problems with inventory window */
-  free_player(pl);
-#ifdef SERVER
-  if(settings.servermode != SERVER_ENABLED)
-#endif
-  if(first_player==NULL) { /* Last player left the game */
+/* clean up everything before exiting */
+void cleanup()
+{
     clean_tmp_files();
     write_book_archive();
     free_all_object_data();
@@ -785,8 +725,28 @@ void leave(player *pl) {
     /* See what the string data that is out there that hasn't been freed. */
 /*    LOG(llevDebug, ss_dump_table(0xff));*/
     exit(0);
-  }
-  new_draw_info(NDI_UNIQUE | NDI_ALL, 5, NULL, buf);
+}
+
+void leave(player *pl) {
+    char buf[MAX_BUF];
+
+    if (pl!=NULL) {
+	/* We do this so that the socket handling routine can do the final
+	 * cleanup.  We also leave that loop to actually handle the freeing
+	 * of the data.
+	 */
+	pl->socket.status=Ns_Dead;
+
+	(void) sprintf(buf,"%s left the game.",pl->ob->name);
+	if (pl->ob->map) {
+	    if (pl->ob->map->in_memory==MAP_IN_MEMORY)
+		pl->ob->map->timeout = MAP_TIMEOUT(pl->ob->map);
+	    pl->ob->map->players--;
+	    pl->ob->map=NULL;
+	}
+	pl->ob->type = DEAD_OBJECT; /* To avoid problems with inventory window */
+    }
+  if (pl!=NULL) new_draw_info(NDI_UNIQUE | NDI_ALL, 5, NULL, buf);
 }
 
 int forbid_play()
@@ -806,6 +766,7 @@ int forbid_play()
 	return 0;
 
     while (fgets (buf, MAX_BUF, fp)) {
+	if (buf[0]=='#') continue;
 	if (!strncmp (buf, "msg", 3)) {
 	    if (forbit)
 		while (fgets (buf, MAX_BUF, fp))  /* print message */
@@ -845,21 +806,12 @@ void do_specials() {
   special_count = 0;
   special2++;
   flush_old_maps();    /* Clears the tmp-files of maps which have reset */
-#if 0
-#ifdef DEBUG_MALLOC_LEVEL
-  if (!malloc_verify())
-	LOG(llevError,"Warning - heap is corrupted\n");
-  else
-	LOG(llevDebug,"Heap checks out OK.\n");
-#endif
-#endif
   if(!(special2%5)) {
     fix_weight();        /* Hack to fix weightproblems caused by bugs */
     if (!(special2%25))
       fix_luck();
   }
   if (!(special2%10)) write_book_archive();
-
 }
 
 /*
@@ -883,16 +835,12 @@ int main(int argc,char **argv)
 
   for(;;) {
     nroferrors = 0;
-#ifdef LONGJUMP
-    setjmp(jump_addr);
-#endif
 
-    check_socket();      /* Check for new connections/data */
     doeric_server();
     process_events(NULL);    /* "do" something with objects with speed */
     check_active_maps(); /* Removes unused maps after a certain timeout */
     do_specials();       /* Routines called from time to time. */
 
-    sleep_delta();
+    sleep_delta();	/* Slepp proper amount of time before next tick */
   }
 }
