@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_monster_c =
- *    "$Id: monster.c,v 1.32 1997/03/09 05:12:29 master Exp master $";
+ *    "$Id: monster.c,v 1.40 1998/09/28 03:29:35 master Exp master $";
  */
 
 /*
@@ -33,15 +33,29 @@
 #include <skills.h>
 #endif
 
-#define MIN_MON_RADIUS 3 /* minimum monster detection radius */ 
+/*
+ * When parsing a message-struct, the msglang struct is used
+ * to contain the values.
+ * This struct will be expanded as new features are added.
+ * When things are stable, it will be parsed only once.
+ */
+
+typedef struct _msglang {
+  char **messages;	/* An array of messages */
+  char ***keywords;	/* For each message, an array of strings to match */
+} msglang;
+
 
 extern spell spells[NROFREALSPELLS];
 
+#define MIN_MON_RADIUS 3 /* minimum monster detection radius */ 
+
 object *get_enemy(object *npc) {
-  if ((npc->move_type & HI4) == 16)
+  if ((npc->move_type & HI4) == 16) {
     if (npc->owner != NULL)
       return npc->enemy = npc->owner->enemy;
     else npc->enemy = NULL;
+  }
   if(npc->enemy) {
     if(QUERY_FLAG(npc->enemy,FLAG_REMOVED)||QUERY_FLAG(npc->enemy,FLAG_FREED)
        ||!(RANDOM()%20)||
@@ -117,274 +131,284 @@ int move_randomly(object *op) {
  */
 
 int move_monster(object *op) {
-  int dir=0,diff;
-  object *part, *owner, *enemy = find_enemy(op);
+    int dir=0,diff;
+    object *part, *owner, *enemy = find_enemy(op);
 
 
-  /* Monsters not on maps don't do anything.  These monsters are thins
-   * Like royal guards in city dwellers inventories.
-   */
-  if (!op->map) return 0;
+    /* Monsters not on maps don't do anything.  These monsters are things
+     * Like royal guards in city dwellers inventories.
+     */
+    if (!op->map) return 0;
 
-  if(QUERY_FLAG(op, FLAG_SLEEP)||QUERY_FLAG(op, FLAG_BLIND)
+
+    if(QUERY_FLAG(op, FLAG_SLEEP)||QUERY_FLAG(op, FLAG_BLIND)
 #ifdef USE_LIGHTING  
-    ||((op->map->darkness>0)&&!QUERY_FLAG(op,FLAG_SEE_IN_DARK)
-	&&!QUERY_FLAG(op,FLAG_SEE_INVISIBLE))
+       ||((op->map->darkness>0)&&!QUERY_FLAG(op,FLAG_SEE_IN_DARK)
+	  &&!QUERY_FLAG(op,FLAG_SEE_INVISIBLE))
 #endif
-  ) {
-    if(!check_wakeup(op,enemy))
-      return 0;
-  }
+       ) {
+	if(!check_wakeup(op,enemy))
+	    return 0;
+    }
 
-  /* check if monster pops out of hidden spot */
-  if(op->hide) do_hidden_move(op);
+    /* check if monster pops out of hidden spot */
+    if(op->hide) do_hidden_move(op);
 
-  if(op->pick_up)
-    monster_check_pickup(op);
+    if(op->pick_up)
+	monster_check_pickup(op);
 
-  if(op->will_apply)
-    monster_apply_below(op); /* Check for items to apply below */
-  /*  generate hp, if applicable */
-  if(op->stats.Con&&op->stats.hp<op->stats.maxhp) {
-    /*  last heal is in funny units.  Dividing by speed puts
-        the regeneration rate on a basis of time instead of
-        #moves the monster makes.  The scaling by 8 is
-        to capture 8th's of a hp fraction regens */
-	 op->last_heal+= (int)((float)(8*op->stats.Con)/FABS(op->speed));
-    op->stats.hp+=op->last_heal/32; /* causes Con/4 hp/tick */
-    op->last_heal%=32;
-    if (QUERY_FLAG(op,FLAG_RUN_AWAY) &&
-        op->stats.hp >= (signed short)(((float)op->run_away/(float)100)*
+    if(op->will_apply)
+	monster_apply_below(op); /* Check for items to apply below */
+
+    /*  generate hp, if applicable */
+    if(op->stats.Con&&op->stats.hp<op->stats.maxhp) {
+
+	/* last heal is in funny units.  Dividing by speed puts
+	 * the regeneration rate on a basis of time instead of
+	 * #moves the monster makes.  The scaling by 8 is
+	 * to capture 8th's of a hp fraction regens 
+	 */
+
+	op->last_heal+= (int)((float)(8*op->stats.Con)/FABS(op->speed));
+	op->stats.hp+=op->last_heal/32; /* causes Con/4 hp/tick */
+	op->last_heal%=32;
+
+	/* So if the monster has gained enough HP that they are no longer afraid */
+	if (QUERY_FLAG(op,FLAG_RUN_AWAY) &&
+	    op->stats.hp >= (signed short)(((float)op->run_away/(float)100)*
                         (float)op->stats.maxhp))
-      CLEAR_FLAG(op, FLAG_RUN_AWAY);
-    if(op->stats.hp>op->stats.maxhp)
-      op->stats.hp=op->stats.maxhp;
-  }
-  /* generate sp, if applicable */
-  if(op->stats.Pow&&op->stats.sp<op->stats.maxsp) {
-    /*  last_sp is in funny units.  Dividing by speed puts
-        the regeneration rate on a basis of time instead of
-        #moves the monster makes.  The scaling by 8 is
-        to capture 8th's of a sp fraction regens */
+	    CLEAR_FLAG(op, FLAG_RUN_AWAY);
+
+	if(op->stats.hp>op->stats.maxhp)
+	    op->stats.hp=op->stats.maxhp;
+    }
+
+    /* generate sp, if applicable */
+    if(op->stats.Pow&&op->stats.sp<op->stats.maxsp) {
+
+	/*  last_sp is in funny units.  Dividing by speed puts
+         * the regeneration rate on a basis of time instead of
+         * #moves the monster makes.  The scaling by 8 is
+         * to capture 8th's of a sp fraction regens 
+	 */
+
 	 op->last_sp+= (int)((float)(8*op->stats.Pow)/FABS(op->speed));
-    op->stats.sp+=op->last_sp/128;  /* causes Pow/16 sp/tick */
-    op->last_sp%=128;
-    if(op->stats.sp>op->stats.maxsp)
-      op->stats.sp=op->stats.maxsp;
-  }
-
-  if(QUERY_FLAG(op, FLAG_SCARED)&&!(RANDOM()%20))
-    CLEAR_FLAG(op,FLAG_SCARED); /* Time to regain some "guts"... */
-
-  if(!enemy) {
-    if(QUERY_FLAG(op, FLAG_ONLY_ATTACK)) {
-      remove_ob(op);
-      free_object(op);
-      return 1;
+	op->stats.sp+=op->last_sp/128;  /* causes Pow/16 sp/tick */
+	op->last_sp%=128;
+	if(op->stats.sp>op->stats.maxsp)
+	    op->stats.sp=op->stats.maxsp;
     }
-    if (op->move_type & HI4) {
-      switch (op->move_type & HI4) {
-        case (PETMOVE):
-          pet_move (op);
-          if(QUERY_FLAG(op, FLAG_REMOVED)) {
-            remove_friendly_object(op);
-            free_object(op);
-            return 1;
-          }
-          break;
-        case (CIRCLE1):
-          circ1_move (op);
-          break;
-        case (CIRCLE2):
-          circ2_move (op);
-          break;
-        case (PACEV):
-          pace_movev(op);
-          break;
-        case (PACEH):
-          pace_moveh(op);
-          break;
-        case (PACEV2):
-          pace2_movev (op);
-          break;
-        case (PACEH2):
-          pace2_moveh (op);
-          break;
-        case (RANDO):
-          rand_move (op);
-          break;
-        case (RANDO2):
-          move_randomly (op);
-          break;
-      }
-      if(QUERY_FLAG(op, FLAG_FREED))
-        return 1;
-      return 0;
-    }
-    if (QUERY_FLAG(op,FLAG_RANDOM_MOVE))
-      (void) move_randomly(op);
-    return 0;
-  }
-  /* There is something to attack */
 
-  if((op->type&HI4) == PETMOVE && (owner = get_owner(op)) != NULL &&
-     op->map != owner->map)
-  {
-    follow_owner(op, owner);
-    if(QUERY_FLAG(op, FLAG_REMOVED) && FABS(op->speed) > 0.00001) {
-      remove_friendly_object(op);
-      free_object(op);
-      return 1;
+    if(QUERY_FLAG(op, FLAG_SCARED)&&!(RANDOM()%20))
+	CLEAR_FLAG(op,FLAG_SCARED); /* Time to regain some "guts"... */
+
+    /* If we don't have an enemy, do special movement or the like */
+    if(!enemy) {
+	if(QUERY_FLAG(op, FLAG_ONLY_ATTACK)) {
+	    remove_ob(op);
+	    free_object(op);
+	    return 1;
+	}
+	if (op->move_type & HI4) {
+	    switch (op->move_type & HI4) {
+		case (PETMOVE):
+		    pet_move (op);
+		    if(QUERY_FLAG(op, FLAG_REMOVED)) {
+			remove_friendly_object(op);
+			free_object(op);
+			return 1;
+		    }
+		    break;
+		case (CIRCLE1):
+		    circ1_move (op);
+		    break;
+		case (CIRCLE2):
+		    circ2_move (op);
+		    break;
+		case (PACEV):
+		    pace_movev(op);
+		    break;
+		case (PACEH):
+		    pace_moveh(op);
+		    break;
+		case (PACEV2):
+		    pace2_movev (op);
+		    break;
+		case (PACEH2):
+		    pace2_moveh (op);
+		    break;
+		case (RANDO):
+		    rand_move (op);
+		    break;
+		case (RANDO2):
+		    move_randomly (op);
+		    break;
+	    } /* move hi */
+	    if(QUERY_FLAG(op, FLAG_FREED))
+		return 1;
+	    return 0;
+	}
+
+	if (QUERY_FLAG(op,FLAG_RANDOM_MOVE))
+	    (void) move_randomly(op);
+	return 0;
+    }
+
+    /* We have an enemy.  Block immediately below is for pets */
+    if((op->type&HI4) == PETMOVE && (owner = get_owner(op)) != NULL &&
+       op->map != owner->map) {
+
+	follow_owner(op, owner);
+	if(QUERY_FLAG(op, FLAG_REMOVED) && FABS(op->speed) > 0.00001) {
+	    remove_friendly_object(op);
+	    free_object(op);
+	    return 1;
+	}
+	return 0;
     }
-    return 0;
-  }
    
-  /* doppleganger code to change monster facing to that of the nearest 
-     player */
-  if( (op->race != NULL)&& strcmp(op->race,"doppleganger") == 0){
-    op->face = enemy->face; 
-    strcpy(op->name,enemy->name);
-  }
+    /* doppleganger code to change monster facing to that of the nearest 
+	player */
+    if( (op->race != NULL)&& strcmp(op->race,"doppleganger") == 0){
+	op->face = enemy->face; 
+	strcpy(op->name,enemy->name);
+    }
 
 #define NEW_MON_ATTACK
 #ifndef NEW_MON_ATTACK
-  part = get_nearest_part(op,enemy);
+    part = get_nearest_part(op,enemy);
 #else
-  for (part=op; part!=NULL; part=part->more) {
+    for (part=op; part!=NULL; part=part->more) {
 #endif
-  dir=find_dir_2(part->x-enemy->x,part->y-enemy->y);
+	dir=find_dir_2(part->x-enemy->x,part->y-enemy->y);
 
-  if(QUERY_FLAG(op,FLAG_IS_TURNING))
-    op->value=(enemy->x>op->x);
+	if(QUERY_FLAG(op, FLAG_SCARED) || QUERY_FLAG(op,FLAG_RUN_AWAY))
+	    dir=absdir(dir+4);
 
+	if(QUERY_FLAG(op,FLAG_CONFUSED))
+	    dir = absdir(dir + RANDOM()%3 + RANDOM()%3 - 2);
 
-  if(QUERY_FLAG(op, FLAG_SCARED) || QUERY_FLAG(op,FLAG_RUN_AWAY))
-    dir=absdir(dir+4);
-
-  if(QUERY_FLAG(op,FLAG_CONFUSED))
-    dir = absdir(dir + RANDOM()%3 + RANDOM()%3 - 2);
-
-  if (!QUERY_FLAG(op, FLAG_SCARED)) {
-    if(QUERY_FLAG(op,FLAG_CAST_SPELL))
-      if(monster_cast_spell(op,part,enemy,dir))
-        return 0;
-    if(QUERY_FLAG(op,FLAG_READY_WAND)&&!(RANDOM()%3))
-      if(monster_use_wand(op,part,enemy,dir))
-        return 0;
-    if(QUERY_FLAG(op,FLAG_READY_ROD)&&!(RANDOM()%4))
-      if(monster_use_rod(op,part,enemy,dir))
-        return 0;
-    if(QUERY_FLAG(op,FLAG_READY_HORN)&&!(RANDOM()%5))
-      if(monster_use_horn(op,part,enemy,dir))
-        return 0;
+	if (!QUERY_FLAG(op, FLAG_SCARED)) {
+	    if(QUERY_FLAG(op,FLAG_CAST_SPELL))
+		if(monster_cast_spell(op,part,enemy,dir))
+		    return 0;
+	    if(QUERY_FLAG(op,FLAG_READY_WAND)&&!(RANDOM()%3))
+		if(monster_use_wand(op,part,enemy,dir))
+		    return 0;
+	    if(QUERY_FLAG(op,FLAG_READY_ROD)&&!(RANDOM()%4))
+		if(monster_use_rod(op,part,enemy,dir))
+		    return 0;
+	    if(QUERY_FLAG(op,FLAG_READY_HORN)&&!(RANDOM()%5))
+		if(monster_use_horn(op,part,enemy,dir))
+		    return 0;
 #ifdef ALLOW_SKILLS
-    if(QUERY_FLAG(op,FLAG_READY_SKILL)&&!(RANDOM()%3))
-      if(monster_use_skill(op,part,enemy,dir))
-	return 0;
+	    if(QUERY_FLAG(op,FLAG_READY_SKILL)&&!(RANDOM()%3))
+		if(monster_use_skill(op,part,enemy,dir))
+		    return 0;
 #endif
-    if(QUERY_FLAG(op,FLAG_READY_BOW)&&!(RANDOM()%2))
-      if(monster_use_bow(op,part,enemy,dir))
-        return 0;
-  }
+	    if(QUERY_FLAG(op,FLAG_READY_BOW)&&!(RANDOM()%2))
+		if(monster_use_bow(op,part,enemy,dir))
+		    return 0;
+	}
 #ifdef NEW_MON_ATTACK
-  }
-  part = get_nearest_part(op,enemy);
-#endif
-  if ((op->move_type & LO4) && !QUERY_FLAG(op, FLAG_SCARED)) {
-     switch (op->move_type & LO4) {
-     case DISTATT:
-       dir = dist_att (dir,op,enemy,part);
-       break;
-     case RUNATT:
-       dir = run_att (dir,op,enemy,part);
-       break;
-     case HITRUN:
-       dir = hitrun_att(dir,op,enemy);
-       break;
-     case WAITATT:
-       dir = wait_att (dir,op,enemy,part);
-       break;
-     case RUSH:
-     case ALLRUN:
-       break; 
-     case DISTHIT:
-       dir = disthit_att (dir,op,enemy,part);
-       break;
-     case WAIT2:
-       dir = wait_att2 (dir,op,enemy,part);
-       break;
-     default:
-       LOG(llevDebug,"Illegal low mon-move: %d\n",op->move_type & LO4);
-     }
-   } 
-   if (!dir)
-     return (0);
-  if (!QUERY_FLAG(op,FLAG_STAND_STILL))
-  {
-    if(move_object(op,dir)) /* Can the monster move directly toward player? */
-      return 0;
-    if(QUERY_FLAG(op, FLAG_SCARED) || !can_hit(part,enemy) || QUERY_FLAG(op,FLAG_RUN_AWAY))
-    {                                    /* Try move around corners if !close */
-      int maxdiff = (QUERY_FLAG(op, FLAG_ONLY_ATTACK) || RANDOM()&1) ? 1 : 2;
-      for(diff = 1; diff <= maxdiff; diff++)
-      {                                  /* try different detours */
-        int m = 1-(RANDOM()&2);          /* Try left or right first? */
-        if(move_object(op,absdir(dir + diff*m)) ||
-           move_object(op,absdir(dir - diff*m)))
-          return 0;
-      }
     }
-  }
-
-/*
- * Eneq(@csd.uu.se): Patch to make RUN_AWAY or SCARED monsters move a random
- * direction if they can't move away.
- */
-
-  if (!QUERY_FLAG(op, FLAG_ONLY_ATTACK)&&(QUERY_FLAG(op,FLAG_RUN_AWAY)||QUERY_FLAG(op, FLAG_SCARED)))
-    if(move_randomly(op))
-      return 0;
-
-/*
- * Monster can't move...now see if it can hit the player...
- *    Eneq(@csd.uu.se): Added check to handle RUN_AWAY and berzerk attack from
- *    RUN_AWAY, locked in monster.
- */
-  if (!QUERY_FLAG(op, FLAG_FRIENDLY) && enemy == op->enemy) {
-    object *nearest_player = get_nearest_player(op);
-    if (nearest_player && nearest_player != enemy && !can_hit(part,enemy)) {
-      op->enemy = NULL;
-      enemy = nearest_player;
+    part = get_nearest_part(op,enemy);
+#endif
+    if ((op->move_type & LO4) && !QUERY_FLAG(op, FLAG_SCARED)) {
+	switch (op->move_type & LO4) {
+	    case DISTATT:
+		dir = dist_att (dir,op,enemy,part);
+		break;
+	    case RUNATT:
+		dir = run_att (dir,op,enemy,part);
+		break;
+	    case HITRUN:
+		dir = hitrun_att(dir,op,enemy);
+		break;
+	    case WAITATT:
+		dir = wait_att (dir,op,enemy,part);
+		break;
+	    case RUSH:
+	    case ALLRUN:
+		break; 
+	    case DISTHIT:
+		dir = disthit_att (dir,op,enemy,part);
+		break;
+	    case WAIT2:
+		dir = wait_att2 (dir,op,enemy,part);
+		break;
+	    default:
+		LOG(llevDebug,"Illegal low mon-move: %d\n",op->move_type & LO4);
+	}
     }
-  }
+    if (!dir)
+	return 0;
 
-  if(!QUERY_FLAG(op, FLAG_SCARED)&&can_hit(part,enemy))
-  {
-    if(QUERY_FLAG(op,FLAG_RUN_AWAY))
-    {
-      signed char tmp = (signed char)((float)part->stats.wc*(float)2);
-      part->stats.wc+=tmp;
+    if (!QUERY_FLAG(op,FLAG_STAND_STILL)) {
+	if(move_object(op,dir)) /* Can the monster move directly toward player? */
+	    return 0;
+	if(QUERY_FLAG(op, FLAG_SCARED) || !can_hit(part,enemy) 
+	   || QUERY_FLAG(op,FLAG_RUN_AWAY)) {
+
+	    /* Try move around corners if !close */
+	    int maxdiff = (QUERY_FLAG(op, FLAG_ONLY_ATTACK) || RANDOM()&1) ? 1 : 2;
+	    for(diff = 1; diff <= maxdiff; diff++) {
+		/* try different detours */
+		int m = 1-(RANDOM()&2);          /* Try left or right first? */
+		if(move_object(op,absdir(dir + diff*m)) ||
+		   move_object(op,absdir(dir - diff*m)))
+		return 0;
+	    }
+	}
+    } /* if monster is not standing still */
+
+    /*
+     * Eneq(@csd.uu.se): Patch to make RUN_AWAY or SCARED monsters move a random
+     * direction if they can't move away.
+     */
+    if (!QUERY_FLAG(op, FLAG_ONLY_ATTACK)&&(QUERY_FLAG(op,FLAG_RUN_AWAY)||QUERY_FLAG(op, FLAG_SCARED)))
+	if(move_randomly(op))
+	    return 0;
+
+    /*
+     * Monster can't move...now see if it can hit the player...
+     *    Eneq(@csd.uu.se): Added check to handle RUN_AWAY and berzerk attack from
+     *    RUN_AWAY, locked in monster.
+     */
+    if (!QUERY_FLAG(op, FLAG_FRIENDLY) && enemy == op->enemy) {
+	object *nearest_player = get_nearest_player(op);
+	if (nearest_player && nearest_player != enemy && !can_hit(part,enemy)) {
+	    op->enemy = NULL;
+	    enemy = nearest_player;
+	}
+    }
+
+    if(!QUERY_FLAG(op, FLAG_SCARED)&&can_hit(part,enemy)) {
+	if(QUERY_FLAG(op,FLAG_RUN_AWAY)) {
+	    signed char tmp = (signed char)((float)part->stats.wc*(float)2);
+	    part->stats.wc+=tmp;
 #ifdef ALLOW_SKILLS
-      (void)skill_attack(enemy,part,0,NULL);
+	    (void)skill_attack(enemy,part,0,NULL);
 #else
-      (void)attack_ob(enemy,part);
+	    (void)attack_ob(enemy,part);
 #endif
-      part->stats.wc-=tmp;
-    } else
+	    part->stats.wc-=tmp;
+	} else
 #ifdef ALLOW_SKILLS
-      (void)skill_attack(enemy,part,0,NULL);
+	    (void)skill_attack(enemy,part,0,NULL);
 #else
-      (void) attack_ob(enemy,part);
+	    (void) attack_ob(enemy,part);
 #endif
-  }
-  if(QUERY_FLAG(part,FLAG_FREED))    /* Might be freed by ghost-attack or hit-back */
-    return 1;
-  if(QUERY_FLAG(op, FLAG_ONLY_ATTACK)) {
-    remove_ob(op);
-    free_object(op);
-    return 1;
-  }
-  return 0;
+    } /* if monster is in attack range */
+
+    if(QUERY_FLAG(part,FLAG_FREED))    /* Might be freed by ghost-attack or hit-back */
+	return 1;
+    if(QUERY_FLAG(op, FLAG_ONLY_ATTACK)) {
+	remove_ob(op);
+	free_object(op);
+	return 1;
+    }
+    return 0;
 }
 
 int can_hit(object *ob1,object *ob2) {
@@ -558,7 +582,7 @@ int monster_use_wand(object *head,object *part,object *pl,int dir) {
     return 0;
   }
   if(wand->stats.food<=0) {
-    apply(head,wand);
+    apply(head,wand,0);
     CLEAR_FLAG(head, FLAG_READY_WAND);
     if (wand->arch) {
       CLEAR_FLAG(wand, FLAG_ANIMATE);
@@ -657,7 +681,7 @@ int monster_use_bow(object *head, object *part, object *pl, int dir) {
   }
   if((arrow=find_arrow(head,bow->race)) == NULL) {
     /* Out of arrows */
-    apply(head,bow);
+    apply(head,bow,0);
     CLEAR_FLAG(head, FLAG_READY_BOW);
     return 0;
   }
@@ -668,7 +692,7 @@ int monster_use_bow(object *head, object *part, object *pl, int dir) {
   arrow->speed = 1;
   update_ob_speed(arrow);
   arrow->speed_left=0;
-  arrow->face=&new_faces[arrow->arch->faces[dir]];
+  SET_ANIMATION(arrow, dir);
   arrow->stats.sp = arrow->stats.wc; /* save original wc and dam */
   arrow->stats.hp = arrow->stats.dam; 
   arrow->stats.dam+= (QUERY_FLAG(bow, FLAG_NO_STRENGTH) ? 0 : head->level)
@@ -692,13 +716,13 @@ int check_good_weapon(object *who, object *item) {
       break;
   if(other_weap==NULL) /* No other weapons */
     return 1;
-  if (!apply(who,item)) {
+  if (!apply(who,item,0)) {
     LOG(llevMonster,"Can't wield %s(%d).\n",item->name,item->count);
     return 0;
   }
   if(who->stats.dam < prev_dam && !QUERY_FLAG(other_weap,FLAG_FREED)) {
     /* New weapon was worse.  (Note ^: Could have been freed by merging) */
-    if (!apply(who,other_weap))
+    if (!apply(who,other_weap,0))
       LOG(llevMonster,"Can't rewield %s(%d).\n",item->name,item->count);
     return 0;
   }
@@ -714,13 +738,13 @@ int check_good_armour(object *who, object *item) {
       break;
   if (other_armour == NULL) /* No other armour, use the new */
     return 1;
-  if (!apply(who, item)) {
+  if (!apply(who, item,0)) {
     LOG(llevMonster, "Can't take off %s(%d).\n",item->name,item->count);
     return 0;
   }
   if(who->stats.ac < prev_ac && !QUERY_FLAG(other_armour,FLAG_FREED)) {
     /* New armour was worse. *Note ^: Could have been freed by merging) */
-    if (!apply(who, other_armour))
+    if (!apply(who, other_armour,0))
       LOG(llevMonster,"Can't rewear %s(%d).\n", item->name, item->count);
     return 0;
   }
@@ -736,6 +760,11 @@ int check_good_armour(object *who, object *item) {
  *
  * Eneq(@csd.uu.se): This can now be defined in the archetypes, added code
  * to handle this.
+ *
+ * This function was seen be continueing looping at one point (tmp->below
+ * became a recursive loop.  It may be better to call monster_check_apply
+ * after we pick everything up, since that function may call others which
+ * affect stacking on this space.
  */
 
 void monster_check_pickup(object *monster) {
@@ -824,15 +853,15 @@ void monster_apply_below(object *monster) {
     switch (tmp->type) {
     case HANDLE:
       if (monster->will_apply&1)
-        apply(monster,tmp);
+        apply(monster,tmp,0);
       break;
     case TREASURE:
       if (monster->will_apply&2)
-        apply(monster,tmp);
+        apply(monster,tmp,0);
       break;
     case SCROLL:  /* Ideally, they should wait until they meet a player */
       if (QUERY_FLAG(monster,FLAG_USE_SCROLL))
-        apply(monster,tmp); 
+        apply(monster,tmp,0); 
       break;
     }
   }
@@ -863,7 +892,7 @@ void monster_check_apply(object *mon, object *item) {
         SET_FLAG(mon, FLAG_READY_BOW);
         LOG(llevMonster,"Found correct bow for arrows.\n");
         if(!QUERY_FLAG(bow, FLAG_APPLIED))
-          apply(mon,bow);
+          apply(mon,bow,0);
         break;
       }
   }
@@ -901,7 +930,7 @@ void monster_check_apply(object *mon, object *item) {
     case SKILL:
 #ifdef ALLOW_SKILLS
       if((flag=QUERY_FLAG(mon,FLAG_CAN_USE_SKILL))) {
-        if(!QUERY_FLAG(item,FLAG_APPLIED)) apply(mon,item);
+        if(!QUERY_FLAG(item,FLAG_APPLIED)) apply(mon,item,0);
         if (item->type==SKILL&&present_in_ob(SKILL,mon)!=NULL)
 	  SET_FLAG(mon, FLAG_READY_SKILL);
       }
@@ -919,7 +948,7 @@ void monster_check_apply(object *mon, object *item) {
     if (((!(mon->can_apply&32))&&flag) ||((mon->can_apply&32)&&(!flag))) {
         /* &32 reverses behaviour. See global.h */
         if(!QUERY_FLAG(item,FLAG_APPLIED))
-          apply(mon,item);
+          apply(mon,item,0);
         if (item->type==BOW&&present_in_ob(item->stats.maxsp,mon)!=NULL)
 	  SET_FLAG(mon, FLAG_READY_BOW);
     }
@@ -1113,27 +1142,70 @@ void check_doors(object *op, int x, int y) {
  */
 
 int move_object(object *op, int dir) {
-  int newx = op->x+freearr_x[dir];
-  int newy = op->y+freearr_y[dir];
-  object *tmp;
-  if(blocked_link(op, newx, newy)) /* Not all features from blocked_two yet */
-    return 0;                      /* (Not efficient enough yet) */
-  if(op->more != NULL && !move_object(op->more, dir))
-    return 0;
-  if(op->will_apply&4)
-    check_earthwalls(op,newx,newy);
-  if(op->will_apply&8)
-    check_doors(op,newx,newy);
-  if(op->head)
+    int newx = op->x+freearr_x[dir];
+    int newy = op->y+freearr_y[dir];
+    object *tmp;
+
+    /* 0.94.2 - we need to set the direction for the new animation code.
+     * it uses it to figure out face to use - I can't see it
+     * breaking anything, but it might.
+     */
+    op->direction = dir;
+    if(blocked_link(op, newx, newy)) /* Not all features from blocked_two yet */
+	return 0;                      /* (Not efficient enough yet) */
+    if(op->more != NULL && !move_object(op->more, dir))
+	return 0;
+    if(op->will_apply&4)
+	check_earthwalls(op,newx,newy);
+    if(op->will_apply&8)
+	check_doors(op,newx,newy);
+
+    /* 0.94.1 - I got a stack trace that showed it crash with remove_ob trying
+     * to remove a removed object, and this function was the culprit.  A possible
+     * guess I have is that check_doors above ran into a trap, killing the
+     * monster.
+     *
+     * Unfortunately, it doesn't appear that the calling functions of move_object
+     * deal very well with op being killed, so all this might do is just
+     * migrate the problem someplace else.
+     */
+
+    if (QUERY_FLAG(op, FLAG_REMOVED)) {
+	LOG(llevDebug,"move_object: monster has been removed - will not process further\n");
+	/* Was not successful, but don't want to try and move again */
+	return 1;
+    }
+    if(op->head)
+	return 1;
+
+    remove_ob(op);
+
+    for(tmp = op; tmp != NULL; tmp = tmp->more)
+	tmp->x+=freearr_x[dir], tmp->y+=freearr_y[dir];
+
+    insert_ob_in_map(op, op->map);
     return 1;
-  remove_ob(op);
-  for(tmp = op; tmp != NULL; tmp = tmp->more)
-    tmp->x+=freearr_x[dir], tmp->y+=freearr_y[dir];
-  insert_ob_in_map(op, op->map);
-  return 1;
 }
 
-msglang *parse_message(char *msg) {
+static void free_messages(msglang *msgs) {
+  int messages, keywords;
+
+  if (!msgs)
+    return;
+  for(messages = 0; msgs->messages[messages]; messages++) {
+    if(msgs->keywords[messages]) {
+      for(keywords = 0; msgs->keywords[messages][keywords]; keywords++)
+        free(msgs->keywords[messages][keywords]);
+      free(msgs->keywords[messages]);
+    }
+    free(msgs->messages[messages]);
+  }
+  free(msgs->messages);
+  free(msgs->keywords);
+  free(msgs);
+}
+
+static msglang *parse_message(char *msg) {
   msglang *msgs;
   int nrofmsgs, msgnr, i;
   char *cp, *line, *last;
@@ -1197,25 +1269,7 @@ msglang *parse_message(char *msg) {
   return msgs;
 }
 
-void free_messages(msglang *msgs) {
-  int messages, keywords;
-
-  if (!msgs)
-    return;
-  for(messages = 0; msgs->messages[messages]; messages++) {
-    if(msgs->keywords[messages]) {
-      for(keywords = 0; msgs->keywords[messages][keywords]; keywords++)
-        free(msgs->keywords[messages][keywords]);
-      free(msgs->keywords[messages]);
-    }
-    free(msgs->messages[messages]);
-  }
-  free(msgs->messages);
-  free(msgs->keywords);
-  free(msgs);
-}
-
-void dump_messages(msglang *msgs) {
+static void dump_messages(msglang *msgs) {
   int messages, keywords;
   for(messages = 0; msgs->messages[messages]; messages++) {
     LOG(llevDebug, "@match ");
@@ -1231,12 +1285,13 @@ void communicate(object *op, char *txt) {
   for(i = 0; i < 24; i++)
     if (!out_of_map(op->map, op->x+freearr_x[i], op->y+freearr_y[i]))
       for(npc = get_map_ob(op->map,op->x+freearr_x[i],op->y+freearr_y[i]);
-          npc != NULL; npc = npc->above)
+          npc != NULL; npc = npc->above) {
         if (npc->type == MAGIC_EAR)
           (void) talk_to_wall(npc, txt); /* Maybe exit after 1. success? */
         else
           if (talk_to_npc(npc,txt))
             return; /* Can be crowded */
+      }
 }
 
 int talk_to_npc(object *npc, char *txt) {
@@ -1376,19 +1431,34 @@ int can_detect_enemy (object *op, object *enemy) {
   if(path_to_player(op->head?op->head:op,enemy,0)==0) return 0;
 #endif
 
-  /* opponent is unseen? */
+  /* opponent is unseen? We still have a chance to find them if
+   * they are 1) standing in dark square, 2) hidden or 3) low-quality
+   * invisible (the basic invisibility spell).
+   */
   if(!can_see_enemy(op,enemy)) {
     object *part = get_nearest_part(op,enemy);
     int radius = MIN_MON_RADIUS;
+    int x_dist = abs(part->x-enemy->x), y_dist = abs(part->y-enemy->y);
+    /* This is percentage change of being discovered while standing
+     * *adjacent* to the monster */
+    int hide_discovery = enemy->hide?op->stats.Int/5:-1;
 
+    /* The rest of this is for monsters. Players are on their own for
+     * finding enemies! */
+    if(op->type==PLAYER) return 0;
 
-    /* detect non-hidden (eg dark/invis enemy) */
-    if(!enemy->hide)  
+    /* Quality invisible? Bah, we wont see them w/o SEE_INVISIBLE
+     * flag (which was already checked). Lets get out of here */
+    if(enemy->invisible && (!enemy->contr || !enemy->contr->tmp_invis))
+       return 0;
+
+    /* Determine Detection radii */
+    if(!enemy->hide)  /* to detect non-hidden (eg dark/invis enemy) */
       radius = (op->stats.Wis/5)+1>MIN_MON_RADIUS?(op->stats.Wis/5)+1:MIN_MON_RADIUS;
     else { /* a level/INT/Dex adjustment for hiding */
       object *sk_hide;
-      int bonus = op->level + op->stats.Int;
-      if(enemy->type==PLAYER)
+      int bonus = (op->level/2) + (op->stats.Int/5);
+      if(enemy->type==PLAYER) {
         if((sk_hide = find_skill(enemy,SK_HIDING)))
           bonus -= sk_hide->level;
         else { 
@@ -1396,42 +1466,91 @@ int can_detect_enemy (object *op, object *enemy) {
           make_visible(enemy);
           radius=radius<MIN_MON_RADIUS?MIN_MON_RADIUS:radius;
         }
+      }
       else
         bonus -= enemy->level;
-      radius += (bonus/10);
+
+      radius += bonus/5;
+      hide_discovery += bonus*5;
     }
 
-    /* the stealth adjustment. Only if you are stealthy 
+    /* Radii stealth adjustment. Only if you are stealthy 
      * will you be able to sneak up closer to creatures */ 
     if(QUERY_FLAG(enemy,FLAG_STEALTH)) 
-      radius = radius/2;
-    else /* never less than minimum radii */ 
-      radius=radius<MIN_MON_RADIUS?MIN_MON_RADIUS:radius;
+      radius = radius/2, hide_discovery = hide_discovery/3;
 
-
-    /* sanity check if the enemy is visible */
-    if(!enemy->invisible) {
+#ifdef USE_LIGHTING
+    /* Radii adjustment for enemy standing in the dark */ 
+    if(op->map->darkness>0 && !stand_in_light(enemy)) {
        /* on dark maps body heat can help indicate location with infravision */
-       if(QUERY_FLAG(op,FLAG_SEE_IN_DARK)&&op->map->darkness>0
-         &&(!enemy->immune&AT_FIRE||QUERY_FLAG(enemy,FLAG_UNDEAD))) 
-	radius += op->map->darkness/2;
+       if(QUERY_FLAG(op,FLAG_SEE_IN_DARK)
+         &&(!enemy->immune&AT_FIRE||is_true_undead(enemy))) 
+	 radius += op->map->darkness/2;
+       else
+	 radius -= op->map->darkness/2;
 
        /* op next to a monster (and not in complete darkness) 
-	* the monster should see you. */
-	if(radius<MIN_MON_RADIUS&&op->map->darkness<5) 
+	* the monster should have a chance to see you. */
+	if(radius<MIN_MON_RADIUS && op->map->darkness<5 && (x_dist<=1||y_dist<=1)) 
 	  radius = MIN_MON_RADIUS;
     }
+#endif
 
-    /* now check if enemy detected/found */ 
-    if(abs(part->x-enemy->x)<radius&&abs(part->y-enemy->y)<radius) { 
-	if(enemy->hide) { /* make hidden enemy visible */
+    /* Lets not worry about monsters that have incredible detection
+     * radii, we only need to worry here about things the player can
+     * (potentially) see. */
+    if(radius>5) radius = 5;
+
+    /* Enemy in range! Now test for detection */
+    if(x_dist<=radius||y_dist<=radius) { 
+       /* ah, we are within range, detected? take cases */
+       if(!enemy->invisible) /* enemy in dark squares... are seen! */
+         return 1;
+       else if(enemy->hide||(enemy->contr&&enemy->contr->tmp_invis)) { 
+         /* hidden or low-quality invisible */  
+
+         /* THere is a a small chance each time we check this function 
+          * that we can detect hidden enemy. This means the longer you stay 
+          * near something, the greater the chance you have of being 
+          * discovered. */
+	 if(enemy->hide && (x_dist<=1||y_dist<=1) && (RANDOM()%100<=hide_discovery)
+         ) { 
 	   make_visible(enemy);
 	   /* inform players of new status */
-	   if(enemy->type==PLAYER) 
-	     new_draw_info(NDI_UNIQUE,0, enemy,"You are discovered!");
-        }
-    } else
+	   if(enemy->type==PLAYER && player_can_view(enemy,op)) 
+	     new_draw_info_format(NDI_UNIQUE,0, enemy,
+               "You are discovered by %s!",op->name);
+           return 1; /* detected enemy */ 
+         }
+
+         /* If the hidden/tmp_invis enemy is nearby we excellerate the time of 
+          * becoming unhidden/visible (ie as finding the enemy is easier)
+ 	  * In order to leave actual discovery (invisible=0 state) to
+	  * be handled above (not here) we only decrement so that 
+	  * enemy->invisible>1 is preserved. */
+          if((enemy->invisible-=RANDOM()%(op->stats.Int+2))<1) 
+            enemy->invisible=1;
+
+       }
+
+       /* MESSAGING: SO we didnt find them (wah!), we may warn a 
+	* player that the monster is getting close to discovering them. 
+  	*
+        * We only warn the player if: 
+        *   1) player has los to the monster
+	*   2) random value based on player Int value
+   	*/
+       if(enemy->type==PLAYER 
+	   && (RANDOM()%(enemy->stats.Int+10)> MAX_STAT/2)
+           && player_can_view(enemy,op)
+       ) { 
+         char buf[MAX_BUF];
+         sprintf(buf,"You see %s noticing your position.", query_name(op));
+	 new_draw_info(NDI_UNIQUE,0, enemy, buf);
+       }
+
        return 0;
+    }
   }
   return 1;
 } 
@@ -1485,7 +1604,7 @@ int can_see_enemy (object *op, object *enemy) {
     if(has_carried_lights(enemy)) { 
       if(enemy->hide) { 
 	make_visible(enemy);
-	new_draw_info(NDI_UNIQUE,0, enemy,
+        new_draw_info(NDI_UNIQUE,0, enemy,
 	  "Your light reveals your hidding spot!");
       }
       return 1;
@@ -1496,13 +1615,17 @@ int can_see_enemy (object *op, object *enemy) {
       &&(is_true_undead(looker)==QUERY_FLAG(enemy,FLAG_UNDEAD)))
       return 0;
 
-  }
+  } else if(looker->type==PLAYER) /* for players, a (possible) shortcut */
+      if(player_can_view(looker,enemy)) return 1;
 
 #ifdef USE_LIGHTING
   /* ENEMY IN DARK MAP. Without infravision, the enemy is not seen 
-   * unless they carry a light or stand in light. */
+   * unless they carry a light or stand in light. Darkness doesnt
+   * inhibit the undead per se (but we should give their archs
+   * CAN_SEE_IN_DARK, this is just a safety  */
   if(looker->map->darkness>0&&!stand_in_light(enemy) 
-     &&(!QUERY_FLAG(looker,FLAG_SEE_IN_DARK)||!QUERY_FLAG(looker,FLAG_XRAYS)))
+     &&(!QUERY_FLAG(looker,FLAG_SEE_IN_DARK)||
+        !is_true_undead(looker)||!QUERY_FLAG(looker,FLAG_XRAYS)))
     return 0;
 #endif
 
