@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_move_c =
- *    "$Id: move.c,v 1.14 1997/03/09 05:12:37 master Exp master $";
+ *    "$Id: move.c,v 1.16 1998/09/28 03:30:21 master Exp master $";
  */
 
 /*
@@ -40,9 +40,6 @@ int move_ob(object *op,int dir) {
     return 0;
   }
 
-  if (op->type==PLAYER)
-    op->contr->freeze_look=1;
-
   /* If this is a multipart object and we are dealing with the
    * head, remove the object.  It needs to be removed, otherwise
    * it may block itself from moving to that space (You can't
@@ -100,8 +97,6 @@ int move_ob(object *op,int dir) {
     /* Re insert object if we removed it above */
     if(op->more!=NULL && op->head==NULL)
       insert_ob_in_map(op,op->map);
-    if (op->type==PLAYER)
-      op->contr->freeze_look=0;
     return 0;
   }
 
@@ -128,25 +123,19 @@ int move_ob(object *op,int dir) {
       op->more=tmp;
       if(op->head==NULL)
         insert_ob_in_map(op,op->map);
-      if (op->type==PLAYER)
-	op->contr->freeze_look=0;
       return 0;
     }
     else op->more=tmp;
   }
   op->x+=freearr_x[dir],op->y+=freearr_y[dir];  /* Move this object */
 
-  if (op->type == PLAYER && op->contr->eric_server > 0) {
-    esrv_map_scroll(op->contr->eric_server,
-                  freearr_x[dir],freearr_y[dir]);
-  }
+  esrv_map_scroll(&op->contr->socket, freearr_x[dir],freearr_y[dir]);
 
   /* If head of object (or single part object), reinsert into map. */
   if(op->head==NULL)
     insert_ob_in_map(op,op->map);
 
   if (op->type==PLAYER) {
-    op->contr->freeze_look=0;
     draw_look(op);
   }
   return 1;
@@ -187,11 +176,11 @@ void teleport(object *teleporter,unsigned char tele_type) {
         continue;
       other_teleporter=get_map_ob(teleporter->map,
                                   teleporter->x+i,teleporter->y+j);
-      if(other_teleporter==NULL
-	 || (other_teleporter->type!=tele_type
-	     && (other_teleporter->above==NULL
-		 || other_teleporter->above->type!=tele_type)))
-        continue;
+      while (other_teleporter) {
+	if (other_teleporter->type == tele_type) break;
+	other_teleporter = other_teleporter->above;
+      }
+      if (!other_teleporter) continue;
       altern[nrofalt++]=other_teleporter;
     }
   if(!nrofalt) {
