@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_resurrection_c =
- *   "$Id: resurrection.c,v 1.5 1997/03/09 05:13:57 master Exp master $";
+ *   "$Id: resurrection.c,v 1.7 1998/09/28 03:41:45 master Exp master $";
  */
 
 /*
@@ -50,11 +50,7 @@ void dead_player(object *op)
   char path[MAX_BUF];
 
 /*  set up our paths/strings...  */
-#ifdef SAVE_HOMEDIR
-  sprintf(path,"%s/%s/%s",(char *) getenv("HOME"),settings.playerdir,op->name);
-#else
   sprintf(path,"%s/%s/%s",settings.libdir,settings.playerdir,op->name);
-#endif
 
   strcpy(filename,path);
   strcat(filename,".pl");
@@ -82,16 +78,20 @@ int cast_raise_dead_spell(object *op,int dir,int spell_type, object * corpseobj)
 
 
   if(corpseobj == NULL){
-      /*  First we need to find a corpse, if any.  */
-      for(temp=get_map_ob(op->map,op->x+freearr_x[dir],op->y+freearr_y[dir]);
-          temp!=NULL;
-            temp=temp->above)
-          /* really need a better way to spot corpseobjects */
-          if(temp->type == 157 && temp->immune==262143)  /* corpses immune to everything, so that they cannot be destroyed.  */
-            break;
-	     } else {
-      temp = corpseobj;
-	     }
+    if (out_of_map(op->map,op->x+freearr_x[dir],op->y+freearr_y[dir]))
+	temp=NULL;
+    else {
+	/*  First we need to find a corpse, if any.  */
+	/* If no object, temp will be set to NULL */
+	for(temp=get_map_ob(op->map,op->x+freearr_x[dir],op->y+freearr_y[dir]);
+	    temp!=NULL; temp=temp->above)
+	    /* really need a better way to spot corpseobjects
+	     * corpses immune to everything, so that they cannot be destroyed.
+	     */
+		if(temp->type == CORPSE && temp->immune==262143) 
+		    break;
+    } 
+  } else temp = corpseobj;
 
    if(temp == NULL&&(spell_type==SP_RAISE_DEAD||spell_type==SP_RESURRECTION)){
         new_draw_info(NDI_UNIQUE, 0,op, "You need a body for this spell.");
@@ -187,11 +187,7 @@ int resurrect_player(object *op,char *playername,int rspell)
   int Con;
 
 /*  set up our paths/strings...  */
-#ifdef SAVE_HOMEDIR
-  sprintf(path,"%s/%s/%s",(char *) getenv("HOME"),settings.playerdir,playername);
-#else
   sprintf(path,"%s/%s/%s",settings.libdir,settings.playerdir,playername);
-#endif
 
   strcpy(newname,path);
   strcat(newname,".pl");
@@ -272,11 +268,7 @@ void dead_character(char *name) {
   char buf[MAX_BUF];
   char buf2[MAX_BUF];
 
-#ifdef SAVE_HOMEDIR
-  sprintf(buf,"%s/%s/%s.pl",(char *) getenv("HOME"),settings.playerdir,name);
-#else
   sprintf(buf,"%s/%s/%s.pl",settings.libdir,settings.playerdir,name);
-#endif
   /*  peterm:  create a .dead filename....  ***.pl.dead  */
   strcpy(buf2,buf);
   strcat(buf,".dead");
@@ -290,11 +282,7 @@ void dead_character(char *name) {
 int dead_player_exists(char *name) {
   char buf[MAX_BUF];
 
-#ifdef SAVE_HOMEDIR
-  sprintf(buf,"%s/%s/%s",(char *) getenv("HOME"),settings.playerdir,name);
-#else
   sprintf(buf,"%s/%s/%s",settings.libdir,settings.playerdir,name);
-#endif
   strcat(buf,".pl.dead");
   return !(access(buf,0));
 }
