@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_rune_c =
- *   "$Id: rune.c,v 1.17 1996/12/01 07:59:53 master Exp $";
+ *   "$Id: rune.c,v 1.19 1998/05/16 02:52:54 master Exp $";
  */
 
 /*
@@ -287,14 +287,14 @@ int trap_see(object *op,object *trap) {
 }
 
 int trap_show(object *trap, object *where) {
-  object *tmp2;
+    object *tmp2;
 
-  if(where==NULL) return 0;
-  tmp2=get_archetype("runedet");
-  tmp2->face=&new_faces[trap->arch->faces[0]];
-  tmp2->x=where->x;tmp2->y=where->y;tmp2->map=where->map;
-  insert_ob_in_map(tmp2,where->map);
-  return 1;
+    if(where==NULL) return 0;
+    tmp2=get_archetype("runedet");
+    tmp2->face=&new_faces[GET_ANIMATION(trap, 0)];
+    tmp2->x=where->x;tmp2->y=where->y;tmp2->map=where->map;
+    insert_ob_in_map(tmp2,where->map);
+    return 1;
 
 }
 
