@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_shop_c =
- *   "$Id: shop.c,v 1.33 1996/09/30 03:56:52 master Exp $";
+ *   "$Id: shop.c,v 1.37 1998/09/28 03:42:03 master Exp master $";
  */
 
 /*
@@ -161,11 +161,11 @@ int query_cost(object *tmp, object *who, int flag) {
       float diff;
 
       charisma = who->stats.Cha;  /* used for SK_BARGAINING modification */ 
-      if(who->contr->shoottype == range_skill && who->chosen_skill)
-         if(who->chosen_skill->stats.sp == SK_BARGAINING) { 
-	    charisma = who->stats.Cha + (who->level+2)/3;
-	    if(charisma>30) charisma = 30;
-         }
+
+      if (find_skill(who,SK_BARGAINING)) {
+	charisma = who->stats.Cha + (who->level+2)/3;
+	if(charisma>30) charisma = 30;
+      }
       if (cha_bonus[charisma]<=1.0) {
 	LOG(llevError,"Illegal charisma bonus, %d <=1.0", cha_bonus[charisma]);
 	diff=0.9;	/*pretty bad case */
@@ -307,7 +307,6 @@ int pay_for_item(object *op,object *pl) {
     for (i=0; i<NUM_COINS; i++) {
 	coin_objs[i] = NULL;
     }
-    pl->contr->freeze_inv=1;
 
     /* This hunk should remove all the money objects from the player */
     while ((tmp=present_in_ob(MONEY, pl))!=NULL) {
@@ -322,14 +321,12 @@ int pay_for_item(object *op,object *pl) {
 			pl->name, coins[NUM_COINS-1-i]);
 		    remove_ob(tmp);
 		    coin_objs[i]->nrof += tmp->nrof;
-		    if (pl->contr->eric_server)
-			esrv_del_item(pl->contr->eric_server, tmp->count);
+		    esrv_del_item(pl->contr, tmp->count);
 		    free_object(tmp);
 		}
 		else {
 		    remove_ob(tmp);
-		    if (pl->contr->eric_server)
-			esrv_del_item(pl->contr->eric_server, tmp->count);
+		    esrv_del_item(pl->contr, tmp->count);
 		    coin_objs[i] = tmp;
 		}
 		break;
@@ -378,14 +375,14 @@ int pay_for_item(object *op,object *pl) {
     for (i=0; i<NUM_COINS; i++) {
 	if (coin_objs[i]->nrof) {
 	    object *tmp = insert_ob_in_ob(coin_objs[i], pl);
-	    if (pl->contr->eric_server > 0)
-		esrv_send_item(pl, tmp);
+	    esrv_send_item(pl, tmp);
 	} else
 	    free_object(coin_objs[i]);
     }
+#ifndef REAL_WIZ
     if(QUERY_FLAG(pl,FLAG_WAS_WIZ))
       SET_FLAG(op, FLAG_WAS_WIZ);
-    pl->contr->freeze_inv=0;
+#endif
     fix_player(pl);
     return 1;
 }
@@ -428,9 +425,9 @@ int get_payment2 (object *pl, object *op) {
 	    new_draw_info_format(NDI_UNIQUE, 0, op,
 		"You paid %s for %s.",buf,query_name(op));
 	    tmp=merge_ob(op,NULL);
-	    if (pl->type == PLAYER && pl->contr->eric_server > 0) {
+	    if (pl->type == PLAYER) {
 		if (tmp) {      /* it was merged */
-		    esrv_del_item (pl->contr->eric_server, c);
+		    esrv_del_item (pl->contr, c);
 		    op = tmp;
 		}
 	        esrv_send_item(pl, op);
@@ -443,13 +440,8 @@ int get_payment2 (object *pl, object *op) {
 int get_payment(object *pl) {
   int ret;
 
-  D_LOCK(pl);
-
   ret = get_payment2 (pl, pl->inv);
 
-  D_UNLOCK(pl);
-  if (pl->type == PLAYER && pl->contr->eric_server <= 0)
-	draw_inventory(pl);
   return ret;
 }
 
@@ -479,7 +471,6 @@ void sell_item(object *op, object *pl) {
     identify(op);
     return;
   }
-  D_LOCK(pl);
   for (count=0; coins[count]!=NULL; count++) {
     at = find_archetype(coins[count]);
     if (at==NULL) LOG(llevError, "Could not find %s archetype", coins[count]);
@@ -489,15 +480,13 @@ void sell_item(object *op, object *pl) {
 	tmp->nrof = i/tmp->value;
 	i -= tmp->nrof * tmp->value;
 	tmp = insert_ob_in_ob(tmp, pl);
-	if(pl->contr->eric_server > 0)
-	    esrv_send_item (pl, tmp);
+	esrv_send_item (pl, tmp);
     }
   }
 
   if (i!=0) 
 	LOG(llevError,"Warning - payment not zero: %d\n", i);
 
-  D_UNLOCK(pl);
   new_draw_info_format(NDI_UNIQUE, 0, pl,
 	"You receive %s for %s.",query_cost_string(op,pl,1),
           query_name(op));
@@ -506,126 +495,112 @@ void sell_item(object *op, object *pl) {
 }
 
 
-/* SHOP_LISTINGS code added by Mark Wedel (master@cats.ucsc.edu)
- *
- * I put the code here, because it did seem shop related.  I suppose it
- * could also be put in the apply.c file.
- *
- * Various thoughts:
- *
- * Perhaps the location of the item relative to the menu sign should be given?
- *
- * Perhaps a better ordering/sorting of the objects could be done? (group
- * all chain armor together, and then order by magic plus value, for
- * example.)
- *
- * Tell the price of the item also?
+typedef struct shopinv {
+    char	*item_sort;
+    char	*item_real;
+    uint16	type;
+    uint32	nrof;
+} shopinv;
+
+/* There are a lot fo extra casts in here just to suppress warnings - it
+ * makes it look uglier than it really it.
+ * The format of the strings we get is type:name.  So we first want to
+ * sort by type (numerical) - if the same type, then sort by name.
  */
-
-void shop_listing_more(object *op)
+static int shop_sort(const void *a1, const void *a2)
 {
-	object *tmp,*old;
-	int lines = 7;
+    shopinv *s1 = (shopinv*)a1, *s2= (shopinv*)a2;
 
-	tmp = op->contr->menu->inv;
-        if (tmp == (object *) NULL) {
-                new_draw_info(NDI_UNIQUE, 0,op, "The Shop is currently empty.");
-	        op->contr->state = ST_PLAYING;	/* reset to normal state */
-                free_object(op->contr->menu);
-                op->contr->menu = NULL;
-                return;
-        }
-	/* If the person is using scroll, a blank line is probably better
-         * than clearing the window.  Clearing the window is better for
-         * people that do not use scroll.  Mark Wedel (master@rahul.net)
-         */
-
-	if (op->contr->scroll)
-		new_draw_info(NDI_UNIQUE, 0,op, "");  /* add a blank line for readability */
-	else
-		clear_win_info(op); /* Seems more readable and faster to me.  -Frank */
-
-	if (op->contr->state==ST_MENU_MORE)
-		new_draw_info(NDI_UNIQUE, 0,op, "The Shop contains (continued):");
-	else
-		new_draw_info(NDI_UNIQUE, 0,op, "The Shop contains:");
-	do {
-		/* mark object as being paid, so that '(unpaid)' isn't
-		added to the end -- By definition, an object being
-		viewed in the shop listing is unpaid.
-		*/
-		CLEAR_FLAG(tmp, FLAG_UNPAID);
-		/* Give a little more detail on some items.
-		 * After all, telling the person that there is 5
-		 * rings in the shop isn't especially useful.
-		 */
-		switch (tmp->type) {
-		    case RING:
-		    case AMULET:
-		    case BRACERS:
-		    case BOOTS:
-		    case GLOVES:
-		    case GIRDLE:
-			new_draw_info_format(NDI_UNIQUE, 0, op,
-			"%s %s",query_name(tmp),
-				describe_item(tmp));
-			break;
-		    default:
-			new_draw_info(NDI_UNIQUE, 0,op, query_name(tmp));
-			break;
-		}
-		old = tmp;
-		tmp = tmp->below;
-		remove_ob(old);
-		free_object(old);
-		/* If we have put so many lines of inventory on the
-		 * screen AND there is more to print, ask for them
-		 * to enter a key.
-		 */
-		if (++lines>op->contr->infolines && tmp)  {
-			new_draw_info(NDI_UNIQUE, 0,op,"Press any key to continue list");
-			op->contr->state = ST_MENU_MORE;
-			if (op->contr->eric_server>0)
-				send_query(op->contr->eric_server,CS_QUERY_SINGLECHAR,"");
-			return;	/* let the routine in player.c handle the rest */
-		}
-
-	} while (tmp);
-	free_object(op->contr->menu);
-	op->contr->menu = NULL;
-	op->contr->state = ST_PLAYING;	/* reset to normal state */
+    if (s1->type<s2->type) return -1;
+    if (s1->type>s2->type) return 1;
+    /* the type is the same (what atoi gets), so do a strcasecmp to sort
+     * via alphabetical order
+     */
+    return strcasecmp(s1->item_sort, s2->item_sort);
 }
 
-
 void shop_listing(object *op)
 {
-	int i,j;
-	char *map_mark = (char *) malloc(op->map->mapx * op->map->mapy);
-	object	*tmp,*stack;
+    int i,j,numitems=0,numallocated=0;
+    char *map_mark = (char *) malloc(op->map->mapx * op->map->mapy),
+	buf[MAX_BUF];
+    object	*stack,*tmp,*menu;
+    shopinv	*items;
 
-	memset(map_mark, 0, op->map->mapx * op->map->mapy);
-	magic_mapping_mark(op, map_mark, 3);
+    /* Should never happen, but just in case a monster does apply a sign */
+    if (op->type!=PLAYER) return;
 
-	if (op->contr->menu!=NULL) {
-		LOG(llevError,"Shop listing being called with non null menu point in player\n");
-		return;
-	}
-	op->contr->menu = get_object();
-	for (i=0; i<op->map->mapx; i++)
-	    for (j=0; j<op->map->mapy; j++) {
-		if (map_mark[i + op->map->mapx * j]) {
-		    stack  =get_map_ob(op->map,i,j);
-		    while (stack) {
-			if (QUERY_FLAG(stack, FLAG_UNPAID)) {
-				tmp = get_object();
-				copy_object(stack, tmp);
-				(void) insert_ob_in_ob(tmp, op->contr->menu);
-			}
-			stack = stack->above;
+    new_draw_info(NDI_UNIQUE, 0, op, "\nThe shop contains:");
+
+    memset(map_mark, 0, op->map->mapx * op->map->mapy);
+    magic_mapping_mark(op, map_mark, 3);
+    items=malloc(40*sizeof(shopinv));
+    numallocated=40;
+    menu=get_object();
+
+    /* Find all the appropriate items */
+    for (i=0; i<op->map->mapx; i++) {
+	for (j=0; j<op->map->mapy; j++) {
+	    if (map_mark[i + op->map->mapx * j]) {
+		stack  =get_map_ob(op->map,i,j);
+		while (stack) {
+		    if (QUERY_FLAG(stack, FLAG_UNPAID)) {
+			tmp=get_object();
+			copy_object(stack, tmp);
+			insert_ob_in_ob(tmp, menu);
 		    }
+		    stack = stack->above;
 		}
 	    }
+	}
+    }
+    if (menu->inv == NULL) {
+	new_draw_info(NDI_UNIQUE, 0, op, "The shop is currently empty.\n");
+	free_object(menu);
 	free(map_mark);
-	shop_listing_more(op);
+	free(items);
+	return;
+    }
+    for (tmp=menu->inv; tmp!=NULL; tmp=tmp->below) {
+	if (numitems==numallocated) {
+	    items=realloc(items, sizeof(shopinv)*(numallocated+10));
+	    numallocated+=10;
+	}
+	/* clear unpaid flag so that doesn't come up in query
+	 * string.  We clear nrof so that we can better sort
+	 * the object names.
+         */
+	CLEAR_FLAG(tmp, FLAG_UNPAID);
+	items[numitems].nrof=tmp->nrof;
+	items[numitems].type=tmp->type;
+	switch (tmp->type) {
+	    case RING:
+	    case AMULET:
+	    case BRACERS:
+	    case BOOTS:
+	    case GLOVES:
+	    case GIRDLE:
+		sprintf(buf,"%s %s",query_base_name(tmp),describe_item(tmp));
+	        items[numitems].item_sort = strdup_local(buf);
+		sprintf(buf,"%s %s",query_name(tmp),describe_item(tmp));
+	        items[numitems++].item_real = strdup_local(buf);
+		break;
+
+	    default:
+	        items[numitems].item_sort = strdup_local(query_base_name(tmp));
+	        items[numitems++].item_real = strdup_local(query_name(tmp));
+		break;
+	}
+    }
+    /* Free_object should also free all the objects in the inventory */
+    free_object(menu);
+    qsort(items, numitems, sizeof(shopinv), (int (*)())shop_sort);
+
+    for (i=0; i<numitems; i++) {
+	new_draw_info(NDI_UNIQUE, 0, op, items[i].item_real);
+	free(items[i].item_sort);
+    }
+    free(map_mark);
+    free(items);
 }
 
