@@ -511,7 +511,10 @@ void read_skill_params () {
     LOG(llevDebug,"done.\n");
 }
 
-/* lookup_skill_by_name() - based on look_up_spell_by_name - b.t. */
+/* lookup_skill_by_name() - based on look_up_spell_by_name - b.t.
+ * Given name, we return the index of skill 'string' in the skill
+ * array, -1 if no skill is found.
+ */
 
 int lookup_skill_by_name(char *string) {
   int skillnr=0, nmlen;
@@ -519,32 +522,13 @@ int lookup_skill_by_name(char *string) {
 
   if(!string) return -1;
   
-#if 0 /* IF we do this, you can't ready_skill sense curse - it find sense
-       * magic instead.  MW 8/3/96
-       */
-
-  /* truncate string at first blank space to find skill name. */
-  nmlen = strcspn(string," ");
-  strncpy(name,string,nmlen); 
-#else
   strcpy(name, string);
   nmlen=strlen(name);
-#endif
 
   for (skillnr=0; skillnr<NROFSKILLS; skillnr++) {
 	if (!strncmp(name,skills[skillnr].name,MIN(strlen(skills[skillnr].name),
 	    nmlen))) return skillnr;
   }
-#if 0
-  while(strncmp(name,skills[skillnr].name,MIN(strlen(skills[skillnr].name),
-	nmlen)) && skillnr <(NROFSKILLS-1) )
-  {
-	skillnr++;
-  }
-
-  if(skillnr==(NROFSKILLS)) skillnr=-1; 
-  return skillnr;
-#endif
   return -1;
 }
  
@@ -704,8 +688,7 @@ int init_player_exp(object *pl) {
            insert_ob_in_ob(tmp,pl);
 	   tmp->stats.exp = pl->stats.exp/nrofexpcat;
            exp_ob[j] = tmp;
-           if (pl->contr->eric_server > 0)
-               esrv_send_item(pl, tmp);
+           esrv_send_item(pl, tmp);
            exp_index++;
         }
   } else if (exp_index!=nrofexpcat) { /* situation for old save file */
@@ -722,8 +705,7 @@ int init_player_exp(object *pl) {
               copy_object(exp_cat[j],tmp);
               insert_ob_in_ob(tmp,pl);
 	      exp_ob[i] = tmp; 
-              if (pl->contr->eric_server > 0)
-                  esrv_send_item(pl, tmp);
+              esrv_send_item(pl, tmp);
               exp_index++;
            }
    }
@@ -971,15 +953,14 @@ object *tmp2;
 # endif /* LINKED_SKILL_LIST */
 
      /* now a random change to learn, based on player Int */
-     if(RANDOM()%100>learn_spell[pl->stats.Int])
-       return 2; /* failure :< */
+    if(RANDOM()%100>learn_spell[pl->stats.Int])
+	return 2; /* failure :< */
 
     /* Everything is cool. Give'em the skill */
-     (void) insert_ob_in_ob(tmp,pl);
-     (void) link_player_skill(pl,tmp);  
-     if (pl->contr->eric_server > 0)
-	esrv_send_item(pl, tmp);
-     return 1;
+    (void) insert_ob_in_ob(tmp,pl);
+    (void) link_player_skill(pl,tmp);  
+    esrv_send_item(pl, tmp);
+    return 1;
 }
 
 /* show_skills() - Meant to allow players to examine
@@ -1018,7 +999,9 @@ void show_skills(object *op) {
 #endif
 
 /* print out skills by category */
-  for (i=0;i<=nrofexpcat;i++) { 
+  for (i=0;i<=nrofexpcat;i++) {
+    char Special[100];
+    Special[0]='\0';
     is_first=1;
     if(i==nrofexpcat) i=EXP_NONE; /* skip to misc exp category */
 #ifdef LINKED_SKILL_LIST
@@ -1039,10 +1022,14 @@ void show_skills(object *op) {
 	      int k=(length-15-strlen(tmp_exp->name)); 
 	      char tmpbuf[40];
 	      strcpy(tmpbuf,tmp_exp->name);
-	      while(k>0) {k--; strcat(tmpbuf,".");} 
+	      while(k>0) {k--; strcat(tmpbuf,".");}
               new_draw_info_format(NDI_UNIQUE,0,op,"%slvl:%3d (xp:%d/%d)",
 		tmpbuf,tmp_exp->level,tmp_exp->stats.exp,
 		level_exp(tmp_exp->level+1, op->expmul));
+	      if (strcmp(tmp_exp->name,"physique")==0)
+		{
+		  sprintf(Special,"You can handle %d weapon improvements.",tmp_exp->level/5+5);
+		}
             } else if(i==EXP_NONE) { 
               new_draw_info(NDI_UNIQUE,0,op,"misc.");
 	    } 
@@ -1059,8 +1046,12 @@ void show_skills(object *op) {
 		skills[tmp->stats.sp].name);
 
           new_draw_info(NDI_UNIQUE,0,op,buf);
-      } 
+      }
     }
+    if (Special[0]!='\0')
+      {
+	new_draw_info(NDI_UNIQUE,0,op,Special);
+      }
   }
 }    
 
@@ -1071,36 +1062,45 @@ void show_skills(object *op) {
  */
 
 int use_skill(object *op, char *string) {
-  int sknum=-1;
+    int sknum=-1;
 
-  /* the skill name appears at the begining of the string,
-   * need to reset the string to next word, if it exists. */
-  /* first eat name of skill and then eat any leading spaces */  
-  if(string && (sknum=lookup_skill_by_name(string))>=0) {
-    int len=strlen(skills[sknum].name);
+    /* the skill name appears at the begining of the string,
+     * need to reset the string to next word, if it exists. */
+    /* first eat name of skill and then eat any leading spaces */  
 
-    if(len>=strlen(string)) {
-      *string=0x0;
-    } else {
-        while(len--){ string++;}
-      while(*string==0x20){string++;}
-      }
+    if(string && (sknum=lookup_skill_by_name(string))>=0) {
+	int len;
 
-    if(strlen(string)==0) string = NULL;
+	if (sknum==-1) {
+	    new_draw_info_format(NDI_UNIQUE, 0, op, 
+		 "Unable to find skill by name %s", string);
+	    return;
+	}
 
-#ifdef SKILL_UTIL_DEBUG
-    if(string) LOG(llevDebug,"use_skill() params: \"%s\"\n",string);
-#endif
-  }
+	len=strlen(skills[sknum].name);
+
+	/* All this logic goes and skips over the skill name to find any
+	 * options given to the skill.
+	 */
+	if(len>=strlen(string)) {
+	    *string=0x0;
+	} else {
+	    while(len--){ string++;}
+	    while(*string==0x20){string++;}
+	}
+	if(strlen(string)==0) string = NULL;
+
+    }
    
 #ifdef SKILL_UTIL_DEBUG
   LOG(llevDebug,"use_skill() got skill: %s\n",sknum>-1?skills[sknum].name:"none");
 #endif
+
+   /* Change to the new skill, then execute it. */
+   if(change_skill(op,sknum))
+	if(do_skill(op,op->facing,string)) return 1;
  
-  if(change_skill(op,sknum))
-    if(do_skill(op,op->facing,string)) return 1;
- 
-  return 0;
+    return 0;
 }
 
 
@@ -1124,7 +1124,6 @@ int change_skill(object *pl, int sk_index) {
  	  && QUERY_FLAG(pl->chosen_skill,FLAG_APPLIED)) { /* already have it prepared */ 
             if(pl->type==PLAYER) {
               pl->contr->shoottype = range_skill;
-              draw_stats(pl);
             }
 	    return 1;	
 	}
@@ -1174,8 +1173,6 @@ int change_skill(object *pl, int sk_index) {
                  new_draw_info_format(NDI_UNIQUE, 0,pl, 
 			"Readied skill: %s.",skills[tmp->stats.sp].name);
                  pl->contr->shoottype = range_skill;
-                 draw_stats(pl);
-		 draw_inventory(pl);
             }
             return 1;
   }       
@@ -1242,9 +1239,8 @@ int attack_hth(object *pl, int dir, char *string) {
         CLEAR_FLAG(pl,FLAG_READY_WEAPON);
         fix_player(pl);
         if(pl->type==PLAYER) {
-          new_draw_info(NDI_UNIQUE, 0,pl,"You unweild your weapon in order to attack.");
-          draw_stats(pl);
-          draw_inventory(pl);
+	    new_draw_info(NDI_UNIQUE, 0,pl,"You unweild your weapon in order to attack.");
+	    esrv_update_item(UPD_FLAGS, pl, weapon);
         }
         break;
     }   
@@ -1281,11 +1277,9 @@ int skill_attack (object *tmp, object *pl, int dir, char *string) {
         if((QUERY_FLAG(tmp,FLAG_ALIVE) && tmp->stats.hp>=0)
            || QUERY_FLAG(tmp, FLAG_CAN_ROLL)
            || tmp->type==LOCKED_DOOR ) {
-#ifdef SIMPLE_PARTY_SYSTEM
                 if((pl->type==PLAYER && tmp->type==PLAYER) && (pl->contr->party_number!=-1
                        && pl->contr->party_number==tmp->contr->party_number))
                                 return 0;
-#endif
                 break;
         }
 
@@ -1351,8 +1345,7 @@ int do_skill_attack(object *tmp, object *op, char *string) {
                 tmp2=arch_to_object(skill);
 		insert_ob_in_ob(tmp2,op);
 		(void) link_player_skill(op,tmp2);
-     		if (op->contr->eric_server > 0)
-        	    esrv_send_item(op, tmp2);
+		esrv_send_item(op, tmp2);
 	    }
 	    /* now try to ready the new skill */
 	    if(!change_skill(op,tmp2->stats.sp)) {  /* oh oh, trouble! */
@@ -1370,7 +1363,6 @@ int do_skill_attack(object *tmp, object *op, char *string) {
 			   int dam=hth_damage(tmp,op); 
                            if(dam!=op->stats.dam) { 
 				op->stats.dam=dam;
-				if(op->type==PLAYER) draw_stats(op);
 			   }
 	  }
 
@@ -1391,7 +1383,6 @@ int do_skill_attack(object *tmp, object *op, char *string) {
 #ifdef NO_AUTO_SKILL_SWITCH
 		if (op->contr->shoottype!=oldrange) {
 		    op->contr->shoottype=oldrange;
-		    draw_stats(op);
 		}
 #endif
 	  }
@@ -1548,24 +1539,3 @@ int get_weighted_skill_stats(object *op) {
 
 }
 
-/* find_skill() - looks for the skill and returns a pointer to it if found */
-
-object *find_skill(object *op, int skillnr) {
-  object *tmp;
-#ifdef LINKED_SKILL_LIST
-  objectlink *obl;
-
-  if(op->sk_list!=NULL) {   /* use fast method */ 
-     for(obl=op->sk_list;obl;obl=obl->next) 
-         if(obl->id==skillnr&&obl->ob) return obl->ob;
-  } 
-#endif
-
-  /* *sigh*; we didnt find it. Perhaps because the skill we requested
-   * is an unapplied tool. Lets search entire inventory */
-  for(tmp=op->inv;tmp;tmp=tmp->below) 
-    if(tmp->type==SKILL&&tmp->stats.sp==skillnr) 
-		return tmp;
-
-  return ((object *) NULL); 
-}
