@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_newspells_c =
- *   "$Id: spell_effect.c,v 1.19 1997/03/09 08:22:30 master Exp master $";
+ *   "$Id: spell_effect.c,v 1.26 1998/09/28 04:01:55 master Exp master $";
  */
 
 
@@ -33,9 +33,7 @@
 #include <sproto.h>
 #endif
 #include <spells.h>
-#ifdef SOUND_EFFECTS
 #include <sounds.h>
-#endif
 
 extern object *objects;
 
@@ -46,37 +44,46 @@ extern object *objects;
 #ifdef SPELL_FAILURE_EFFECTS
 void spell_failure(object *op, int failure,int power)
 {  
-  if(failure<= -20&&failure > -40) /* wonder */
+    if(failure<= -20&&failure > -40) /* wonder */
     {
-     new_draw_info(NDI_UNIQUE, 0,op,"Your spell causes an unexpected effect.");
-     cast_cone(op,op,0,10,SP_WOW,spellarch[SP_WOW],0);
+	new_draw_info(NDI_UNIQUE, 0,op,"Your spell causes an unexpected effect.");
+	cast_cone(op,op,0,10,SP_WOW,spellarch[SP_WOW],0);
     }
 
-  else if (failure <= -40&&failure > -60) /* confusion */
-   {
-    new_draw_info(NDI_UNIQUE, 0,op,"Your magic recoils on you!");
-    confuse_player(op,op,99);
-   }
-  else if (failure <= -60&&failure> -80) /* paralysis */
-  {
-    new_draw_info(NDI_UNIQUE, 0,op,"Your magic recoils on you!");
-    paralyze_player(op,op,99);
-  }
-  else if (failure <= -80) /* blast the immediate area */
-  { object *tmp;
-   new_draw_info(NDI_UNIQUE, 0,op,"You lose control of the mana!  The uncontrolled magic blasts you!");
-   tmp=get_archetype("loose_magic");
-   
-/*
-   tmp->level=op->level;
- */
-   tmp->level=SK_level(op);
-   tmp->x=op->x;tmp->y=op->y;
-   tmp->stats.hp+=power/5;  /* increase the area of destruction a little for more powerful spells */
-   tmp->stats.dam=power; /* nasty recoils! */
-   tmp->stats.maxhp=tmp->count; /*??*/
-   insert_ob_in_map(tmp,op->map);
-  }
+    else if (failure <= -40&&failure > -60) /* confusion */
+    {
+	new_draw_info(NDI_UNIQUE, 0,op,"Your magic recoils on you!");
+	confuse_player(op,op,99);
+    }
+    else if (failure <= -60&&failure> -80) /* paralysis */
+    {
+	new_draw_info(NDI_UNIQUE, 0,op,"Your magic recoils on you!");
+	paralyze_player(op,op,99);
+    }
+    else if (failure <= -80) /* blast the immediate area */
+    { 
+	object *tmp;
+	/* Safety check to make sure we don't get any mana storms in scorn */
+	if (blocks_magic(op->map, op->x, op->y)) {
+	    new_draw_info(NDI_UNIQUE, 0, op, "The magic warps and you are turned inside out!");
+	    hit_player(tmp,9999,op,AT_INTERNAL);
+
+	} else {
+	    new_draw_info(NDI_UNIQUE, 0,op,"You lose control of the mana!  The uncontrolled magic blasts you!");
+	    tmp=get_archetype("loose_magic");
+	    tmp->level=SK_level(op);
+	    tmp->x=op->x;tmp->y=op->y;
+
+	    /* increase the area of destruction a little for more powerful spells */
+	    tmp->stats.hp+=ISQRT(power);
+
+	    if (power>25) tmp->stats.dam = 25 + ISQRT(power);
+	    else tmp->stats.dam=power; /* nasty recoils! */
+
+	    tmp->stats.maxhp=tmp->count; /*??*/
+	    insert_ob_in_map(tmp,op->map);
+	}
+    }
 }
 #endif
 
@@ -123,14 +130,25 @@ void prayer_failure(object *op, int failure,int power)
    new_draw_info(NDI_UNIQUE, 0,op,"God smites you!");
 #endif
    
-   cast_mana_storm(op,power);
+   cast_magic_storm(op,get_archetype("god_power"), power);
 
   }
 }
-    
-void cast_mana_storm(object *op, int lvl) {
-  object *tmp=get_archetype("loose_magic"); 
 
+/* Should really just replace all calls to cast_mana_storm to call
+ * cast_magic_storm directly.
+ */
+
+void cast_mana_storm(object *op, int lvl) {
+    object *tmp=get_archetype("loose_magic"); 
+
+    cast_magic_storm(op, tmp, lvl);
+}
+
+    
+void cast_magic_storm(object *op, object *tmp, int lvl) {
+
+    if (!tmp) return;	/* error */
     tmp->level=SK_level(op);
     tmp->x=op->x;tmp->y=op->y; 
     tmp->stats.hp+=lvl/5;  /* increase the area of destruction */
@@ -172,9 +190,7 @@ int recharge(object *op) {
   if(!(RANDOM()%4)) {
     new_draw_info_format(NDI_UNIQUE, 0, op,
 	"The %s vibrates violently, then explodes!",query_name(wand));
-#ifdef SOUND_EFFECTS
     play_sound_map(op->map, op->x, op->y, SOUND_OB_EXPLODE);
-#endif
     spell_effect(SP_DESTRUCTION, op->x, op->y, op->map);
     remove_ob(wand);
     free_object(wand);
@@ -193,153 +209,289 @@ int recharge(object *op) {
   return 1;
 }
 
-#define MAX_ALT 80
+
+/******************************************************************************
+ * Start of polymorph related functions.
+ *
+ * Changed around for 0.94.3 - it will now look through and use all the
+ * possible choices for objects/monsters (before it was teh first 80 -
+ * arbitrary hardcoded limit in this file.)  Doing this will be a bit
+ * slower however - while before, it traversed the archetypes once and
+ * stored them into an array, it will now potentially traverse it
+ * an average of 1.5 times.  This is probably more costly on the polymorph
+ * item function, since it is possible a couple lookups might be needed before
+ * an item of proper value is generated.
+ */
+
+/* polymorph_living - takes a living object (op) and turns it into 
+ * another monster of some sort.  Currently, we only deal with single
+ * space monsters.
+ */
 
 void polymorph_living(object *op) {
-  archetype *altern[MAX_ALT], *at;
-  int nr = 0, x = op->x, y = op->y, choice, friendly;
-  mapstruct *map = op->map;
-  object *tmp, *next, *owner;
+    archetype *at;
+    int nr = 0, x = op->x, y = op->y, numat=0, choice,friendly;
+    mapstruct *map = op->map;
+    object *tmp, *next, *owner;
 
-  if(op->head != NULL || op->more != NULL)
-    return;
-  for(at = first_archetype ; at != NULL; at = at->next)
-    if(QUERY_FLAG((&at->clone),FLAG_MONSTER) == QUERY_FLAG(op, FLAG_MONSTER) &&
-       QUERY_FLAG((&at->clone),FLAG_GENERATOR) == QUERY_FLAG(op,FLAG_GENERATOR) &&
-       at->more == NULL && EDITABLE((&at->clone)))
+    if(op->head != NULL || op->more != NULL)
+	return;
+
+    /* First, count up the number of legal matches */
+    for(at = first_archetype ; at != NULL; at = at->next)
+	if(QUERY_FLAG((&at->clone),FLAG_MONSTER) == QUERY_FLAG(op, FLAG_MONSTER) &&
+	   QUERY_FLAG((&at->clone),FLAG_GENERATOR) == QUERY_FLAG(op,FLAG_GENERATOR) &&
+	   at->more == NULL && EDITABLE((&at->clone)))
     {
-      altern[nr] = at;
-      if(++nr == MAX_ALT)
-        break;
+	numat++;
     }
-  if(!nr)
-    return;
-  choice = RANDOM()%nr;
-  for(tmp = op->inv; tmp != NULL; tmp = next) {
-    next = tmp->below;
-    if(QUERY_FLAG(tmp, FLAG_APPLIED))
-      apply(op,tmp);
-    if(tmp->type == ABILITY) {
-      remove_ob(tmp);
-      free_object(tmp);
+    if (!numat) return;	/* no valid matches? if so, return */
+
+    /* Next make a choice, and loop through until we get to it */
+    choice = RANDOM()%numat;
+    for(at = first_archetype ; at != NULL; at = at->next)
+	if(QUERY_FLAG((&at->clone),FLAG_MONSTER) == QUERY_FLAG(op, FLAG_MONSTER) &&
+	   QUERY_FLAG((&at->clone),FLAG_GENERATOR) == QUERY_FLAG(op,FLAG_GENERATOR) &&
+	   at->more == NULL && EDITABLE((&at->clone)))
+    {
+	if (!choice) break;
+	else choice--;
+    }
+
+    /* Look through the monster.  Unapply anything they have applied, 
+     * and remove any abilities.
+     */
+    for(tmp = op->inv; tmp != NULL; tmp = next) {
+	next = tmp->below;
+	if(QUERY_FLAG(tmp, FLAG_APPLIED))
+	    apply(op,tmp,0);
+	if(tmp->type == ABILITY) {
+	    remove_ob(tmp);
+	    free_object(tmp);
+	}
+    }
+
+    /* Remove the object, preserve some values for the new object */
+    remove_ob(op);
+    owner = get_owner(op);
+    friendly = QUERY_FLAG(op, FLAG_FRIENDLY);
+    if (friendly)
+	remove_friendly_object(op);
+
+    copy_object(&(at->clone),op);
+    if (owner != NULL)
+	set_owner(op,owner);
+    if (friendly) {
+	SET_FLAG(op, FLAG_FRIENDLY);
+	op->move_type = PETMOVE;
+	add_friendly_object(op);
+    }
+
+    /* Put the new creature on the map */
+    op->x = x; op->y = y;
+    insert_ob_in_map(op,map);
+
+    /* It is possible that there is something on the map that kills
+     * the object when inserted.  Unlikely, but worth checking. */
+    if (QUERY_FLAG(op, FLAG_FREED)) return;
+
+    /*update_object(op);*/ /*I believe insert_ob_in_map already does this */
+
+
+    if(op->randomitems != NULL)
+	create_treasure(op->randomitems,op,GT_INVISIBLE,map->difficulty,0);
+
+    /* Apply any objects.  This limits it to the first 20 items, which
+     * I guess is reasonable.
+     */
+    for(tmp = op->inv, nr = 0; tmp != NULL && ++nr < 20; tmp = next) {
+	next = tmp->below;
+	(void) monster_check_apply(op,tmp);
     }
-  }
-  remove_ob(op);
-  owner = get_owner(op);
-  friendly = QUERY_FLAG(op, FLAG_FRIENDLY);
-  if (friendly)
-    remove_friendly_object(op);
-  copy_object(&(altern[choice]->clone),op);
-  if (owner != NULL)
-    set_owner(op,owner);
-  if (friendly) {
-    SET_FLAG(op, FLAG_FRIENDLY);
-    op->move_type = PETMOVE;
-    add_friendly_object(op);
-  }
-  op->x = x; op->y = y;
-  insert_ob_in_map(op,map);
-  update_object(op);
-  if(op->randomitems != NULL)
-    create_treasure(op->randomitems,op,GT_INVISIBLE,map->difficulty,0);
-  for(tmp = op->inv, nr = 0; tmp != NULL && ++nr < 20; tmp = next) {
-    next = tmp->below;
-    (void) monster_check_apply(op,tmp);
-  }
 }
 
 
-/* Destroys item from polymorph failure */
+/* polymorph_melt Destroys item from polymorph failure 
+ * who is the caster of the polymorph, op is the
+ * object destroyed.  We should probably do something
+ * more clever ala nethack - create an iron golem or
+ * something.
+ */
 void polymorph_melt(object *who, object *op)
 {
 
     new_draw_info_format(NDI_UNIQUE, 0, who,
 	"%s%s glows red, melts and evaporates!",
             op->nrof?"":"The ",query_name(op));
-#ifdef SOUND_EFFECTS
     play_sound_map(op->map, op->x, op->y, SOUND_OB_EVAPORATE);
-#endif
     remove_ob(op);
     free_object(op);
     return;
 }
 
+/* polymorph_item - changes an item to another item of similar type.
+ * who is the caster of spell, op is the object to be changed.
+ */
+
 void polymorph_item(object *who, object *op) {
-  archetype *altern[MAX_ALT], *at;
-  int nr = 0, max_value, difficulty, tries=0,choice, charges=op->stats.food;
-  object *new_ob;
+    archetype *at;
+    int max_value, difficulty, tries=0,choice, charges=op->stats.food,numat=0;
+    object *new_ob;
 
-  max_value = op->value * 2;
-  if(max_value > 20000)
-    max_value = 20000 + (max_value - 20000) / 2;
 
-  for(at = first_archetype ; at != NULL; at = at->next)
-    if(at->clone.type == op->type)
-    {
-      altern[nr] = at;
-      if(++nr == MAX_ALT)
-        break;
+    /* We try and limit the maximum value of the changd object. */
+    max_value = op->value * 2;
+    if(max_value > 20000)
+	max_value = 20000 + (max_value - 20000) / 2;
+
+    /* Look through and try to find matching items.  Can't turn into something
+     * invisible.  Also, if the value is too high now, it would almost
+     * certainly be too high below.
+     */
+    for(at = first_archetype ; at != NULL; at = at->next) {
+	if(at->clone.type == op->type && !at->clone.invisible && 
+	   at->clone.value < max_value &&
+	   !QUERY_FLAG(&at->clone, FLAG_NO_DROP))
+		numat++;
     }
-  if(!nr)
-    return;
 
-  difficulty = op->magic * 5;
-  if (difficulty<0) difficulty=0;
-  new_ob = get_object();
-  do {
-      choice = RANDOM()%nr;
-      copy_object(&(altern[choice]->clone),new_ob);
-      fix_generated_item(new_ob,op,difficulty,FABS(op->magic));
-      ++tries;
-  } while (new_ob->value > max_value && tries<10);
-  if (tries==10) {
+    if(!numat)
+	return;
+
+    difficulty = op->magic * 5;
+    if (difficulty<0) difficulty=0;
+    new_ob = get_object();
+    do {
+	choice = RANDOM()%numat;
+	for(at = first_archetype ; at != NULL; at = at->next) {
+	    if(at->clone.type == op->type && !at->clone.invisible && 
+	       at->clone.value < max_value && 
+	       !QUERY_FLAG(&at->clone, FLAG_NO_DROP)) {
+		if (!choice) break;
+		else choice--;
+	    }
+	}
+	copy_object(&(at->clone),new_ob);
+	fix_generated_item(new_ob,op,difficulty,FABS(op->magic));
+	++tries;
+    } while (new_ob->value > max_value && tries<10);
+    if (new_ob->invisible) {
+	LOG(llevError,"polymorph_item: fix_generated_object made %s invisible?!\n", new_ob->name);
+    }
+
+    /* Unable to generate an acceptable item?  Melt it */
+    if (tries==10) {
 	polymorph_melt(who, op);
 	free_object(new_ob);
 	return;
-  }
+    }
 
-  if(op->nrof && new_ob->nrof) {
-    new_ob->nrof = op->nrof;
-    /* decrease the number of items */
-    if (new_ob->nrof>2) new_ob->nrof -= RANDOM() % (op->nrof/2);
-  }
-  /* We don't want rings to keep sustenance/hungry status. There are propably
-     other cases too that should be checked. */
-  if(charges && op->type != RING && op->type != FOOD)
-      op->stats.food = charges;
-  new_ob->x = op->x;
-  new_ob->y = op->y;
-  remove_ob(op);
-  free_object(op);
-  insert_ob_in_map_simple(new_ob,who->map);
+    if(op->nrof && new_ob->nrof) {
+	new_ob->nrof = op->nrof;
+	/* decrease the number of items */
+	if (new_ob->nrof>2) new_ob->nrof -= RANDOM() % (op->nrof/2);
+    }
+
+    /* We don't want rings to keep sustenance/hungry status. There are propably
+       other cases too that should be checked. */
+    if(charges && op->type != RING && op->type != FOOD)
+	op->stats.food = charges;
+
+    new_ob->x = op->x;
+    new_ob->y = op->y;
+    remove_ob(op);
+    free_object(op);
+    /* insert_ob_in_map_simple is caused so that objects don't get
+     * merged/rearranged.
+     */
+    insert_ob_in_map_simple(new_ob,who->map);
 }
 
+/* polymorh - caster who has hit object op. */
 void polymorph(object *op, object *who) {
 
-  int tmp;
+    int tmp;
 
-  if(op->type == PLAYER)
-    return; /* WILL add this later 8) */
+    /* Can't polymorph players right now */
+    if(op->type == PLAYER)
+	return;
 
-  if(QUERY_FLAG(op, FLAG_MONSTER) || QUERY_FLAG(op,FLAG_GENERATOR)) {
-    polymorph_living(op);
-    return;
-  }
-  if(QUERY_FLAG(op, FLAG_ALIVE))
-    return;
-  if(FABS(op->speed) > 0.001 && !QUERY_FLAG(op, FLAG_ANIMATE))
-    return; /* Don't want to morph flying arrows, etc... */
+    if(QUERY_FLAG(op, FLAG_MONSTER) || QUERY_FLAG(op,FLAG_GENERATOR)) {
+	polymorph_living(op);
+	return;
+    }
+    /* If it is a living object of some other type, don't handle
+     * it now.
+     */
+    if(QUERY_FLAG(op, FLAG_ALIVE))
+	return;
 
-  if(op->type == 0 || op->type==PLAYER || op->arch == NULL || 
-     QUERY_FLAG(op,FLAG_NO_PICK) 
-     || QUERY_FLAG(op, FLAG_NO_PASS) || op->type == TREASURE)
-    return;
+    /* Don't want to morph flying arrows, etc... */
+    if(FABS(op->speed) > 0.001 && !QUERY_FLAG(op, FLAG_ANIMATE))
+	return;
 
-  tmp = RANDOM() % 8;
-  if (tmp) polymorph_item(who, op);
-  else polymorph_melt(who, op);
+    /* Do some sanity checking here.  type=0 is unkown, objects
+     * without archetypes are not good.  As are a few other
+     * cases.
+     */
+    if(op->type == 0 || op->arch == NULL || 
+       QUERY_FLAG(op,FLAG_NO_PICK) 
+       || QUERY_FLAG(op, FLAG_NO_PASS) || op->type == TREASURE)
+	return;
+
+    tmp = RANDOM() % 8;
+    if (tmp) polymorph_item(who, op);
+    else polymorph_melt(who, op);
 }
 
+
+/* cast_polymorph - object *op has cast it
+ *dir is the direction.
+ * Returns 0 on illegal cast, otherwise 1.
+ */
+
+int cast_polymorph(object *op, int dir) {
+    object *tmp, *next;
+    int range;
+    archetype *poly;
+
+    if(dir == 0)
+	return 0;
+
+    poly = find_archetype("polymorph");
+    /* Range is until it hits a wall */
+    for(range = 1;;range++) {
+	int x=op->x+freearr_x[dir]*range,y=op->y+freearr_y[dir]*range;
+	object *image;
+
+	if(wall(op->map,x,y) || blocks_magic(op->map,x,y))
+	    break;
+
+	/* Get the top most object */
+	for(tmp = get_map_ob(op->map,x,y); tmp != NULL && tmp->above != NULL;
+	    tmp = tmp->above);
+
+	/* Now start polymorphing the objects, top down */
+	while (tmp!=NULL) {
+	    /* Once we find the floor, no need to go further */
+	    if (QUERY_FLAG(tmp, FLAG_IS_FLOOR)) break;
+	    next = tmp->below;
+	    polymorph(tmp, op);
+	    tmp = next;
+	}
+	image = arch_to_object(poly);
+	image->x = x; image->y = y;
+	/* I am not sure what this below does with the
+	 * range - msw 7/4/1998
+	 */
+	image->stats.food += range;
+	image->speed_left = 0.1;
+	insert_ob_in_map(image,op->map);
+    }
+    return 1;
+}
+
+
 /*  allows the choice of what sort of food object to make.
     If stringarg is NULL, it will create food dependent on level  --PeterM*/
 int cast_create_food(object *op,object *caster, int dir, char *stringarg)
@@ -392,6 +544,8 @@ int cast_create_food(object *op,object *caster, int dir, char *stringarg)
   new_op = get_object();
   copy_object(&at->clone, new_op);
   new_op->nrof = food_value;
+  new_op->value = 0;
+  SET_FLAG(new_op, FLAG_STARTEQUIP);
   if (new_op->nrof<1) new_op->nrof = 1;
 
   cast_create_obj(op, caster,new_op, dir);
@@ -399,36 +553,6 @@ int cast_create_food(object *op,object *caster, int dir, char *stringarg)
 }
   
   
-
-int cast_polymorph(object *op, int dir) {
-  object *tmp, *next;
-  int range;
-  archetype *poly;
-
-  if(dir == 0)
-    return 0;
-  poly = find_archetype("polymorph");
-  for(range = 1;;range++) {
-    int x=op->x+freearr_x[dir]*range,y=op->y+freearr_y[dir]*range;
-    object *image;
-    if(wall(op->map,x,y) || blocks_magic(op->map,x,y))
-      break;
-    for(tmp = get_map_ob(op->map,x,y); tmp != NULL && tmp->above != NULL;
-        tmp = tmp->above);
-    while (tmp!=NULL) {
-      next = tmp->below;
-      polymorph(tmp, op);
-      tmp = next;
-    }
-    image = arch_to_object(poly);
-    image->x = x; image->y = y;
-    image->stats.food += range;
-    image->speed_left = 0.1;
-    insert_ob_in_map(image,op->map);
-  }
-  return 1;
-}
-
 int cast_speedball(object *op, int dir, int type) {
   object *spb;
   if(blocked(op->map,op->x+freearr_x[dir],op->y+freearr_y[dir]))
@@ -571,29 +695,35 @@ int cast_wow(object *op, int dir, int ability, SpellTypeFrom item) {
 }
 
 int perceive_self(object *op) {
-  char *cp=describe_item(op), buf[MAX_BUF];
-  archetype *at=find_archetype("depletion");
-  object *tmp;
-  int i;
+    char *cp=describe_item(op), buf[MAX_BUF];
+    archetype *at=find_archetype("depletion");
+    object *tmp;
+    int i;
 
-  tmp=present_arch_in_ob(at,op);
+    tmp=find_god(determine_god(op));
+    if (tmp) 
+	new_draw_info_format(NDI_UNIQUE, 0, op, "You worship %s", tmp->name);
+    else
+	new_draw_info(NDI_UNIQUE, 0,op,"You worship no god");
 
-  if(*cp=='\0' && tmp==NULL)
-    new_draw_info(NDI_UNIQUE, 0,op,"You feel very mundane");
-  else {
-    new_draw_info(NDI_UNIQUE, 0,op,"You have:");
-    new_draw_info(NDI_UNIQUE, 0,op,cp);
-    if (tmp!=NULL) {
-      for (i=0; i<7; i++) {
-	if (get_attr_value(&tmp->stats, i)<0) {
-	  sprintf(buf,"Your %s is depleted by %d", statname[i],
-	    -(get_attr_value(&tmp->stats,i)));
-	  new_draw_info(NDI_UNIQUE, 0,op, buf);
+    tmp=present_arch_in_ob(at,op);
+
+    if(*cp=='\0' && tmp==NULL)
+	new_draw_info(NDI_UNIQUE, 0,op,"You feel very mundane");
+    else {
+	new_draw_info(NDI_UNIQUE, 0,op,"You have:");
+	new_draw_info(NDI_UNIQUE, 0,op,cp);
+	if (tmp!=NULL) {
+	    for (i=0; i<7; i++) {
+		if (get_attr_value(&tmp->stats, i)<0) {
+		    sprintf(buf,"Your %s is depleted by %d", statname[i],
+			    -(get_attr_value(&tmp->stats,i)));
+		    new_draw_info(NDI_UNIQUE, 0,op, buf);
+		}
+	    }
 	}
-      }
     }
-  }
-  return 1;
+    return 1;
 }
 
 int cast_destruction(object *op, object *caster, int dam, int attacktype) {
@@ -733,6 +863,11 @@ int magic_wall(object *op,object *caster,int dir,int spell_type) {
   tmp->x=op->x+freearr_x[dir],tmp->y=op->y+freearr_y[dir];
   insert_ob_in_map(tmp,op->map);
 
+  if (QUERY_FLAG(tmp, FLAG_REMOVED)) {
+    new_draw_info(NDI_UNIQUE, 0,op,"Something destroys your wall");
+    return 0;
+  }
+
   /*  This code causes the wall to extend to a distance of 5 in
 		each direction, or until an obstruction is encountered. 
 		posblocked and negblocked help determine how far the
@@ -820,57 +955,75 @@ int cast_light(object *op,object *caster,int dir) {
 }
 
 int dimension_door(object *op,int dir) {
-  int dist;
-  if(!dir) {
-    new_draw_info(NDI_UNIQUE, 0,op,"In what direction?");
-    return 0;
-  }
-  if(op->type!=PLAYER)
-    return 0;
-  if(op->contr->count) {
-    for(dist=0;dist<op->contr->count&&
-               !blocks_magic(op->map,op->x+freearr_x[dir]*(dist+1),
-                             op->y+freearr_y[dir]*(dist+1));
-        dist++);
-    if(dist<op->contr->count) {
-      new_draw_info(NDI_UNIQUE, 0,op,"Something blocks your magic.\n");
-      op->contr->count=0;
-      return 0;
+    int dist;
+
+    if(op->type!=PLAYER)
+	return 0;
+
+    if(!dir) {
+	new_draw_info(NDI_UNIQUE, 0,op,"In what direction?");
+	return 0;
     }
-    op->contr->count=0;
-    if(blocked(op->map,op->x+freearr_x[dir]*dist,
-                       op->y+freearr_y[dir]*dist))
-    { /* Now the fun starts... 8) */
-      int x=RANDOM()%op->map->mapx,y=RANDOM()%op->map->mapy;
-      if(blocked(op->map,x,y)) { /* Lucky after all... */
-        new_draw_info(NDI_UNIQUE, 0,op,"You cast your spell, but nothing happens.\n");
-        return 1; /* Maybe the penalty should be more severe... */
-      }
-      remove_ob(op);
-      op->x=x,op->y=y;
-      insert_ob_in_map(op,op->map);
-      draw(op);
-      return 1;
-    }
-  } /* if op->contr->count*/ else {
-    for(dist=0;!blocks_view (op->map,op->x+freearr_x[dir]*(dist+1),
+
+    if(op->contr->count) {
+	for(dist=0;dist<op->contr->count; dist++)
+	    if (blocks_magic(op->map,op->x+freearr_x[dir]*(dist+1),
+                             op->y+freearr_y[dir]*(dist+1))) break;
+
+	if(dist<op->contr->count) {
+	    new_draw_info(NDI_UNIQUE, 0,op,"Something blocks your magic.\n");
+	    op->contr->count=0;
+	    return 0;
+	}
+	op->contr->count=0;
+
+	/* If the player is trying to dimension door to solid rock, choose
+	 * a random place on the map to put the player.
+	 * Changed in 0.94.3 so that the player can not get put in
+	 * a no magic spot.
+	 */
+	if(blocked(op->map,op->x+freearr_x[dir]*dist, op->y+freearr_y[dir]*dist)){
+	    int x=RANDOM()%op->map->mapx,y=RANDOM()%op->map->mapy;
+
+	    if(blocked(op->map,x,y) || blocks_magic(op->map,x,y)) {
+		new_draw_info(NDI_UNIQUE, 0,op,"You cast your spell, but nothing happens.\n");
+		return 1; /* Maybe the penalty should be more severe... */
+	    }
+
+	    remove_ob(op);
+	    op->x=x,op->y=y;
+	    insert_ob_in_map(op,op->map);
+	    draw(op);
+	    return 1;
+	}
+    } else { /* Player didn't specify a distance, so lets see how far
+	      * we can move the player.
+	      */
+	for(dist=0;!blocks_view (op->map,op->x+freearr_x[dir]*(dist+1),
                                      op->y+freearr_y[dir]*(dist+1))&&
                !blocks_magic(op->map,op->x+freearr_x[dir]*(dist+1),
                                      op->y+freearr_y[dir]*(dist+1));
-        dist++);
-    for(;dist>0&&blocked(op->map,op->x+freearr_x[dir]*dist,
+	    dist++);
+
+	/* If the destinate is blocked, keep backing up until we
+	 * find a place for the player.
+	 */
+	for(;dist>0&&blocked(op->map,op->x+freearr_x[dir]*dist,
                                  op->y+freearr_y[dir]*dist);dist--);
-    if(!dist) {
-      new_draw_info(NDI_UNIQUE, 0,op,"Your spell failed!\n");
-      return 0;
+
+	if(!dist) {
+	    new_draw_info(NDI_UNIQUE, 0,op,"Your spell failed!\n");
+	    return 0;
+	}
     }
-  }
-  remove_ob(op);
-  op->x+=freearr_x[dir]*dist,op->y+=freearr_y[dir]*dist;
-  insert_ob_in_map(op,op->map);
-  draw(op);
-  op->speed_left= -FABS(op->speed)*5; /* Freeze them for a short while */
-  return 1;
+
+    /* Actually move the player now */
+    remove_ob(op);
+    op->x+=freearr_x[dir]*dist,op->y+=freearr_y[dir]*dist;
+    insert_ob_in_map(op,op->map);
+    draw(op);
+    op->speed_left= -FABS(op->speed)*5; /* Freeze them for a short while */
+    return 1;
 }
 
 
@@ -952,10 +1105,7 @@ cast_heal(object *op,int dir,int spell_type) {
   tmp->stats.hp+=heal;
   if(tmp->stats.hp>tmp->stats.maxhp)
     tmp->stats.hp=tmp->stats.maxhp;
-  if(tmp->type==PLAYER)
-    draw_stats(tmp);
-  if(op!=tmp && op->type==PLAYER)
-    draw_stats(op);
+
   op->speed_left= -FABS(op->speed)*3; /* Freeze them for a short while */
   return 1;
 }
@@ -968,8 +1118,6 @@ int cast_regenerate_spellpoints(object *op) {
   if(tmp==NULL) return 0;
 
   tmp->stats.sp = tmp->stats.maxsp;
-  if (tmp->type == PLAYER)
-    draw_stats(tmp);
   new_draw_info(NDI_UNIQUE, 0,tmp, "Magical energies surge through your body!");
   return 1;
 }
@@ -1209,8 +1357,7 @@ cast_change_attr(object *op,object *caster,int dir,int spell_type) {
   change_abil(tmp,force); /* Mostly to display any messages */
   fix_player(tmp);        /* This takes care of some stuff that change_abil() */
 			  /* unfortunately is incapable off. */
-  if(tmp->type==PLAYER)
-    draw_stats(tmp);
+
   return 1;
 }
 
@@ -1371,7 +1518,7 @@ void animate_bomb(object *op) {
   int i;
   object *env;
   archetype *at = find_archetype("splint");
-  if(op->state!=op->arch->animations-1)
+  if(op->state!=NUM_ANIMATIONS(op)-1)
     return;
   for(env=op;env->env!=NULL;env=env->env);
   if (op->env) {
@@ -1408,11 +1555,8 @@ int fire_cancellation(object *op,int dir,archetype *at, int magic) {
     tmp->attacktype|=AT_MAGIC;
   set_owner(tmp,op);
   if(op->type==PLAYER)
-    D_LOCK(op);
   insert_ob_in_map(tmp,op->map);
   move_cancellation(tmp);
-  if(op->type==PLAYER)
-    D_UNLOCK(op);
   return 1;
 }
 
@@ -1442,11 +1586,6 @@ void cancellation(object *op)
     for(tmp=op->inv;tmp!=NULL;tmp=tmp->below)
       if (!did_make_save_item(tmp, AT_CANCELLATION))
 	cancellation(tmp);
-    if(op->type==PLAYER && op->contr->eric_server <= 0) {
-      draw_inventory(op);
-      draw_look(op);
-      fix_player(op);	/* Remember that wc/dam has probably changed */
-    }
   }
   else				/* Nullify this object. */
     if(FABS(op->magic)<=(RANDOM()%6)) {
@@ -1455,11 +1594,9 @@ void cancellation(object *op)
       CLEAR_FLAG(op, FLAG_CURSED);
       CLEAR_FLAG(op, FLAG_KNOWN_MAGICAL);
       CLEAR_FLAG(op, FLAG_KNOWN_CURSED);
-      if (op->env && op->env->type == PLAYER)
-	if (op->env->contr->eric_server > 0) 
-	    esrv_send_item (op->env, op);
-	else
-	    draw_inventory(op->env);
+      if (op->env && op->env->type == PLAYER) {
+	esrv_send_item (op->env, op);
+      }
     }
 }
 
@@ -1737,14 +1874,13 @@ int remove_curse(object *op, int type, SpellTypeFrom src) {
         CLEAR_FLAG(tmp, FLAG_CURSED);
         CLEAR_FLAG(tmp, FLAG_KNOWN_CURSED);
         tmp->value = 0; /* Still can't sell it */
-	if (op->type == PLAYER && op->contr->eric_server > 0)
+	if (op->type == PLAYER)
 	    esrv_send_item(op, tmp);
       }
     }
   if (op->type==PLAYER) {
     if (success) {
       new_draw_info(NDI_UNIQUE, 0,op, "You feel like someone is helping you.");
-      draw_inventory(op);
     } else
       if (was_one)
         new_draw_info(NDI_UNIQUE, 0,op, "You failed to remove the curse.");
@@ -1778,8 +1914,7 @@ int cast_identify(object *op) {
 	  new_draw_info(NDI_UNIQUE, 0,op, "The item has a story:");
 	  new_draw_info(NDI_UNIQUE, 0,op, tmp->msg);
 	}
-	if (op->contr->eric_server > 0)
-	    esrv_send_item(op, tmp);
+	esrv_send_item(op, tmp);
       }
 
       if ((random_val=RANDOM()%chance) > (chance - ++success - 2))
@@ -1802,8 +1937,7 @@ int cast_identify(object *op) {
 	  new_draw_info(NDI_UNIQUE, 0,op, "The item has a story:");
 	  new_draw_info(NDI_UNIQUE, 0,op, tmp->msg);
 	}
-	if (op->contr->eric_server > 0)
-	    esrv_send_item(op, tmp);
+	esrv_send_item(op, tmp);
       }
       if (RANDOM() %chance > (chance - ++success - 2))
 	break;
@@ -1813,8 +1947,6 @@ int cast_identify(object *op) {
     new_draw_info(NDI_UNIQUE, 0,op, "You can't reach anything unidentified.");
   else {
     spell_effect(SP_IDENTIFY, op->x, op->y, op->map);
-    if (op->type == PLAYER && op->contr->eric_server <= 0)
-      draw_inventory(op);
   }
   return success;
 }
@@ -1875,9 +2007,20 @@ int cast_detection(object *op, int type) {
 #endif
           break; } 
         case SP_SHOW_INVIS:
-            done_one = tmp->invisible;
-	    /* if(RANDOM()%(op->level) > tmp->level/4) tmp->invisible=0; */ 
-	    if(RANDOM()%(SK_level(op)) > tmp->level/4) tmp->invisible=0;
+	    /* Might there be other objects that we can make visibile? */
+	    if (tmp->invisible && (QUERY_FLAG(tmp, FLAG_MONSTER) || 
+		tmp->type==PLAYER || tmp->type==HANDLE || 
+		tmp->type==TRAPDOOR || tmp->type==EXIT || tmp->type==HOLE ||
+		tmp->type==BUTTON || tmp->type==TELEPORTER ||
+		tmp->type==GATE || tmp->type==LOCKED_DOOR ||
+		tmp->type==WEAPON || tmp->type==ALTAR || tmp->type==SIGN ||
+		tmp->type==TRIGGER_PEDESTAL || tmp->type==SPECIAL_KEY ||
+		tmp->type==TREASURE || tmp->type==BOOK)) {
+		if(RANDOM()%(SK_level(op)) > tmp->level/4) {
+		    tmp->invisible=0;
+		    done_one = 1;
+		}
+	    }
           break;
         case SP_DETECT_CURSE:
           if (!QUERY_FLAG(tmp, FLAG_KNOWN_CURSED) &&
@@ -1906,8 +2049,7 @@ int cast_detection(object *op, int type) {
         case SP_DETECT_MAGIC:
           if (is_magical(tmp) && !QUERY_FLAG(tmp,FLAG_KNOWN_MAGICAL)) {
             SET_FLAG(tmp,FLAG_KNOWN_MAGICAL);
-	    if (op->contr->eric_server > 0)
-		esrv_send_item (op, tmp);
+	    esrv_send_item (op, tmp);
             done_one = 1;
           }
           break;
@@ -1916,14 +2058,11 @@ int cast_detection(object *op, int type) {
 		 (QUERY_FLAG(tmp, FLAG_CURSED) ||
 		 QUERY_FLAG(tmp, FLAG_DAMNED))) {
 	    SET_FLAG(tmp, FLAG_KNOWN_CURSED);
-	    if (op->contr->eric_server > 0)
-		esrv_send_item (op, tmp);
+	    esrv_send_item (op, tmp);
             done_one = 1;
           }
           break;
         }
-    if (done_one && op->contr->eric_server <= 0)
-      draw_inventory(op);
   }
   return 1;
 }
@@ -2078,6 +2217,10 @@ int create_the_feature(object *op, object *caster,int dir, int spell_effect)
 				 of the wall builder */
   tmp->x=op->x;tmp->y=op->y;
   if(putflag) { tmp->x+=freearr_x[dir];tmp->y+=freearr_y[dir];}
+  if (out_of_map(op->map, tmp->x, tmp->y)) {
+    free_object(tmp);
+    return 0;
+  }
   insert_ob_in_map(tmp,op->map);
   if(QUERY_FLAG(tmp, FLAG_BLOCKSVIEW))
     update_all_los(op->map);
@@ -2096,19 +2239,21 @@ int create_the_feature(object *op, object *caster,int dir, int spell_effect)
     head explode if they supercharge too much, though.  */
 
 int cast_transfer(object *op,int dir) {
-    object *plyr;
+    object *plyr=NULL;
 /*  see if we can find someone to give sp to.  */
-     for(plyr=get_map_ob(op->map,op->x+freearr_x[dir],op->y+freearr_y[dir]);
-	 plyr!=NULL;
-	 plyr=plyr->above)
-	if(QUERY_FLAG(plyr, FLAG_ALIVE))
-        break;
+    if (!out_of_map(op->map, op->x+freearr_x[dir], op->y+freearr_y[dir])) {
+	for(plyr=get_map_ob(op->map,op->x+freearr_x[dir],op->y+freearr_y[dir]);
+	    plyr!=NULL; plyr=plyr->above)
+		if(QUERY_FLAG(plyr, FLAG_ALIVE))
+		    break;
+    }
      /*  If we did not find a player in the specified direction, transfer
 	to anyone on top of us. */
     if(plyr==NULL)
-         for(plyr=get_map_ob(op->map,op->x,op->y); plyr!=NULL; plyr=plyr->above)
-	if(QUERY_FLAG(plyr,FLAG_ALIVE))
-        break;
+        for(plyr=get_map_ob(op->map,op->x,op->y); plyr!=NULL; plyr=plyr->above)
+	    if(QUERY_FLAG(plyr,FLAG_ALIVE))
+		break;
+
     if(plyr) {
 	int maxsp=plyr->stats.maxsp;
 	int sp=(plyr->stats.sp+=8);
@@ -2116,7 +2261,7 @@ int cast_transfer(object *op,int dir) {
 	new_draw_info(NDI_UNIQUE, 0,plyr,"You feel energy course through you.");
 	if(sp>=maxsp*2) {
             new_draw_info(NDI_UNIQUE, 0,plyr,"Your head explodes!");
-            fire_arch (plyr, plyr,0, spellarch[SP_L_FIREBALL], SP_L_FIREBALL, 0);
+            fire_arch (op, plyr,0, spellarch[SP_L_FIREBALL], SP_L_FIREBALL, 0);
                 /* Explodes a large fireball centered at player */
 /*            hit_player(plyr, 9999, op, AT_PHYSICAL);*/
 	    plyr->stats.sp = 2*maxsp;
@@ -2172,40 +2317,50 @@ void counterspell(object *op)
     int nflag=0;
     
     for(tmp=get_map_ob(op->map,op->x,op->y); tmp!=NULL; tmp=tmp->above,nflag=0)
-    {  if(tmp->material==0 && tmp->attacktype&AT_MAGIC &&!
-	(tmp->attacktype&AT_COUNTERSPELL)) nflag=1;
-	else
-	switch(tmp->type) {
+    {	  
+	/* Basially, if the object is magical and not counterspell, 
+	 * we will more or less remove the object.  Changed in 0.94.3
+	 * so that we don't kill monsters with this (some monsters attacktype
+	 * has magic in it).
+         */
+	if(tmp->material==0 && tmp->attacktype&AT_MAGIC &&
+	   !(tmp->attacktype&AT_COUNTERSPELL) && 
+	   !QUERY_FLAG(tmp,FLAG_MONSTER))
+		nflag=1;
+
+	else switch(tmp->type) {
 	    case CONE: case FBALL: case LIGHTNING:
 	    case FBULLET: case MMISSILE: case SPEEDBALL:
 	    case BOMB: case POISONCLOUD: case CANCELLATION:
 	    case SWARM_SPELL:
 	    case BALL_LIGHTNING:
-	    nflag=1;
-	    break;
+		nflag=1;
+		break;
+
 	    case RUNE:
-	    nflag=2;
-	    break;
+		nflag=2;
+		break;
+
 	    default:
-	    nflag=0;
+		nflag=0;
 	}
 	switch(nflag) {
-	    case 1:
+	    case 1: {
 	    /* { if(op->level > tmp->level) { */ 
-	    { if(SK_level(op) > tmp->level) {
-		remove_ob(tmp);
-		free_object(tmp);
-		}
-		break;
-	    }
-	    case 2:
-	    { if(RANDOM()%150 == 0) {
-		tmp->stats.hp--;  /* weaken the rune */
-		if(!tmp->stats.hp) {
+	        if(SK_level(op) > tmp->level) {
 		    remove_ob(tmp);
 		    free_object(tmp);
 		}
+		break;
 	    }
+	    case 2: {
+		if(RANDOM()%150 == 0) {
+		    tmp->stats.hp--;  /* weaken the rune */
+		    if(!tmp->stats.hp) {
+			remove_ob(tmp);
+			free_object(tmp);
+		    }
+		}
 		break;
 	    }
 	}
@@ -2664,7 +2819,7 @@ int finger_of_death(object *op, object *caster, int dir) {
   if(QUERY_FLAG(target,FLAG_UNDEAD)) {
       success = 0;
       if(RANDOM()%3) { 
-        new_draw_info(NDI_UNIQUE,0,op,"Idiot! Your spell boomerangs!"); 	
+        new_draw_info(NDI_UNIQUE,0,op,"Idiot! Your spell boomerangs!"); 
 	hitter->x=op->x;
 	hitter->y=op->y;	
       } else { 
@@ -2769,8 +2924,8 @@ int staff_to_snake(object *op,object *caster,int dir, archetype *at, int spellnu
   remove_ob(weapon);
   insert_ob_in_ob(weapon,tmp);
   fix_player(op);
-  draw_inventory(op);
-  draw_stats(op);
+  esrv_send_item(op, weapon);
+
   new_draw_info(NDI_UNIQUE, 0,op,"Your staff becomes a serpent and leaps to the ground!");
 
   /*  make experience increase in proportion to the strength of
