@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_spells_c =
- *   "$Id: spell_util.c,v 1.20 1997/03/09 08:18:22 master Exp master $";
+ *   "$Id: spell_util.c,v 1.27 1998/09/28 04:08:40 master Exp master $";
  */
 
 /*
@@ -33,9 +33,8 @@
 #ifndef __CEXTRACT__
 #include <sproto.h>
 #endif
-#ifdef SOUND_EFFECTS
 #include <sounds.h>
-#endif
+
 #ifdef sequent
 /* stoopid sequent includes don't do this like they should */
 extern char * sys_errlist[];
@@ -169,6 +168,10 @@ int cast_spell(object *op,object *caster,int dir,int type,int ability,SpellTypeF
     LOG(llevError,"Error, unknown spell: %d\n",type);
     return 0;
   }
+  if(item==spellPotion) {  /*  if the potion casts an onself spell, 
+		    don't use the facing direction (given by apply.c)*/
+	 if( spells[type].onself) dir = 0;
+  }
 
   if(!(QUERY_FLAG(op, FLAG_WIZ))&&
 	  (op->type==PLAYER)&&
@@ -292,9 +295,7 @@ int cast_spell(object *op,object *caster,int dir,int type,int ability,SpellTypeF
   if(item == spellNormal && op->type==PLAYER&&s->cleric&&
      RANDOM()%100< s->level*2 - op->level + cleric_chance[op->stats.Wis]-
 	op->stats.luck*3) {
-#ifdef SOUND_EFFECTS
-    play_sound_player_only(op->contr, SOUND_FUMBLE_SPELL);
-#endif
+    play_sound_player_only(op->contr, SOUND_FUMBLE_SPELL,0,0);
     new_draw_info(NDI_UNIQUE, 0,op,"You fumble the spell.");
 #ifdef CASTING_TIME
   op->casting = -1;
@@ -740,9 +741,7 @@ if (item == spellNormal && !ability ){
     break;
 
   }
-#ifdef SOUND_EFFECTS
   play_sound_map(op->map, op->x, op->y, SOUND_CAST_SPELL_0 + type);
-#endif
 #ifdef CASTING_TIME
 	  /* free the spell arg */
 	  if(stringarg) {free(stringarg);stringarg=NULL; };
@@ -822,15 +821,75 @@ int summon_monster(object *op,object *caster,int dir,archetype *at,int spellnum)
 }
 
 
-int ok_to_put_more(mapstruct *m,int x,int y,object *op,int immune_stop) {
-  object *tmp,*head;
-  for(tmp=get_map_ob(m,x,y);tmp!=NULL;tmp=tmp->above) {
+/* Returns true if it is ok to put spell *op on the space/may provided.
+ * Not sure what immune_stop was supposed to do - the only functions that
+ * current call this are move_cone and explosion, and both are just
+ * passing op->attacktype on.
+ * My guess judging from the old code is that if the space has a monster
+ * immune to immune_stop, the spell should not be placed.  But that
+ * doesn't make a lot of sense to me (a dragon immune to fire should not
+ * cause a fireball not to explode
+ *
+ * 0.94.2 - rewrote this to include the wall check instead of the calling
+ * function doing so.
+ */
+
+static int ok_to_put_more(mapstruct *m,int x,int y,object *op,int immune_stop) {
+    object *tmp;
+
+    /* No check for out of map was here before - probably caught when the
+     * calling function tries to insert the object.  But a check here
+     * should only be a minor performance hit, and is a good thing.
+     */
+
+    if (out_of_map(m,x,y)) return 0;
+
+    /* if there is a wall, certainly can't put the spell there.  The blocks
+     * view check is historic from the old calling functions - my personal
+     * view is that blocks view should not affect spells in any way, but
+     * I will leave it in for now.
+     */
+    if(wall(m,x,y) || blocks_view(m,x,y)) return 0;
+
+    for(tmp=get_map_ob(m,x,y);tmp!=NULL;tmp=tmp->above) {
+	/* If there is a counterspell on the space, and this
+	 * object is using magic, don't progess.  I believe we could
+	 * leave this out and let in progress, and other areas of the code
+	 * will then remove it, but that would seem to to use more
+	 * resources, and may not work as well if a player is standing
+	 * on top of a counterwall spell (may hit the player before being
+	 * removed.)  On the other hand, it may be more dramatic for the
+	 * spell to actually hit the counterwall and be sucked up.
+	 *
+	 * We should probably check the type of the object, just in case
+	 * a player somehow gets a counterspell attacktype.
+	 */
+	if ((tmp->attacktype & AT_COUNTERSPELL) &&
+	    (immune_stop & AT_MAGIC)) return 0;
+
+	/* This is to prevent 'out of control' spells.  Basically, this
+	 * limits one spell effect per space per spell.  This is definately
+	 * needed for performance reasons, and just for playability I believe.
+	 * there are no such things as multispaced spells right now, so
+	 * we don't need to worry about the head.
+	 */
+	if ((tmp->stats.maxhp == op->stats.maxhp) && (tmp->type == op->type))
+	    return 0;
+
+	/* Perhaps we should also put checks in for no magic and unholy
+	 * ground to prevent it from moving along?
+	 */
+#if 0
+	/* Remove code that determines if it stops because it hits
+	 * a type of monster.  I don't think that should affect it.
+	 */
     head=tmp->head==NULL?tmp:tmp->head;
-    if((QUERY_FLAG(op, FLAG_ALIVE) && head->immune & immune_stop) ||
-       (head->stats.maxhp == op->stats.maxhp && head->type == op->type))
+    if((QUERY_FLAG(head, FLAG_ALIVE) && head->immune & immune_stop))
       return 0;
-  }
-  return 1;
+#endif
+    }
+    /* If it passes the above tests, it must be OK */
+    return 1;
 }
 
 
@@ -853,7 +912,7 @@ int fire_bolt(object *op,object *caster,int dir,int type,int magic) {
   tmp->x=op->x,tmp->y=op->y;
   tmp->direction=dir;
   if(QUERY_FLAG(tmp, FLAG_IS_TURNABLE))
-    tmp->face=&new_faces[tmp->arch->faces[dir]];
+    SET_ANIMATION(tmp, dir);
   set_owner(tmp,op);
 #if 0
   if(op->type==PLAYER)
@@ -878,6 +937,10 @@ int fire_bolt(object *op,object *caster,int dir,int type,int magic) {
 /*  peterm  added a type field to fire_arch.  Needed it for making
     fireball etall level dependent.
     Later added a ball-lightning firing routine.
+ * dir is direction, at is spell we are firing.  Type is index of spell
+ * array.  If magic is 1, then add magical attacktype to spell.
+ * op is the owner of the spell (player who gets exp), caster is the
+ * casting object.
  */
 
 int fire_arch(object *op,object *caster,int dir,archetype *at, int type, int magic) {
@@ -904,13 +967,11 @@ int fire_arch(object *op,object *caster,int dir,archetype *at, int type, int mag
   if(magic)
     tmp->attacktype|=AT_MAGIC;
   if(QUERY_FLAG(tmp, FLAG_IS_TURNABLE))
-    tmp->face=&new_faces[tmp->arch->faces[dir]];
+    SET_ANIMATION(tmp, dir);
 #if 0
   if(op->type==PLAYER)
     tmp->stats.hp=(op->contr->shootstrength-10)/10+10;
 #endif
-  if(op->type==PLAYER)
-    D_LOCK(op);
   insert_ob_in_map(tmp,op->map);
   /* object was used when it was inserted into the map - don't do anything
    * more
@@ -928,8 +989,6 @@ int fire_arch(object *op,object *caster,int dir,archetype *at, int type, int mag
     default:
     move_fired_arch(tmp);
   }
-  if(op->type==PLAYER)
-    D_UNLOCK(op);
   return 1;
 }
 
@@ -955,9 +1014,9 @@ cast_cone(object *op, object *caster,int dir, int strength, int spell_type,arche
     tmp->level=SK_level(op); /* need to use the cleric level, not overall value -b.t. */ 
     tmp->x=x,tmp->y=y;
 #ifdef MULTIPLE_GODS /* holy word stuff */                
-    if((tmp->attacktype&AT_HOLYWORD)||(tmp->attacktype&AT_GODPOWER))
+    if((tmp->attacktype&AT_HOLYWORD)||(tmp->attacktype&AT_GODPOWER)) {
             if(!tailor_god_spell(tmp,op)) return 0;  
-    else /* god/holy word isnt really 'magic' */
+    } else /* god/holy word isnt really 'magic' */
 #endif
     if(magic)
       tmp->attacktype|=AT_MAGIC;  /* JWI cone attacks should be considered
@@ -1016,8 +1075,7 @@ void move_cone(object *op) {
   for(i= -1;i<2;i++) {
     int x=op->x+freearr_x[absdir(op->stats.sp+i)],
         y=op->y+freearr_y[absdir(op->stats.sp+i)];
-    if(!wall(op->map,x,y)&&ok_to_put_more(op->map,x,y,op,op->attacktype) &&
-       !blocks_view(op->map,x,y)) {
+    if(ok_to_put_more(op->map,x,y,op,op->attacktype)) {
       object *tmp=arch_to_object(op->arch);
       set_owner(tmp,op->owner);
       tmp->x=x, tmp->y=y;
@@ -1047,7 +1105,7 @@ void fire_a_ball(object *op,int dir,int strength) {
   tmp->speed = 1;
   update_ob_speed(tmp);
   tmp->stats.hp=strength;
-  tmp->face=&new_faces[tmp->arch->faces[dir]];
+  SET_ANIMATION(tmp, dir);
   SET_FLAG(tmp, FLAG_FLYING);
   insert_ob_in_map(tmp,op->map);
   move_fired_arch(tmp);
@@ -1263,7 +1321,7 @@ void move_missile(object *op) {
   i=find_dir(op->map,op->x,op->y,get_owner(op));
   if(i&&i!=op->direction){
     op->direction=absdir(op->direction+((op->direction-i+8)%8<4?-1:1));
-    op->face=&new_faces[op->arch->faces[op->direction]];
+    SET_ANIMATION(op, op->direction);
   }
   insert_ob_in_map(op,op->map);
 }
@@ -1289,16 +1347,23 @@ int explode_object(object *op) {
         if(QUERY_FLAG(victim,FLAG_ALIVE))
           break;
       hit_map(op,0,op->attacktype);
+
+#if 0
+    /* Hit_map will also do a hit_player for us.  Leaving this call in
+     * effectively doubles the amount of damage the bullet is doing.
+     */
       /* Should hit_map also be doing this?  Why call hit_player
        * again?  Also, make sure victim has not been killed - it
        * is possible that hit_map killed the object.
        */
       if(victim!=NULL && !QUERY_FLAG(victim,FLAG_FREED))
 	 hit_player(victim,op->stats.dam,op,op->attacktype);
+#endif
     }
 
   /*  peterm:  hack added to make fireballs and other explosions level
-  **  dependent:  mark*/
+   *  dependent:
+   */
 
   /*  op->stats.sp stores the spell which made this object here. */
   if(op->owner)
@@ -1356,61 +1421,70 @@ void check_fired_arch(object *op) {
 }
 
 void move_fired_arch(object *op) {
-  remove_ob(op);
+    remove_ob(op);
 
-  /* peterm:  added to make comet leave a trail of burnouts 
+    /* peterm:  added to make comet leave a trail of burnouts 
 	it's an unadulterated hack, but the effect is cool.	*/
-  if(op->stats.sp == SP_METEOR)
-  {  
-      object * tmp1=arch_to_object(find_archetype("fire_trail"));
+    if(op->stats.sp == SP_METEOR) {
+	object * tmp1=arch_to_object(find_archetype("fire_trail"));
+
         tmp1->x = op->x; tmp1->y = op->y;
         insert_ob_in_map(tmp1,op->map);
-  }  /* end addition.  */
+    } /* end addition.  */
 
-  op->x+=DIRX(op),op->y+=DIRY(op);
-  if(!op->direction||wall(op->map,op->x,op->y)) {
-    if(explode_object(op))
-      return;
-    free_object(op);
-    return;
-  }
-  if(reflwall(op->map,op->x,op->y)) {
-    op->direction=absdir(op->direction+4);
-    insert_ob_in_map(op,op->map);
-    update_turn_face(op);
-    return;
-  }
-  if(blocked(op->map,op->x,op->y)) {
-    object *tmp;
-    if(out_of_map(op->map,op->x,op->y)) {
+    op->x+=DIRX(op),op->y+=DIRY(op);
+    if(!op->direction||wall(op->map,op->x,op->y)) {
+	if(explode_object(op))
+	    return;
 	free_object(op);
 	return;
     }
-    if(explode_object(op))
-      return;
-    for(tmp=get_map_ob(op->map,op->x,op->y);tmp!=NULL;tmp=tmp->above)
-      if(QUERY_FLAG(tmp, FLAG_ALIVE))
-        break;
-    if(tmp!=NULL) {
-      /* Certain items, like speedballs, have attacktype ghosthit.
-       * hit_player wants to remove the object after it hits the player.
-       * Since it is already removed, just don't make it ghosthit, and
-       * remove it here
-       */
-      if (op->attacktype & AT_GHOSTHIT) {
-	hit_player(tmp,op->stats.dam,op,(op->attacktype & ~AT_GHOSTHIT));
-	free_object(op);
+
+    if(reflwall(op->map,op->x,op->y)) {
+	op->direction=absdir(op->direction+4);
+	insert_ob_in_map(op,op->map);
+	update_turn_face(op);
 	return;
-      }
-      else
-        op->stats.dam-=hit_player(tmp,op->stats.dam,op,op->attacktype);
     }
     if(blocked(op->map,op->x,op->y)) {
-      free_object(op);
-      return;
-    }
-  }
-  insert_ob_in_map(op,op->map);
+	object *tmp;
+
+	if(out_of_map(op->map,op->x,op->y)) {
+	    free_object(op);
+	    return;
+	}
+
+	if(explode_object(op))
+	    return;
+
+	for(tmp=get_map_ob(op->map,op->x,op->y);tmp!=NULL;tmp=tmp->above)
+	    if(QUERY_FLAG(tmp, FLAG_ALIVE))
+		break;
+
+	if(tmp!=NULL) {
+	    /* Certain items, like speedballs, have attacktype ghosthit.
+	     * hit_player wants to remove the object after it hits the player.
+	     * Since it is already removed, just don't make it ghosthit, and
+	     * remove it here
+	     */
+
+	    if (op->attacktype & AT_GHOSTHIT) {
+		hit_player(tmp,op->stats.dam,op,(op->attacktype & ~AT_GHOSTHIT));
+		free_object(op);
+		return;
+	    }
+	    else
+		op->stats.dam-=hit_player(tmp,op->stats.dam,op,op->attacktype);
+	}
+	/* I guess this can be applicable if the object blocking the
+	 * space was destroyed?
+	 */
+	if(blocked(op->map,op->x,op->y)) {
+	    free_object(op);
+	    return;
+	}
+    } /* if space is blocked */
+    insert_ob_in_map(op,op->map);
 }
 
 
@@ -1652,6 +1726,9 @@ void move_swarm_spell(object *op)
 
 }
 
+
+
+
 /*  fire_swarm:  peterm */
 /*  The following routine creates a swarm of objects.  It actually
     sets up a specific swarm object, which then fires off all
@@ -1719,27 +1796,39 @@ int look_up_spell_by_name(object *op,char *spname) {
 
 
 void put_a_monster(object *op,char *monstername) {
-  object *tmp;
+  object *tmp,*head=NULL,*prev=NULL;
   archetype *at;
   int dir;
-  int nx,ny;
   
   /* find a free square nearby */
   /* first we check the closest square for free squares */
   if((at=find_archetype(monstername))==NULL) return;
   dir=find_first_free_spot(at,op->map,op->x,op->y);
-  if(dir!=-1 && (tmp=arch_to_object(at))!=NULL) {
-    nx=op->x+freearr_x[dir];
-    ny=op->y+freearr_y[dir];
-    tmp->x=nx;
-    tmp->y=ny;
-    tmp->map = op->map;
-    insert_ob_in_map(tmp,op->map);
+  if(dir!=-1) {
+    /* This is basically grabbed for generate monster.  Fixed 971225 to
+     * insert multipart monsters properly
+     */
+    while (at!=NULL) {
+	tmp=arch_to_object(at);
+	tmp->x=op->x+freearr_x[dir]+at->clone.x;
+	tmp->y=op->y+freearr_y[dir]+at->clone.y;
+	if (head) {
+	    tmp->head=head;
+	    prev->more=tmp;
+	}
+	insert_ob_in_map(tmp,op->map);
+	/* If something happens on the insert, don't insert anymore parts */
+	if (QUERY_FLAG(tmp,FLAG_FREED)) break;
+	if (!head) head=tmp;
+	prev=tmp;
+	at=at->more;
+    }
+
     /* thought it'd be cool to insert a burnout, too.*/
     tmp=get_archetype("burnout");
     tmp->map = op->map;
-    tmp->x=nx;
-    tmp->y=ny;
+    tmp->x=op->x+freearr_x[dir];
+    tmp->y=op->y+freearr_y[dir];
     insert_ob_in_map(tmp,op->map);
     }
 }
@@ -1801,11 +1890,7 @@ void shuffle_attack(object *op,int change_face)
     i=RANDOM()%22;
     op->attacktype|=ATTACKS[i].attacktype|AT_MAGIC;
     if(change_face) {
-	op->face=&new_faces[op->arch->faces[ATTACKS[i].face]];
-#if 0
-	op->face.fg=ATTACKS[i].fg;
-	op->face.bg=ATTACKS[i].bg;
-#endif
+	SET_ANIMATION(op, ATTACKS[i].face);
     }
 }
 
@@ -1959,10 +2044,11 @@ object *get_pointed_target(object *op, int dir) {
      ;!out_of_map(op->map,x,y)&&!blocks_view(op->map,x,y)
      &&!wall(op->map,x,y);x+=freearr_x[dir],y+=freearr_y[dir])
         for(target=get_map_ob(op->map,x,y);target;target=target->above) {
-              if(QUERY_FLAG(target->head?target->head:target,FLAG_MONSTER))
+              if(QUERY_FLAG(target->head?target->head:target,FLAG_MONSTER)) {
                    if(!blocks_magic(op->map,x,y))
                           return target;
                    else break;
+	      }
         }
  
   return ((object *) NULL);
@@ -2000,14 +2086,15 @@ int cast_smite_spell (object *op, object *caster,int dir, int type) {
   /* tailor the effect by priest level and worshipped God */
    effect->level=SK_level(op);
 #ifdef MULTIPLE_GODS
-   if(effect->attacktype&AT_HOLYWORD||effect->attacktype&AT_GODPOWER)
+   if(effect->attacktype&AT_HOLYWORD||effect->attacktype&AT_GODPOWER) {
         if(tailor_god_spell(effect,op))
            new_draw_info_format(NDI_UNIQUE,0,op,
                 "%s answers your call!",determine_god(op));
         else {
            new_draw_info(NDI_UNIQUE,0,op,"Your request is ignored.");
            return 0;
-        }
+	}
+   }
 #endif 
  
    /* size of the area of destruction */
