@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_swap_c =
- *    "$Id: swap.c,v 1.11 1997/03/09 08:20:06 master Exp master $";
+ *    "$Id: swap.c,v 1.16 1998/09/28 04:10:08 master Exp master $";
  */
 
 /*
@@ -47,22 +47,19 @@ static void write_map_log()
 	return;
     }
     for (map=first_map; map!=NULL; map=map->next) {
-	if (map->in_memory != MAP_IN_MEMORY)
-/* The format is the same (so recompiling won't really affect this.
- * However, in one case that element of the structure is available, so
- * put it in.
- */
-#if defined(UNIQUE_ITEMS) && defined(LOCK_ITEMS)
-	    fprintf(fp,"%s:%s:%ld:%d:%d:%d:%d:%d\n", map->path, map->tmpname,
-		    map->reset_time-current_time, map->lock_items,
+	/* If tmpname is null, it is probably a unique player map,
+	 * so don't save information on it.
+	 */
+	if (map->in_memory != MAP_IN_MEMORY && (map->tmpname !=NULL)) {
+	    /* the 0 written out is a leftover from the lock number for
+	     * unique items.  Keep using it so that old temp files continue
+	     * to work.
+	     */
+		fprintf(fp,"%s:%s:%ld:0:%d:%d:%d:%d\n", map->path, map->tmpname,
+		    (map->reset_time==-1 ? -1: map->reset_time-current_time),
 		    map->encounter, map->difficulty, map->do_los,
 		    map->darkness);
-#else
-	    fprintf(fp,"%s:%s:%ld:0:%d:%d:%d:%d\n", map->path, map->tmpname,
-		    map->reset_time-current_time, map->lock_items,
-		    map->encounter, map->difficulty, map->do_los,
-		    map->darkness);
-#endif
+	}
     }
     fclose(fp);
 }
@@ -73,7 +70,7 @@ void read_map_log()
     mapstruct *map;
     char buf[MAX_BUF],*cp,*cp1;
     long current_time=time(NULL);
-    int encounter, do_los, darkness;
+    int encounter, do_los, darkness, lock;
 
     sprintf(buf,"%s/temp.maps", settings.libdir);
     if (!(fp=fopen(buf,"r"))) {
@@ -92,22 +89,14 @@ void read_map_log()
 	cp1=strchr(cp,':');
 	*cp1++='\0';
 	map->tmpname=strdup_local(cp);
-/* The format is the same (so recompiling won't really affect this.
- * However, in one case that element of the structure is available, so
- * put it in.
- */
-#if defined(UNIQUE_ITEMS) && defined(LOCK_ITEMS)
+
+	/* Lock is left over from the lock items - we just toss it now */
 	sscanf(cp1,"%ld:%d:%d:%hd:%d:%d\n",
-		    &map->reset_time, &map->lock_items,
+		    &map->reset_time, &lock,
 		    &encounter, &map->difficulty, &do_los,
 		    &darkness);
-#else
-	sscanf(cp1,"%ld:%d:%d:%hd:%d:%d\n",
-		    &map->reset_time, &map->lock_items,
-		    &encounter, &map->difficulty, &do_los,
-		    &darkness);
-#endif
-	map->reset_time += current_time;
+	if (map->reset_time != -1)
+	    map->reset_time += current_time;
 	map->in_memory=MAP_SWAPPED;
 	map->darkness=darkness;
 	/* The following two are bit fields, so can't be done above */
@@ -138,6 +127,10 @@ void swap_map(mapstruct *map) {
 
   if (new_save_map (map, 0) == -1) {
     LOG(llevError, "Failed to swap map %s.\n", map->path);
+    /* need to reset the in_memory flag so that delete map will also
+     * free the objects with it.
+     */
+    map->in_memory = MAP_IN_MEMORY;
     delete_map(map);
   } else
     free_map(map,1);
@@ -147,9 +140,10 @@ void swap_map(mapstruct *map) {
 }
 
 void check_active_maps() {
-  mapstruct *map;
+  mapstruct *map, *next;
 
-  for(map=first_map;map!=NULL;map=map->next) {
+  for(map=first_map;map!=NULL;map=next) {
+    next = map->next;
     if(map->in_memory != MAP_IN_MEMORY)
       continue;
     if(map->need_refresh) {
@@ -160,7 +154,10 @@ void check_active_maps() {
       continue;
     if( --(map->timeout) > 0)
       continue;
+    /* If LWM is set, we only swap maps out when we run out of objects */
+#ifndef MAX_OBJECTS_LWM
     swap_map(map);
+#endif
   }
 }
 
@@ -185,17 +182,24 @@ mapstruct *map_least_timeout(char *except_level) {
  */
 
 void swap_below_max(char *except_level) {
-  mapstruct *map;
+    mapstruct *map;
 
-  for(;;) {
     if(nrofallocobjects - nroffreeobjects < MAX_OBJECTS)
-      return;
-    if ((map = map_least_timeout(except_level)) == NULL)
-      return;
-    LOG(llevDebug,"Trying to swap out %s before its time.\n", map->path);
-    map->timeout=0;
-    swap_map(map);
-  }
+	return;
+    for(;;) {
+#ifdef MAX_OBJECTS_LWM
+	if(nrofallocobjects - nroffreeobjects < MAX_OBJECTS_LWM)
+	    return;
+#else
+	if(nrofallocobjects - nroffreeobjects < MAX_OBJECTS)
+	    return;
+#endif
+	if ((map = map_least_timeout(except_level)) == NULL)
+	    return;
+	LOG(llevDebug,"Trying to swap out %s before its time.\n", map->path);
+	map->timeout=0;
+	swap_map(map);
+    }
 }
 
 /*
