@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_time_c =
- *    "$Id: time.c,v 1.33 1997/03/09 08:20:46 master Exp master $";
+ *    "$Id: time.c,v 1.40 1998/09/28 04:10:33 master Exp master $";
  */
 
 /*
@@ -190,18 +190,18 @@ void poison_more(object *op) {
   if(op->env->type==PLAYER) {
     op->env->stats.food--;
     new_draw_info(NDI_UNIQUE, 0,op->env,"You feel very sick...");
-    draw_stats(op->env);
   }
   (void)hit_player(op->env,
                    op->stats.dam,
-                   op,AT_PHYSICAL);
+                   op,AT_INTERNAL);
 }
 
 
 void move_gate(object *op) { /* 1 = going down, 0 = goind up */
   object *tmp;
-  if(op->stats.wc < 0 || (int)op->stats.wc  > (int)op->arch->animations-1) {
-    LOG(llevError,"Gate error: animation was %d\n",op->stats.wc);
+  if(op->stats.wc < 0 || (int)op->stats.wc  >= NUM_ANIMATIONS(op)) {
+    LOG(llevError,"Gate error: animation was %d, max=%d\n",op->stats.wc,
+	NUM_ANIMATIONS(op));
     dump_object(op);
     LOG(llevError,"%s\n",errmsg);
     op->stats.wc=0;
@@ -215,21 +215,26 @@ void move_gate(object *op) { /* 1 = going down, 0 = goind up */
 	op->speed = 0;
 	update_ob_speed(op);
       }
+#if 0	/* Dont think this is needed, as the section below should be hit */
+      CLEAR_FLAG(op, FLAG_NO_PASS);
+      CLEAR_FLAG(op, FLAG_BLOCKSVIEW);
+      update_all_los(op->map);
+#endif
+    }
+    if((int)op->stats.wc < (NUM_ANIMATIONS(op)/2+1)) {
       CLEAR_FLAG(op, FLAG_NO_PASS);
       CLEAR_FLAG(op, FLAG_BLOCKSVIEW);
       update_all_los(op->map);
     }
-    if((int)op->stats.wc <= ((int)op->arch->animations-1)/2+1) {
-      CLEAR_FLAG(op, FLAG_NO_PASS);
-      CLEAR_FLAG(op, FLAG_BLOCKSVIEW);
-      update_all_los(op->map);
-    }
-    op->face=&new_faces[op->arch->faces[op->stats.wc]];
+    SET_ANIMATION(op, op->stats.wc);
     update_object(op);
     return;
   }
   /* We're going up */
-  if((unsigned char) op->stats.wc==(op->arch->animations-1)&&op->above==NULL) {
+
+  /* First, lets see if we are already at the top */
+  if((unsigned char) op->stats.wc==(NUM_ANIMATIONS(op)-1)
+     &&op->above==NULL) {
     if(op->arch->clone.speed)
       op->value=1;
     else {
@@ -239,14 +244,16 @@ void move_gate(object *op) { /* 1 = going down, 0 = goind up */
     return;
   }
   if(op->stats.food) {    /* The gate is going temporarily down */
-    if(--op->stats.wc<=0) /* Gone all the way down? */
-      op->stats.food=0,   /* Then let's try again */
+    if(--op->stats.wc<=0) { /* Gone all the way down? */
+      op->stats.food=0;	    /* Then let's try again */
       op->stats.wc=0;
+    }
   } else {                /* The gate is still going up */
     op->stats.wc++;
-    if((int)op->stats.wc > ((int)op->arch->animations-1))
-      op->stats.wc=(signed char)op->arch->animations-1;
-    if((int)op->stats.wc >= (int)op->arch->animations/2) {
+    if((int)op->stats.wc >= (NUM_ANIMATIONS(op)))
+      op->stats.wc=(signed char)NUM_ANIMATIONS(op)-1;
+
+    if((int)op->stats.wc >= NUM_ANIMATIONS(op)/2) {
       /* Halfway or further, check blocks */
       for(tmp=op->above;tmp!=NULL&&tmp->above!=NULL;tmp=tmp->above); /* top */
       if(tmp!=NULL) {
@@ -274,7 +281,7 @@ void move_gate(object *op) { /* 1 = going down, 0 = goind up */
       }
     }
   }
-  op->face=&new_faces[op->arch->faces[(unsigned char)op->stats.wc]];
+  SET_ANIMATION(op, op->stats.wc);
   update_object(op);
 }
 
@@ -301,13 +308,57 @@ void move_timed_gate(object *op)
   }
 }
 
+/*  slaying:    name of the thing the detector is to look for
+ *	 speed:      frequency of 'glances'
+ *	 connected:  connected value of detector
+ *  sp:         1 if detection sets buttons
+ *              -1 if detection unsets buttons
+ */
+
+void move_detector(object *op) 
+{
+  object *tmp;
+  int last = op->value;
+  int detected;
+  detected = 0;
+  for(tmp = get_map_ob(op->map,op->x,op->y);tmp!=NULL&&!detected;tmp=tmp->above)
+    if (!strcmp(op->slaying,tmp->name))
+		{
+		  detected = 1;
+		}
+  
+  if(op->stats.sp == 1) 
+	 {
+		if(detected && last == 0) {
+		  op->value = 1;
+		  update_button(op);
+		}
+		if(!detected && last == 1) {
+		  op->value = 0;
+		  update_button(op);
+		}
+	 }
+  else
+	 {
+		if(detected && last == 1) {
+		  op->value = 0;
+		  update_button(op);
+		}
+		if(!detected && last == 0) {
+		  op->value = 1;
+		  update_button(op);
+		}
+	 }
+}
+
+
 void animate_trigger (object *op)
 {
-  if((unsigned char)++op->stats.wc >= op->arch->animations-1) {
+  if((unsigned char)++op->stats.wc >= NUM_ANIMATIONS(op)) {
     op->stats.wc = 0;
     check_trigger(op);
   }
-  op->face=&new_faces[op->arch->faces[op->stats.wc]];
+  SET_ANIMATION(op, op->stats.wc);
   update_object(op);
 }
 
@@ -318,21 +369,21 @@ void move_hole(object *op) { /* 1 = opening, 0 = closing */
       op->speed = 0;
       update_ob_speed(op);
       SET_FLAG(op, FLAG_WALK_ON);
-      while(op->above!=NULL && apply(op->above,op))
+      while(op->above!=NULL && apply(op->above,op,0))
         ;
     }
-    op->face=&new_faces[op->arch->faces[op->stats.wc]];
+    SET_ANIMATION(op, op->stats.wc);
     update_object(op);
     return;
   }
   /* We're closing */
   CLEAR_FLAG(op, FLAG_WALK_ON);
   op->stats.wc++;
-  if((int)op->stats.wc > (int)op->arch->animations-1)
-    op->stats.wc=op->arch->animations-1;
-  op->face=&new_faces[op->arch->faces[op->stats.wc]];
+  if((int)op->stats.wc >= NUM_ANIMATIONS(op))
+    op->stats.wc=NUM_ANIMATIONS(op)-1;
+  SET_ANIMATION(op, op->stats.wc);
   update_object(op);
-  if((unsigned char) op->stats.wc==op->arch->animations-1) {
+  if((unsigned char) op->stats.wc==(NUM_ANIMATIONS(op)-1)) {
     op->speed = 0;
     update_ob_speed(op); /* closed, let's stop */
     return;
@@ -348,144 +399,175 @@ void move_hole(object *op) { /* 1 = opening, 0 = closing */
 
 void stop_arrow(object *op,object *tmp) {
 
-  if(wall(op->map,op->x,op->y))
-    op->x-=DIRX(op),op->y-=DIRY(op);
-  if(wall(op->map, op->x, op->y))
-  {
-    free_object(op);
-    return;
-  }
-  if(RANDOM() % 100 < op->stats.food) {
-    /* Small chance of breaking */
-    free_object(op);
-    return;
-  }
-  op->direction=0;
-  CLEAR_FLAG(op, FLAG_WALK_ON);
-  CLEAR_FLAG(op, FLAG_FLY_ON);
-  CLEAR_FLAG(op, FLAG_FLYING);
-  op->speed = 0;
-  update_ob_speed(op);
-  op->stats.wc = op->stats.sp;
-  op->stats.dam= op->stats.hp;
-  /* Reset these to zero, so that CAN_MERGE will work properly */
-  op->stats.sp = 0;
-  op->stats.hp = 0;
-  op->face=op->arch->clone.face;
+    if(wall(op->map,op->x,op->y))
+	op->x-=DIRX(op),op->y-=DIRY(op);
+    if(wall(op->map, op->x, op->y)) {
+	free_object(op);
+	return;
+    }
 
-  /* this happens for thrown objects, which 'carry' the
-   * real object in inventory. */ 
-  if(op->inv) {
-    object *old=op,*new=op->inv;
-    remove_ob(op->inv);
-    new->map = old->map; 
-    free_object(old);
-    op = new;
-  }
+    if(RANDOM() % 100 < op->stats.food) {
+	/* Small chance of breaking */
+	free_object(op);
+	return;
+    }
 
-  /* If the missile hit a player, we insert it in their inventory.
-   * However, if the missile is heavy, we don't do so (assume it falls
-   * to the ground after a hit).  What a good value for this is up to
-   * debate - 5000 is 5 kg, so arrows, knives, and other light weapons
-   * stick around.
-   */
-  if(op->weight <= 5000 && tmp!=NULL&&tmp->stats.hp>=0) {
-    if(tmp->head != NULL)
-      tmp = tmp->head;
-      op = insert_ob_in_ob(op,tmp);
-    if (tmp->type== PLAYER && tmp->contr->eric_server > 0)
-      esrv_send_item (tmp, op);
-  } else
-    insert_ob_in_map(op,op->map);
-  op->owner=NULL; /* So that stopped arrows will be saved */
+    op->direction=0;
+    CLEAR_FLAG(op, FLAG_WALK_ON);
+    CLEAR_FLAG(op, FLAG_FLY_ON);
+    CLEAR_FLAG(op, FLAG_FLYING);
+    op->speed = 0;
+    update_ob_speed(op);
+    op->stats.wc = op->stats.sp;
+    op->stats.dam= op->stats.hp;
+    /* Reset these to zero, so that CAN_MERGE will work properly */
+    op->stats.sp = 0;
+    op->stats.hp = 0;
+    op->face=op->arch->clone.face;
+
+    /* this happens for thrown objects, which 'carry' the
+     * real object in inventory. */ 
+    if(op->inv) {
+	object *old=op,*new=op->inv;
+	remove_ob(op->inv);
+	new->map = old->map; 
+	free_object(old);
+	op = new;
+    }
+
+    /* If the missile hit a player, we insert it in their inventory.
+     * However, if the missile is heavy, we don't do so (assume it falls
+     * to the ground after a hit).  What a good value for this is up to
+     * debate - 5000 is 5 kg, so arrows, knives, and other light weapons
+     * stick around.
+     */
+    if(op->weight <= 5000 && tmp!=NULL&&tmp->stats.hp>=0) {
+	if(tmp->head != NULL)
+	    tmp = tmp->head;
+	op = insert_ob_in_ob(op,tmp);
+	if (tmp->type== PLAYER)
+	    esrv_send_item (tmp, op);
+    } else
+	insert_ob_in_map(op,op->map);
+    op->owner=NULL; /* So that stopped arrows will be saved */
 }
 
-/* now involves thrown objects too */
+
+/* Move an arrow along its course.  op is the arrow or thrown object.
+ */
+
 void move_arrow(object *op) {
-  object *tmp;
+    object *tmp;
+
+    if(op->map==NULL) {
+	LOG(llevError,"Arrow had no map.\n");
+	remove_ob(op);
+	free_object(op);
+	return;
+    }
 
-  if(op->map==NULL) {
-    LOG(llevError,"Arrow had no map.\n");
     remove_ob(op);
-    free_object(op);
-    return;
-  }
-  remove_ob(op);
 
-  /* we need to stop thrown objects at some point. Like here. */ 
-  if(op->type==THROWN_OBJ) {
-     if(op->last_sp-- < 0) { 
- 	stop_arrow(op, NULL); 
-	return; 
-     } 
-  }
+    /* we need to stop thrown objects at some point. Like here. */ 
+    if(op->type==THROWN_OBJ) {
+	if(op->last_sp-- < 0) { 
+	    stop_arrow(op, NULL); 
+	    return; 
+	}
+    }
 
-  if(wall(op->map,op->x+DIRX(op),op->y+DIRY(op)))
-    if(!QUERY_FLAG(op, FLAG_REFLECTING) || !(RANDOM()%20)) {
-      stop_arrow(op,NULL);
-      return;
-    } else {
-      if(op->direction&1)
-	op->direction=absdir(op->direction+4);
-      else {
-        int left= wall(op->map,op->x+freearr_x[absdir(op->direction-1)],
-		       op->y+freearr_y[absdir(op->direction-1)]),
-	right=wall(op->map,op->x+freearr_x[absdir(op->direction+1)],
-		   op->y+freearr_y[absdir(op->direction+1)]);
-        if(left==right)
-          op->direction=absdir(op->direction+4);
-        else if(left)
-          op->direction=absdir(op->direction+2);
-        else if(right)
-          op->direction=absdir(op->direction-2);
-      }
-      if(wall(op->map,op->x+DIRX(op),op->y+DIRY(op)))
-	{
-	  int left= wall(op->map,op->x+freearr_x[absdir(op->direction-1)],
-			 op->y+freearr_y[absdir(op->direction-1)]),
-	  right=wall(op->map,op->x+freearr_x[absdir(op->direction+1)],
-		     op->y+freearr_y[absdir(op->direction+1)]);
-	  if(!left)
-	    op->direction=absdir(op->direction-1);
-	  else if(!right)
-	    op->direction=absdir(op->direction+1);
-	  else {		/* is this possible? */
+
+    if(wall(op->map,op->x+DIRX(op),op->y+DIRY(op))) {
+	/* if the object doesn't reflect, stop the arrow from moving */
+	if(!QUERY_FLAG(op, FLAG_REFLECTING) || !(RANDOM()%20)) {
 	    stop_arrow(op,NULL);
 	    return;
-	  }
-	}
-      /* many thrown objects *don't* have more than one face */
-      if(op->arch->faces)
-	op->face=&new_faces[op->arch->faces[op->direction]];
-  }
-  op->x+=DIRX(op),op->y+=DIRY(op);
-  tmp=get_map_ob(op->map,op->x,op->y);
-  while(tmp!=NULL&&!QUERY_FLAG(tmp, FLAG_ALIVE))
-    tmp=tmp->above;
+	} else {    /* object is reflected */
+	    /* If one of the major directions (n,s,e,w), just reverse it */
+	    if(op->direction&1) {
+		op->direction=absdir(op->direction+4);
+	    } else {
+		/* The below is just logic for figuring out what direction
+		 * the object should now take.
+		 */
+	
+		int left= wall(op->map,op->x+freearr_x[absdir(op->direction-1)],
+		       op->y+freearr_y[absdir(op->direction-1)]),
+		right=wall(op->map,op->x+freearr_x[absdir(op->direction+1)],
+		   op->y+freearr_y[absdir(op->direction+1)]);
 
-  if(tmp==NULL) {
-    insert_ob_in_map(op,op->map);
-    return;
-  }
-  if (QUERY_FLAG(tmp, FLAG_REFL_MISSILE)){
-    int number = op->face->number;
-    op->direction=absdir(op->direction+4),op->state=0;
-    if(op->arch->faces) { 
-      number+=4;
-      if(number > op->arch->faces[8])
-        number-=8;
-      op->face = &new_faces[number];
+		if(left==right)
+		    op->direction=absdir(op->direction+4);
+		else if(left)
+		    op->direction=absdir(op->direction+2);
+		else if(right)
+		    op->direction=absdir(op->direction-2);
+	    }
+	    /* Is the new direction also a wall?  If show, shuffle again */
+	    if(wall(op->map,op->x+DIRX(op),op->y+DIRY(op))) {
+		int left= wall(op->map,op->x+freearr_x[absdir(op->direction-1)],
+			 op->y+freearr_y[absdir(op->direction-1)]),
+		right=wall(op->map,op->x+freearr_x[absdir(op->direction+1)],
+		     op->y+freearr_y[absdir(op->direction+1)]);
+
+		if(!left)
+		    op->direction=absdir(op->direction-1);
+		else if(!right)
+		    op->direction=absdir(op->direction+1);
+		else {		/* is this possible? */
+		    stop_arrow(op,NULL);
+		    return;
+		}
+	    }
+	    /* update object image for new facing */
+	    /* many thrown objects *don't* have more than one face */
+	    if(GET_ANIM_ID(op))
+		SET_ANIMATION(op, op->direction);
+	} /* object is reflected */
+    } /* object ran into a wall */
+
+    op->x+=DIRX(op),op->y+=DIRY(op);
+    tmp=get_map_ob(op->map,op->x,op->y);
+
+    /* See if there is any living object on this space */
+    while(tmp!=NULL&&!QUERY_FLAG(tmp, FLAG_ALIVE))
+	tmp=tmp->above;
+
+    /* Nothing alive?  Insert arrow and return */
+    if(tmp==NULL) {
+	insert_ob_in_map(op,op->map);
+	return;
+    }
+
+    /* Found living object, but it is reflecting the missile.  Update
+     * as below.
+     */
+    if (QUERY_FLAG(tmp, FLAG_REFL_MISSILE)) {
+	int number = op->face->number;
+
+	op->direction=absdir(op->direction+4),op->state=0;
+	if(GET_ANIM_ID(op)) {
+	    number+=4;
+	    if(number > GET_ANIMATION(op, 8))
+		number-=8;
+	    op->face = &new_faces[number];
+	}
+	insert_ob_in_map(op,op->map);
+	return;
+    }
+
+    /* Attach the object.  IF successful, stop the arrow */
+    if(attack_ob(tmp,op))
+	stop_arrow(op,tmp);
+    else {
+	/* if we miss, insert it back into the map.  If no direction,
+	 * stop the arrow.
+	 */
+	if(op->direction)
+	    insert_ob_in_map(op,op->map);
+	else 
+	    stop_arrow(op,NULL);
     }
-    insert_ob_in_map(op,op->map);
-    return;
-  }
-  if(attack_ob(tmp,op))
-    stop_arrow(op,tmp);
-  else 
-    if(op->direction)
-      insert_ob_in_map(op,op->map);
-    else 
-      stop_arrow(op,NULL);
 }
 
 /* This routine doesnt seem to work for "inanimate" objects that
@@ -517,7 +599,7 @@ void change_object(object *op) { /* Doesn`t handle linked objs yet */
     } else {
         j=find_first_free_spot(tmp->arch,op->map,op->x,op->y);
 	if (j==-1)  /* No free spot */
-	    free(tmp);
+	    free_object(tmp);
 	else {
 	    tmp->x=op->x+freearr_x[j],tmp->y=op->y+freearr_y[j];
 	    insert_ob_in_map(tmp,op->map);
@@ -566,20 +648,22 @@ void move_firechest(object *op) {
 
 
 /*  move_player_mover:  this function takes a "player mover" as an
-argument, and performs the function of a player mover, which is:
+ * argument, and performs the function of a player mover, which is:
 
-  a player mover finds any players that are sitting on it.  It
-moves them in the op->stats.sp direction.  speed is how often it'll move.
-If attacktype is nonzero it will paralyze the player.  If lifesave is set,
-it'll dissapear after hp+1 moves.  If hp is set and attacktype is set,
-it'll paralyze the victim for hp*his speed/op->speed
+ * a player mover finds any players that are sitting on it.  It
+ * moves them in the op->stats.sp direction.  speed is how often it'll move.
+ * If attacktype is nonzero it will paralyze the player.  If lifesave is set,
+ * it'll dissapear after hp+1 moves.  If hp is set and attacktype is set,
+ * it'll paralyze the victim for hp*his speed/op->speed
 
 */
 void move_player_mover(object *op) {
   object *victim, *nextmover;
+
   for(victim=get_map_ob(op->map,op->x,op->y); victim !=NULL; victim=victim->above)
     if(QUERY_FLAG(victim, FLAG_ALIVE)&& (!(QUERY_FLAG(victim,FLAG_FLYING))||op->stats.maxhp))
       {
+
       if(QUERY_FLAG(op,FLAG_LIFESAVE)&&op->stats.hp--<0) {
 	remove_ob(op);
 	free_object(op);
@@ -594,10 +678,20 @@ void move_player_mover(object *op) {
 	}
       }
 
-      if(victim->type==PLAYER)  /*  only level >=1 movers move people */
-         { if(op->level) move_player(victim, op->stats.sp?op->stats.sp:(RANDOM()%8)+1);
-		else return;
-	 }
+      if(victim->type==PLAYER) { 
+	/*  only level >=1 movers move people */
+	    if(op->level) {
+		/* Following is a bit of hack.  We need to make sure it
+		 * is cleared, otherwise the player will get stuck in
+		 * place.  This can happen if the player used a spell to
+		 * get to this space.
+		 */
+		victim->contr->fire_on=0;
+		victim->speed_left=-FABS(victim->speed);
+		move_player(victim, op->stats.sp?op->stats.sp:(RANDOM()%8)+1);
+	    }
+	    else return;
+      }
       else move_object(victim,op->stats.sp?op->stats.sp:(RANDOM()%8)+1);
 
       if(!op->stats.maxsp&&op->attacktype) op->stats.maxsp=2.0;
@@ -623,6 +717,10 @@ void move_creator(object *op) {
   if(op->stats.hp < 0 && !QUERY_FLAG(op,FLAG_LIFESAVE)) 
 	{ op->stats.hp=-1;return;}
   tmp=arch_to_object(op->other_arch);
+  if(op->slaying) {
+	 tmp->name = add_string(op->slaying);
+	 tmp->title = add_string(op->slaying);
+  }
   tmp->x=op->x;tmp->y=op->y;tmp->map=op->map;tmp->level=op->level;
   insert_ob_in_map(tmp,op->map);
 }
@@ -740,6 +838,8 @@ int process_object(object *op) {
   case TRIGGER_ALTAR:
     animate_trigger(op);
     return 0;
+  case DETECTOR:
+	 move_detector(op);
   case DIRECTOR:
     if (op->stats.maxsp)
       animate_turning(op);
@@ -765,6 +865,9 @@ int process_object(object *op) {
   case PLAYERMOVER:
     move_player_mover(op);
     return 0;
+  case CREATOR:
+	 move_creator(op);
+	 return 0;
   }
 
   return 0;
